// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package algo;

import java.util.Enumeration;
import java.util.Vector;

import model.Haltestelle;
import model.IFahrt;
import model.Linie;
import model.RoutenAbschnitt;

public class DataWrapperReverse implements IDataWrapper {

	public Haltestelle getNextHaltestelle(IFahrt aufFahrt, Haltestelle haltestelle) {
		return aufFahrt.getNextHaltestelleReverse(haltestelle);
	}

	public int getTimeAtNextHaltestelle(IFahrt fahrt, Haltestelle aktHaltestelle, int zeit, Linie linie) {
		return fahrt.getTimeAtNextHaltestelleReverse(aktHaltestelle, zeit, linie);
	}

	public boolean isSmallerThan(int newDistance, int oldDistance) {
		// Note: This is reverse!
		return newDistance > oldDistance;
	}
	
	public boolean isGreaterThan(int newDistance, int oldDistance) {
		// Note: This is reverse!
		return newDistance < oldDistance;
	}

	public boolean isEqualOrGreaterThan(int newDistance, int oldDistance) {
		// Note: This is reverse!
		return newDistance < oldDistance || newDistance == oldDistance;
	}

	public boolean isEqualOrSmallerThan(int newDistance, int oldDistance) {
		// Note: This is reverse!
		return newDistance > oldDistance || newDistance == oldDistance;
	}
	
	public int addTime(int time, int add) {
		// Note: This is reverse!
		return time - add;
	}

	public RoutenAbschnitt[] buildRoutenAbschnittArray(Vector routenAbschnitte) {
		// Reihenfolge wird nicht umgedreht, weil Algo schon rückwärts lief.
		// Dafür muss man aber in jedem RoutenAbschnitt von und nach mit
		// Ankunfts- und Abfahrtszeit vertauschen.
		RoutenAbschnitt[] routenAbschnitteAsArray = new RoutenAbschnitt[routenAbschnitte.size()];
		int i = 0;
		for (Enumeration e = routenAbschnitte.elements(); e.hasMoreElements();) {
			RoutenAbschnitt r = (RoutenAbschnitt) e.nextElement();
			routenAbschnitteAsArray[i] =
					new RoutenAbschnitt(
						r.getNach(),
						r.getVon(),
						r.getAnkunftsZeit(),
						r.getAbfahrtsZeit(),
						r.getFahrt());
			i++;
		}
		return routenAbschnitteAsArray;
	}

	public int unendlichSpaet() {
		return Integer.MIN_VALUE;
	}
}
