// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package control;

import java.io.DataInputStream;

import model.Laufweg;
import model.Verkehrsmittel;
import data.persistence.UserSettingsInt;
import dev.Logger;

public final class Global {
	
	// Wenn der push&ride Code von extern verwendet wird, muss das hier auf false gesetzt werden!!!
	public static boolean ISMIDLET = true;
	
	public static boolean IS_FAKE_ANDROID = false;
	
	// ******************* VERSIONSVERWALTUNG ********************
	private static final String SVNLastChangedRevision = "$LastChangedRevision: 2763 $";
	public static final String REVISION = SVNLastChangedRevision.substring(22, SVNLastChangedRevision.length() - 2);

	// ******************* LOGGING *******************
	public static final boolean SHOW_DEBUG_OUTPUT = false;
	public static final boolean SHOW_MEMORY_INFO = false;
	public static final boolean SHOW_MEMINFO = false;

	// ******************* GUI *******************
	public static final boolean REMINDER_ALWAYS_VISIBLE = false;
	public static final boolean STARTSCREEN_BERECHNE_ROUTEN = true;
	public static boolean OVERRIDE_GFX_SET_LOW_QUALITY = false;

	// ******************* ALGORITHMUS *******************
	// Fasst Streckenabschnitte zusammen, wenn sie auf der gleichen Linie liegen
	public static boolean USE_KOMPAKTE_ROUTENABSCHNITTE = true;

	// Zeit die zum Umsteigen angenommen wird (in Minuten)
	private static final byte INIT_UMSTEIGEZEIT = 2;
	private static final byte INIT_UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL = 3;
	private static final byte INIT_UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL_VONAUF_BUS = 4;
	public static int UMSTEIGEZEIT = INIT_UMSTEIGEZEIT;
	public static int UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL = INIT_UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL;
	public static int UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL_VONAUF_BUS = INIT_UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL_VONAUF_BUS;

	// Bestimmt, ob der Routen Cache aktiviert wird
	public static boolean USE_ROUTE_CACHE = true;

	// Im Cache sind Routen Objekte gespeichert - der benötigte Speicher ist
	// doppelt so groß wie hier angegeben, da hin und rück ein eigener Cache ist.
	public static final int ROUTE_CACHE_SIZE = 30; // * 2

	// ******************* PERSISTENCE *******************
	public static final boolean FORCE_EMPTY_RECORDSTORE = false;
	// data.persistence.LastHaltestellen
	public static final int NUM_LAST_HALTESTELLEN_SAVED = 20;
	// data.persistence.LastHaltestellen
	public static final int NUM_RETUNED_LAST_HALTESTELLEN = 4;

	// ******************* TAGE *******************
	public static final int TAG_UNBEKANNT = 0;
	public static final int TAG_WOCHENTAG_FERIEN = 1;
	public static final int TAG_SAMSTAG = 2;
	public static final int TAG_SONN_FEIERTAG = 3;
	public static final int TAG_WOCHENTAG_SCHULE = 4;
	public static final int[] TAGE_NUMMERN = { TAG_UNBEKANNT, TAG_WOCHENTAG_FERIEN, TAG_SAMSTAG, TAG_SONN_FEIERTAG, TAG_WOCHENTAG_SCHULE };
	public static final String[] TAGE_NAMEN = { "TAG_UNBEKANNT", "TAG_WOCHENTAG_FERIEN", "TAG_SAMSTAG", "TAG_SONN_FEIERTAG", "TAG_WOCHENTAG_SCHULE" };

	// ******************* ZEIT *******************
	public static final boolean SET_TIME_TO_DEFAULT_VALUE = false;
	public static final int TIME_ERROR = -987654321;

	// ******************* DELAY *******************
	public static final int TIME_MIN_STARTUP_MS = 2000;
	public static final int TIME_MIN_BERECHNE_MS = 2000;

	// ******************* REPLIKATION *******************
	//	public static final String SELF_RESOURCE_NAME = "/res/data.bin";
	public static final String SELF_RESOURCE_NAME = "/res/PushnRide.jar";
	public static final String SELF_REPLICATE_HEADERNAME = "/res/binary1";
	public static final String SELF_REPLICATE_NAME = "/res/binary2";
	public static final int REPLICATE_UNKNOWN = -1;
	public static final int REPLICATE_NONE = 0;
	public static final int REPLICATE_ONCE = 1;
	public static final int REPLICATE_SPECIAL = 2;
	public static final String PREFIX_FULL_VERSION = "aja.";

	// ************** URLs **************************//
	// Download Link der per SMS versendet wird.
	public static final String PUSHNRIDE_DOWNLOADLINK = "http://m.pushandride.de/";
	// Handy Nummer fuer Feedback SMS
	public static final String FEEDBACK_TELEPHON_NUMBER = "123";

	// ************** REMINDER **************************//
	public static final int REMINDER_LAUFZEIT[] = { 5, 7, 10, 30 };

	// ******************* LAUFWEGE *******************
	// LAUFWEGE_ZEIT_FAKTOR == 50 entspricht 5
	// LAUFWEGE_ZEIT_FAKTOR == 10 entspricht 1
	// LAUFWEGE_ZEIT_FAKTOR == 1 entspricht 0.1
	public static int LAUFWEGE_ZEIT_FAKTOR = -1;

	public static void setNewLaufzeitFaktor(int value) {
		if (LAUFWEGE_ZEIT_FAKTOR < 0 || LAUFWEGE_ZEIT_FAKTOR != value) {
			LAUFWEGE_ZEIT_FAKTOR = value;

			// Umsteigezeiten berechnen
			Global.UMSTEIGEZEIT = Laufweg.calcLaufzeit(Global.INIT_UMSTEIGEZEIT, Global.LAUFWEGE_ZEIT_FAKTOR);
			Global.UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL = Laufweg.calcLaufzeit(Global.INIT_UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL, Global.LAUFWEGE_ZEIT_FAKTOR);
			Global.UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL_VONAUF_BUS = Laufweg.calcLaufzeit(Global.INIT_UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL_VONAUF_BUS, Global.LAUFWEGE_ZEIT_FAKTOR);
		}
	}

	public static int getLaufzeitFaktor() {
		if (LAUFWEGE_ZEIT_FAKTOR == -1) {
			if (!ISMIDLET) {
				setNewLaufzeitFaktor(10);
			} else {
				int neuerFaktor = UserSettingsInt.get(UserSettingsInt.SETTINGS_LAUFZEIT_FAKTOR);
				if (neuerFaktor == UserSettingsInt.NULL_VALUE) {
					neuerFaktor = 10;
				}
				setNewLaufzeitFaktor(neuerFaktor);
			}
		}
		return LAUFWEGE_ZEIT_FAKTOR;
	}

	public static String getMidletVersion() {
		int revision = Integer.parseInt(REVISION);
		if (revision < 1000 || revision > 9999) {
			return "0.0.1";
		} else {
			String major = "1";
			String minor = Integer.parseInt(REVISION.substring(0, 2)) + "";
			String subminor = Integer.parseInt(REVISION.substring(2)) + "";
			return major + "." + minor + "." + subminor;
		}
	}

	// ******************* DATENSATZ *******************
	public static String haltestelleFlughafenUniqueId = "2103";
	
	public static final int BINARY_INFO_URL = 0;
	public static final int BINARY_INFO_SETS = 1;
	public static final int BINARY_INFO_LINIEN = 2;
	public static final int BINARY_INFO_SETS_PART = 3;

	private static String[] binaryInfo = null;

	// TODO: Die Funktionen gehören eigentlich in die Data-Klasse, die auch "static" ist.
	/**
	 * Gibt die beim Download ausgewählten Linien sowie die URL zurück
	 */
	public static String[] getBinaryInfo() {
		if (binaryInfo != null) {
			return binaryInfo;
		}
		String linienString;
		String downloadUrl;
		String setsString;
		String setsPartString;
		DataInputStream is = null;
		try {
			is = new DataInputStream(Global.class.getResourceAsStream("/res/bininfo"));
			setsString = is.readUTF();
			setsPartString = is.readUTF();
			linienString = is.readUTF();
			downloadUrl = is.readUTF();
		} catch (Exception e) {
			setsString = "";
			linienString = "keine Information vorhanden";
			downloadUrl = "jad7.de";
			setsPartString = "";
			Logger.fatalException(Global.class, e, "konnte /res/bininfo nicht richtig lesen (downloadUrl)");
		}
		try {
			is.close();
		} catch (Exception e) {
			Logger.fatalException(Global.class, e, "asxis");
		}
		String[] result = new String[4];
		result[BINARY_INFO_SETS] = setsString;
		result[BINARY_INFO_LINIEN] = linienString;
		result[BINARY_INFO_URL] = downloadUrl;
		result[BINARY_INFO_SETS_PART] = setsPartString; 
		binaryInfo = result;
		return result;
	}
	
	/**
	 * Gibt die download-URL der installierten Version zurück (z.B: dgj.jad7.de)
	 */
	public static String getDownloadUrl() {
		String s = getBinaryInfo()[BINARY_INFO_URL];
		if (s.equals("")) {
			return "keine/unbekannt";
		} else {
			return s;
		}
	}

	/**
	 * Gibt die komplett enthaltenen Fahrmittel als String zurück
	 */
	public static String getSetInfo() {
		String setsString = getBinaryInfo()[BINARY_INFO_SETS];
		if (setsString.equals("")) {
			return "keine/unbekannt";
		} else {
			String result = "";
			int anz = setsString.length();
			if (anz > 1) {
				boolean first = true;
				for (int i = 0; i < anz - 1; i++) {
					if (!first) {
						result += ", ";
					}
					result += Verkehrsmittel.VERKEHRSMITTEL_NAMEN[Integer.parseInt(setsString.charAt(i) + "")];
					first = false;
				}
				result += " und ";
			}
			result += Verkehrsmittel.VERKEHRSMITTEL_NAMEN[Integer.parseInt(setsString.charAt(anz - 1) + "")];
			return result;
		}
	}
	
	static {
		if (IS_FAKE_ANDROID) {
			Logger.info(Global.class, "!!!!!! FAKE_ANDROID AKTIVIERT !!!!!!");
			Logger.info(Global.class, "!!!!!! FAKE_ANDROID AKTIVIERT !!!!!!");
			Logger.info(Global.class, "!!!!!! FAKE_ANDROID AKTIVIERT !!!!!!");
		}
	}
}
