// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package control;

import gui.launch.GameCanvasInstance;

import java.util.Vector;

import javax.microedition.lcdui.Canvas;

import utils.StringUtils;
import dev.Logger;

/**
 * Diese Klasse verwaltet alles was mit den Tasten zu tun hat
 */
public class Keys {
	public static final int UNKNOWN = -1;
	
	// hier müssen alle Tastencodes-Arrays angegeben werden 
	public static final int OK = 0;
	public static final int MENU_LEFT = 1;
	public static final int MENU_RIGHT = 2;
	public static final int UP = 3;
	public static final int DOWN = 4;
	public static final int LEFT = 5;
	public static final int RIGHT = 6;
	public static final int C = 7;
	public static final int BACK = 8;
	private static final int KEY_MAX_ID = BACK; 
	
	public static final int NUMBER = 9;
	public static final int ALPHA = 10;
	
	public static final int ACTION_UNKNOWN = -1;
	public static final int ACTION_FORWARD = 0;
	public static final int ACTION_BACKWARD = 1;
	
	private static boolean isInitialized = false;
	
	private static final int[][] assignedKeys;

	static {
		assignedKeys = new int[KEY_MAX_ID + 1][];
		for (int i = 0; i <= KEY_MAX_ID; i++) {
			assignedKeys[i] = new int[0];
		}
	}
	
	private static Vector usedKeys = new Vector();
	
	private static boolean addKey(int type, int keyCode) {
		// prüfe auf Gültigkeit des Typs
		if (!isValidKeyCollection(type)) {
			try {
				throw new IllegalArgumentException("type=" + type);
			} catch (IllegalArgumentException e) {
				Logger.fatalException(Keys.class, e, "s89nsA");
				return false;
			}
		}
		
		// prüfe ob der Keycode ein ZahlenCode ist
		if (keyCode >= '0' && keyCode <= '9') {
			try {
				throw new IllegalArgumentException("numKey" + keyCode);
			} catch (IllegalArgumentException e) {
				Logger.errorException(Keys.class, e, "sxas8a");
				return false;
			}
		}
		// prüfe ob der Keycode ein Buchstaben ist
		if (keyCode >= 'a' && keyCode <= 'z') {
			try {
				throw new IllegalArgumentException("alphKey" + keyCode);
			} catch (IllegalArgumentException e) {
				Logger.errorException(Keys.class, e, "x8wndsdr");
				return false;
			}
		}
		
		// Ist der Key schon hinzugefügt?
		if (usedKeys.contains(new Integer(keyCode))) {
			return false;
		}
		
		usedKeys.addElement(new Integer(keyCode));
		int[] oldKeyArray = assignedKeys[type];
		int[] newKeyArray = new int[oldKeyArray.length + 1];
		System.arraycopy(oldKeyArray, 0, newKeyArray, 0, oldKeyArray.length);
		newKeyArray[newKeyArray.length - 1] = keyCode;
		assignedKeys[type] = newKeyArray;
		
		return true;
	}
	
	private static void removeKey(int keyCode) {
		for (int i = 0; i < assignedKeys.length; i++) {
			for (int i2 = 0; i2 < assignedKeys[i].length; i2++) {
				if (assignedKeys[i][i2] == keyCode) {
					int[] oldKeyArray = assignedKeys[i];
					int[] newKeyArray = new int[oldKeyArray.length - 1];
					System.arraycopy(oldKeyArray, 0, newKeyArray, 0, i2);
					System.arraycopy(oldKeyArray, i2 + 1, newKeyArray, i2, newKeyArray.length - i2);
					assignedKeys[i] = newKeyArray;
					i2--;
				}
			}
		}
		if (usedKeys.contains(new Integer(keyCode))) {
			usedKeys.removeElement(new Integer(keyCode));
		}
	}
	
	synchronized public static void init() {
		if (isInitialized) {
			return;
		}
		
		Canvas canvas = GameCanvasInstance.getInstance();
		
		// native J2ME Keys
		addKey(UP, canvas.getKeyCode(Canvas.UP));
		addKey(DOWN, canvas.getKeyCode(Canvas.DOWN));
		addKey(LEFT, canvas.getKeyCode(Canvas.LEFT));
		addKey(RIGHT, canvas.getKeyCode(Canvas.RIGHT));
		addKey(OK, canvas.getKeyCode(Canvas.FIRE));
		
		// non native KEYS
		
		int KEY_MENU_LEFT_DEFAULT = -6;
		int KEY_MENU_RIGHT_DEFAULT = -7;
		int KEY_C_DEFAULT = -8;
		
		addKey(MENU_LEFT, KEY_MENU_LEFT_DEFAULT);
		addKey(MENU_RIGHT, KEY_MENU_RIGHT_DEFAULT);
		addKey(C, KEY_C_DEFAULT);

		// specific keys
		int KEY_BACK_SONY = -11;
		addKey(BACK, KEY_BACK_SONY);

		int KEY_MENU_LEFT_HTC_P3600 = 57345;
		int KEY_MENU_RIGHT_HTC_P3600 = 57346;
		
		addKey(MENU_LEFT, KEY_MENU_LEFT_HTC_P3600);
		addKey(MENU_RIGHT, KEY_MENU_RIGHT_HTC_P3600);
		
		int KEY_MENU_LEFT_MOTOROLA_V8 = -21;
		int KEY_MENU_RIGHT_MOTOROLA_V8 = -22;
		
		addKey(MENU_LEFT, KEY_MENU_LEFT_MOTOROLA_V8);
		addKey(MENU_RIGHT, KEY_MENU_RIGHT_MOTOROLA_V8);

		// specific keys (conflicting)
		
		String devicePlatformLowercase = Device.getPlatform().toLowerCase();
		
		if (!StringUtils.startsWith("nokia", devicePlatformLowercase)) {
			// Siemens
			int KEY_MENU_LEFT_SIEMENS = -1;
			int KEY_MENU_RIGHT_SIEMENS = -4;
			addKey(MENU_LEFT, KEY_MENU_LEFT_SIEMENS);
			addKey(MENU_RIGHT, KEY_MENU_RIGHT_SIEMENS);
		} 
		
		// Nokia
		addKey(UP, -1);
		addKey(DOWN, -2);
		addKey(LEFT, -3);
		addKey(RIGHT, -4);
		addKey(OK, -5);
		
		// Emulator
		addKey(C, 8);
		
		// LG-GM310
		addKey(C, -16);

		// Microemulator-Android
		if (Device.isAndroid && !Global.IS_FAKE_ANDROID) {
			removeKey(-4);
			addKey(BACK, -4);
			addKey(C, 127);
			addKey(OK, 10);
		}

		isInitialized = true;
	}
	
	public static String getKeyString(int key) {
		init();
		int keyType = getKeyType(key);
		switch (keyType) {
		case MENU_LEFT:
			return "MENU_LEFT";
		case MENU_RIGHT:
			return "MENU_RIGHT";
		case OK:
			return "OK";
		case C:
			return "C";
		case UP:
			return "UP";
		case DOWN:
			return "DOWN";
		case LEFT:
			return "LEFT";
		case RIGHT:
			return "RIGHT";
		case BACK:
			return "BACK";
		case NUMBER:
			char c = (char) key;
			return "NUMBER '" + c + "'";
		case ALPHA:
			char c2 = (char) key;
			return "ALPHA '" + c2 + "'";
		}

		return "UNBEKANNT";
	}

	private static boolean isValidKeyCollection(int type) {
		return (type >= 0 && type <= KEY_MAX_ID);
	}
	
	private static int getKeyType(int key) {
		if (key >= '0' && key <= '9') {
			return NUMBER;
		}
		for (int i = 0; i <= KEY_MAX_ID; i++) {
			if (inKeyCollection(assignedKeys[i], key)) {
				return i;
			}
		}
		if (key >= 32 && key <= 126) {
			return ALPHA;
		}

		return UNKNOWN;
	}
	
	private static boolean inKeyCollection(int[] collection, int key) {
		for (int i = 0; i < collection.length; i++) {
			if (collection[i] == key) {
				return true;
			}
		}
		return false;
	}


		
	public static boolean isKey(int key, int keyType) {
		init();
		if (getKeyType(key) == keyType) {
			return true;
		} else {
			return false;
		}
	}
	
	private static int keyOfCollection(int[] collection) {
		if (collection.length == 0) {
			unknownKeyWarning();
			return UNKNOWN;
		} else {
			return collection[0];
		}
		
	}
	
	public static int getKeyCode(int keyType) {
		init();
		if (isValidKeyCollection(keyType)) {
			return keyOfCollection(assignedKeys[keyType]);
		} else {
			unknownKeyWarning();
			return UNKNOWN;
		}
	}
	
	public static boolean isKeyAction(int key, int keyAction) {
		init();
		int keyType = getKeyType(key);
		if (keyAction == ACTION_FORWARD) {
			if (keyType == OK || keyType == RIGHT) {
				return true;
			} else {
				return false;
			}
		}
		if (keyAction == ACTION_BACKWARD) {
			if (keyType == BACK || keyType == LEFT) {
				return true;
			} else {
				return false;
			}
		}
		return false;
	}
	
	private static void unknownKeyWarning() {
		System.err.println("UnknownKeyWarning");
	}


	public static String getKeySettings() {
		init();
		String s = "";
		
		Canvas canvas = GameCanvasInstance.getInstance();
		
		// native J2ME Keys
		int j2meUp = canvas.getKeyCode(Canvas.UP);
		int j2meDown = canvas.getKeyCode(Canvas.DOWN);
		int j2meLeft = canvas.getKeyCode(Canvas.LEFT);
		int j2meRight = canvas.getKeyCode(Canvas.RIGHT);
		int j2meFire = canvas.getKeyCode(Canvas.FIRE);

		s += "UP " + StringUtils.arrayToString(assignedKeys[UP]) + "[" + j2meUp + "]" + "\n";
		s += "DOWN " + StringUtils.arrayToString(assignedKeys[DOWN]) + "[" + j2meDown + "]" + "\n";
		s += "LEFT " + StringUtils.arrayToString(assignedKeys[LEFT]) + "[" + j2meLeft + "]" + "\n";
		s += "RIGHT " + StringUtils.arrayToString(assignedKeys[RIGHT]) + "[" + j2meRight + "]" + "\n";
		s += "OK " + StringUtils.arrayToString(assignedKeys[OK]) + "[" + j2meFire + "]" + "\n";
		s += "MENU_LEFT " + StringUtils.arrayToString(assignedKeys[MENU_LEFT]) + "\n";
		s += "MENU_RIGHT " + StringUtils.arrayToString(assignedKeys[MENU_RIGHT]) + "\n";
		s += "BACK " + StringUtils.arrayToString(assignedKeys[BACK]) + "\n";
		s += "C " + StringUtils.arrayToString(assignedKeys[C]);

		return s;
	}
	
	public static char getLowerCaseAlphaKey(char c) {
		switch (c) {
		case 'ä':
		case 'Ä':
			return 'ä';
		case 'ö':
		case 'Ö':
			return 'ö';
		case 'ü':
		case 'Ü':
			return 'ü';
		case 'ß':
			return c;
		default:
			// to lowercase
			if (c >= 'A' && c <= 'Z') {
				c = (char)((byte)c - 32);
			}
			if (c >= 'a' && c <= 'z') {
				return c;
			} else {
				return ' '; 
			}
		}
	}
}
