// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package control;

import gui.BgStartScreen;
import gui.launch.GameCanvasInstance;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import control.reminder.Reminder;
import data.persistence.Persistence;
import dev.Logger;

public class Pushandride extends MIDlet {

	private static Pushandride pushandride;

	public static GameCanvasInstance launchScreen;

	private static long startupTime;

	public static long STARTUP_TOTAL_MEMORY;
	public static long STARTUP_FREE_MEMORY;

	public Pushandride() {
		Runtime runtime = Runtime.getRuntime();
		STARTUP_FREE_MEMORY = runtime.freeMemory();
		STARTUP_TOTAL_MEMORY = runtime.totalMemory();

		Pushandride.pushandride = this;
		
		launchPushandride();
	}

	private void launchPushandride() {
		try {
			startupTime = System.currentTimeMillis();
			launchScreen = new GameCanvasInstance(this);
		} catch (OutOfMemoryError e) {
			PushandrideMidlet.OutOfMemoryException(this.getClass(), e, "x9s,w");
		} catch (Exception e) {
			Logger.errorException(this, e, "x9s,w");
		}
	}

	public static void restartPushandride() {
		finishThreads();
		pushandride.launchPushandride();
	}

	/**
	 * startApp()
	 */
	protected void startApp() throws MIDletStateChangeException {
	}

	/**
	 * pauseApp()
	 */
	protected void pauseApp() {
	}

	/**
	 * destroyApp()
	 */
	protected void destroyApp(boolean unconditional) throws MIDletStateChangeException {
		finishThreads();
	}

	public static void finishThreads() {
		Persistence.finishRSThread();
		PushandrideMidlet.finishKeyThread();
		BgStartScreen.quit();
		Reminder.reset();
		// Die Deaktivierung des Cache hat hier eigentlich nichts verloren - ist aber ein Quickfix dafür, dass beim Neustart Probleme mit dem Cache auftreten können.
		Global.USE_ROUTE_CACHE = false;
	}

	/**
	 * Beendet das Programm.
	 */
	public void exitProgramm() {
		try {
			destroyApp(false);
		} catch (Exception e) {
			Logger.error(this, "Fehler beim Beenden (destroyApp): " + e.getMessage());
		}
		try {
			notifyDestroyed();
		} catch (Exception e) {
			Logger.error(this, "Fehler beim Beenden (notifyDestroyed): " + e.getMessage());
		}
	}

	public static Pushandride getMidlet() {
		return pushandride;
	}

	public static long getStartupTime() {
		return startupTime;
	}
}
