// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package control;

import gui.Manager;
import gui.utils.AlertMessage;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import control.locale.Locale;
import data.Data;
import data.FileData;
import data.persistence.Persistence;
import data.persistence.RemoteDataBinary;
import data.persistence.UserSettingsInt;
import dev.DebugInfo;
import dev.Logger;
import dev.bootProfiler.BootProfiler;
import dev.bootProfiler.ZeitMessung;

public class PushandrideMidlet {

	public static int replicateStatus = Global.REPLICATE_UNKNOWN;
	private static Manager manager;

	public static void init() {
		if (Global.FORCE_EMPTY_RECORDSTORE) {
			Persistence.deleteRecordStore();
			Logger.info(Pushandride.class, "deleting record store - forced by Global.FORCE_EMPTY_RECORDSTORE");
		}

		ZeitMessung profil_loadDataStagePre = ZeitMessung.getStartedInstance("Lade Datensatz | Pre");
		if (!Device.isAndroid) {
			Data.loadDataStagePre("/res/data.bin");
		} else {
			int datensatzTyp = UserSettingsInt.get(UserSettingsInt.SETTINGS_ID_DATENSATZ_TYP);
			if (datensatzTyp == UserSettingsInt.NULL_VALUE) {
				datensatzTyp = Data.DATSET_KLEIN;
				UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_DATENSATZ_TYP, datensatzTyp);
			}
			FileData.overrideInputStream(null);
			
			// Überprüfe ob eine gülte individuelle Version im Record-Store liegt und lösche ungültige
			if (RemoteDataBinary.getBinaryFromRecordstore().length != 0) {
				if (Data.VERSION != RemoteDataBinary.getVersion() || Data.UPDATE > RemoteDataBinary.getUpdate()) {
					Logger.info(PushandrideMidlet.class, "lösche individuellen Datensatz - ungültige Version");
					RemoteDataBinary.removeBinary();
					if (datensatzTyp == Data.DATSET_INDIVIDUELL) {
						UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_DATENSATZ_TYP, Data.DATSET_KLEIN);
						datensatzTyp = Data.DATSET_KLEIN;
						RemoteDataBinary.showInfoDueToVersionConfilct = true;
					}
				}
			}
			
			Data.setDatasetType(datensatzTyp);
			if (datensatzTyp == Data.DATSET_KLEIN) {
				Data.loadDataStagePre("/res/dataSmall.bin");
			} else {
				if (datensatzTyp == Data.DATSET_ALLES) {
					Data.loadDataStagePre("/res/data.bin");
				} else {
					if (datensatzTyp == Data.DATSET_INDIVIDUELL) {
						byte[] b = RemoteDataBinary.getBinaryFromRecordstore();
						if (b.length < 1024 * 100) {
							Logger.error(PushandrideMidlet.class, "Datensatz im Recordstore zu klein: " + b.length);
							Data.loadDataStagePre("/res/dataSmall.bin");
						} else {
							FileData.overrideInputStream(new ByteArrayInputStream(b));
							Data.loadDataStagePre("");
						}
					} else {
						Logger.error(PushandrideMidlet.class, "SETTINGS_ID_DATENSATZ_TYP ungültig");
						Data.loadDataStagePre("/res/dataSmall.bin");
					}
				}
			}
		}
		profil_loadDataStagePre.stop();
		BootProfiler.addLeerzeile();

		// Start GUI
		initReplicate();
		ZeitMessung profil_instanziereManager = ZeitMessung.getStartedInstance("Instanziere Manager (gesamt)");
		manager = new Manager(Pushandride.getMidlet());
		profil_instanziereManager.stop();
		BootProfiler.addLeerzeile();

		// INIT - Thread
		new Init().start();

		// Stats (wenn alles andere fertig ist)
		data.persistence.Stats.inc(data.persistence.Stats.STARTS);
	}

	/**
	 * Prüft, ob das JAR sich selbst verschicken kann
	 */
	private static void initReplicate() {
		String sourceName = Global.SELF_RESOURCE_NAME;
		String replicateName = Global.SELF_REPLICATE_NAME;
		InputStream is;
		try {
			if ((is = Pushandride.class.getResourceAsStream(sourceName)) == null) {
				if ((is = Pushandride.class.getResourceAsStream(replicateName)) == null) {
					PushandrideMidlet.replicateStatus = Global.REPLICATE_NONE;
				} else {
					is.close();
					PushandrideMidlet.replicateStatus = Global.REPLICATE_SPECIAL;
				}
			} else {
				is.close();
				PushandrideMidlet.replicateStatus = Global.REPLICATE_ONCE;
			}
		} catch (Exception e) {
			Logger.fatalException(PushandrideMidlet.class, e, "xsaxc888");
		}
	}

	public static void OutOfMemoryException(Class c, Throwable exception, String s) {
		try {
			Logger.fatalException(c, exception, s);
			Logger.info(PushandrideMidlet.class, "Nutzbarer Arbeitsspeicher von Java: " + (Runtime.getRuntime().totalMemory() / 1024) + "kb");
			if (!Device.isAndroid) {
				if (Device.getGfxQuality() != Device.GFX_QUALITY_LOW) {
					Device.setGfxQuality(Device.GFX_QUALITY_LOW);
					try {
						// Eine Sekunde warten, damit der CommitThread im Hintergrund die Änderung in den RecordStore schreiben kann.
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					new AlertMessage("Fehler", Locale.HinweisOutOfMemoryExceptionIsHighGfx + "\n\nInfo\nNutzbarer Arbeitsspeicher von Java: " + (Runtime.getRuntime().totalMemory() / 1024) + "kb" + "\n\n" + DebugInfo.getAllDebugMessagesAsString(), Locale.Beenden).setVisible(true);
				} else {
					new AlertMessage("Fehler", Locale.HinweisOutOfMemoryExceptionIsLowGfx, Locale.Beenden).setVisible(true);
				}
			} else {
				new AlertMessage("Fehler", Locale.HinweisOutOfMemorExceptionyAndroid, Locale.Beenden).setVisible(true);
			}
		} catch (Exception e) {
			
		}
	}

	public static Manager getManager() {
		return manager;
	}

	public static void finishKeyThread() {
		if (manager != null) {
			manager.finishKeyThread();
		}
	}
}
