// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package control.connections.bluetooth;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
import javax.bluetooth.UUID;

public class BluetoothSearchObexService extends Thread implements Runnable, DiscoveryListener {
	private final RemoteDevice remoteDevice;
	private final BluetoothSearchObexService owner;
	private final BluetoothManager bluetoothManager;
	private Thread searchingForServicesThread;
	private String connectionUrl = "";
	private LocalDevice local;

	public BluetoothSearchObexService(BluetoothManager bluetoothManager, RemoteDevice remoteDevice) {
		this.remoteDevice = remoteDevice;
		this.owner = this;
		this.bluetoothManager = bluetoothManager;

		bluetoothManager.setDebugState(getClass(), "SearchObex - Konstruktor");

		try {
			local = LocalDevice.getLocalDevice();
			searchForServices(remoteDevice.getBluetoothAddress());
		} catch (BluetoothStateException BSe) {
			bluetoothManager.failure(getClass(), BSe, " Konstruktor", BluetoothManager.FAILURE_BLUETOOTH_EXCEPTION);
		}
	}

	private void searchForServices(String btConnectionURL) {
		// retrieve the discovery agent 
		searchingForServicesThread = new Thread(new Runnable() {
			public void run() {
				// 0x1105 is the UUID for the Object Push Profile
				UUID[] uuidSet = { new UUID(0x1105) };

				// 0x0100 is the attribute for the service name element in the service record
				int[] attrSet = { 0x0100 };

				DiscoveryAgent agent = local.getDiscoveryAgent();

				bluetoothManager.setDebugState(getClass(), "SearchObex - preSearch");
				try {
					synchronized (searchingForServicesThread) {
						agent.searchServices(attrSet, uuidSet, remoteDevice, owner);
						searchingForServicesThread.wait();
					}
					bluetoothManager.setDebugState(getClass(), "SearchObex - postSearch");
				} catch (Exception e) {
					bluetoothManager.failure(getClass(), e, " Thread - searchForService", BluetoothManager.FAILURE_BLUETOOTH_NO_OBEX);
					return;
				}

				// finished - report to callback function
				if (connectionUrl != "") {
					bluetoothManager.finishedObexServiceSearch(connectionUrl, true);
				} else {
					bluetoothManager.finishedObexServiceSearch(connectionUrl, false);
				}
			}
		});
		bluetoothManager.setDebugState(getClass(), "SearchObex - preStart");
		searchingForServicesThread.start();
	}

	public void deviceDiscovered(RemoteDevice arg0, DeviceClass arg1) {
	}

	public void inquiryCompleted(int arg0) {
	}

	public void serviceSearchCompleted(int transID, int respCode) {
		bluetoothManager.setDebugState(getClass(), "SearchObex - search Completed");
		switch (respCode) {
		case DiscoveryListener.SERVICE_SEARCH_COMPLETED:
			break;

		case DiscoveryListener.SERVICE_SEARCH_DEVICE_NOT_REACHABLE:
		case DiscoveryListener.SERVICE_SEARCH_ERROR:
		case DiscoveryListener.SERVICE_SEARCH_NO_RECORDS:
			break;
		case DiscoveryListener.SERVICE_SEARCH_TERMINATED:
			break;
		}
		bluetoothManager.setDebugState(getClass(), "SearchObex - searchCompleted");

		synchronized (searchingForServicesThread) {
			searchingForServicesThread.notify();
		}
	}

	public void servicesDiscovered(int transID, ServiceRecord[] servRecord) {
		bluetoothManager.setDebugState(getClass(), "SearchObex - serviceFound");
		// wenn ein Device mehrere OBEX möglichkeiten bietet wird immer das letzte genommen
		for (int i = 0; i < servRecord.length; i++) {
			connectionUrl = servRecord[i].getConnectionURL(ServiceRecord.NOAUTHENTICATE_NOENCRYPT, false);
		}
	}
}
