// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package control.connections.bluetooth;

import java.io.ByteArrayInputStream;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Hashtable;

import javax.microedition.io.Connector;
import javax.obex.ClientSession;
import javax.obex.HeaderSet;
import javax.obex.Operation;

import utils.StringUtils;
import control.Global;
import control.PushandrideMidlet;
import dev.Logger;

public class BluetoothSendFileViaObex {

	public static final int SENDJAR = 0;
	public static final int SENDJAD = 1;

	private static final int BUFFERSIZE = 4 * 1024;

	private final String btConnectionURL;
	private final BluetoothManager bluetoothManager;
	private final int sendType;
	private final static Hashtable cacheSizes = new Hashtable();

	private long currentPos = 0;
	
	private boolean cancel = false;
	
	synchronized private boolean isCanceled() {
		return cancel;
	}
	
	synchronized public void cancel() {
		cancel = true;
	}

	public BluetoothSendFileViaObex(BluetoothManager bluetoothManager, String btConnectionURL, int sendType) {
		currentPos = 0;
		this.bluetoothManager = bluetoothManager;
		bluetoothManager.setDebugState(getClass(), "SendFile - Konstruktor");
		this.btConnectionURL = btConnectionURL;
		this.sendType = sendType;
	}

	synchronized private static long getSizeOfResource(BluetoothManager bluetoothManager, String resourceName) {
		if (cacheSizes.containsKey(resourceName)) {
			return ((Long) cacheSizes.get(resourceName)).longValue();
		}

		bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - getSize " + resourceName);
		long resourceSize = 0;
		try {
			InputStream sourceStream;
			if ((sourceStream = BluetoothSendFileViaObex.class.getResourceAsStream(resourceName)) == null) {
				System.err.println("resource not available: " + resourceName);
				bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - getSize - Error " + resourceName + " not avaiable");
				bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - getSize - Error " + resourceName + " not avaiable");
				return -1;
			}

			int lastChunkSize = 0;
			byte[] buffer = new byte[BUFFERSIZE];
			do {
				lastChunkSize = sourceStream.read(buffer);
				resourceSize += lastChunkSize;
			} while (lastChunkSize == buffer.length);
			sourceStream.close();
		} catch (Exception e) {
			bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - getSize - Exception " + e.getMessage() + " " + e.toString());
			bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - getSize - Exception " + e.getMessage() + " " + e.toString());
			bluetoothManager.failure(BluetoothSendFileViaObex.class, e, null, BluetoothManager.FAILURE_BLUETOOTH_NOT_WORKING);
			return -1;
		}
		if (PushandrideMidlet.replicateStatus == Global.REPLICATE_SPECIAL) {
			try {
				InputStream sourceStream;
				if ((sourceStream = BluetoothSendFileViaObex.class.getResourceAsStream(Global.SELF_REPLICATE_HEADERNAME)) == null) {
					System.err.println("resource not available: " + resourceName);
					bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - getSize - Error " + resourceName + " not avaiable");
					bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - getSize - Error " + resourceName + " not avaiable");
					return -1;
				}
				int headerSize = 0;
				byte[] buffer = new byte[100];
				headerSize = sourceStream.read(buffer);
				sourceStream.close();
				resourceSize += headerSize;
			} catch (Exception e) {
				bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - getSize - Exception2 " + e.getMessage() + " " + e.toString());
				bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - getSize - Exception2 " + e.getMessage() + " " + e.toString());
				bluetoothManager.failure(BluetoothSendFileViaObex.class, e, null, BluetoothManager.FAILURE_BLUETOOTH_NOT_WORKING);
				return -1;
			}

		}
		bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - Size: " + resourceSize);
		bluetoothManager.setDebugState(BluetoothSendFileViaObex.class, "SendFile - Size: " + resourceSize);

		cacheSizes.put(resourceName, new Long(resourceSize));

		return resourceSize;
	}

	private int getByteOfLong(long value, int byteNo) {
		value = value >> 8 * byteNo;
		return (int) (value & (long) 0xFF);
	}

	private static byte[] createJad(long jarSize) {
		// das muss mit dem Generator-Skript übereinstimmen
		StringBuffer sb = new StringBuffer();
		String jar;
		String icon;
		String name;
		if (StringUtils.startsWith(Global.PREFIX_FULL_VERSION, Global.getDownloadUrl())) {
			jar = "pushandride_komplett.jar";
			icon = "midleticon_komplett.png";
			name = "push&ride Stuttgart komplett";
		} else {
			jar = "pushandride.jar";
			icon = "midleticon.png";
			name = "push&ride Stuttgart";
		}
		
		sb.append("MIDlet-Jar-URL: " + jar + "\n");
		sb.append("MIDlet-Icon: /res/" + icon + "\n");
		sb.append("MicroEdition-Configuration: CLDC-1.1\n");
		sb.append("MIDlet-Version: " + Global.getMidletVersion() + "\n");
		sb.append("MIDlet-Name: " + name + "\n");
		sb.append("MIDlet-Vendor: GNU General Public License\n");
		sb.append("MIDlet-Info-URL: http://pushandride.de/\n");
		sb.append("MicroEdition-Profile: MIDP-2.0\n");
		sb.append("MIDlet-Jar-Size: " + jarSize + "\n");
		sb.append("MIDlet-1: " + name + ", /res/" + icon + ", control.Pushandride\n");
		sb.append("Nokia-MIDlet-On-Screen-Keypad: no");
		return sb.toString().getBytes();
	}

	private static String getSourceName() {
		switch (PushandrideMidlet.replicateStatus) {
		case Global.REPLICATE_ONCE:
			return Global.SELF_RESOURCE_NAME;
		case Global.REPLICATE_SPECIAL:
			return Global.SELF_REPLICATE_NAME;
		default:
			return "";
		}
	}

	public static long getSizeOfJar(BluetoothManager bluetoothManager) {
		String sourceName = getSourceName();
		return getSizeOfResource(bluetoothManager, sourceName);
	}

	public void sendFile() {
		try {
			String sourceName = getSourceName();
			long jarSize = getSizeOfJar(bluetoothManager);

			bluetoothManager.setDebugState(getClass(), "SendFile - preOpen");
			bluetoothManager.setDebugState(getClass(), "SendFile - postOpen");
			// connection obtained

			// now, let's create a session and a headerset objects
			ClientSession cs = (ClientSession) Connector.open(btConnectionURL);
//			ClientSession cs = (ClientSession) OBEXConnector.open(btConnectionURL); // for use with aventana JSR-82 implementation
			HeaderSet hs = cs.createHeaderSet();

			// now let's send the connect operation
			cs.connect(hs);
			bluetoothManager.setDebugState(getClass(), "SendFile - postConnect");

			hs.setHeader(HeaderSet.TYPE, "application/java-archive");

			long sendSize;
			if (sendType == SENDJAR) {
				String jarname;
				if (StringUtils.startsWith(Global.PREFIX_FULL_VERSION, Global.getDownloadUrl())) {
					jarname = "pushandride_komplett.jar";
				} else {
					jarname = "pushandride.jar";
				}
				hs.setHeader(HeaderSet.NAME, jarname);
				sendSize = jarSize;
			} else {
				String jadname;
				if (StringUtils.startsWith(Global.PREFIX_FULL_VERSION, Global.getDownloadUrl())) {
					jadname = "pushandride_komplett.jad";
				} else {
					jadname = "pushandride.jad";
				}
				hs.setHeader(HeaderSet.NAME, jadname);
				byte[] jad = createJad(jarSize);
				sendSize = jad.length;
			}

			hs.setHeader(HeaderSet.LENGTH, new Long(sendSize));

			Operation putOperation = cs.put(hs);

			OutputStream outputStream = putOperation.openOutputStream();

			if (sendType == SENDJAR) {
				// send header first
				if (PushandrideMidlet.replicateStatus == Global.REPLICATE_SPECIAL) {
					bluetoothManager.setDebugState(getClass(), "SendFile - Header");
					InputStream sourceStream;
					if ((sourceStream = getClass().getResourceAsStream(Global.SELF_REPLICATE_HEADERNAME)) == null) {
						System.err.println("resource not available: " + Global.SELF_REPLICATE_HEADERNAME);
						bluetoothManager.finishedBluetoothSendFileViaObex(false, sendType);
						return;
					}
					byte[] buffer = new byte[100];
					int headerlength = sourceStream.read(buffer);
					if (headerlength < 0x20) {
						System.err.println("resource not correct: " + Global.SELF_REPLICATE_HEADERNAME);
						bluetoothManager.finishedBluetoothSendFileViaObex(false, sendType);
						return;
					}
					for (int dup = 0; dup < 2; dup++) {
						for (int byteNo = 0; byteNo < 4; byteNo++) {
							buffer[0x12 + 4 * dup + 3 - byteNo] = (byte) getByteOfLong(sendSize, byteNo);
						}
					}
					bluetoothManager.setDebugState(getClass(), "SendFile - HeaderSend");
					outputStream.write(buffer, 0, headerlength);
				}
			}

			bluetoothManager.setDebugState(getClass(), "SendFile - Main Content");

			InputStream sourceStream;
			if (sendType == SENDJAR) {
				if ((sourceStream = getClass().getResourceAsStream(sourceName)) == null) {
					Logger.fatal(getClass(), "no way");
					bluetoothManager.finishedBluetoothSendFileViaObex(false, sendType);
					return;
				}
			} else {
				sourceStream = new ByteArrayInputStream(createJad(jarSize));
			}

			long sentSize = 0;
			int lastChunkSize = 0;
			byte[] buffer = new byte[BUFFERSIZE];
			do {
				bluetoothManager.watchDogUpdate();
				if (isCanceled()) {
					try {
						sourceStream.close();
						outputStream.close();
						putOperation.close();
						cs.disconnect(null);
						cs.close();
					} catch (Exception e) {
					}
					return;
				}
				try {
					bluetoothManager.callBackSendingFile();
					lastChunkSize = sourceStream.read(buffer);
					outputStream.write(buffer, 0, lastChunkSize);
				} catch (IOException e) {
					bluetoothManager.failure(getClass(), e, null, BluetoothManager.FAILURE_EXCEPTION);
					return;
				}
				sentSize += lastChunkSize;
				currentPos = sentSize;
			} while (lastChunkSize == buffer.length);

			// file push complete

			bluetoothManager.setDebugState(getClass(), "SendFile - beforeClose");

			sourceStream.close();
			outputStream.close();
			putOperation.close();

			cs.disconnect(null);
			cs.close();

			bluetoothManager.setDebugState(getClass(), "SendFile - beforeCallback");

			bluetoothManager.finishedBluetoothSendFileViaObex(true, sendType);
		} catch (Exception e) {
			bluetoothManager.setDebugState(getClass(), "SendFile - Exception" + e.getMessage() + " " + e.toString());
			bluetoothManager.finishedBluetoothSendFileViaObex(false, sendType);
		}
	}

	public synchronized long getCurrentPos() {
		return currentPos;
	}
	
}
