// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package control.locale;

import utils.StringUtils;

/**
 * LOCALE
 * In dieser Klasse sind alle Dinge untergebracht, die Sprach, Ort oder
 * Kulturabhängig sind wie Sprache, Datumsformat, Zeitformat etc.
 * Datum und Zeit werden durch entsprechende Funktionen lokalisiert.
 * TextResourcen werden mit dem TextResourcenManager verwaltet und der
 * generierte Code hier eingefügt. Die TR werden direkt als public Variablen
 * angeboten, so das man sie direkt im Code einbinden kann, einen sprechenden
 * Namen hat, ohne nochmal eine Abstraktionsebene einzufügen, welche
 * Resourcen kostet.
 * 
 * Version: VVS VVS VVS VVS VVS VVS VVS VVS VVS VVS VVS VVS VVS VVS VVS
 */
public class Locale {

	/**
	 * #########################################################################
	 * TEXT RESOURCEN
	 * (Dieser Teil wird durch den TextResourcenManager generiert.)
	 * #########################################################################
	 */

	// Globale Text Ressourcen zur wiederverwendung unten
	public static final String Verkehrsverbund = "VVS";
	public static final String Stadt = "Stuttgart";

	// Kategorie: Navigation
	public static final String Back = "Zurück";
	public static final String Laufgeschwindigkeit = "Laufgeschwindigkeit";
	public static final String SW_Keyboard = "Software-Keyboard";
	public static final String GraphicsQuality = "Grafikqualität";
	public static final String ResetSettings = "Alles zurücksetzen";
	public static final String ResetSettings_Success = "Daten wurden zurückgesetzt!";
	public static final String WriteFeedback = "Feedback schreiben";
	public static final String CheckForNewVersion = "Auf neue Version prüfen";
	public static final String SendLoggings = "Loggings an Entwickler senden";
	public static final String Reminder = "Erinnerung";
	public static final String ReminderActivate = "Erinnerung";
	public static final String ReminderActive = "$1$' vor";

	// Kategorie: Allgemeine Worte
	public static final String Info = "Info";
	public static final String Error = "Fehler";
	public static final String About = "Über push&ride";
	public static final String Route = "Route";
	public static final String Haltestelle = "Haltestelle";
	public static final String normal = "normal";
	public static final String fast = "schnell";
	public static final String high = "hoch";
	public static final String mid = "mittel";
	public static final String low = "niedrig";
	public static final String active = "aktiviert";
	public static final String inactive = "deaktiviert";
	public static final String Nutzungsbedingungen = "Nutzungsbedingungen";
	public static final String Beenden = "Beenden";
	public static final String Hinweis = "Hinweis";

	// Kategorie: Fehler und andere Meldungen
	public static final String Msg_VonNachEqual = "Als VON und NACH Haltestelle wurde '$1$' gewählt!";
	public static final String Msg_SupportedInLaterVersion = "Diese Funktion wird erst in einer späteren Version von push&ride unterstützt!";
	public static final String HinweisBildschirmDrehen = "push&ride unterstützt nicht das Drehen des Bildschirms während des Betriebs.\nBitte drehe das Handy zurück oder beende das Programm und starte es neu.";
	public static final String BitteWarten = "... bitte warten ...";
	public static final String HinweisOutOfMemoryExceptionIsHighGfx = "Nicht genügend Arbeitsspeicher vorhanden.\nBitte starte das Programm erneut, die Grafikqualität wurde automatisch von push&ride reduziert.\nAlternativ kann auch eine Version mit weniger Bussen installiert werden.";
	public static final String HinweisOutOfMemoryExceptionIsLowGfx = "Nicht genügend Arbeitsspeicher vorhanden.\nBitte beende das Programm und installiere eine Version mit weniger Bussen.";
	public static final String HinweisOutOfMemorExceptionyAndroid = "Nicht genügend Arbeitsspeicher vorhanden.\nDies kann durch den mehrfachen Wechsel des Datensatzes entstehen. Bitte das Programm neu starten.";
	public static final String[] KeineVerbindungGefunden = { "keine", "Verbindung", "gefunden" };
	
	// Kategorie: Bluetooth
	public static final String BluetoothNichtAktiv = "Falls Bluetooth ausgeschalten ist, bitte im Handy aktivieren. Wenn es bereits aktiviert ist kann sich push&ride leider nicht per Bluetooth verschicken.";
	public static final String BluetoothSucheNachDevices = "Es wird nach erreichbaren Geräten gesucht.\nBitte warten";

	/**
	 * #########################################################################
	 * TEXT RESOURCEN LOADER
	 * (Dieser Teil wird durch den TextResourcenManager generiert.)
	 * #########################################################################
	 */

	private static String loadedLanguage = "DE";
	
	public static void loadLanguage(String key) {
		// Wenn die Sprache schon geladen ist brauchen wir nichts laden.
		if(loadedLanguage.equals(key)) {
			return;
		}
		
		// Lade die Sprache
		if (key.equals("DE")) {

		} else if (key.equals("EN")) {

		}
		
		// loadedLangeuage neu setzen
		loadedLanguage = key;
	}

	/**
	 * #########################################################################
	 * TEXT RESOURCEN TOOLS
	 * #########################################################################
	 */

	/**
	 * Tool das beim Zusammensetzen von TextResourcen hilft.
	 * 
	 * @param textResource
	 *            String in welchem ersetzt werden soll.
	 * @param key
	 *            String der ersetzt werden soll.
	 * @param value
	 *            String mit jedes Vorkommen des key ersetzt werden soll.
	 * @return Die TextResource mit den gewünschten Ersetzungen.
	 */
	public static String replace(String textResource, int key, String value) {
		return StringUtils.replace(textResource, "$" + key + "$", value);
	}

	/**
	 * Tool das beim Zusammensetzen von TextResourcen hilft.
	 * 
	 * @param textResource
	 *            String in welchem ersetzt werden soll.
	 * @param key
	 *            String der ersetzt werden soll.
	 * @param value
	 *            String mit jedes Vorkommen des key ersetzt werden soll.
	 * @return Die TextResource mit den gewünschten Ersetzungen.
	 */
	public static String replace(String textResource, int key, int value) {
		return StringUtils.replace(textResource, "$" + key + "$", value + "");
	}

	/**
	 * #########################################################################
	 * OTHER LOCALIZATION
	 * #########################################################################
	 */

	/**
	 * Gibt den GMT Offset der Einsatzstandortes zurück
	 * 
	 * @return int mit dem Offset
	 */
	public static int getGMTOffset() {
		return +1;
	}

	/**
	 * Formatiert die Zeit für den Einsatzort
	 * 
	 * @param hour_on_24Scale
	 *            int Stude des Tages im 24-Studen Format. Zulässig 0-23.
	 * @param minute
	 *            int Minute der Stunde. 1- oder 2-stellig. Zulässig 0-59
	 * @return String mit der formatierten Zeit.
	 */
	public static String getFormatedTime(int hour_on_24Scale, int minute) {
		// Helper (evtl. nicht benötigte auskommentieren)
		// String hour_12Scale = ((hour_on_24Scale > 12) ? hour_on_24Scale - 12
		// : hour_on_24Scale) + "";
		String hour_24Scale = utils.StringUtils
				.fillInt(hour_on_24Scale, '0', 2);
		// String am_pm = (hour_on_24Scale > 12) ? "pm" : "am";
		String min = utils.StringUtils.fillInt(minute, '0', 2);

		return hour_24Scale + ":" + min;
	}

	/**
	 * Formatiert das Datum für den Einsatzort
	 * 
	 * @param inDay
	 *            int Nummer des Tages im Montas. 1- oder 2-stellig. Zulässig
	 *            1-31
	 * @param inMonth
	 *            int Nummer des Monats im Jahr. 1- oder 2-stellig. Zulässig
	 *            1-12
	 * @param year_4letter
	 *            int Nummer des Jahre im 4-stelligen Format. z.B. 2008
	 * @return String mit dem formatierten Datum.
	 */
	public static String getFormatedDate(int inDay, int inMonth,
			int year_4letter) {
		// Helper
		String day = (utils.StringUtils.fillInt(inDay, '0', 2)) + "";
		String month = (utils.StringUtils.fillInt(inMonth, '0', 2)) + "";
		String year_2letter = (year_4letter + "").substring(2);

		return day + "." + month + "." + year_2letter;
	}
}
