// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data;

import java.io.IOException;
import java.io.InputStream;

import dev.bootProfiler.ZeitMessung;

/**
 * Klasse zum Laden von Daten vom Handy mit Buffering
 */
public class BinResourceStream extends InputStream {

	private final InputStream sourceStream;
	private int bufferPos;
	private boolean endFound;
	private int endPos;

	private byte[] buffer;
	private static final int BUFFERSIZE = 65536;

	private static final short cryptKey = 0x5A;
	
	private static final ZeitMessung zeitMessungReadBufferChunk = ZeitMessung.getInstance("Lade Datensatz | Pre - Chunk");

	public BinResourceStream(String sourceName) throws IOException {
		super();
		if ((sourceStream = getClass().getResourceAsStream(sourceName)) == null) {
			System.err.println("resource not available: " + sourceName);
		}
		init();
	}
	
	public BinResourceStream(InputStream inputStream) throws IOException {
		super();
		if (inputStream == null) {
			System.err.println("InputStream == null");
		}
		this.sourceStream = inputStream;
		init();
	}
	
	private void init() {
		buffer = new byte[BUFFERSIZE];
		bufferPos = BUFFERSIZE;

		endFound = false;
		endPos = -1;		
	}

	private void readBufferChunk() throws IOException {
		zeitMessungReadBufferChunk.start();
		int inCnt;
		if ((inCnt = sourceStream.read(buffer)) < BUFFERSIZE) {
			if (inCnt == 0) {
				endFound = true;
				endPos = 0;
				sourceStream.close();
			} else {
				endFound = true;
				endPos = inCnt;
				sourceStream.close();
			}
		}

		for (int i = 0; i < inCnt; i++) {
			int value = buffer[i];
			if (value < 0) {
				value += 256;
			}
			buffer[i] = (byte) (value ^ cryptKey);
		}

		bufferPos = 0;
		zeitMessungReadBufferChunk.stop();
	}

	public int read(byte[] b, int off, int len) throws IOException {
		int bytesRead = 0;

		if ((endFound) && (bufferPos == endPos)) {
			return -1;
		}

		boolean finished = false;
		while (!finished) {
			if (bufferPos == BUFFERSIZE) {
				readBufferChunk();
			}

			int dataInBuffer;
			if (endFound) {
				dataInBuffer = endPos - bufferPos;
				finished = true;
			} else {
				dataInBuffer = BUFFERSIZE - bufferPos;
			}
			int chunkSize;
			if (dataInBuffer <= len - bytesRead) {
				chunkSize = dataInBuffer;
			} else {
				chunkSize = len - bytesRead;
				finished = true;
			}

			System.arraycopy(buffer, bufferPos, b, off + bytesRead, chunkSize);
			bufferPos += chunkSize;
			bytesRead += chunkSize;
		}
		return bytesRead;
	}

	public int read() throws IOException {
		if (bufferPos == BUFFERSIZE) {
			readBufferChunk();
		}
		if (endFound && endPos == bufferPos) {
			return -1;
		} else {
			int value = buffer[bufferPos++];
			if (value < 0) {
				value += 256;
			}
			return value;
		}
	}

}
