// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data;

import java.io.InputStream;

import model.DatumZeit;
import model.Haltestelle;
import model.IFahrt;
import model.Linie;
import model.Ort;
import control.Global;
import control.PushandrideMidlet;
import dev.Logger;
import dev.bootProfiler.ZeitMessung;

public class FileData {
	
	public static void showData() {
		viewDebugData = true;
	}

	private static boolean viewDatasetInfo = false;

	private static boolean viewDebugData = false;

	private static boolean finished = false;

	private static BinResourceStream input;
	private static InputStream overrideInputStream = null;
	private static MyDataInputStream dis;
	private static String fileName;

	public static void overrideInputStream(InputStream inputStream) {
		FileData.overrideInputStream = inputStream;
	}

	public static void loadDataStagePre(String datafile) {
		finished = false;

		fileName = datafile;

		try {
			if (overrideInputStream == null) {
				if (fileName.charAt(0) != '/') {
					fileName = '/' + fileName;
				}
				input = new BinResourceStream(fileName);
			} else {
				input = new BinResourceStream(overrideInputStream);
			}

			dis = new MyDataInputStream(input);

			ZeitMessung profil_FileData_ReadFirstByte = ZeitMessung.getStartedInstance("Lade Datensatz | ReadFirstByte");
			// erstes Byte muss 0x02 sein
			if (dis.readByte() != 2) {
				System.out.println("keine Gültige Datei (anfang)" + fileName);
				return;
			}
			profil_FileData_ReadFirstByte.stop();
			if (viewDebugData) {
				System.out.println();
				System.out.println("Einlesen: " + fileName);
			}

			// Data-Version
			String dataVersion = dis.readUTF();
			Data.setVersion(dataVersion);
			if (viewDebugData) {
				System.out.println();
				System.out.println("Version: " + dataVersion);
			}

			// gültig von - bis
			// Format: YYYY-MM-DD
			String zeitString = dis.readUTF();
			String year = zeitString.substring(0, 4);
			String month = zeitString.substring(5, 7);
			String day = zeitString.substring(8);
			int zeit = DatumZeit.createDatumZeit(Integer.valueOf(year).intValue(), Integer.valueOf(month).intValue(), Integer.valueOf(day).intValue(), 0, 0);
			int zeitGueltigVon = zeit;
			Data.setGueltigVon(zeitGueltigVon);

			zeitString = dis.readUTF();
			year = zeitString.substring(0, 4);
			month = zeitString.substring(5, 7);
			day = zeitString.substring(8);
			zeit = DatumZeit.createDatumZeit(Integer.valueOf(year).intValue(), Integer.valueOf(month).intValue(), Integer.valueOf(day).intValue(), 23, 59);

			int zeitGueltigBis = zeit;
			Data.setGueltigBis(zeitGueltigBis);
			if (viewDebugData) {
				System.out.println();
				System.out.println("Gültig von " + DatumZeit.toDateTimeString(zeitGueltigVon) + " bis " + DatumZeit.toDateTimeString(zeitGueltigBis));
			}

			// Orte
			ZeitMessung profil_FileData_Orte = ZeitMessung.getStartedInstance("Lade Datensatz | Pre - Orte");
			Ort[] orteArray = RawdataToClass.readOrteArray(dis);
			if (viewDebugData) {
				System.out.println();
				System.out.println("Orte: " + orteArray.length);
				for (int i = 0; i < orteArray.length; i++) {
					System.out.println("#" + i + ": " + orteArray[i].getScreenName());
				}
			}
			Data.setAlleOrte(orteArray);
			profil_FileData_Orte.stop();

			// Haltestellen
			ZeitMessung profil_FileData_Haltestellen = ZeitMessung.getStartedInstance("Lade Datensatz | Pre - Haltestellen");
			Haltestelle[] haltestellenArray = RawdataToClass.readHaltestellenInitArray(dis);
			if (viewDebugData) {
				System.out.println();
				System.out.println("Haltestellen: " + haltestellenArray.length);
				for (int i = 0; i < haltestellenArray.length; i++) {
					System.out.println("#" + i + ": " + haltestellenArray[i].getScreenName());
				}
			}
			Data.setAlleHaltestellen(haltestellenArray);
			profil_FileData_Haltestellen.stop();
		} catch (OutOfMemoryError oom) {
			PushandrideMidlet.OutOfMemoryException(FileData.class, oom, "0ßSAAs");
		} catch (Exception e) {
			Logger.fatalException(FileData.class, e, "0ßSAAs");
		}
		
		if (viewDebugData) {
			System.out.println();
			System.out.println("finished loadDataStagePre");
			System.out.println();
		}
	}

	public static void loadDataStageFinal() {
		if (viewDebugData) {
			System.out.println();
			System.out.println("start loadDataStageFinal");
			System.out.println();
		}
		
		// laden des Faktors aus den User Settings
		Global.getLaufzeitFaktor();

		try {
			Haltestelle[] haltestellenArray = Data.getAlleHaltestellen();

			// Linien
			ZeitMessung profil_FileData_Linien = ZeitMessung.getStartedInstance("Lade Datensatz | Final - Linien");
			Linie[] linienArray = RawdataToClass.readLinienArray(dis);
			if (viewDebugData) {
				System.out.println();
				System.out.println("Linien: " + linienArray.length);
				for (int i = 0; i < linienArray.length; i++) {
					System.out.println("#" + i + ": " + linienArray[i].getScreenName());
				}
			}
			Data.setAlleLinien(linienArray);
			profil_FileData_Linien.stop();

			// Fahrten
			ZeitMessung profil_FileData_Fahrten = ZeitMessung.getStartedInstance("Lade Datensatz | Final - Fahrten");
			IFahrt[] fahrtenArray = RawdataToClass.readFahrtenArray(dis);
			if (viewDebugData) {
				System.out.println();
				System.out.println("Fahrt: " + fahrtenArray.length);
				for (int i = 0; i < fahrtenArray.length; i++) {
					System.out.println("#" + i + ": " + fahrtenArray[i]);
				}
			}
			Data.setAlleFahrten(fahrtenArray);
			profil_FileData_Fahrten.stop();

			// verknüpfe die Haltestellen mit den Fahrten
			ZeitMessung profil_FileData_FahrtVerknuepfung = ZeitMessung.getStartedInstance("Lade Datensatz | Final - FahrtVerknuepfung");
			try {
				short anzHaltestellen = dis.readShort();
				if (anzHaltestellen < 0) {
					throw new Exception("falsche Elementanzahl");
				}
				// Haltestellen
				for (int haltestellenNr = 0; haltestellenNr < anzHaltestellen; haltestellenNr++) {
					short idHaltestellen = dis.readShort();
					IFahrt[] fahrten = dis.readShortFahrtenOfHaltestelleArray();

					haltestellenArray[idHaltestellen].setFahrten(fahrten);
				}
			} catch (Exception e) {
				e.printStackTrace();
				System.err.println(e.toString());
			}
			profil_FileData_FahrtVerknuepfung.stop();

			// Endbyte ??
			int endByte = dis.readByte();
			if (endByte != 3) {
				System.out.println("keine Gültige Datei (ende)" + fileName);
				return;
			}

			if (viewDebugData) {
				System.out.println();
				System.out.println("Einlesen - Fertig");
			}

			dis.close();
			input.close();

			if (viewDebugData) {
				System.out.println();
				System.out.println("finished loadDataStageFinal");
				System.out.println();
			}

			if (viewDatasetInfo) {
				System.out.println(Data.createDataSetInfo());
			}
			
			// DATEN SIND GELADEN
			Data.createHaltestellenOfLinienArray();
			
		} catch (OutOfMemoryError oom) {
			PushandrideMidlet.OutOfMemoryException(FileData.class, oom, "s09ns");
		} catch (Exception e) {
			Logger.fatalException(FileData.class, e, "s09ns");
		}

		finished = true;
	}

	public static boolean isLoaded() {
		return finished;
	}

	/**
	 * Läd den Datensatz mit einem Funktionsaufruf
	 */
	public static void loadDataComplete(String datafile) {
		loadDataStagePre(datafile);
		loadDataStageFinal();
	}

}
