// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data;

public class LoadMapping {
	private final int fixOrteSize;
	private final int fixHaltestellenSize;
	private final boolean[] useOrt;
	private final boolean[] useHaltestelle;
	private final int[] mappingOrt;
	private final int[] mappingHaltestelle;
	private int finalHaltestellenSize = -1;
	private int finalOrteSize = -1;
	private boolean isMapped = false;
	
	public LoadMapping(int allOrteSize, int fixOrteSize, int allHaltestellenSize, int fixHaltestellenSize) {
		this.fixOrteSize = fixOrteSize;
		this.fixHaltestellenSize = fixHaltestellenSize;
		useOrt = new boolean[allOrteSize - fixOrteSize];
		for (int i = 0; i < useOrt.length; i++) {
			useOrt[i] = false;
		}
		useHaltestelle = new boolean[allHaltestellenSize - fixHaltestellenSize];
		for (int i = 0; i < useHaltestelle.length; i++) {
			useHaltestelle[i] = false;
		}
		mappingOrt = new int[useOrt.length];
		for (int i = 0; i < mappingOrt.length; i++) {
			mappingOrt[i] = -1;
		}
		mappingHaltestelle = new int[useHaltestelle.length];
		for (int i = 0; i < mappingHaltestelle.length; i++) {
			mappingHaltestelle[i] = -1;
		}
	}
	
	private final int getArrayPosOfOrtId(int id) {
		return id - fixOrteSize;
	}

	private final int getArrayPosOfHaltestellenId(int id) {
		return id - fixHaltestellenSize;
	}
	
	public void useOrte(int[] idOrte) {
		for (int i = 0; i < idOrte.length; i++) {
			useOrt[getArrayPosOfOrtId(idOrte[i])] = true;
		}
	}

	public void useHaltestellen(int[] idHaltestellen) {
		for (int i = 0; i < idHaltestellen.length; i++) {
			useOrt[getArrayPosOfHaltestellenId(idHaltestellen[i])] = true;
		}
	}
	
	private void createMapping() {
		int newOrtId = fixOrteSize;
		for (int i = 0; i < useOrt.length; i++) {
			if (useOrt[i]) {
				mappingOrt[i] = newOrtId++;
			}
		}
		finalOrteSize = newOrtId;
		
		int newHaltestellenId = fixHaltestellenSize;
		for (int i = 0; i < useHaltestelle.length; i++) {
			if (useHaltestelle[i]) {
				mappingHaltestelle[i] = newHaltestellenId++;
			}
		}
		finalHaltestellenSize = newHaltestellenId;
		
		isMapped = true;
	}
	
	public int getOrteSize() {
		if (!isMapped) {
			createMapping();
		}
		return finalOrteSize;
	}
	
	public int getHaltestellenSize() {
		if (!isMapped) {
			createMapping();
		}
		return finalHaltestellenSize;
	}
	
	public int getMappedOrtId(int dataOrtId) {
		return mappingOrt[dataOrtId];
	}

	public int getMappedHaltestellenId(int dataHaltestelleId) {
		return mappingOrt[dataHaltestelleId];
	}
	
}
