// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data;

import gui.utils.IntelligentKey;
import model.Fahrt;
import model.Haltestelle;
import model.IFahrt;
import model.Laufweg;
import model.Linie;
import model.Ort;
import control.Global;
import dev.Logger;

public final class RawdataToClass {
	private static Ort readOrt(MyDataInputStream dis) {
		try {
			String OrtName = dis.readUTF();
			String OrtKurzName = dis.readUTF();
			byte Anschluss = dis.readByte();
			return new Ort(Anschluss, OrtName, OrtKurzName);
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println("EXCEPTION: " + e.toString());
			return null;
		}
	}

	public static Ort[] readOrteArray(MyDataInputStream dis) {
		try {
			int elemente = dis.readShort();
			if (elemente < 0) {
				throw new Exception("falsche Elementanzahl");
			}
			Ort[] orteArray = new Ort[elemente];
			// Elemente einlesen
			for (int i = 0; i < elemente; i++) {
				int id = dis.readShort();
				orteArray[id] = readOrt(dis);
				orteArray[id].setIntelligentKeyCondensedValues(IntelligentKey.generateCondensedValue(orteArray[id].getScreenName()));
			}
			return orteArray;
		} catch (Exception e) {
			Logger.fatalException(RawdataToClass.class, e, "s8sjns");
			return null;
		}
	}

	private static Linie readLinie(MyDataInputStream dis, int id) {
		try {
			String linienName = dis.readUTF();
			String fahrstrecke = dis.readUTF();
			return new Linie(linienName, fahrstrecke, id);
		} catch (Exception e) {
			Logger.fatalException(RawdataToClass.class, e, "xs9jns");
			return null;
		}
	}

	public static Linie[] readLinienArray(MyDataInputStream dis) {
		try {
			int elemente = dis.readShort();
			if (elemente < 0) {
				throw new Exception("falsche Elementanzahl");
			}

			Linie[] linienArray = new Linie[elemente]; 

			// Elemente einlesen
			for (int i = 0; i < elemente; i++) {
				int id = dis.readShort();
				linienArray[id] = readLinie(dis, id);
			}
			return linienArray;
		} catch (Exception e) {
			Logger.fatalException(RawdataToClass.class, e, "s,kjnas");
			return null;
		}
	}

	/**
	 * Liest die Haltestellen an sich ein - die Referenzen werden erst später
	 * gesetzt
	 */
	public static Haltestelle[] readHaltestellenInitArray(MyDataInputStream dis) {
		try {
			int elemente = dis.readShort();
			if (elemente < 0) {
				throw new Exception("falsche Elementanzahl");
			}
			Haltestelle[] haltestelleArray = new Haltestelle[elemente];
			for (int i = 0; i < elemente; i++) {
				int id = dis.readShort();
				String name = dis.readUTF();
				byte Anschluss = dis.readByte();
				int ortId = dis.readShort();
		
				// workaround - Id als short laden
				int uniqueIdInt = dis.readShort();
				if (uniqueIdInt < 0) {
					uniqueIdInt = uniqueIdInt + 65536;
				}
				String uniqueId = uniqueIdInt + "";
				String tarifZone = dis.readUTF();
				if (uniqueId.equals(Global.haltestelleFlughafenUniqueId)) {
					name = "Flughafen/Messe";
				}
				haltestelleArray[id] = new Haltestelle(Anschluss, name, Data.getOrt(ortId), uniqueId, tarifZone);
				haltestelleArray[id].setIntelligentKeyCondensedValues(IntelligentKey.generateCondensedValue(haltestelleArray[id].getScreenName()));
			}
			return haltestelleArray;
		} catch (Exception e) {
			Logger.fatalException(RawdataToClass.class, e, "saDnsi");
			return null;
		}
	}

	public static IFahrt[] readFahrtenArray(MyDataInputStream dis) {
		try {
			int elemente = dis.readShort();
			if (elemente < 0) {
				throw new Exception("falsche Elementanzahl");
			}
			IFahrt[] fahrtenArray = new IFahrt[elemente];

			int elementeLaufwege = dis.readShort();
			int elementeFahrt = elemente - elementeLaufwege;
			if (elementeLaufwege < 0) {
				throw new Exception("falsche Elementanzahl (Laufwege)");
			}
			if (elementeFahrt < 0) {
				throw new Exception("falsche Elementanzahl (Fahrt)");
			}

			// Laufwege einlesen
			for (int i = 0; i < elementeLaufwege; i++) {
				int id = dis.readShort();
				fahrtenArray[id] = readLaufweg(dis);
			}

			// Fahrten einlesen
			for (int i = elementeLaufwege; i < elemente; i++) {
				int id = dis.readShort();
				fahrtenArray[id] = readFahrt(dis);
			}
			return fahrtenArray;
		} catch (Exception e) {
			Logger.fatalException(RawdataToClass.class, e, "dcSSSK");
			return null;
		}
	}

	//	public static int cntA = 0;
	//	public static int cntB = 0;
	private static IFahrt readFahrt(MyDataInputStream dis) {
		try {
			int idLinie = dis.readShort();
			Linie linie = Data.getLinie(idLinie);
			Haltestelle[] haltestellen = dis.readShortHaltestellenOfFahrtArray();
			short[] azeitenMoFrSchule = dis.readShortAbfahrtszeitenArray();
			short[] azeitenMoFrFerien = dis.readShortAbfahrtszeitenArray();
			short[] azeitenSa = dis.readShortAbfahrtszeitenArray();
			short[] azeitenSo = dis.readShortAbfahrtszeitenArray();
			int[] fzeiten = dis.readByteFahrzeitenArray();
			return new Fahrt(linie, haltestellen, azeitenMoFrSchule, azeitenMoFrFerien, azeitenSa, azeitenSo, fzeiten);
		} catch (Exception e) {
			Logger.fatalException(RawdataToClass.class, e, "aXS)s");
			return null;
		}
	}

	private static IFahrt readLaufweg(MyDataInputStream dis) {
		try {
			int idHaltestelleVon = dis.readShort();
			Haltestelle haltestelleVon = Data.getHaltestelle(idHaltestelleVon);
			int idHaltestelleNach = dis.readShort();
			Haltestelle haltestelleNach = Data.getHaltestelle(idHaltestelleNach);
			byte laufzeit = dis.readByte();
			return new Laufweg(haltestelleVon, haltestelleNach, laufzeit);
		} catch (Exception e) {
			Logger.fatalException(RawdataToClass.class, e, "asdSSX)s");

			return null;
		}
	}
}
