// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data.persistence;

import gui.guis.GuiSelectionHome;
import model.Haltestelle;
import model.HaltestellenPaar;
import data.Data;
import data.persistence.serialization.Serialization_StringArray;
import dev.Logger;

/**
 * Favoriten sind HaltestellenPaare
 * Die Paare werden als zwei ints in ein int[] serialisiert.
 */
public class Favoriten {

	/**
	 * Adds a HaltestellenPaar at the given position
	 * 
	 * @param haltestellen
	 * @param position:
	 *            0 >= position < GuiSelectionStartScreen.getFreeHaltestellenPaareSlots()
	 */
	public static void setFavorit(HaltestellenPaar paar, int position) {
		int anzFavoritenSlots = GuiSelectionHome.getFreeHaltestellenPaareSlots();
		if (position < 0 || position >= anzFavoritenSlots) {
			position = 0;
			Logger.error(Favoriten.class, "Given position out of bound! setting to 0");
		}

		String[] data = Serialization_StringArray.deserialize(Persistence.get(Persistence.SLOT_FAVORITEN));
		int anzFavoriten = Math.max(anzFavoritenSlots, data.length / 2);
		// hartes Limit für die Favoriten
		if (anzFavoriten > 20) {
			Logger.error(Favoriten.class, "zu viele Favoriten " + anzFavoriten);
			anzFavoriten = 20;
		}

		// neue Favoritenarray erstellen und initialisieren
		String[] dataNew = new String[anzFavoriten * 2];
		for (int i = 0; i < dataNew.length; i++) {
			dataNew[i] = Haltestelle.NULL.getUniqueId();
		}

		// altes Favoritenarray drüberkopieren
		if (data != null) {
			System.arraycopy(data, 0, dataNew, 0, Math.min(dataNew.length, data.length));
		}

		if (paar != HaltestellenPaar.NULL) {
			// falls dasselbe HaltestellenPaar bereits an
			// anderer Position gepseichert ist, gebe diesen Slot frei
			HaltestellenPaar paare[] = new HaltestellenPaar[dataNew.length / 2]; 
			for (int i = 0; i < paare.length; i++) {
				if (i == position) {
					continue;
				}
				Haltestelle von = Data.getHaltestelleByUniqueId(dataNew[2 * i]);
				Haltestelle nach = Data.getHaltestelleByUniqueId(dataNew[2 * i + 1]);
	
				HaltestellenPaar haltestellenPaar = new HaltestellenPaar(von, nach);
				if (haltestellenPaar.isValid()) {
					if (paar.equals(haltestellenPaar) || paar.equals(haltestellenPaar.switchVonNach())) {
						dataNew[i * 2] = Haltestelle.NULL.getUniqueId();
						dataNew[i * 2 + 1] = Haltestelle.NULL.getUniqueId();
					}
				} 
			}
		}
		
		// neuen Favorit speichern
		dataNew[2 * position] = paar.getVon().getUniqueId();
		dataNew[2 * position + 1] = paar.getNach().getUniqueId();

		Persistence.set(Persistence.SLOT_FAVORITEN, Serialization_StringArray.serialize(dataNew), Persistence.RS_COMMIT_IMMEDIATLY);
	}

	/**
	 * Gibt ein Array mit den HaltestellenPaar-Favoriten zurueck. Die Größe des zurückgegebenen
	 * Arrays ist immer GuiSelectionStartScreen.getFreeHaltestellenPaareSlots()! Wenn an einer Position kein Favorit gepspeichert
	 * wurde ist HaltestellenPaar.NULL an dieser Stelle.
	 * 
	 * @return
	 */
	public static HaltestellenPaar[] getFavoriten() {
		int anzFavoritenSlots = GuiSelectionHome.getFreeHaltestellenPaareSlots();

		HaltestellenPaar[] paare = new HaltestellenPaar[anzFavoritenSlots];
		for (int i = 0; i < anzFavoritenSlots; i++) {
			paare[i] = HaltestellenPaar.NULL;
		}
		
		byte[] savedBytes = Persistence.get(Persistence.SLOT_FAVORITEN);
		if (savedBytes == null || savedBytes.length == 0) {
			return paare;
		}
		
		String[] data = Serialization_StringArray.deserialize(savedBytes);
		
		int loadSlots = Math.min(anzFavoritenSlots, data.length / 2);

		for (int i = 0; i < loadSlots; i++) {
			Haltestelle von = Data.getHaltestelleByUniqueId(data[2 * i]);
			Haltestelle nach = Data.getHaltestelleByUniqueId(data[2 * i + 1]);

			HaltestellenPaar haltestellenPaar = new HaltestellenPaar(von, nach);
			if (haltestellenPaar.isValid()) {
				paare[i] = haltestellenPaar;
			} else {
				paare[i] = HaltestellenPaar.NULL;
			}
		}

		return paare;
	}

}
