// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data.persistence;

import gui.items.ItemLocationLastUsed;

import java.util.Vector;

import model.Haltestelle;
import model.HaltestellenPaar;
import control.Global;
import data.Data;
import data.persistence.serialization.Serialization_StringArray;

public class LastHaltestellenPaare {

	public static void addLastHaltestellen(Haltestelle von, Haltestelle nach) {
		String[] last = LastHaltestellenPaare.getLastHaltestellen();
		String[] toWrite = new String[Math.min(last.length + 2, Global.NUM_LAST_HALTESTELLEN_SAVED)];
		
		toWrite[0] = von.getUniqueId();
		toWrite[1] = nach.getUniqueId();
		
		int written = 2;
		for (int i = 0; i < last.length; i++) {
			toWrite[written] = last[i];
			written++;

			// Limit the number of last Haltestellen
			if (written >= Global.NUM_LAST_HALTESTELLEN_SAVED) {
				break;
			}
		}

		// Save the byte[]
		Persistence.set(Persistence.SLOT_LASTHALTESTELLEN, Serialization_StringArray.serialize(toWrite), Persistence.RS_COMMIT_IMMEDIATLY);
	}

	/**
	 * liest die zuletzt verwendeten Haltestellen aus dem RecordStore
	 * ACHTUNG: kann IDs zurückliefern von Haltestellen, die nicht mehr existieren
	 */
	private static String[] getLastHaltestellen() {
		byte[] savedBytes = Persistence.get(Persistence.SLOT_LASTHALTESTELLEN);
		if (savedBytes == null || savedBytes.length == 0) {
			return new String[0];
		}
		return Serialization_StringArray.deserialize(savedBytes);
	}
	
	/**
	 * Gibt die zuletzt gewählten Haltestellen zurück, wobei keine Haltestelle
	 * doppelt vorkommt.
	 * ACHTUNG: kann IDs zurückliefern von Haltestellen, die nicht mehr existieren
	 */
	public static String[] getLastHaltestellenDedublicated() {
		String[] lastHaltestellen = LastHaltestellenPaare.getLastHaltestellen();
		
		// Dedublizieren
		Vector dedublizierteIDs = new Vector();
		for(int i = 0; i < lastHaltestellen.length; i++) {
			String obj = new String(lastHaltestellen[i]);
			if(!dedublizierteIDs.contains(obj)) {
				dedublizierteIDs.addElement(obj);
			}
		}
	
		// Rückgabe
		String[] result = new String[Math.min(dedublizierteIDs.size(), Global.NUM_RETUNED_LAST_HALTESTELLEN)];
		for(int i = 0; i < result.length; i++) {
			result[i] = ((String)dedublizierteIDs.elementAt(i));
		}
		
		return result;
	}
	
	/**
	 * Gibt die zuletzt gewählten HaltestellenPaare zurück.
	 * @return Vector mit HaltestellenPaar
	 */
	public static Vector getLastHaltestellenPaare() {
		String[] lastHaltestellen = LastHaltestellenPaare.getLastHaltestellen();
		
		// Es muss eine gerade Zahl an Haltestellen im Speicher sein.
		if(lastHaltestellen.length % 2 != 0 || lastHaltestellen.length == 0) {
			return new Vector();
		}
		
		Vector result = new Vector();
		for(int i = 0; i < lastHaltestellen.length / 2; i++) {
			Haltestelle von = Data.getHaltestelleByUniqueId(lastHaltestellen[2*i]);
			Haltestelle nach = Data.getHaltestelleByUniqueId(lastHaltestellen[2*i+1]);
			HaltestellenPaar haltestellenPaar = new HaltestellenPaar(von, nach);
			if (haltestellenPaar.isValid()) {
				result.addElement(haltestellenPaar);
			}
		}
		return result;
	}

	/**
	 * Returns an array of ItemLastHaltestelle objects, which represent the last X selected
	 * Haltestellen. Used in GUI.
	 * 
	 * @return
	 */
	public static ItemLocationLastUsed[] getHaltestellenItems() {
		String[] lastUsedHaltestellenIDs = LastHaltestellenPaare.getLastHaltestellenDedublicated();
		Vector v = new Vector();
		for (int i = 0; i < lastUsedHaltestellenIDs.length; i++) {
			Haltestelle haltestelle = Data.getHaltestelleByUniqueId(lastUsedHaltestellenIDs[i]);
			if (haltestelle != Haltestelle.NULL) {
				v.addElement(new ItemLocationLastUsed(haltestelle));
			}
		}
		ItemLocationLastUsed[] result = new ItemLocationLastUsed[v.size()];
		v.copyInto(result);
		return result;
	}
}
