// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data.persistence;

import gui.Manager;
import gui.guis.GuiNewDataset;
import gui.guis.GuiSelectionDatasetSelect;

import java.util.Hashtable;

import utils.DynamicIntArray;
import control.Device;
import control.connections.internet.NetConnection;
import data.Data;
import data.persistence.serialization.Serialization_StringArray;
import dev.Logger;

public class RemoteDataBinary implements Runnable {
	
	public static boolean showInfoDueToVersionConfilct = false;

	public static String[] getCustomLinien() {
		String[] s = Serialization_StringArray.deserialize(Persistence.get(Persistence.SLOT_DATASET_CUSTOM_LINES));
		//		System.out.println("loading " + s.length + " elements");
		return s;
	}

	public static void saveCustomLinien(String[] s) {
		//		System.out.println("saving " + s.length + " elements");
		Persistence.set(Persistence.SLOT_DATASET_CUSTOM_LINES, Serialization_StringArray.serialize(s), Persistence.RS_COMMIT_IMMEDIATLY);
	}

	public static void storeBinaryInRecordstore(byte[] data, int version, int update) {
		Persistence.set(Persistence.SLOT_DATASET_CUSTOM_BINARY, data, Persistence.RS_COMMIT_AFTER_300ms);
		UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_DATENSATZ_VERSION, version);
		UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_DATENSATZ_UPDATE, update);
	}

	public static byte[] getBinaryFromRecordstore() {
		byte[] data = Persistence.get(Persistence.SLOT_DATASET_CUSTOM_BINARY);
		if (data == null) {
			return new byte[0];
		}
		return data;
	}

	private static GuiNewDataset guiNewDataset;
	private static RemoteDataBinary downloadTask;
	public boolean canceled = false;

	public static void getRemoteBinary() {
		guiNewDataset = Manager.downloadDatasetWrapper();
		//		Manager.showMessage("HINWEIS", "Datensatz wird heruntergeladen - bitte warten.", GuiMessageScreen.TYPE_MESSAGE_FULLSCREEN);
		downloadTask = new RemoteDataBinary();
		new Thread(downloadTask).start();
	}
	
	public static int getVersion() {
		int version = UserSettingsInt.get(UserSettingsInt.SETTINGS_ID_DATENSATZ_VERSION);
		if (version == DynamicIntArray.NULL_VALUE) {
			if (Persistence.get(Persistence.SLOT_DATASET_CUSTOM_BINARY) != null) {
				// in der ersten Version wurde die Info nicht gespeichert
				return 0;
			} else {
				// nicht vorhanden
				return -1;
			}
		} else {
			return version;
		}
	}

	public static int getUpdate() {
		int update = UserSettingsInt.get(UserSettingsInt.SETTINGS_ID_DATENSATZ_UPDATE);
		if (update == DynamicIntArray.NULL_VALUE) {
			if (Persistence.get(Persistence.SLOT_DATASET_CUSTOM_BINARY) != null) {
				// in der ersten Version wurde die Info nicht gespeichert
				return 0;
			} else {
				// nicht vorhanden
				return -1;
			}
		} else {
			return update;
		}
	}

	private void returnWithoutError() {
		guiNewDataset.triggerRestart();
	}

	public static void cancelDownload() {
		downloadTask.canceled = true;
	}

	public void run() {
		String[] linien = getCustomLinien();

		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < linien.length; i++) {
			if (i != 0) {
				sb.append(",");
			}
			sb.append(linien[i]);
		}

		Hashtable ht = NetConnection.sendRequest("http://pushandride.appspot.com/a/0,1,2/"+ sb.toString() + "/create");
		if (canceled) {
			return;
		}

		if (!ht.containsKey("status") || !((String) ht.get("status")).equals("ok")) {
			Logger.error(RemoteDataBinary.class, "post: status");
			returnWithError();
			return;
		}

		if (!ht.containsKey("size")) {
			Logger.error(RemoteDataBinary.class, "post: size");
			returnWithError();
			return;
		}

		if (!ht.containsKey("version")) {
			Logger.error(RemoteDataBinary.class, "post: version");
			returnWithError();
			return;
		}

		if (!ht.containsKey("update")) {
			Logger.error(RemoteDataBinary.class, "post: update");
			returnWithError();
			return;
		}

		int remoteDataVersion = Integer.parseInt((String) ht.get("version"));
		if (Data.VERSION > remoteDataVersion) {
			Logger.error(RemoteDataBinary.class, "remote Dataversion too old");
			returnWithError();
			return;
		}
		if (Data.VERSION < remoteDataVersion) {
			Logger.error(RemoteDataBinary.class, "remote Dataversion too new");
			returnInvalidVersion();
			return;
		}

		int remoteUpdateVersion = Integer.parseInt((String) ht.get("update"));

		String downloadUrl = (String) ht.get("url");
		int size = Integer.parseInt((String) ht.get("size"));

		guiNewDataset.startDownload(size);

		byte[] data = NetConnection.downloadBinary(downloadUrl, guiNewDataset, this);

		if (canceled) {
			return;
		}

		if (data.length != size) {
			Logger.error(RemoteDataBinary.class, "unterschiedliche Dateigröße: data.length" + data.length + " referenz: " + size);
			returnWithError();
			return;
		}

		if (canceled) {
			return;
		}

		RemoteDataBinary.storeBinaryInRecordstore(data, remoteDataVersion, remoteUpdateVersion);
		returnWithoutError();
	}

	private void returnWithError() {
		guiNewDataset.error();
	}

	private void returnInvalidVersion() {
		guiNewDataset.invalidVersion();
	}

	public static void handleCustomSelection(String[] s) {
		if (s.length == 0) {
			saveCustomLinien(s);

			Logger.info(RemoteDataBinary.class, "keine individuelle Linie ausgewählt -> kleiner Datensatz");

			Manager.datasetSelectWrapper(GuiSelectionDatasetSelect.ITEM_ID_DATASET_SMALL);
			return;
		}

		String[] last = getCustomLinien();
		if (last.length == s.length) {
			boolean isSame = true;
			for (int i = 0; i < s.length; i++) {
				if (!s[i].equals(last[i])) {
					isSame = false;
					break;
				}
			}
			// selber Datensatz wie letztes mal
			if (isSame && getBinaryFromRecordstore().length != 0) {
				UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_DATENSATZ_TYP, Data.DATSET_INDIVIDUELL);
				Manager.showRestartGuiWrapper();
				return;
			}
		}

		saveCustomLinien(s);
		// for safety
		UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_DATENSATZ_TYP, Data.DATSET_INDIVIDUELL);
		removeBinary();
		getRemoteBinary();
	}
	
	public static void removeBinary() {
		storeBinaryInRecordstore(new byte[0], -1, -1);
	}
	
	public static String getUniqueIdFromRecordStore() {
		byte[] b = Persistence.get(Persistence.SLOT_UNIQUE_ID);
		if (b == null || b.length == 0) {
			return Device.UNIQUE_ID_UNKNOWN;
		} else {
			return new String(b);
		}
	}

	public static void saveUniqueIdInRecordStore(String uniqueId) {
		Persistence.set(Persistence.SLOT_UNIQUE_ID, uniqueId.getBytes(), Persistence.RS_COMMIT_IMMEDIATLY);
	}

}
