// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data.persistence;

import data.persistence.serialization.Serialization_IntegerArray;

public class Stats {

	private static final int ANZAHL = 4; // Aktualisieren nicht vergessen!!!

	public static final int STARTS = 0;
	public static final int BERECHNETE_ROUTEN_NEUEROUTE = 1;
	public static final int BERECHNETE_ROUTEN_FAVORITEN = 2;
	public static final int BERECHNETE_ROUTEN_ZULETZTVERWENDET = 3;

	/**
	 * Erhoeht die gewaehlte Statistik um eins.
	 */
	public static void inc(int type) {
		int[] data = Stats.getStats();
		data[type] = data[type] + 1;
		Stats.saveStats(data);
	}

	/**
	 * Gibt den aktuellen Wert der uebergebenen Statistik zurueck.
	 */
	public static int get(int type) {
		int[] data = Stats.getStats();
		return data[type];
	}

	/**
	 * Setzt alle Statistiken auf 0 zurueck.
	 */
	public static void resetAll() {
		int[] newData = new int[Stats.ANZAHL];
		for (int i = 0; i < newData.length; i++)
			newData[i] = 0;
		Stats.saveStats(newData);
	}

	private static void saveStats(int[] stats) {
		Persistence.set(Persistence.SLOT_STATS,
			Serialization_IntegerArray.serialize(stats),
			Persistence.RS_COMMIT_AFTER_300ms);
	}

	/**
	 * Fehlerfall: int[] mit lauter 0ern
	 */
	private static int[] getStats() {
		byte[] savedBytes = Persistence.get(Persistence.SLOT_STATS);
		if (savedBytes != null) {
			int[] deserialized = Serialization_IntegerArray.deserialize(savedBytes);
			if (deserialized.length == Stats.ANZAHL)
				return deserialized;
		}

		// Irgendwas war nicht ok, gebe ein int[] mit lauter 0ern zurueck.
		int[] ret = new int[Stats.ANZAHL];
		for (int i = 0; i < ret.length; i++)
			ret[i] = 0;
		return ret;
	}
	
	public static String getStatsInfo() {
		StringBuffer sb = new StringBuffer();
		sb.append("Starts: " + get(STARTS) + "\n");
		sb.append("Route (Neu): " + get(BERECHNETE_ROUTEN_NEUEROUTE) + "\n");
		sb.append("Route (Fav): " + get(BERECHNETE_ROUTEN_FAVORITEN) + "\n");
		sb.append("Route (Last): " + get(BERECHNETE_ROUTEN_ZULETZTVERWENDET));
		return sb.toString();
	}
}
