// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data.persistence;

import utils.DynamicIntArray;
import data.persistence.serialization.Serialization_IntegerArray;

public class UserSettingsInt {

	// IDs
	public final static int SETTINGS_ID_SOFTWAREKEYBOARD = 0;
	public final static int SETTINGS_ID_GFX_QUALITY = 1;
	public final static int SETTINGS_TIME_OFFSET = 2;
	public final static int SETTINGS_LAUFZEIT_FAKTOR = 3;
	public final static int SETTINGS_ID_HINWEISE_MASK = 4;
	public final static int SETTINGS_ID_HIDDENSOFTWAREKEYBOARD = 5;
	public final static int SETTINGS_ID_TOUCH_SIZE = 6;
	// Werbung ist obsolete - wird aber beibehalten zwecks Kombatibilität
	public final static int SETTINGS_WERBUNG_CURRENT_SLOT = 7;
	public final static int SETTINGS_ID_DATENSATZ_TYP = 8;
	public final static int SETTINGS_ID_DATENSATZ_VERSION = 9;
	public final static int SETTINGS_ID_DATENSATZ_UPDATE = 10;

	// Null Value
	public final static int NULL_VALUE = DynamicIntArray.NULL_VALUE;

	// Read Cache
	private static DynamicIntArray readCache = null;

	/**
	 * Setzt das User Setting für den gegebenen Wert. Der Key muss eine der
	 * Konstanten die in {@link UserSettingsInt} definiert sind sein!
	 */
	public static void set(int key, int value) {
		readCache.set(key, value);
		saveUserSettings();
	}

	/**
	 * Ermittelt eine User Setting. Der Key muss eine der Konstanten die in
	 * {@link UserSettingsInt} definiert sind sein!
	 * 
	 * @param key
	 * @return Den hinterlegten Wert als int. UserSettingsInt.NULL_VALUE falls
	 *         der Wert nicht gesetzt ist.
	 */
	public static int get(int key) {
		if (readCache == null) {
			// Das Stepping MUSS 1 sein, damit wir nicht unnötige Felder speichern!
			readCache = new DynamicIntArray(getUserSettings(), 1);
		}
		return readCache.get(key);
	}

	protected static void clearCache() {
		UserSettingsInt.readCache = null;
	}

	private static void saveUserSettings() {
		Persistence.set(Persistence.SLOT_SETTINGS_INT, Serialization_IntegerArray.serialize(readCache.getArray()), Persistence.RS_COMMIT_AFTER_300ms);
	}

	private static int[] getUserSettings() {
		// Falls der Cache leer ist.
		byte[] savedBytes = Persistence.get(Persistence.SLOT_SETTINGS_INT);
		if (savedBytes == null || savedBytes.length == 0) {
			return new int[0];
		}
		return Serialization_IntegerArray.deserialize(savedBytes);
	}
}
