// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data.persistence;

import utils.DynamicStringArray;
import data.persistence.serialization.Serialization_StringArray;

public class UserSettingsString {

	// IDs
	public final static int SETTINGS_ID_PLATFORM = 0;
	public final static int SETTINGS_ID_MESSAGING = 1;

	// Null Value
	public final static String NULL_VALUE = DynamicStringArray.NULL_VALUE;

	// Read Cache
	private static DynamicStringArray readCache = null;

	/**
	 * Setzt das User Setting für den gegebenen Wert. Der Key muss eine der Konstanten die in
	 * {@link UserSettingsString} definiert sind sein!
	 */
	public static void set(int key, String value) {
		readCache.set(key, value);
		saveUserSettings();
	}

	/**
	 * Ermittelt eine User Setting. Der Key muss eine der Konstanten die in {@link UserSettingsString}
	 * definiert sind sein!
	 * 
	 * @param key
	 * @return Den hinterlegten Wert als int. DynamicIntArray.NULL_VALUE falls der Wert nicht gesetzt ist.
	 */
	public static String get(int key) {
		if (readCache == null) {
			// Das Stepping MUSS 1 sein, damit wir nicht unnötige Felder speichern!
			readCache = new DynamicStringArray(getUserSettings(), 1);
		}
		return readCache.get(key);
	}
	
	protected static void clearCache() {
		UserSettingsString.readCache = null;
	}

	private static void saveUserSettings() {
		Persistence.set(Persistence.SLOT_SETTINGS_STRING,
			Serialization_StringArray.serialize(readCache.getArray()),
			Persistence.RS_COMMIT_AFTER_300ms);
	}

	private static String[] getUserSettings() {
		// Falls der Cache leer ist .
		byte[] savedBytes = Persistence.get(Persistence.SLOT_SETTINGS_STRING);
		if (savedBytes == null || savedBytes.length == 0) {
			return new String[0];
		}
		return Serialization_StringArray.deserialize(savedBytes);
	}
}
