// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package data.persistence.serialization;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import control.Global;
import dev.Logger;

public class Serialization_IntegerArray {

	public static int INT_SIZE = 4; // in Byte

	/**
	 * @param int
	 *            to serialize
	 * @return
	 */
	public static byte[] serialize(int[] data) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream outputStream = new DataOutputStream(baos);
		if (data == null || data.length == 0) {
			return new byte[0];
		}
		try {
			for (int i = 0; i < data.length; i++) {
				outputStream.writeInt(data[i]);
				if (Global.SHOW_DEBUG_OUTPUT) {
					System.out.println("Serialize: " + data[i]);
				}
			}
			return baos.toByteArray();
		} catch (Exception e) {
			Logger.fatalException(Serialization_IntegerArray.class, e, "asx8smk");
			return new byte[0];
		}
	}

	/**
	 * 
	 * @param int[]
	 *            serialization byte[]
	 * @return int[0] if deserialization fails.
	 */
	public static int[] deserialize(byte[] data) {
		if (data == null || data.length == 0) {
			return new int[0];
		}
		ByteArrayInputStream bais = new ByteArrayInputStream(data);
		DataInputStream inputStream = new DataInputStream(bais);
		try {
			int numOfInts = inputStream.available() / INT_SIZE; // a int consists out of 4 bytes
			int[] out = new int[numOfInts];
			for (int i = 0; i < numOfInts; i++) {
				out[i] = inputStream.readInt();
				if (Global.SHOW_DEBUG_OUTPUT) {
					System.out.println("Serialize: " + out[i]);
				}
			}
			return out;
		} catch (Exception e) {
			Logger.fatalException(Serialization_IntegerArray.class, e, "asxasdx");
			return new int[0];
		}
	}
}
