// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package dev;

import gui.launch.GameCanvasInstance;

import java.util.Vector;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;

import control.Pushandride;

public class DebugInfo implements CommandListener {
    private static Displayable lastDisplay;
    private static Vector debugStrings;
    private static Form form;
    static final Command cmdBack = new Command("back", Command.BACK, 1);
    static final Command cmdClear = new Command("clear", Command.ITEM, 2);
    static final Command cmdLine = new Command("line", Command.ITEM, 3);
    
    private static final boolean showExceptions = true;
    private static final boolean limitItems = true;
    private static final int maxItems = 20;

    private static void drawElements() {
        form.deleteAll();
        if (debugStrings == null) {
            form.append("no debug data");
        } else {
            for (int i = 0; i < debugStrings.size(); i++) {
                form.append((String) debugStrings.elementAt(i) + "\n");
            }
        }
    }
    
    /**
     * Zeigt die Debug-Daten an
     */
    public static void showDebugInfo(Displayable last) {
        lastDisplay = last;
        form = new Form("Debug");
        drawElements();
        form.addCommand(cmdBack);
        form.addCommand(cmdClear);
        form.addCommand(cmdLine);
        form.setCommandListener(new DebugInfo());
        Display.getDisplay(Pushandride.getMidlet()).setCurrent(form);
    }

    /**
     * Fügt einene String zum Debugscreen hinzu
     */
    public static void addDebugString(String s) {
    	add(s);
        if (s != null) {
            Logger.info(DebugInfo.class, "DebugInfo: " + s);
        } else {
        	Logger.error(DebugInfo.class, "addDebugString(null)");
        }
    }

    private static void add(String s) {
        if (debugStrings == null) {
            debugStrings = new Vector();
        }
        if (s != null) {
        	if (debugStrings.size() == maxItems && limitItems) {
        		debugStrings.removeElementAt(0);
        	}
            debugStrings.addElement(s);
        }
    }
    
    public static void addDebugStringException(String s) {
    	if (showExceptions) {
    		add(s);
    	}
    }

    /**
     * Löscht alle Elemente im Debug Screen
     */
    public static void clearDebugScreen() {
        if (debugStrings == null) {
            debugStrings = new Vector();
        }
        debugStrings.removeAllElements();
    }

    public void commandAction(Command cmd, Displayable dis) {
        if (cmd == cmdBack) {
            Display.getDisplay(Pushandride.getMidlet()).setCurrent(lastDisplay);
			GameCanvasInstance.getInstance().setFullScreenMode(true);
        }
        if (cmd == cmdClear) {
            debugStrings.removeAllElements();
            drawElements();
        }
        if (cmd == cmdLine) {
            debugStrings.addElement(" - - - - - - - ");
            drawElements();
        }
    }

    /**
     * Gibt einen String mit allen aktuellen DebugMeldungen zur Anzeige außerhalb des DebugScreens zurück.
     */
	public static String getAllDebugMessagesAsString() {
		String ret = "";
		for (int i = 0; i < debugStrings.size(); i++) {
            ret += (String) debugStrings.elementAt(i) + "\n";
        }
		return ret;
	}
}
