// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package dev;

import java.util.Hashtable;

public class Logger {

	// Gibt an ob bei einer normalen Nachricht ein StackTrace mitgeliefert werden soll.
	private static final boolean APPEND_STACK_TRACE = false;
	
	// Der LOGGER_OUTPUT ist eine flexible Lösung um die Ausgaben des Loggers umzuleiten.
	// Setze den Default Logger Output auf System.err der aktuellen JVM.
	public static ILoggerOutput LOGGER_OUTPUT = new ILoggerOutput() {
		public void log(String msg) {
			System.err.println(msg);
		}
	};
	
	// Das Interface das der Output Kanal implementieren muss.
	public interface ILoggerOutput {
		public void log(String msg);
	}

	// ////////////////////////////////////////////////////////////////////////
	// LOG LEVELS
	// ////////////////////////////////////////////////////////////////////////
	/** Ein Fehler mit dem man nicht mehr weiterabeiten kann. (Eine logisch nicht erreichbare Code-Zeile
	 * wurde erreicht) */
	public static final int LEVEL_FATAL = 3;
	
	/** Ein Fehler */
	public static final int LEVEL_ERROR = 2;
	
	/** Eine Warnung die auf einen potentiellen Fehler oder fehlerbegünstigende Situation hinweist. */
	public static final int LEVEL_WARNING = 1;
	
	/** Eine Info für einen Normalen Ablauf der keinene Fehlerfall darstellt. */
	public static final int LEVEL_INFO = 0;
	
	/** Default Level, wenn für eine Komponente kein andere definiert ist. **/
	private static final int LEVEL_DEFAULT = LEVEL_INFO;

	// ////////////////////////////////////////////////////////////////////////
	// SCHWELLWERTE DER EINZELNEN KOMPONENTEN
	// ////////////////////////////////////////////////////////////////////////
	private static Hashtable SCHWELLWERTE = null;
	private static boolean showLogMessage(int level, Class component) {
		if (SCHWELLWERTE == null) {
			// Initialisiere Schwellwerte falls noch nicht geschehen
			SCHWELLWERTE = new Hashtable();
			
			// <DEFINIERE INDIVIDUELLE SCHWELLWERTE für Komponenten>
			// Bsp: SCHWELLWERTE.put(Algorithmus.class, new Integer(Logger.LEVEL_INFO));
			// </DEFINIERE INDIVIDUELLE SCHWELLWERTE für Komponenten>
		}

		// Wenn der Level größer oder gleich des Schwellwertes ist, dann zeige die Meldung
		Object schwellwertObj = SCHWELLWERTE.get(component);
		if (schwellwertObj == null) {
			// Default
			return level >= Logger.LEVEL_DEFAULT;
		} else {
			int schwellwert = ((Integer) schwellwertObj).intValue();
			return level >= schwellwert;
		}
	}
	
	// ////////////////////////////////////////////////////////////////////////
	// INFO
	// ////////////////////////////////////////////////////////////////////////

	public static void info(Class component, String msg) {
		Logger.doLogging(LEVEL_INFO, component, msg);
	}
	
	public static void info(Object component, String msg) {
		Logger.doLogging(LEVEL_INFO, component.getClass(), msg);
	}
	
	public static void infoException(Class component, Throwable throwableOrException, String s) {
		addToDebugInfo(LEVEL_INFO, component, throwableOrException, s);
		Logger.doLogging(LEVEL_INFO, component, throwableOrException);
		throwableOrException.printStackTrace();
	}
	
	public static void infoExeption(Object component, Throwable throwableOrException, String s) {
		addToDebugInfo(LEVEL_INFO, component.getClass(), throwableOrException, s);
		Logger.doLogging(LEVEL_INFO, component.getClass(), throwableOrException);
		throwableOrException.printStackTrace();
	}

	// ////////////////////////////////////////////////////////////////////////
	// WARN
	// ////////////////////////////////////////////////////////////////////////
	
	public static void warn(Class component, String msg) {
		Logger.doLogging(LEVEL_WARNING, component, msg);
	}
	
	public static void warn(Object component, String msg) {
		Logger.doLogging(LEVEL_WARNING, component.getClass(), msg);
	}
	
	public static void warnException(Class component, Throwable throwableOrException, String s) {
		addToDebugInfo(LEVEL_WARNING, component, throwableOrException, s);
		Logger.doLogging(LEVEL_WARNING, component, throwableOrException);
		throwableOrException.printStackTrace();
	}
	
	public static void warnException(Object component, Throwable throwableOrException, String s) {
		addToDebugInfo(LEVEL_WARNING, component.getClass(), throwableOrException, s);
		Logger.doLogging(LEVEL_WARNING, component.getClass(), throwableOrException);
		throwableOrException.printStackTrace();
	}

	// ////////////////////////////////////////////////////////////////////////
	// ERROR
	// ////////////////////////////////////////////////////////////////////////

	public static void error(Class component, String msg) {
		Logger.doLogging(LEVEL_ERROR, component, msg);
	}
	
	public static void error(Object component, String msg) {
		Logger.doLogging(LEVEL_ERROR, component.getClass(), msg);
	}
	
	public static void errorException(Class component, Throwable throwableOrException, String s) {
		addToDebugInfo(LEVEL_ERROR, component, throwableOrException, s);
		Logger.doLogging(LEVEL_ERROR, component, throwableOrException);
		throwableOrException.printStackTrace();
	}
	
	public static void errorException(Object component, Throwable throwableOrException, String s) {
		addToDebugInfo(LEVEL_ERROR, component.getClass(), throwableOrException, s);
		Logger.doLogging(LEVEL_ERROR, component.getClass(), throwableOrException);
		throwableOrException.printStackTrace();
	}

	// ////////////////////////////////////////////////////////////////////////
	// FATAL
	// ////////////////////////////////////////////////////////////////////////
	
	public static void fatal(Class component, String msg) {
		Logger.doLogging(LEVEL_FATAL, component, msg);
	}
	
	public static void fatal(Object component, String msg) {
		Logger.doLogging(LEVEL_FATAL, component.getClass(), msg);
	}
	
	public static void fatalException(Class component, Throwable throwableOrException, String s) {
		addToDebugInfo(LEVEL_FATAL, component, throwableOrException, s);
		Logger.doLogging(LEVEL_FATAL, component, throwableOrException);
		throwableOrException.printStackTrace();
	}
	
	public static void fatalException(Object component, Throwable throwableOrException, String s) {
		addToDebugInfo(LEVEL_FATAL, component.getClass(), throwableOrException, s);
		Logger.doLogging(LEVEL_FATAL, component.getClass(), throwableOrException);
		throwableOrException.printStackTrace();
	}

	// ////////////////////////////////////////////////////////////////////////
	// INTERNAL
	// ////////////////////////////////////////////////////////////////////////
	
	private static void doLogging(int level, Class component, String msg) {
		if (showLogMessage(level, component)) {
			LOGGER_OUTPUT.log("[" + component.getName() + "] " + msg);
			
			// Da J2ME einen nicht auf den StackTrace zugreifen lässt, müssen wir es leider direkt ausgeben.
			if(APPEND_STACK_TRACE) {
				try {
					throw new Exception("Zeige StacktTace wie gewünscht in Logger.APPEND_STACK_TRACE");
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	private static void doLogging(int level, Class component, Throwable t) {
		if (showLogMessage(level, component)) {
			LOGGER_OUTPUT.log("[" + component.getName() + "] " + t.getMessage());
			t.printStackTrace();
		}
	}
	
	private static void addToDebugInfo(int level, Class component, Throwable throwableOrException, String message) {
		DebugInfo.addDebugStringException("EX" + level + " " + component + " " + throwableOrException + "@" + message);
	}
}
