// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package dev;

public class Memory {
	private static int arraybase = 16;
	private static int bytesize = 1;
	private static int shortsize = 2;
	private static int intsize = 4;
	private static int longsize = 8;
	private static int objectsize = 4;
	private static int stringcharsize = 1;
	private static int stringbase = objectsize + stringcharsize;
	private static boolean initialized = true;
	
	public static int getShortSize() {
		if (!initialized) {
			init();
		}
		return shortsize;
	}

	public static int getByteSize() {
		if (!initialized) {
			init();
		}
		return bytesize;
	}

	public static int getIntSize() {
		if (!initialized) {
			init();
		}
		return intsize;
	}

	public static int getLongSize() {
		if (!initialized) {
			init();
		}
		return longsize;
	}

	public static int getObjSize() {
		if (!initialized) {
			init();
		}
		return objectsize;
	}

	public static int getArrayBaseSize() {
		if (!initialized) {
			init();
		}
		return arraybase;
	}	
	
	public static int getStringBaseSize() {
		if (!initialized) {
			init();
		}
		return stringbase;
	}

	public static int getStringCharakterSize() {
		if (!initialized) {
			init();
		}
		return stringcharsize;
	}

	public static String getSizeInfo() {
		if (!initialized) {
			init();
		}
		String result = "";
		result += "array: " + arraybase + "\n";
		result += "byte: "  + bytesize + "\n";
		result += "short: " + shortsize + "\n";
		result += "int:   " + intsize + "\n";
		result += "long:  " + longsize + "\n";
		result += "obj:   " + objectsize + "\n";

		return result;
	}

	public static void init() {
		Runtime.getRuntime().gc();
		long before = Runtime.getRuntime().freeMemory();
		long after = Runtime.getRuntime().freeMemory();
		long diff = before - after;

		int none = 0;
		int one = 0;
		int two = 0;
		int three = 0;

		if (before < 0 || after < 0 || diff < -999) {
			System.err.println("can't be here");
		}
		Runtime.getRuntime().gc();
		before = Runtime.getRuntime().freeMemory();
		Runtime.getRuntime().gc();
		after = Runtime.getRuntime().freeMemory();
		diff = before - after;
		none = (int) diff;

		Runtime.getRuntime().gc();
		before = Runtime.getRuntime().freeMemory();
		int i[] = new int[1];
		Runtime.getRuntime().gc();
		after = Runtime.getRuntime().freeMemory();
		if (i[0] < 0) {
			System.err.println("aaa");
		}
		diff = before - after;
		one = (int) diff;

		Runtime.getRuntime().gc();
		before = Runtime.getRuntime().freeMemory();
		int j[] = new int[2];
		Runtime.getRuntime().gc();
		after = Runtime.getRuntime().freeMemory();
		if (j[0] < 0) {
			System.err.println("aaa");
		}
		diff = before - after;
		two = (int) diff;

		Runtime.getRuntime().gc();
		before = Runtime.getRuntime().freeMemory();
		int k[] = new int[3];
		Runtime.getRuntime().gc();
		after = Runtime.getRuntime().freeMemory();
		if (k[0] < 0) {
			System.err.println("aaa");
		}
		diff = before - after;
		three = (int) diff;

		Runtime.getRuntime().gc();
		before = Runtime.getRuntime().freeMemory();
		int l[] = new int[1000];
		Runtime.getRuntime().gc();
		after = Runtime.getRuntime().freeMemory();
		if (l[0] < 0) {
			System.err.println("aaa");
		}
		diff = before - after;
		intsize = (short) diff;

		Runtime.getRuntime().gc();
		before = Runtime.getRuntime().freeMemory();
		byte x[] = new byte[1000];
		Runtime.getRuntime().gc();
		after = Runtime.getRuntime().freeMemory();
		if (x[0] < 0) {
			System.err.println("aaa");
		}
		diff = before - after;
		bytesize = (int) diff;

		Runtime.getRuntime().gc();
		before = Runtime.getRuntime().freeMemory();
		short m[] = new short[1000];
		Runtime.getRuntime().gc();
		after = Runtime.getRuntime().freeMemory();
		if (m[0] < 0) {
			System.err.println("aaa");
		}
		diff = before - after;
		shortsize = (int) diff;

		Runtime.getRuntime().gc();
		before = Runtime.getRuntime().freeMemory();
		long n[] = new long[1000];
		Runtime.getRuntime().gc();
		after = Runtime.getRuntime().freeMemory();
		if (n[0] < 0) {
			System.err.println("aaa");
		}
		diff = before - after;
		longsize = (int) diff;

		Runtime.getRuntime().gc();
		before = Runtime.getRuntime().freeMemory();
		Object o[] = new Object[1000];
		Runtime.getRuntime().gc();
		after = Runtime.getRuntime().freeMemory();
		if (o[0] != null) {
			System.err.println("aaa");
		}
		diff = before - after;
		objectsize = (int) diff;

		int single = two - one;
		arraybase = one - single;
		if (none != 0) {
			System.err.println("error calculating Memory (none)");
		}
		if (three != arraybase + 3 * single) {
			System.err.println("error calculating Memory (three)");
		}
		if (intsize != arraybase + 1000 * single) {
			System.err.println("error calculating Memory (int)");
		}

		bytesize = (bytesize - arraybase) / 1000;
		shortsize = (shortsize - arraybase) / 1000;
		intsize = (intsize - arraybase) / 1000;
		longsize = (longsize - arraybase) / 1000;
		objectsize = (objectsize - arraybase) / 1000;
		
		stringbase = objectsize + stringcharsize;
		
		initialized = true;

	}
}
