// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package dev;

import java.util.Vector;

import utils.StringUtils;

public class Statistics {
	private final Vector data;

	// Breite und Höhe der gfx-Stats (nur ungefähr, maximal Faktor 2,5 größer)
	private final int gfxHeight = 20;
	private final int gfxWidth = 100;

	private int minValue;
	private int maxValue;

	private final String name;

	public Statistics(String name) {
		data = new Vector(1000, 0);
		this.name = name;
		clear();
	}

	public void add(int i) {
		data.addElement(new Integer(i));
		minValue = Math.min(minValue, i);
		maxValue = Math.max(maxValue, i);
	}

	public int getAnzahl() {
		return data.size();
	}

	public double getMittelwert() {
		double result = 0;
		double dataSize = data.size();
		for (int i = 0; i < data.size(); i++) {
			result += ((Integer) data.elementAt(i)).doubleValue() / dataSize;
		}
		return result;
	}

	public double getStandardabweichung() {
		double result = 0;
		double dataSize = data.size();
		double mittelWert = getMittelwert();
		for (int i = 0; i < data.size(); i++) {
			double diff = ((Integer) data.elementAt(i)).intValue() - mittelWert;
			result += diff * diff / dataSize;
		}
		return Math.sqrt(result);
	}

	public void clear() {
		data.removeAllElements();
		maxValue = Integer.MIN_VALUE;
		minValue = Integer.MAX_VALUE;
	}

	private final String barActive = "XX";
	private final String barSemiactive = " X";
	private final String barInactive = "  ";
	private final String barSpace = "  ";
	private final int barWidth = barActive.length();
	private final int barSpacing = barSpace.length();
	
	private void sortData() {
		int size = data.size();
		for (int i = 0; i < size; i++) {
			for (int j = i + 1; j < size; j++) {
				Integer a = (Integer) data.elementAt(i);
				Integer b = (Integer) data.elementAt(j);
				if (a.intValue() > b.intValue()) {
					data.setElementAt(a, j);
					data.setElementAt(b, i);
				}
			}
		}
	}

	public String showGfxStats() {
		String outputName = name;

		if ((name == null) || (name == "")) {
			outputName = "NONAME";
		}

		StringBuffer result = new StringBuffer("Gfx-Statistics  ### " + outputName + " ###\n");
		result.append("  % ^\n");

		if (data.size() == 0) {
			result.append("keine Daten vorhanden");
			return result.toString();
		}

		sortData();
		
		// 90% Wert wird als letzter Wert anvisiert
		int visibleMaxValue = ((Integer) data.elementAt(data.size() * 9 / 10)).intValue(); 

		int showDiff = visibleMaxValue - minValue;
		
		if (showDiff == 0) {
			showDiff = maxValue - minValue;
		}

		if (showDiff == 0) {
			result.append("alle Werte haben dieselbe Größe: " + visibleMaxValue);
			return result.toString();
		}
		
		// zuerst werden grob abschätzt wieviele Spalten benötigt werden
		int ungefaehreAnzahlBars = gfxWidth / (barWidth + barSpacing);

		int ungefaehresIntervall = (int) Math.ceil(showDiff / (double) ungefaehreAnzahlBars);
		
//		int diff2 = data.elementAt();
//		int ungefaehresIntervall2 = (int) Math.ceil(diff / (double) ungefaehreAnzahlBars);

		// die Spalten werden jetzt so gewählt, dass sie auf ein sinnvolles Intervall haben
		// int[] ai = {1,2,5};
		int[] ai = { 1, 2, 5, 10, 15, 20, 25, 30, 40, 50, 60, 70, 80, 90 };

		int pow10 = 1;
		int base = 0;
		int lastValue = 0;
		int currentValue = 0;
		do {
			lastValue = currentValue;
			base = (base + 1) % ai.length;
			if (base == 0) {
				pow10 = pow10 * 10;
			}
			currentValue = pow10 * ai[base];
		} while (currentValue <= ungefaehresIntervall);

		int interval = Math.max(1, lastValue);
		int intervalMinus = (interval) / 2;

		int firstMult = (minValue / interval) * interval;

		/* der code hier war für irgend etwas nützlich - aber für was ? */
//		if ((firstMult * interval - intervalMinus) >= minValue) {
//			firstMult++;
//		}

	
		int firstValue = firstMult * interval - intervalMinus;

		// Spalten-Objekte erstellen und füllen

		int anzahlBarsMin = (visibleMaxValue - firstValue) / interval + 1;
		int anzahlBarsMax = (maxValue - firstValue) / interval + 1;
		
		int anzahlBars = Math.min(anzahlBarsMax, ungefaehreAnzahlBars);
		anzahlBars = Math.max(anzahlBars, anzahlBarsMin);

//		 System.out.println("ungefAnzahlBars = " + ungefaehreAnzahlBars);
//		 System.out.println("ungefInterval   = " + ungefaehresIntervall);
//		 System.out.println("anzahlBarsMin   = " + anzahlBarsMin);
//		 System.out.println("anzahlBarsMax   = " + anzahlBarsMax);
//		 System.out.println("anzahlBars      = " + anzahlBars);
//		 System.out.println("minValue        = " + minValue);
//		 System.out.println("firstValue      = " + firstValue);
//		 System.out.println("maxValue        = " + maxValue);
//		 System.out.println("visibleMaxValue = " + visibleMaxValue);
//		 System.out.println("interval        = " + interval);

		Bar[] bars = new Bar[anzahlBars];
		for (int i = 0; i < bars.length; i++) {
			bars[i] = new Bar((firstMult + i) * interval);
		}

		for (int i = 0; i < data.size(); i++) {
			int value = ((Integer) data.elementAt(i)).intValue();

			int barNo = (value - firstValue) / interval;
			
//			System.out.println("barNo = " + barNo);

			bars[Math.min(barNo, bars.length - 1)].increase();
		}

		// Grafische Ausgabe
		
		int leftSpace = 6;
		String leftSpaceString = "";
		for (int i = 0; i < leftSpace; i++) {
			leftSpaceString += " ";
		}

		int barMax = 0;
		for (int barNo = 0; barNo < bars.length; barNo++) {
			barMax = Math.max(barMax, bars[barNo].getCount());
		}

		int yInt = barMax / gfxHeight;
		yInt = Math.max(yInt, 1);
		int yIntMinus = yInt / 2;

		int firstBarX = 0;
		
		// Zeilen ausgeben
		for (int y = gfxHeight; y > 0; y--) {
			int percent = (yInt * y * 100) / data.size();
			if (percent > 100) {
				continue;
			}
			StringBuffer zeile = new StringBuffer(StringUtils.fillInt(percent, ' ', 3) + " | ");
			firstBarX = zeile.length();
			for (int barNo = 0; barNo < bars.length; barNo++) {
				zeile.append(bars[barNo].draw(yInt * y - yIntMinus, yInt * y));
				zeile.append(barSpace);
			}

			zeile.append("\n");
			result.append(zeile);
		}

		// X-Achse
		StringBuffer s = new StringBuffer("    +-");
		int zwidth = bars.length * (barSpace.length() + barActive.length());
		for (int i = 0; i < zwidth; i++) {
			s.append("-");
		}
		s.append(">\n");

		result.append(s);

		// x-Beschriftung
		int xPos = firstBarX + barActive.length() / 2;

		String firstLine = "";
		String secondLine = "";

		for (int barNo = 0; barNo < bars.length; barNo++) {
			if (barNo % 2 == 0) {
				if (barNo == bars.length - 1) {
					// last
					firstLine = drawCentered(firstLine, ">=" +bars[barNo].getCenter() + "", xPos + 1);
				} else {
					firstLine = drawCentered(firstLine, bars[barNo].getCenter() + "", xPos);
				}
			} else {
				if (barNo == bars.length - 1) {
					// last
					secondLine = drawCentered(secondLine, ">="+bars[barNo].getCenter() + "", xPos + 1);
				} else {
					secondLine = drawCentered(secondLine, bars[barNo].getCenter() + "", xPos);
				}
			}

			xPos += (barSpace.length() + barActive.length());
		}

		result.append(firstLine + "\n");
		result.append(secondLine + "\n");

		// x-Beschriftung (Anzahl der Elemente)
		result.append("Anzahl:\n");

		firstLine = "";
		secondLine = "";
		xPos = firstBarX + barActive.length() / 2;

		for (int barNo = 0; barNo < bars.length; barNo++) {
			if (barNo % 2 == 0) {
				firstLine = drawCentered(firstLine, bars[barNo].getCount() + "", xPos);
			} else {
				secondLine = drawCentered(secondLine, bars[barNo].getCount() + "", xPos);
			}

			xPos += (barSpace.length() + barActive.length());
		}

		result.append(firstLine + "\n");
		result.append(secondLine + "\n");
		
		// statistische Größen 
		result.append("\\ Summe Anzahl der Elemente: " + StringUtils.formatDouble(data.size(), 7, 0) + "\n");
		result.append("\\ Mittelwert:                " + StringUtils.formatDouble(getMittelwert(), 7, 1) + "\n");
		result.append("\\ Standardabweichung:        " + StringUtils.formatDouble(getStandardabweichung(), 7, 1) + "\n");
		result.append("\\ Minimalwert:               " + StringUtils.formatDouble(minValue, 7, 0) + "\n");
		result.append("\\ Maximalwert:               " + StringUtils.formatDouble(maxValue, 7, 0) + "\n");

		return result.toString();
	}
	
	public String showTextStats() {
		StringBuffer result = new StringBuffer();
		
		result.append(StringUtils.safeSubString(name                                                   , 0, 19) + " ");
		result.append(StringUtils.safeSubString(StringUtils.formatDouble(getMittelwert(), 7, 1)        , 0, 9 ) + " ");
		result.append(StringUtils.safeSubString(StringUtils.formatDouble(getStandardabweichung(), 7, 1), 0, 9 ) + " ");
		result.append(StringUtils.safeSubString(StringUtils.formatDouble(minValue, 7, 0)               , 0, 9 ) + " ");
		result.append(StringUtils.safeSubString(StringUtils.formatDouble(maxValue, 7, 0)               , 0, 9 ) + " ");
		result.append(StringUtils.safeSubString(StringUtils.formatDouble(data.size(), 7, 0)            , 0, 9 ));
		
		return result.toString();
	}
	
	public static String showTextStatsHead() {
		StringBuffer result = new StringBuffer();
		
		result.append(StringUtils.safeSubString("Name"      , 0, 19) + " ");
		result.append(StringUtils.safeSubString("Mittelwert", 0, 9 ) + " ");
		result.append(StringUtils.safeSubString("StdAbw"    , 0, 9 ) + " ");
		result.append(StringUtils.safeSubString("Min"       , 0, 9 ) + " ");
		result.append(StringUtils.safeSubString("Max"       , 0, 9 ) + " ");
		result.append(StringUtils.safeSubString("#Elemente" , 0, 9 ));
		
		return result.toString();
	}
	

	private String drawCentered(String string, String text, int pos) {
		StringBuffer sb = new StringBuffer(string);
		int startPos = pos - text.length() / 2;
		for (int i = 0; i < text.length(); i++) {
			int currentPos = startPos + i;
			char nextChar = text.charAt(i);
			while (sb.length() < currentPos) {
				sb.append(" ");
			}
			if (sb.length() == currentPos) {
				sb.append(nextChar);
			} else {
				sb.deleteCharAt(currentPos);
				sb.insert(currentPos, nextChar);
			}
		}
		return sb.toString();
	}

	/**
	 * Hilfsklasse die die Säulen repräsentiert
	 */
	private class Bar {
		// center ist der Wert, der die Mitte der Säule auf der x-Achse repräsentiert
		private final int center;
		// count gibt die Anzahl der Elemente wieder die in der Säule sind
		private int count = 0;

		public Bar(int center) {
			this.center = center;
		}

		public void increase() {
			count++;
		}

		public int getCount() {
			return count;
		}

		public int getCenter() {
			return center;
		}
		
		/**
		 * Zeichnet die Säulenzeile
		 * falls cntMindestens <= count <  cntBig :  X 
		 * falls                  count <= cntBig : XX 
		 */
		public String draw(int cntMindestens, int cntBig) {
			if (count >= cntMindestens) {
				if (count >= cntBig) {
					return barActive;
				} else {
					return barSemiactive;
				}
			} else {
				return barInactive;
			}
		}
	}
}
