// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package dev.bootProfiler;

import dev.Logger;

public class ZeitMessung {

	private long startTime = -1;
	private long duration = 0;
	private String description;

	public static ZeitMessung getStartedInstance(String description) {
		return new ZeitMessung(description, true);
	}

	public static ZeitMessung getInstance(String description) {
		return new ZeitMessung(description, false);
	}

	private ZeitMessung(String description, boolean start) {
		this.description = description;

		BootProfiler.registerZeitmessung(this);
		
		if (start) {
			start();
		}
	}

	public void start() {
		if (startTime != -1) {
			Logger.warn(this, "Zeitmessung bereits gestartet");
			return;
		}
		this.startTime = System.currentTimeMillis();
	}

	public void stop() {
		if (startTime == -1) {
			Logger.warn(this, "Zeitmessung noch nicht gestartet");
			return;
		}
		duration += (System.currentTimeMillis() - startTime);
		this.startTime = -1;
	}

	public long getDuration() {
		return duration;
	}

	public String getDescription() {
		return description;
	}
}
