// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import gui.guis.GuiSelectionHome;
import gui.items.ItemHomeHaltestellenPaar;
import model.DatumZeit;
import model.HaltestellenPaar;
import model.Route;
import algo.Algorithmus;
import control.Global;
import control.PushandrideMidlet;
import dev.Logger;

public class BgStartScreen extends Thread {
	private final GuiSelectionHome guiSelectionStartScreen;

	private ItemHomeHaltestellenPaar[] items;
	private static boolean finish = false;
	private static Thread thisThread; 

	public BgStartScreen(GuiSelectionHome guiSelectionStartScreen) {
		finish = false;
		this.guiSelectionStartScreen = guiSelectionStartScreen;
		this.items = guiSelectionStartScreen.getAllItemHaltestellenPaare();
		thisThread = this;
	}

	public static void quit() {
		finish = true;
	}

	public static void forceTimeInvalid() {
		resetTime = true;
		if (thisThread != null) {
			try {
				synchronized (thisThread) {
					thisThread.notify();
				}
			} catch (Exception e) {
				Logger.errorException(BgStartScreen.class, e, "yxsa8js");
			}
		}
	}
	
	private static boolean resetTime = false;
	
	public void run() {
		
		try {
		boolean restart;
		if (Global.STARTSCREEN_BERECHNE_ROUTEN) {
			do {
				if (resetTime) {
					for (int i = 0; i < items.length; i++) {
						items[i].setInvalidTime();
					}
					resetTime = false;
				}
				
				this.items = guiSelectionStartScreen.getAllItemHaltestellenPaare();
				restart = false;

				// State 1 - Fahrzeiten berechnen
				for (int i = 0; i < items.length; i++) {
					if (!items[i].hasTime()) {
						if (finish) {
							break;
						}
						HaltestellenPaar hsp = items[i].getHaltestellenPaar();
						Route route = Algorithmus.calcStartScreenRoute(hsp.getVon(), hsp.getNach());

						if (items[i].tryToUpdateNaechsteFahrmoeglichkeit(route, hsp)) {
							guiSelectionStartScreen.redrawMain();
						}
						restart = true;
						break;
					}
				}

				if (restart || finish || resetTime) {
					continue;
				}

				// State 2 - Fahrzeiten der Rücklinien berechnen
				for (int i = 0; i < items.length; i++) {
					if (!items[i].hasReversedTime()) {
						if (finish) {
							break;
						}
						HaltestellenPaar hsp = items[i].getHaltestellenPaar();
						Route route = Algorithmus.calcStartScreenRoute(hsp.getNach(), hsp.getVon());
						if (items[i].tryToUpdateNaechsteFahrmoeglichkeit(route, hsp.switchVonNach())) {
							guiSelectionStartScreen.redrawMain();
						}
						restart = true;
						break;
					}
				}

				if (restart || finish || resetTime) {
					continue;
				}

				// State 3 - Zeiten (Hinfahrt) updaten 				
				int now = DatumZeit.Now();
				for (int i = 0; i < items.length; i++) {
					if (items[i].getAbfahrtsZeit() == Global.TIME_ERROR) {
						continue;
					}
					if (items[i].getAbfahrtsZeit() + 1 < now) {
						if (finish) {
							break;
						}
						HaltestellenPaar hsp = items[i].getHaltestellenPaar();
						Route route = Algorithmus.calcStartScreenRoute(hsp.getVon(), hsp.getNach());
						now = DatumZeit.Now();
						if (items[i].tryToUpdateNaechsteFahrmoeglichkeit(route, hsp)) {
							guiSelectionStartScreen.redrawMain();
						}
						restart = true;
						break;
					}
				}

				if (restart || finish || resetTime) {
					continue;
				}

				// State 4 - Zeiten (Rückfahrt) updaten 				
				for (int i = 0; i < items.length; i++) {
					if (items[i].getAbfahrtsZeitReversed() == Global.TIME_ERROR) {
						continue;
					}
					if (items[i].getAbfahrtsZeitReversed() + 1 < now) {
						if (finish) {
							break;
						}
						HaltestellenPaar hsp = items[i].getHaltestellenPaar();
						Route route = Algorithmus.calcStartScreenRoute(hsp.getNach(), hsp.getVon());
						now = DatumZeit.Now();
						if (items[i].tryToUpdateNaechsteFahrmoeglichkeit(route, hsp.switchVonNach())) {
							guiSelectionStartScreen.redrawMain();
						}
						restart = true;
						break;
					}
				}
				
				if (restart || finish || resetTime) {
					continue;
				}

				try {
					synchronized (this) {
						// 1 Minute warten
						this.wait(60 * 1000);
					}
				} catch (InterruptedException e) {
					Logger.errorException(this.getClass(), e, "asd9ijkm");
				}
			} while (!finish);

			items = null;
		}
		} catch (OutOfMemoryError e) {
			PushandrideMidlet.OutOfMemoryException(this.getClass(), e, "s9jnsjs");
		} catch (Exception e) {
			Logger.errorException(this, e, "s9jnsjs");
		}
	}
}
