// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import gui.guis.GuiSelection;
import gui.guis.GuiSelectionUebersicht;

import java.util.Vector;

import model.Haltestelle;
import model.HaltestellenPaar;
import model.Route;
import algo.Algorithmus;
import dev.Logger;

/**
 * FIXME: Nicht vergessen diesen Thread zu beenden! (interrupt)
 */
public class BgWorker extends Thread {
	private HaltestellenPaar haltestellenPaar;
	private GuiSelectionUebersicht gui;
	private Route earliestRoute;
	private Route latestRoute;
	private Vector normalQueue;
	private Vector priorityQueue;
	private BgWorker oldWorker;
	private final int firstQueryTime;

	public BgWorker(HaltestellenPaar haltestellenPaar, GuiSelectionUebersicht gui, BgWorker oldWorker, int firstQueryTime) {
		this.haltestellenPaar = haltestellenPaar;
		this.gui = gui;
		this.oldWorker = oldWorker;
		normalQueue = new Vector();
		priorityQueue = new Vector();
		this.firstQueryTime = firstQueryTime;
	}

	/**
	 * Errechnet die erste Route und wartet anschließend auf eingehende Anfragen
	 */
	public void run() {
		Haltestelle von = haltestellenPaar.getVon();
		Haltestelle nach = haltestellenPaar.getNach();
		if (oldWorker != null && oldWorker.earliestRoute != null) {
			earliestRoute = oldWorker.earliestRoute;
		} else {
			earliestRoute = Algorithmus.calcRoute(von, nach, firstQueryTime);
		}
		latestRoute = earliestRoute;
		if (oldWorker == null) {
			addToGui(earliestRoute, GuiSelection.EINFUEGEN_HINTEN);
		} else {
			normalQueue = oldWorker.normalQueue;
			priorityQueue = oldWorker.priorityQueue;
		}
		while (true) {
			synchronized (this) {
				try {
					if (normalQueue.size() == 0 && priorityQueue.size() == 0)
						this.wait();
				} catch (InterruptedException ie) {
				}
			}
			processQueues();
		}
	}

	/**
	 * Fuegt einen weiteren Job zu einer der Warteschlangen (Priority oder Normal Queue) hinzu
	 * 
	 * @param priority -
	 *            true=priorityQueue, false=normalQueue
	 * @param pos -
	 *            GuiSelection.EINFUEGEN_VORNE oder GuiSelection.EINFUEGEN_HINTEN
	 */
	public void addToQueue(boolean priority, int pos) {
		if (priority)
			priorityQueue.addElement(new Integer(pos));
		else
			normalQueue.addElement(new Integer(pos));
		synchronized (this) {
			this.notify();
		}
	}

	/**
	 * 
	 */
	private void processQueues() {
		while (normalQueue.size() > 0 || priorityQueue.size() > 0) {
			if (priorityQueue.size() > 0) {
				process((Integer) priorityQueue.firstElement(), true);
			} else {
				process((Integer) normalQueue.firstElement(), false);
			}
		}
	}

	/**
	 * 
	 * @param i
	 * @param priority
	 */
	private void process(Integer i, boolean priority) {
		int pos = i.intValue();
		add(priority, pos);
	}

	/**
	 * 
	 * @param priority
	 * @param pos
	 */
	private void add(boolean priority, int pos) {
		Route actualRoute = null;
		switch (pos) {
		case (GuiSelection.EINFUEGEN_VORNE):
			actualRoute = Algorithmus.calcFormerRoute(earliestRoute);
			earliestRoute = actualRoute;
			break;
		case (GuiSelection.EINFUEGEN_HINTEN):
			actualRoute = Algorithmus.calcLaterRoute(latestRoute);
			latestRoute = actualRoute;
			break;
		default:
			Logger.error(this.getClass(), "ungueltige");
		}
		addToGui(actualRoute, pos);
		if (priority)
			priorityQueue.removeElement(new Integer(pos));
		else
			normalQueue.removeElement(new Integer(pos));
	}

	/**
	 * 
	 * @param route
	 * @param einfuegenPosition
	 */
	private void addToGui(Route route, int einfuegenPosition) {
		if (!gui.addFinalRoute(route, einfuegenPosition)) {
			this.interrupt();
		}
	}
}
