// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

public class Color {
	final static public int WHITE = gray2color(255);
	final static public int BLACK = gray2color(0);
	final static public int RED = rgb2color(255, 0, 0);
	final static public int GREEN = rgb2color(0, 255, 0);
	final static public int BLUE = rgb2color(0, 0, 255);
	final static public int COLOR_BG_FINAL = rgb2color(0,96,181);
	final static public int COLOR_BG_ACTIVE = Color.rgb2color(152, 0, 0);
	
	/**
	 * erstellt einen linearen Farbverlauf
	 * 
	 * @param source
	 *            Quellfarbe
	 * @param dest
	 *            Zielfarbe
	 * @param pos
	 *            Übergansposition (0..1)
	 * @return Farbwert für setColor(int)
	 */
	public static int mixColor(int source, int dest, double pos) {
		int sColor[] = new int[3];
		int dColor[] = new int[3];
		int rColor[] = new int[3];
		for (int i = 0; i < 3; i++) {
			//SourceColor
			sColor[i] = (source >> (8 * i)) & 255;
			//DestColor
			dColor[i] = (dest >> (8 * i)) & 255;
			//ReturnColor
			rColor[i] = (int) ((double) sColor[i] * (1.0 - pos) + (double) dColor[i] * pos);
		}
		int ret = 0;
		for (int i = 0; i < 3; i++) {
			ret |= rColor[i] << (8 * i);
		}
		return ret;
	}

	/**
	 * Farbwert für setColor(int)
	 */
	public final static int rgb2color(int red, int green, int blue) {
		return ((red << 16) | (green << 8) | blue);
	}

	/**
	 * Farbwert für setColor(int)
	 */
	public final static int gray2color(int gray) {
		return ((gray << 16) | (gray << 8) | gray);
	}
}
