// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import java.util.Timer;

import javax.microedition.lcdui.Graphics;

public class Cursor {
	private static final int COLOR = Color.WHITE;
	private static final int BLINK_RATE = 650;
	private final int cursorHeight;
	private boolean showing;
	private Timer timer;
	private ICursorable parent;
	private final int addY;

	public Cursor(int fontSize, ICursorable parent) {
		this.cursorHeight = FontMetrics.getRealHeight(fontSize);
		this.addY = FontMetrics.getTopSpace(fontSize);
		this.showing = false;
		this.parent = parent;
	}

	/**
	 * schaltet den Cursor-Timer an
	 */
	public void activate() {
		timer = new Timer();
		timer.scheduleAtFixedRate(new CursorTask(this), BLINK_RATE, BLINK_RATE);
		showing = true;
		parent.updateCursor();
	}

	/**
	 * zeichnet den Cursor an die übergebene Position und Zeichenfläche
	 */
	public void drawTo(Point position, Graphics g) {
		g.setColor(COLOR);
		g.drawLine(position.X, position.Y + addY, position.X, position.Y + addY + cursorHeight - 1);
	}

	/**
	 * hält den Cursor-Timer an
	 */
	public void deactivate() {
		if (timer != null) {
			timer.cancel();
		}
		timer = null;
		showing = false;
		parent.updateCursor();
	}

	/**
	 * schaltet den Zustand des Cursors um (sichtbar / nicht sichtbar)
	 */
	synchronized public void switchState() {
		showing = !showing;
		parent.updateCursor();
	}

	public boolean isVisible() {
		return showing;
	}
}
