// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import gui.guis.Gui;
import gui.guis.GuiDetail;
import gui.guis.GuiDetailReminder;
import gui.guis.GuiSelectionDatasetSelect;
import gui.guis.GuiSelectionLocation;
import gui.guis.GuiSelectionOptions;
import gui.guis.GuiSelectionUebersicht;
import gui.utils.GfxUtils;
import gui.utils.ImageScaler;

import java.io.IOException;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import model.Verkehrsmittel;
import control.Device;
import dev.Logger;

public class Imager {
	private static Image footerImage;
	private static Image keyImage;
	private static Image keyImagePressed;

	private final static int idNoImage = 0;
	private final static int idPushandrideImage = 1;
	private final static int idErinnerungImage = 2;
	private final static int idOptionenImage = 3;
	private final static int idRouteImage = 4;
	private final static int idFahrtwahlImage = 5;
	private final static int idHaltestellenImage = 6;
	private static Image[] headerImages = new Image[7];

	public static final int ICON_NEXT = 0;
	public static final int ICON_EINSTELLUNGEN = 1;
	public static final int ICON_X = 2;
	public static final int ICON_BLUETOOTH = 3;
	public static final int ICON_FAVORIT0 = 4;
	public static final int ICON_FAVORIT1 = 5;
	public static final int ICON_INFO = 6;
	public static final String[] iconNames = {"next", "settings", "x", "bluetooth", "fav0", "fav1", "info", "fa"};
	private static Image[][] icons = new Image[iconNames.length][3];

	public static final int ICON_FOOTER_NONE = -1;
	public static final int ICON_FOOTER_REMINDER = 0;
	public static final int ICON_FOOTER_RUECKFAHRT = 1;
	public static final int ICON_FOOTER_FAVORITEN = 2;
	public static final int ICON_FOOTER_LISTE = 3;
	public static final int ICON_FOOTER_ZEIT = 4;
	public static final int ICON_FOOTER_SMS = 5;
	public static final int ICON_FOOTER_WEB = 6;
	public static final int ICON_FOOTER_CALL = 7;
	public static final int ICON_FOOTER_INFO = 8;
	public static final String[] iconFooterNames = {"remind", "rueckfahrt", "fav", "list", "zeit", "sms", "web", "call", "info"};
	private static Image[] iconsFooter = new Image[iconFooterNames.length];
	
	private static Image zuletztVerwendet;
	private static Image[][] anschlussPic;

	private static void createAnschlussPic(int typ, int color) {
		int height = Manager.HEADER_BASE_UNIT;
		char c;
		c = Verkehrsmittel.ANSCHLUSSTYP[typ];
		height = height - GfxUtils.scaleWithMinimun(height, 3, 1) * 2;
		int index = getColorIndex(color);
		char name = Manager.LOGO_BACKGROUND_COLOR_NAMES[index];
		anschlussPic[typ][index] = ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/" + c + name + ".png"), height);
		//
	}

	private static final int BG_COLOR_ZULETZT_VERWENDET = Color.rgb2color(0x00, 0x60, 0xb5);

	public static Image getZuletztVerwendet() {
		if (zuletztVerwendet == null) {
			createZuletztVerwendet();
		}
		// x
		return zuletztVerwendet;
	}

	private static void createZuletztVerwendet() {
		int width = Manager.SCREEN_WIDTH;
		int height = Manager.HEADER_BASE_UNIT;
		zuletztVerwendet = Image.createImage(width, height);
		Image scaledPic = ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/zuletztVerwendet30px.png"), height);
		Graphics g = zuletztVerwendet.getGraphics();
		g.setColor(BG_COLOR_ZULETZT_VERWENDET);
		GfxUtils.fillFullRectangle(g, 0, height);
		g.setColor(Color.WHITE);
		g.drawImage(scaledPic, 0, 0, Graphics.LEFT | Graphics.TOP);
		g.setFont(FontMetrics.getFont(Manager.ITEMS_FONT_SIZE));
		GfxUtils.drawText(g, "zuletzt Verwendete:", scaledPic.getWidth(), 0);
	}

	/*
	 * TODO: Eigentlich müsste man von Anfang an nur mit einem quasi Enum arbeiten und nur dort wo es
	 * gebraucht wird auf die tatsächlichen Farbwerte gehen.
	 */
	
	private static int getColorIndex(int color) {
		for (int i = 0; i < Manager.LOGO_BACKGROUND_COLORS.length; i++) {
			if (color == Manager.LOGO_BACKGROUND_COLORS[i]) {
				return i;
			}
		}		
		return 0;
	}
	
	public static Image getAnschlussPic(int typ, boolean selected, int color) {
		int index = getColorIndex(color);
		if (typ >= Verkehrsmittel.ANSCHLUSSTYP.length) {
			System.err.println("getAnschlussPic - falscher Index: " + typ);
			return null;
		}
		if (anschlussPic == null) {
			anschlussPic = new Image[Verkehrsmittel.ANSCHLUSSTYP.length][Manager.LOGO_BACKGROUND_COLORS.length];
		}
		if (anschlussPic[typ][index] == null) {
			createAnschlussPic(typ,color);
		}
		return anschlussPic[typ][index];
	}
	
	public static Image getAnschlussPic(int typ) {
		return getAnschlussPic(typ, false, Manager.ITEMS_COLOR_BACKGROUND_DEF);
	}
	

	public static void clearKeyImage() {
		keyImage = null;
		keyImagePressed = null;
	}
	
	public static Image getKeyImage() {
		if (keyImage == null) {
			createKeyImage();
		}
		return keyImage;
	}

	public static Image getKeyPressedImage() {
		if (keyImagePressed == null) {
			createKeyImage();
		}
		return keyImagePressed;
	}
	
	public static Image getKeyPressedImage(int keyNo) {
		Image unpressed = getKeyImage();
		if (!GuiSelectionLocation.softKeysZweizeilig) {
			return unpressed;
		}
		if (keyNo < 0 || keyNo > 9) {
			Logger.info(Imager.class, "getKeyPressedImage - keyNo=" + keyNo);
			return unpressed;
		}
		Image pressed = getKeyPressedImage();
		Image result = Image.createImage(unpressed.getWidth(), unpressed.getHeight());
		Graphics g = result.getGraphics();  
		g.drawImage(unpressed, 0, 0, Graphics.TOP|Graphics.LEFT);
		int width = Manager.SCREEN_WIDTH;
		int itemHeight = pressed.getHeight();
		int i = keyNo;
		int x1 = width * (i%5) / 5;
		int x2 = width * (i%5 + 1) / 5;
		int y = itemHeight / 2 * (i / 5);
		g.drawRegion(pressed, x1, y, x2 - x1, itemHeight / 2, 0, x1, y, Graphics.LEFT | Graphics.TOP);
		return result;
	}

	public static Image getHeaderBackgroundImage() {
		if (headerImages[idNoImage] == null) {
			int width = Manager.SCREEN_WIDTH;
			int height = Manager.getHeaderHeight();
			headerImages[idNoImage] = Image.createImage(width, height);
			Image scaledSpacePic = ImageScaler.scaleBresenhams(GfxUtils.loadImage("/res/" + getImageNameById(idNoImage)), width, height - 1);
			Graphics g = headerImages[idNoImage].getGraphics();
			g.drawImage(scaledSpacePic, 0, 0, Graphics.LEFT | Graphics.TOP);
			g.setColor(Manager.HEADER_COLOR_OUTLINE);
			GfxUtils.drawFullLineHoriz(g, height - 1);
		}
		return headerImages[idNoImage];
	}

	private static void createHeaderImage(int id) {
		if (id == idNoImage) {
			getHeaderBackgroundImage();
			return;
		}
		if (id == idHaltestellenImage) {
			getHeaderBackgroundImage();
			return;
		}

		int width = Manager.SCREEN_WIDTH;
		int height = Manager.getHeaderHeight();
		Image scaledSpacePic = getHeaderBackgroundImage();
		headerImages[id] = Image.createImage(width, height);
		Image scaledPic = ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/" + getImageNameById(id)), Manager.HEADER_HEIGHT - 1);
		Graphics g = headerImages[id].getGraphics();
		g.drawImage(scaledSpacePic, 0, 0, Graphics.LEFT | Graphics.TOP);
		g.drawImage(scaledPic, (width - scaledPic.getWidth()) / 2, 0, Graphics.LEFT | Graphics.TOP);
		g.setColor(Manager.HEADER_COLOR_OUTLINE);
		GfxUtils.drawFullLineHoriz(g, height - 1);
	}

	private static void createKeyImage() {
		String keyName;
		if (Manager.SCREEN_WIDTH >= Manager.SCREEN_HEIGHT) {
			GuiSelectionLocation.softKeysZweizeilig = false;
		} else {
			GuiSelectionLocation.softKeysZweizeilig = true;
		}
		
		if (Device.getPlatform().toLowerCase().startsWith("nokia5800")) {
			if (Manager.SCREEN_WIDTH <= 360) {
				GuiSelectionLocation.softKeysZweizeilig = true;
				Logger.info(Imager.class, "override Softkey (zweizeilig)");
			}
		}
		if (GuiSelectionLocation.softKeysZweizeilig) {
			if (Device.increaseHeightForTouch()) {
				keyName = "keyTouch";
			} else {
				keyName = "key2";
			}
		} else {
			keyName = "key1";
		}
		
		int width = Manager.SCREEN_WIDTH;
		
		String[] keys = new String[]{"abc", "def", "ghi", "jkl", "mno", "pqrs", "tuv", "wxyz", "_", "<-"};

		Image scaledKeyboard = null;
		Image scaledKeyboardPressed = null;
		
		if (GuiSelectionLocation.softKeysZweizeilig) {
			Font f = null;
			for (int i = 0; i < FontMetrics.sizes.length; i++) {
				int size = FontMetrics.sizes[i];
				Font fTest = FontMetrics.getFont(size);
				if (fTest.stringWidth("wxyz") < width / 5) {
					f = fTest;
					if (Manager.ITEMS_FONT_SIZE == size) {
						break;
					}
				} else {
					break;
				}
			}
			if ( f != null ) {
				int itemHeight = FontMetrics.getHeight(gui.Manager.ITEMS_FONT_SIZE);
				
				if (Device.increaseHeightForTouch()) {
					itemHeight = GfxUtils.getHeightForTouch(itemHeight);
				}
				
				int topSpace = (itemHeight - f.getHeight()) / 2; 

				itemHeight = itemHeight * 2;
				
				scaledKeyboard = Image.createImage(width, itemHeight);
				scaledKeyboardPressed = Image.createImage(width, itemHeight);
				for (int no = 0; no < 2; no++) {
					Graphics g;
					if (no == 0) {
						g = scaledKeyboard.getGraphics();
						g.setColor(Color.WHITE);
					} else {
						g = scaledKeyboardPressed.getGraphics();
						g.setColor(Color.rgb2color(255, 204, 68));
					}
					g.setFont(f);
					GfxUtils.fillFullRectangle(g, 0, itemHeight);
					g.setColor(Color.BLACK);
					g.drawRect(0, 0, width, itemHeight - 1);
					g.drawLine(0, itemHeight/2, width, itemHeight / 2);
					for (int i = 0; i < keys.length; i++) {
						String s = keys[i];
						int x1 = width * (i%5) / 5;
						int x2 = width * (i%5 + 1) / 5;
						int y = itemHeight / 2 * (i / 5) + topSpace;
						GfxUtils.drawTextInBetween(g, s, x1, x2, y);
	
						if (i % 5 != 0) {
							g.drawLine(x1, 0, x1, itemHeight);
						}
					}
				}
			} else {
				Logger.info(Imager.class, "Can't create custom SoftKeyboard - Font too big");
			}
		}

		if (scaledKeyboard == null) {
			scaledKeyboard = ImageScaler.scaleBilinearByWidth(GfxUtils.loadImage("/res/" + keyName + ".png"), width);
			scaledKeyboardPressed = ImageScaler.scaleBilinearByWidth(GfxUtils.loadImage("/res/" + keyName + "pressed.png"), width);
		}

		keyImage = scaledKeyboard;
		keyImagePressed = scaledKeyboardPressed;
	}

	private static String getImageNameById(int id) {
		String suffix = "_30px.png";
		String base = null;
		switch (id) {
		case idErinnerungImage:
			base = "erinnerung";
			break;
		case idFahrtwahlImage:
			base = "fahrtwahl";
			break;
		case idNoImage:
			base = "headerSpace";
			break;
		case idOptionenImage:
			base = "optionen";
			break;
		case idPushandrideImage:
			base = "pushandride";
			break;
		case idRouteImage:
			base = "route";
			break;
		}
		return base + suffix;
	}

	private static void createFooter() {
		int width = Manager.SCREEN_WIDTH;
		//		int height = Manager.getFooterHeight();

		footerImage = Image.createImage(width, Manager.FOOTER_UPPER_SPACE);
		Graphics g = footerImage.getGraphics();
		int picHeight = footerImage.getHeight();
		double maxPos = (picHeight);
		for (int i = 0; i < picHeight; i++) {
			double pos = i / maxPos;
			// System.out.println("P: " + pos);
			// g.setColor(Color.mixColor(Color.WHITE, Color.BLACK, pos));
			g.setColor(Color.mixColor(Color.WHITE, Manager.FOOTER_COLOR_BACKGROUND, pos));
			GfxUtils.drawFullLineHoriz(g, i);
		}
	}

	public static Image getHeaderImage(Gui gui) {
		//		System.out.println("getHeaderImage " + gui.getName());
		int id = getImageId(gui);
		//		System.out.println("imageId " + id);
		if (id == idHaltestellenImage) {
			return getHeaderBackgroundImage();
		}
		if (headerImages[id] == null) {
			getHeaderBackgroundImage();
			createHeaderImage(id);
		}
		return headerImages[id];
	}

	private static int getImageId(Gui gui) {
		if (gui instanceof GuiDetailReminder) {
			return idErinnerungImage;
		}
		if (gui instanceof GuiDetail) {
			return idRouteImage;
		}
		if (gui instanceof GuiSelectionUebersicht) {
			return idFahrtwahlImage;
		}
		if (gui instanceof GuiSelectionOptions) {
			return idOptionenImage;
		}
		if (gui instanceof GuiSelectionDatasetSelect) {
			return idOptionenImage;
		}
		if (gui instanceof GuiSelectionLocation) {
			return idHaltestellenImage;
		}
		return idPushandrideImage;
	}

	public static Image getFooterImage() {
		if (footerImage == null) {
			createFooter();
		}
		return footerImage;
	}

	private static final int[] logoHandWidths = { 300, 240, 192, 153, 122, 97, 77 };
	private static final int[] logoHandHeights = { 359, 287, 230, 183, 146, 116, 92 };

	private static Image logoHand = null;

	public static Image createLogoHand(int maxWidth, int maxHeight, int targetWidth) {
		if (logoHand == null) {
			int i = 0;
			for (i = 0; i < logoHandWidths.length; i++) {
				if (targetWidth > logoHandWidths[i]) {
					break;
				}
			}

			// prüfe welches Näher ist
			if (i != 0) {
				int diffA = logoHandWidths[i - 1] - targetWidth;
				int diffB = targetWidth - logoHandWidths[i];
				if (diffA < diffB) {
					i--;
				}
			}

			// maxWidth checken
			while (maxWidth < logoHandWidths[i]) {
				if (i == logoHandWidths.length - 1) {
					Logger.warn(Imager.class, "logoHand zu Groß (breit) für Bildschirm");
					
					break;
				}
				i++;
			}

			// maxHeight checken
			while (maxHeight < logoHandHeights[i]) {
				if (i == logoHandHeights.length - 1) {
					Logger.warn(Imager.class, "logoHand zu Groß (hoch) für Bildschirm");
					break;
				}
				i++;
			}
			
			String filename = "/res/logo/hand_" + logoHandWidths[i] + ".png";
			try {
				logoHand = Image.createImage(filename);
			} catch (IOException e) {
				Logger.error(Imager.class, "konnte "+ filename + " nicht laden");
			}
			
			if (logoHand.getWidth() > maxWidth || logoHand.getHeight() > maxHeight) {
				// force scaling;
				logoHand = ImageScaler.scaleBilinearByMaxWidthHeight(logoHand, maxWidth, maxHeight);
			}
		}
		return logoHand;
	}

	/**
	 * gibt das entsprechende Icon zurück (KEIN FOOTER ICON)
	 * als Prefix wird vor die Datei wird ein 'i' gestellt, falls es ein ausgewähltes Item ist als Postfix ein 's'
	 * falls es eins mit dunklem hintergrund ist (und nicht selected) wird am ende ein d eingefügt 
	 */
	public static Image getIcon(int iconId, boolean selected, boolean dark) {
		int typeId;
		if (selected) {
			typeId = 0;
		} else {
			if (dark) {
				typeId = 2;
			} else {
				typeId = 1;	
			}
		}
		if (icons[iconId][typeId] == null) {
			if (Device.getGfxQuality() != Device.GFX_QUALITY_LOW) {
				String iconName = iconNames[iconId];
				if (selected) {
					iconName += "s";
				} else if (dark) {
					iconName += "d";
				}
				Image unscaledIcon = GfxUtils.loadImage("/res/i" + iconName + ".png");
				Image scaledIcon = ImageScaler.scaleBilinearByHeight(unscaledIcon, FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE));
				icons[iconId][typeId] = scaledIcon;
			}
		}
		return icons[iconId][typeId];
	}
	
	public static Image getIcon(int iconId, boolean selected) {
		return getIcon(iconId, selected, false);
	}
	

	/**
	 * gibt das entsprechende FOOTER Icon zurück 
	 * als Prefix wird vor die Datei wird ein 'if' gestellt
	 */
	public static Image getFooterIcon(int iconFooterId) {
		if (iconFooterId == ICON_FOOTER_NONE) {
			return null;
		}
		if (iconsFooter[iconFooterId] == null) {
			if (Device.getGfxQuality() != Device.GFX_QUALITY_LOW) {
				String iconName = iconFooterNames[iconFooterId];
				Image unscaledIcon = GfxUtils.loadImage("/res/if" + iconName + ".png");
				Image scaledIcon = ImageScaler.scaleBilinearByHeight(unscaledIcon, FontMetrics.getHeight(Manager.FOOTER_FONT_SIZE));
				iconsFooter[iconFooterId] = scaledIcon;
			}
		}
		return iconsFooter[iconFooterId];
	}
}
