// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import gui.guis.Gui;
import gui.guis.GuiAbout;
import gui.guis.GuiBedienungsHinweis;
import gui.guis.GuiBerechne;
import gui.guis.GuiDetail;
import gui.guis.GuiDetailReminder;
import gui.guis.GuiMessageScreen;
import gui.guis.GuiNewDataset;
import gui.guis.GuiSelectionDatasetCustomize;
import gui.guis.GuiSelectionDatasetSelect;
import gui.guis.GuiSelectionDayTime;
import gui.guis.GuiSelectionHome;
import gui.guis.GuiSelectionLocation;
import gui.guis.GuiSelectionLocationHaltestellen;
import gui.guis.GuiSelectionLocationOrte;
import gui.guis.GuiSelectionOptions;
import gui.guis.GuiSelectionReminderTime;
import gui.guis.GuiSelectionUebersicht;
import gui.guis.GuiSelectionWeitergeben;
import gui.guis.GuiWaitScreen;
import gui.items.Item;
import gui.items.ItemHomeExecute;
import gui.items.ItemHomeHaltestellenPaar;
import gui.items.ItemWeitergebenExecute;
import gui.launch.GameCanvasInstance;
import gui.launch.GuiLaunchScreen;
import gui.options.AdviceFriend;
import gui.options.BluetoothChooseDeviceScreen;
import gui.options.dev.TestKeys;
import gui.utils.GfxUtils;
import gui.utils.ImageScaler;

import java.io.IOException;

import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;

import model.DatumZeit;
import model.Haltestelle;
import model.HaltestellenPaar;
import model.Route;
import algo.Algorithmus;
import control.Device;
import control.Global;
import control.Keys;
import control.Pushandride;
import control.PushandrideMidlet;
import control.locale.Locale;
import control.reminder.Reminder;
import data.Data;
import data.persistence.Persistence;
import data.persistence.RemoteDataBinary;
import data.persistence.Stats;
import data.persistence.UserSettingsInt;
import dev.DebugInfo;
import dev.Logger;
import dev.Memory;
import dev.bootProfiler.BootProfiler;
import dev.bootProfiler.ZeitMessung;

public class Manager implements Runnable {

	private Thread keyThread;
	private boolean finish_flag = false;

	private final Pushandride pushandride;

	private static Manager manager;

	private boolean singleKey = false;

	private boolean firstAfterMagixxHelp = false;

	private final GameCanvas gc;

	public Manager(Pushandride pushandride) {
		this.pushandride = pushandride;
		Manager.manager = this;

		if (Global.SHOW_MEMORY_INFO) {
			Memory.init();
			System.out.println(Memory.getSizeInfo());
		}

		this.gc = GameCanvasInstance.getInstance();

		Graphics g = GameCanvasInstance.returnGraphics();
		ZeitMessung profil_manager_init = ZeitMessung.getStartedInstance("Manager | init");
		init(g);
		profil_manager_init.stop();

		ZeitMessung profil_manager_calcHeight = ZeitMessung.getStartedInstance("Manager | calcHeight");
		calculateHeaderHeight();
		profil_manager_calcHeight.stop();

		guis = new Gui[Manager.GUI_ANZAHL];

		// Erstellung der verfügbaren Guis
		ZeitMessung profil_manager_guisErstellen = ZeitMessung.getStartedInstance("Manager | Guis Erstellen");
		guiHome = new GuiSelectionHome(gc, g);
		guis[STATE_HOME] = guiHome;

		guiOrtVon = new GuiSelectionLocationOrte(GuiSelectionLocation.TYPE_VON_ORT, gc, g);
		guis[STATE_ORT_VON] = guiOrtVon;

		guiOrtNach = new GuiSelectionLocationOrte(GuiSelectionLocation.TYPE_NACH_ORT, gc, g);
		guis[STATE_ORT_NACH] = guiOrtNach;

		guiHaltestelleVon = new GuiSelectionLocationHaltestellen(GuiSelectionLocation.TYPE_VON_HALTESTELLE, gc, g);
		guis[STATE_HALTESTELLE_VON] = guiHaltestelleVon;

		guiHaltestelleNach = new GuiSelectionLocationHaltestellen(GuiSelectionLocation.TYPE_NACH_HALTESTELLE, gc, g);
		guis[STATE_HALTESTELLE_NACH] = guiHaltestelleNach;

		guiMessageScreen = new GuiMessageScreen(gc, g);
		guis[STATE_MESSAGESCREEN] = guiMessageScreen;

		guiSelectionOptions = new GuiSelectionOptions(gc, g);
		guis[STATE_OPTIONS] = guiSelectionOptions;

		guiSelectionOptionDataset = new GuiSelectionDatasetSelect(gc, g);
		guis[STATE_OPTION_DATASET] = guiSelectionOptionDataset;

		guiSelectionWeitergeben = new GuiSelectionWeitergeben(gc, g);
		guis[STATE_WEITERGEBEN] = guiSelectionWeitergeben;

		profil_manager_guisErstellen.stop();

		ZeitMessung profil_manager_startScreen = ZeitMessung.getStartedInstance("Manager | Start-Screen anzeigen");
		currentState = STATE_HOME;
		getActiveGui().show();
		getActiveGui().drawTo(g);

		keyThread = new Thread(this);
		keyThread.start();

		// Startupscreen anzeigen gegen Homescreen ersetzen
		GameCanvasInstance.getInstance().redirectGameCanvasEvents(this);

		profil_manager_startScreen.stop();

		showFirst();
	}

	public void initByThread() {
		Graphics g = GameCanvasInstance.returnGraphics();

		guiUebersicht = new GuiSelectionUebersicht(gc, g);
		guis[STATE_UEBERSICHT] = guiUebersicht;

		guiDetail = new GuiDetail(gc, g);
		guis[STATE_DETAILS] = guiDetail;

		guiReminder = new GuiDetailReminder(gc, g);
		guis[STATE_REMINDER] = guiReminder;

		guiBerechne = new GuiBerechne(gc, g);
		guis[STATE_BERECHNE] = guiBerechne;

		// Zeit akualisierung
		new ZeitAkualisierung(this);

	}

	private void calculateHeaderHeight() {
		Font f = GuiLaunchScreen.getDefaultFont(Manager.SCREEN_WIDTH, Manager.SCREEN_HEIGHT);

		// HEADER
		HEADER_FONT_SIZE = f.getSize();
		HEADER_BASE_UNIT = f.getHeight();

		HEADER_TOP_SPACE = GfxUtils.scale(HEADER_BASE_UNIT, 10);
		HEADER_MID_SPACE = GfxUtils.scale(HEADER_BASE_UNIT, 10);
		HEADER_BOTTOM_SPACE = GfxUtils.scale(HEADER_BASE_UNIT, 10);
		HEADER_BOTTOM_LINE = 1;
		HEADER_HEIGHT = HEADER_TOP_SPACE + HEADER_MID_SPACE + HEADER_BOTTOM_SPACE + HEADER_BOTTOM_LINE + HEADER_BASE_UNIT * 2;

		ITEMS_FONT_SIZE = HEADER_FONT_SIZE;
		FOOTER_FONT_SIZE = HEADER_FONT_SIZE;

		int headerTextHeight = FontMetrics.getHeight(HEADER_FONT_SIZE);

		HEADER_INNER_SPACE = GfxUtils.scale(headerTextHeight, 10);

		HEADER_TEXTPOS_Y = HEADER_INNER_SPACE;

		HEADER_TEXTCENTERPOS_X = SCREEN_WIDTH / 2;

		int footerTextHeight = FontMetrics.getHeight(FOOTER_FONT_SIZE);

		FOOTER_UPPER_SPACE = GfxUtils.scaleWithMinimun(footerTextHeight, 30, 2);

		FOOTER_LOWER_SPACE = GfxUtils.scaleWithMinimun(footerTextHeight, 20, 1);

		FOOTER_HEIGHT = FOOTER_UPPER_SPACE + footerTextHeight + FOOTER_LOWER_SPACE;

		FOOTER_LEFT_SPACE = GfxUtils.scaleWithMinimun(MAIN_LEFT_SPACE, 70, 1);
	}

	public static int getHeaderHeight() {
		return HEADER_HEIGHT;
	}

	private GuiSelectionHome guiHome;
	private GuiSelectionUebersicht guiUebersicht;
	private GuiSelectionLocationOrte guiOrtVon;
	private GuiSelectionLocationOrte guiOrtNach;
	private GuiSelectionLocationHaltestellen guiHaltestelleVon;
	private GuiSelectionLocationHaltestellen guiHaltestelleNach;
	private GuiDetail guiDetail;
	private GuiMessageScreen guiMessageScreen;
	private GuiSelectionOptions guiSelectionOptions;
	private GuiSelectionDatasetSelect guiSelectionOptionDataset;
	private GuiSelectionWeitergeben guiSelectionWeitergeben;
	private GuiDetailReminder guiReminder;
	private GuiSelectionDayTime guiDayTime;
	private GuiSelectionReminderTime guiReminderTime;
	private GuiWaitScreen guiWaitScreen;
	private GuiBerechne guiBerechne;

	public static int SCREEN_WIDTH = 0;
	public static int SCREEN_HEIGHT = 0;

	public static int HEADER_FONT_SIZE = Font.SIZE_SMALL;
	public static final int HEADER_FONT_STYLE = Font.STYLE_BOLD;
	public static int HEADER_BASE_UNIT;
	public static int HEADER_TOP_SPACE;
	public static int HEADER_MID_SPACE;
	public static int HEADER_BOTTOM_SPACE;
	public static int HEADER_BOTTOM_LINE;
	public static int HEADER_HEIGHT;

	public static int HEADER_INNER_SPACE;
	public static int HEADER_TEXTCENTERPOS_X;
	public static int HEADER_TEXTPOS_Y;
	public static int HEADER_COLOR_BACKGROUND = Color.rgb2color(0, 161, 255);
	public static int HEADER_COLOR_FONT = Color.BLACK;
	public static int HEADER_COLOR_FONT_SHADOW = Color.gray2color(128);
	public static int HEADER_COLOR_OUTLINE = Color.gray2color(170);

	public static int ITEMS_FONT_SIZE = Font.SIZE_SMALL;
	public static final int ITEMS_FONT_STYLE = Font.STYLE_PLAIN;
	public static final int ITEMS_COLOR_BACKGROUND = Color.WHITE;
	public static final int ITEMS_COLOR_FONT = Color.BLACK;
	public static final int ITEMS_COLOR_FONT_DISABLED = Color.gray2color(200);
	public static final int ITEMS_COLOR_BACKGROUND_SELECTED = Color.rgb2color(187, 238, 255);
	public static final int ITEMS_COLOR_BACKGROUND_INTELLIGENTKEYMATCH = Color.rgb2color(221, 255, 255);
	public static final int ITEMS_COLOR_BACKGROUND_DISABLED = Color.gray2color(50);

	final static public int ITEMS_COLOR_BACKGROUND_DARK = Color.gray2color(204);
	final static public int ITEMS_COLOR_BACKGROUND_MID = Color.gray2color(221);
	final static public int ITEMS_COLOR_BACKGROUND_LIGHT = Color.gray2color(238);
	final static public int ITEMS_COLOR_BACKGROUND_DEF = Color.gray2color(255);

	final static public int[] LOGO_BACKGROUND_COLORS = { ITEMS_COLOR_BACKGROUND_DEF, ITEMS_COLOR_BACKGROUND_SELECTED, ITEMS_COLOR_BACKGROUND_DARK, ITEMS_COLOR_BACKGROUND_MID, ITEMS_COLOR_BACKGROUND_LIGHT };
	final static public char[] LOGO_BACKGROUND_COLOR_NAMES = { 'w', 's', 'd', 'm', 'l' };

	public static int HEADER2_UEBERSICHT_COLOR_BACKGROUND = ITEMS_COLOR_BACKGROUND_SELECTED;
	public static final int HEADER2_UEBERSICHT_COLOR_FONT = Color.gray2color(0);

	public static int FOOTER_HEIGHT;
	public static int FOOTER_UPPER_SPACE;
	public static int FOOTER_LOWER_SPACE;
	public static int FOOTER_LEFT_SPACE;
	// public static final int FOOTER_COLOR_BACKGROUND =
	// Color.rgb2color(173,243,255);
	public static final int FOOTER_COLOR_BACKGROUND = Color.rgb2color(220, 250, 255);
	public static final int FOOTER_COLOR_FONT = Color.BLACK;
	public static final int FOOTER_COLOR_OUTLINE = Color.gray2color(128);
	public static int FOOTER_FONT_SIZE = Font.SIZE_SMALL;
	public static final int FOOTER_FONT_STYLE = Font.STYLE_PLAIN;

	public static final int FADER_TYPE_SPLITTER = 0;
	public static final int FADER_TYPE_MOVER = 1;
	public static final int FADER_TYPE_SPLITTOHALTESTELLEN = 2;
	public static final int FADER_TYPE_OVERLAY = 3;

	private static String footerDebugString = "";

	public static int MAIN_INNER_SPACE;
	public static int MAIN_LEFT_SPACE;

	public static Graphics mainScreen;

	public static int currentState = -1;
	public static final int STATE_HOME = 0;
	public static final int STATE_ORT_VON = 1;
	public static final int STATE_HALTESTELLE_VON = 2;
	public static final int STATE_ORT_NACH = 3;
	public static final int STATE_HALTESTELLE_NACH = 4;
	public static final int STATE_BERECHNE = 5;
	public static final int STATE_UEBERSICHT = 6;
	public static final int STATE_DETAILS = 7;
	public static final int STATE_MESSAGESCREEN = 8;
	public static final int STATE_OPTIONS = 9;
	public static final int STATE_REMINDER = 10;
	public static final int STATE_REMINDER_TIME = 11;
	public static final int STATE_SELECT_DAYTIME = 12;
	public static final int STATE_WAIT_SCREEN = 13;
	public static final int STATE_ABOUT = 14;
	public static final int STATE_BEDIENUNGSHINWEIS = 15;
	public static final int STATE_WEITERGEBEN = 16;
	public static final int STATE_OPTION_DATASET = 17;
	public static final int STATE_DATASET_COSTUMIZE = 18;
	public static final int STATE_NEW_DATASET = 19;

	private static final int GUI_ANZAHL = 20;

	public static Gui guis[];

	public static final int ANZAHL_BERECHNUNGEN = 7;

	public static final boolean PRESSED_OK = false;
	public static final boolean PRESSED_BACK = true;

	private boolean isLocked = false;

	public static int SHOW_HINWEISE_STATE = 0xFFFF;

	// Hinweise
	private void init(Graphics mainScreen) {

		// SCREEN
		Manager.mainScreen = mainScreen;

		SCREEN_WIDTH = mainScreen.getClipWidth();
		SCREEN_HEIGHT = mainScreen.getClipHeight();

		// MAIN
		MAIN_INNER_SPACE = GfxUtils.scale(FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE), 40);
		MAIN_LEFT_SPACE = GfxUtils.scale(SCREEN_WIDTH, 5);
	}

	/**
	 * Beendet den Key-Thread (Scrolling-Thread) regulaer
	 */
	public void finishKeyThread() {
		if (keyThread != null) {
			finish_flag = true;
			synchronized (keyThread) {
				keyThread.notify();
			}
		}
	}

	/**
	 * Thread um festzustellen, ob eine Taste gedrückt bleibt
	 */
	public void run() {
		int cnt = 0;
		int sleeptime = 70;
		boolean first = true;
		while (true) {
			int keyState = gc.getKeyStates();
			if (first) {
				keyState = 0;
				first = false;
			}
			if ((keyState & (GameCanvas.UP_PRESSED | GameCanvas.DOWN_PRESSED)) == 0) {
				try {
					synchronized (keyThread) {
						keyThread.wait();
					}
				} catch (InterruptedException ie) {
				}
				cnt = 0;
			}
			if (finish_flag)
				break;
			if (cnt > 2) {
				if ((keyState & GameCanvas.UP_PRESSED) != 0) {
					keyHandler(Keys.getKeyCode(Keys.UP));
				}
				if ((keyState & GameCanvas.DOWN_PRESSED) != 0) {
					keyHandler(Keys.getKeyCode(Keys.DOWN));
				}
			}
			try {
				Thread.sleep(sleeptime);
			} catch (InterruptedException ie) {
			}
			cnt++;
		}
	}

	/**
	 * @return aktuell angezeigte Gui
	 */
	public Gui getActiveGui() {
		return guis[currentState];
	}

	private void switchToState(int state, boolean key_status) {
		switchToState(state, key_status, FADER_TYPE_MOVER);
	}

	/**
	 * Speichert den Rücklink für die neue Gui und zeichnet den Übergang der Gui
	 * 
	 * @param state
	 *            STATE_xyz
	 * @param key_status
	 *            - PRESSED_OK / PRESSED_BACK
	 */

	private long lastSwitchTime = 0;
	private long lastUpDownTime = 0;

	private void switchToState(int state, boolean key_status, int SCROLL_TYPE) {

		// Vermeiden von zu schnellem hin und her gehen beim Android-Stick
		if (Device.isAndroid) {
			long currentTime = System.currentTimeMillis();
			if (currentTime - lastSwitchTime < 150) {
				lastSwitchTime = System.currentTimeMillis();
				return;
			}
			if (currentTime - lastUpDownTime < 150) {
				return;
			}
		}
		lastSwitchTime = System.currentTimeMillis();

		// abfangen falls direkt beim ersten Startscreen zurück gedrückt wird
		if (key_status == PRESSED_BACK) {
			if (currentState == STATE_HOME) {
				if (getActiveGui().getPreviousGui() == STATE_HOME) {
					return;
				}
			}
		}

		while (!lockScreen()) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				System.err.println(e.getMessage());
			}
		}

		Gui newGui;

		newGui = guis[state];

		newGui.beforeSwitchTo();

		getActiveGui().hide();

		if (Device.getGfxQuality() == Device.GFX_QUALITY_HIGH) {
			if (key_status == PRESSED_OK) {
				switch (SCROLL_TYPE) {
				case FADER_TYPE_MOVER:
					while (Slider.move(this, newGui, Slider.DIRECTION_RIGHT) != Slider.STATE_FINISHED) {
					}
					break;
				case FADER_TYPE_SPLITTER:
					while (Splitter.split(this, newGui, Splitter.DIRECTION_MOVEIN) != Splitter.STATE_FINISHED) {
					}
					break;
				case FADER_TYPE_SPLITTOHALTESTELLEN:
					while (SplitToHaltestellen.move(this, newGui, SplitToHaltestellen.DIRECTION_TO_HALTESTELLE) != Splitter.STATE_FINISHED) {
					}
					break;
				case FADER_TYPE_OVERLAY:
					if (guis[state] instanceof IOverlayable) {
						while (Overlay.move(this, newGui, Overlay.FROM_LEFT) != Splitter.STATE_FINISHED) {
						}
					} else {
						while (Overlay.move(this, newGui, Overlay.TO_RIGHT) != Splitter.STATE_FINISHED) {
						}
					}
					break;
				}
			} else {
				switch (SCROLL_TYPE) {
				case FADER_TYPE_MOVER:
					while (Slider.move(this, newGui, Slider.DIRECTION_LEFT) != Slider.STATE_FINISHED) {
					}
					break;
				case FADER_TYPE_SPLITTER:
					while (Splitter.split(this, newGui, Splitter.DIRECTION_MOVEOUT) != Splitter.STATE_FINISHED) {
					}
					break;
				case FADER_TYPE_SPLITTOHALTESTELLEN:
					while (SplitToHaltestellen.move(this, newGui, SplitToHaltestellen.DIRECTION_TO_ORT) != Splitter.STATE_FINISHED) {
					}
					break;
				case FADER_TYPE_OVERLAY:
					while (Overlay.move(this, newGui, Overlay.TO_LEFT) != Splitter.STATE_FINISHED) {
					}
					break;
				}
			}
		}

		int oldState = currentState;
		currentState = state;

		// nur falls nach vorne gedrückt wurde die previous Gui evtl.
		// überschreiben
		if (key_status == PRESSED_OK) {
			if (!(guis[oldState] instanceof IOverlayable)) {
				// Gui Speichern, von der er aufgerufen wurde
				getActiveGui().setPreviousGui(oldState);
			}

			// falls der letzte Screen ein Waitscreen war darf er nicht
			// angezeigt werden wenn man mit zurück auf den letzten Bildschirm
			// geht
			if (oldState == STATE_WAIT_SCREEN) {
				getActiveGui().setPreviousGui(guiWaitScreen.getPreviousGui());
			}
			// falls der letzte Screen ein Berechne war darf er nicht angezeigt werden wenn man mit zurück auf den letzten Bildschirm geht
			if (oldState == STATE_BERECHNE) {
				getActiveGui().setPreviousGui(guiBerechne.getPreviousGui());
			}
		}

		getActiveGui().show();
		getActiveGui().drawTo(GameCanvasInstance.returnGraphics());

		lastSwitchTime = System.currentTimeMillis();

		unlockScreen();

		// evtl. Bedienungshinweise anzeigen
		activateBedinungshinweise(currentState);

	}

	private void activateBedinungshinweise(int state) {
		if (Device.isAndroid) {
			int alreadySeenMask = UserSettingsInt.get(UserSettingsInt.SETTINGS_ID_HINWEISE_MASK);
			if (alreadySeenMask != 1 && Data.getCurrentDatasetType() == Data.DATSET_ALLES) {
				UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_HINWEISE_MASK, 1);
				Manager.showMessage("Achtung", "Das Programm wird aufgrund der großen Anzahl an Bussen deutlich langamer rechnen. Du kannst auch einzelne individuelle Busse hinzufügen um die hohe Performance zu erhalten.", GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
			}
			if (RemoteDataBinary.showInfoDueToVersionConfilct) {
				Manager.showMessage("Hinweis", "Der individuelle Datensatz ist nicht mehr aktuell - bitte lade dir unter 'Busse hinzufügen' den aktuellen herunter oder aktiviere alle Busse. Im Moment ist der Datensatz OHNE Busse aktiviert.", GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
				RemoteDataBinary.showInfoDueToVersionConfilct = false;
			}
		} else {
			if (!Device.showSoftwareKeyboard()) {
				Gui hinweis = GuiBedienungsHinweis.getBedienungsHinweis(gc, GameCanvasInstance.returnGraphics(), state);
				if (hinweis != null) {
					guis[STATE_BEDIENUNGSHINWEIS] = hinweis;
					switchToState(STATE_BEDIENUNGSHINWEIS, PRESSED_OK, FADER_TYPE_OVERLAY);
				}
			}
		}
	}

	public static int getKeyNumber(int keyCode) {
		if (!Keys.isKey(keyCode, Keys.NUMBER)) {
			return -1;
		}
		return keyCode - 48;
	}

	public static int cntPressed = 1;

	private static int cntMagicKeyPressed = 0;
	private static int cntDebugScreen = 0;

	synchronized private void keyHandler(int keyCode) {
		Graphics g = GameCanvasInstance.returnGraphics();

		if (singleKey) {
			System.out.println("singleKey " + keyCode);
			singleKey = false;
			g.setColor(Color.WHITE);
			g.fillRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
			g.setColor(Color.BLACK);
			g.drawString("keyCode " + keyCode, 0, 0, 0);
			gc.repaint();
			return;
		}

		if (firstAfterMagixxHelp) {
			firstAfterMagixxHelp = false;
			getActiveGui().drawTo(g);
			gc.repaint();
		}

		if (Keys.isKey(keyCode, Keys.UP) || Keys.isKey(keyCode, Keys.DOWN)) {
			lastUpDownTime = System.currentTimeMillis();
		}

		Gui currentGui = getActiveGui();

		if ((currentState == STATE_HOME) && (keyCode == 48)) {
			cntDebugScreen++;
			if (cntDebugScreen == 3) {
				DebugInfo.showDebugInfo(gc);
				cntDebugScreen = 0;
				return;
			}
		}

		// FIXME warum werden ständig Keycodes gesendet wenn ich z.B. im Debugmenü bin ?
		// System.out.println(keyCode);
		if (currentState == STATE_WAIT_SCREEN) {
			// ignore all keys
			return;
		}

		if (Device.isAndroid && (currentState == STATE_HOME) && (Keys.isKey(keyCode, Keys.BACK))) {
			Pushandride.getMidlet().exitProgramm();
			return;
		}

		if ((currentState == STATE_HOME) && ((keyCode == 48 + 9) || (cntMagicKeyPressed == 3))) {
			if (cntMagicKeyPressed == 3) {
				cntMagicKeyPressed = 0;
				if (keyCode == 48 + 0) {
					showProfilerResults();
				}
				if (keyCode == 48 + 1) {
					while (!Data.isLoaded()) {
						try {
							Thread.sleep(100);
						} catch (InterruptedException e) {
							System.err.println("Exception" + e.toString() + " " + e.getMessage());
						}
					}
					DebugInfo.clearDebugScreen();
					long timeBeforeCalculation = System.currentTimeMillis();
					Haltestelle von = Data.getHaltestelle(7);
					Haltestelle nach = Data.getHaltestelle(10);
					Algorithmus.calcRoute(von, nach, DatumZeit.Now());
					long duration_algo = System.currentTimeMillis() - timeBeforeCalculation;
					DebugInfo.addDebugString("TestRoute - Dauer: " + duration_algo + " ms\n");
					DebugInfo.showDebugInfo(gc);
				}
				if (keyCode == 48 + 2) {
					new TestKeys();
				}
				if (keyCode == 48 + 3) {
					singleKey = true;
				}
				if (keyCode == 48 + 4) {
					DebugInfo.showDebugInfo(gc);
				}
				if (keyCode == 48 + 5) {
					Device.setGfxQuality(Device.GFX_QUALITY_LOW);
				}
				if (keyCode == 48 + 6) {
					Device.drawInfoTo(g, gc);
				}
				if (keyCode == 48 + 7) {
					showStats();
				}
				if (keyCode == 48 + 8) {
					GameCanvasInstance.setRequiresFlushGraphics();

					g.setColor(Color.WHITE);
					g.fillRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
					g.setColor(Color.BLACK);
					Font f = FontMetrics.getFont(Font.SIZE_SMALL);
					int fontHeight = f.getHeight();
					int y = 0;
					g.drawString("Bitte senden Sie eine Email an", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("bugfix@pushandride.de", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("mit folgendem Inhalt:", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString(Device.getPlatform(), 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("Der Fehler wird in der", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("nächsten Version behoben.", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("Beliebige Taste für Weiter:", 0, (y++) * fontHeight, Graphics.LEFT | Graphics.TOP);

					gc.flushGraphics();
				}
				if (keyCode == 48 + 9) {
					g.setColor(Color.WHITE);
					g.fillRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
					g.setColor(Color.BLACK);
					int fontHeight = g.getFont().getHeight();
					int lineNo = 0;
					int leftOffset = 5;
					g.drawString("Magixx Keys 999?", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					leftOffset = 10;
					g.drawString("9990 Profiler", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("9991 einzelne Route berechnen", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("9992 KeyChecker & Device info", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("9993 <Key> single KeyChecker", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("9994 show Debug Info", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("9995 Low Quality Mode", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("9996 ShowDeviceMemInfo", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("9997 ShowStats", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("9998 enable flushgraphics", leftOffset, lineNo++ * fontHeight, Graphics.LEFT | Graphics.TOP);
					g.drawString("9999 Magixx Key Help", leftOffset, ++lineNo * fontHeight, Graphics.LEFT | Graphics.TOP);
					gc.repaint();

					firstAfterMagixxHelp = true;
				}
				return;
			} else {
				if (keyCode == 48 + 9) {
					cntMagicKeyPressed++;
				}
			}
		} else {
			cntMagicKeyPressed = 0;
		}

		// evtl. Dinge auch in pointerPressed eintragen....
		if (currentState == STATE_MESSAGESCREEN) {
			if (guiMessageScreen.getFaderType() != GuiMessageScreen.TYPE_MESSAGE_FULLSCREEN) {
				switchToState(getActiveGui().getPreviousGui(), PRESSED_OK, guiMessageScreen.getFaderType());
			} else {
				switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK, guiMessageScreen.getFaderType());
			}
			return;
		}

		if (currentState == STATE_BEDIENUNGSHINWEIS) {
			switchToState(getActiveGui().getPreviousGui(), PRESSED_OK, FADER_TYPE_OVERLAY);
			return;
		}

		if (currentState == STATE_SELECT_DAYTIME) {
			if ((Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD) || Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD) || Keys.isKey(keyCode, Keys.C) || Keys.isKey(keyCode, Keys.MENU_LEFT) || Keys.isKey(keyCode, Keys.MENU_RIGHT)) && ((!Keys.isKey(keyCode, Keys.LEFT) && (!Keys.isKey(keyCode, Keys.RIGHT))))) {
				if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD) || Keys.isKey(keyCode, Keys.MENU_RIGHT)) {
					int newTime = guiDayTime.getTime();
					if (newTime != guiUebersicht.getFirstQueryTime()) {
						guiUebersicht.dataInvalid();
						guiUebersicht.prepare(guiUebersicht.getHaltestellenPaar(), newTime);
					}
					switchToState(getActiveGui().getPreviousGui(), PRESSED_OK, FADER_TYPE_OVERLAY);
				} else {
					if (Keys.isKey(keyCode, Keys.C)) {
						guiDayTime.resetTime();
					} else {
						switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK, FADER_TYPE_OVERLAY);
					}
				}
				return;
			}
			getActiveGui().keyPressed(keyCode, gc);
			return;
		}

		if (currentState == STATE_NEW_DATASET) {
			GuiNewDataset gui = (GuiNewDataset) guis[STATE_NEW_DATASET];
			int state = gui.getState();
			if (state == GuiNewDataset.STATE_RESTART) {
				return;
			}
			if (state == GuiNewDataset.STATE_ERROR) {
				switchToState(STATE_HOME, PRESSED_BACK);
				return;
			}
			if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD) || Keys.isKey(keyCode, Keys.MENU_LEFT)) {
				RemoteDataBinary.cancelDownload();
				gui.cancel();
				switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				return;
			}
		}

		if (currentState == STATE_REMINDER_TIME) {
			if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD) || Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD) || Keys.isKey(keyCode, Keys.C) || Keys.isKey(keyCode, Keys.MENU_LEFT)) {
				if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD) || Keys.isKey(keyCode, Keys.MENU_LEFT)) {
					int newTime = guiReminderTime.getTime();
					if (newTime == -1) {
						Reminder.reset();
					} else {
						Reminder.enable(guiDetail.getRoute(), newTime);
					}
					guiDetail.updateReminderString();
					switchToState(getActiveGui().getPreviousGui(), PRESSED_OK, FADER_TYPE_OVERLAY);
				} else {
					switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK, FADER_TYPE_OVERLAY);
				}
				return;
			}
			getActiveGui().keyPressed(keyCode, gc);
			return;
		}

		if (currentState == STATE_WEITERGEBEN) {
			if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
				Item selectedItem = guiSelectionWeitergeben.getSelected();
				if (selectedItem instanceof ItemWeitergebenExecute) {
					int itemType = ((ItemWeitergebenExecute) selectedItem).getType();
					switch (itemType) {
					case ItemWeitergebenExecute.WEITERGEBEN_ITEM_SEND_VIA_BLUETOOTH:
						try {
							new BluetoothChooseDeviceScreen();
						} catch (Exception e) {
							Manager.showMessage("Info", "Bitte Bluetooth aktvieren. Falls dieses bereits aktiv ist kann push&ride nicht versendet werden.", GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
						}
						return;
					case ItemWeitergebenExecute.WEITERGEBEN_ITEM_SEND_VIA_SMS:
						Displayable last = Display.getDisplay(Pushandride.getMidlet()).getCurrent();
						new AdviceFriend(last).setVisible(true);
						return;
					}
				}
			}
			if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD) || Keys.isKey(keyCode, Keys.C) || Keys.isKey(keyCode, Keys.MENU_LEFT)) {
				switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK, FADER_TYPE_SPLITTER);
				return;
			}
		}

		if (Keys.isKey(keyCode, Keys.NUMBER)) {
			int no = getKeyNumber(keyCode);
			switch (currentState) {
			case (STATE_HOME):
				if (guiHome.selectHaltestellenPaar(no)) {
					keyHandler(Keys.getKeyCode(Keys.OK));
				}
				return;
			}
		}

		if (Keys.isKey(keyCode, Keys.MENU_RIGHT)) {
			switch (currentState) {
			case (STATE_UEBERSICHT):
				if (guiUebersicht.isGueltigeStrecke()) {
					guiDayTime = new GuiSelectionDayTime(gc, g, guiUebersicht.getFirstQueryTime());
					guis[STATE_SELECT_DAYTIME] = guiDayTime;
					switchToState(STATE_SELECT_DAYTIME, PRESSED_OK, FADER_TYPE_OVERLAY);
				}
				return;
			case (STATE_DATASET_COSTUMIZE):
				String s[] = ((GuiSelectionDatasetCustomize) guis[STATE_DATASET_COSTUMIZE]).getSelectedLinien();
				RemoteDataBinary.handleCustomSelection(s);
				return;
			}
		}

		if (Keys.isKey(keyCode, Keys.MENU_LEFT)) {
			switch (currentState) {
			case (STATE_HOME):
				guiHome.flipHaltestellen();
				return;
			case (STATE_OPTIONS):
				returnFromOptions();
				return;
			case (STATE_OPTION_DATASET):
				switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				return;
			case (STATE_DATASET_COSTUMIZE):
				switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				return;
			case (STATE_UEBERSICHT):
				// nothing
				return;
			case (STATE_DETAILS):
				if (!guiDetail.isRemindable()) {
					return;
				}
				guiReminderTime = new GuiSelectionReminderTime(gc, g, guiDetail.getRoute());
				guis[STATE_REMINDER_TIME] = guiReminderTime;
				switchToState(STATE_REMINDER_TIME, PRESSED_OK, FADER_TYPE_OVERLAY);
				return;
			case (STATE_ABOUT):
				switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				return;
			}
		}

		if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD) || Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD)) {
			switch (currentState) {
			case (STATE_HOME):
				GuiSelectionHome guiSelectionHome = (GuiSelectionHome) currentGui;
				Item selectedItem = guiSelectionHome.getSelectedItem();
				if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {

					markFollowingGuisInvalid(currentState);

					if (selectedItem instanceof ItemHomeExecute) {
						int itemType = ((ItemHomeExecute) selectedItem).getType();
						switch (itemType) {
						case ItemHomeExecute.HOME_ITEM_NEW:
							switchToState(STATE_ORT_VON, PRESSED_OK);
							break;
						case ItemHomeExecute.HOME_ITEM_WEITERGEBEN:
							switchToState(STATE_WEITERGEBEN, PRESSED_OK, FADER_TYPE_SPLITTER);
							break;
						case ItemHomeExecute.HOME_ITEM_OPTIONS:
							guiSelectionOptions.prepare();
							switchToState(STATE_OPTIONS, PRESSED_OK, FADER_TYPE_SPLITTER);
							break;
						case ItemHomeExecute.HOME_ITEM_DATASET:
							switchToState(STATE_OPTION_DATASET, PRESSED_OK, FADER_TYPE_SPLITTER);
							break;
						case ItemHomeExecute.HOME_ITEM_EXIT:
							pushandride.exitProgramm();
							break;
						}
					}
					if (selectedItem instanceof ItemHomeHaltestellenPaar) {
						guiUebersicht.prepare(((ItemHomeHaltestellenPaar) selectedItem).getHaltestellenPaar());
						switch (((ItemHomeHaltestellenPaar) selectedItem).getTyp()) {
						case ItemHomeHaltestellenPaar.TYP_FAVORITEN:
							data.persistence.Stats.inc(data.persistence.Stats.BERECHNETE_ROUTEN_FAVORITEN);
							break;
						case ItemHomeHaltestellenPaar.TYP_ZULETZTVERWENDETE:
							data.persistence.Stats.inc(data.persistence.Stats.BERECHNETE_ROUTEN_ZULETZTVERWENDET);
							break;
						}
						switchToState(STATE_BERECHNE, PRESSED_OK);
					}
				}
				if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD)) {
					switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				}

				break;
			case (STATE_ORT_VON):
				if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
					/*
					 * evtl. kann der nächste Screen übersprungen werden da
					 * entweder nur eine Haltestelle am Ort ist oder eine
					 * gespeicherte Haltestelle ausgewählt wurde
					 */
					if (guiOrtVon.skipHaltestelleScreen()) {
						switchToState(STATE_ORT_NACH, PRESSED_OK);
					} else {
						switchToState(STATE_HALTESTELLE_VON, PRESSED_OK, FADER_TYPE_SPLITTOHALTESTELLEN);
					}
				}
				if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD)) {
					switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				}
				break;
			case (STATE_HALTESTELLE_VON):
				if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
					switchToState(STATE_ORT_NACH, PRESSED_OK);
				}
				if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD))
					switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK, FADER_TYPE_SPLITTOHALTESTELLEN);
				break;
			case (STATE_ORT_NACH):
				if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
					/*
					 * evtl. kann der nächste Screen übersprungen werden da
					 * entweder nur eine Haltestelle am Ort ist oder eine
					 * gespeicherte Haltestelle ausgewählt wurde
					 */
					if (guiOrtNach.skipHaltestelleScreen()) {
						Haltestelle haltestelleVon = (Haltestelle) guiHaltestelleVon.getSelected();
						Haltestelle haltestelleNach = (Haltestelle) guiHaltestelleNach.getSelected();
						HaltestellenPaar haltestellenPaar = new HaltestellenPaar(haltestelleVon, haltestelleNach);

						if (haltestelleVon.equals(haltestelleNach)) {
							showMessage(Locale.Info, Locale.replace(Locale.Msg_VonNachEqual, 1, haltestelleVon.getFullName()), GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
						} else {
							guiUebersicht.prepare(haltestellenPaar);
							data.persistence.Stats.inc(data.persistence.Stats.BERECHNETE_ROUTEN_NEUEROUTE);
							switchToState(STATE_BERECHNE, PRESSED_OK);
							PushandrideMidlet.getManager().setMainScreenInvalid();
						}
					} else {
						switchToState(STATE_HALTESTELLE_NACH, PRESSED_OK, FADER_TYPE_SPLITTOHALTESTELLEN);
					}
				}
				if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD))
					switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				break;
			case (STATE_HALTESTELLE_NACH):
				if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
					Haltestelle haltestelleVon = (Haltestelle) guiHaltestelleVon.getSelected();
					Haltestelle haltestelleNach = (Haltestelle) guiHaltestelleNach.getSelected();
					HaltestellenPaar haltestellenPaar = new HaltestellenPaar(haltestelleVon, haltestelleNach);
					if (haltestelleVon.equals(haltestelleNach)) {
						showMessage(Locale.Info, Locale.replace(Locale.Msg_VonNachEqual, 1, haltestelleVon.getFullName()), GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
					} else {
						guiUebersicht.prepare(haltestellenPaar);
						data.persistence.Stats.inc(data.persistence.Stats.BERECHNETE_ROUTEN_NEUEROUTE);
						switchToState(STATE_BERECHNE, PRESSED_OK);
						PushandrideMidlet.getManager().setMainScreenInvalid();
					}
				}
				if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD))
					switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK, FADER_TYPE_SPLITTOHALTESTELLEN);
				break;
			case (STATE_BERECHNE):
				if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD)) {
					berechneScreenCancel();
					switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				}
				break;
			case (STATE_UEBERSICHT):
				/*
				 * if (Manager.isKeyMenuLeft(keyCode)) { ItemUebersicht
				 * itemUebersicht = (ItemUebersicht) items[selectedId];
				 * HaltestellenPaar hp =
				 * itemUebersicht.getRoute().getHaltestellenPaar(); new
				 * FavoritenList(hp); }
				 */
				if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
					Route route;
					if (guiUebersicht.isGueltigeStrecke()) {
						route = guiUebersicht.getSelectedRoute();
						if (route != model.Route.NULL) {
							if (route.isOptimierungAbgebrochen()) {
								activateWaitScreen("Strecke wird optimiert");
								route = Algorithmus.calcRoute(route.getHaltestelleVon(), route.getHaltestelleNach(), route.getAbfahrtszeit(), Algorithmus.OPTIMIERUNG_LANG);
							}
							guiDetail.activate(route);
							switchToState(STATE_DETAILS, PRESSED_OK);
						}
					} else {
						switchToState(STATE_HOME, PRESSED_OK);
					}
				}
				if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD))
					switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				break;
			case (STATE_DETAILS):
				if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
					switchToState(STATE_HOME, PRESSED_OK);
				}
				if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD)) {
					switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				}
				break;
			case (STATE_OPTIONS):
				if (!getActiveGui().keyPressed(keyCode, gc)) {
					if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD)) {
						PushandrideMidlet.getManager().returnFromOptions();
					}
				}
				break;
			case (STATE_OPTION_DATASET):
				if (!getActiveGui().keyPressed(keyCode, gc)) {
					if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD)) {
						switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK, FADER_TYPE_SPLITTER);
					}
				}
				break;
			case (STATE_DATASET_COSTUMIZE):
				if (!getActiveGui().keyPressed(keyCode, gc)) {
					if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD)) {
						switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
					}
				}
				break;
			case (STATE_REMINDER):
				switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK, FADER_TYPE_SPLITTER);
				break;
			case (STATE_ABOUT):
				switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK);
				break;
			}
		} else {
			/*
			 * es wurde eine Taste gedrückt die nicht hoch oder runter ist daher
			 * werden in alle folgenden Guis die Daten als ungültig deklariert
			 */
			if (currentState != STATE_HOME) {
				markFollowingGuisInvalid(currentState);
			}
			getActiveGui().keyPressed(keyCode, gc);
		}
	}

	private void showProfilerResults() {
		while (!Data.isLoaded()) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				System.err.println("Exception" + e.toString() + " " + e.getMessage());
			}
		}
		DebugInfo.clearDebugScreen();
		// Test-Route
		long timeBeforeCalculation = System.currentTimeMillis();
		Haltestelle von = Data.getHaltestelle(7);
		Haltestelle nach = Data.getHaltestelle(10);
		Algorithmus.calcRoute(von, nach, DatumZeit.Now());
		long duration_algo = System.currentTimeMillis() - timeBeforeCalculation;
		// Test Bild-Skalieren
		timeBeforeCalculation = System.currentTimeMillis();
		Image source1 = null;
		try {
			source1 = Image.createImage("/res/pnr60px.png");
		} catch (IOException e) {
			Logger.warnException(getClass(), e, "as89y");
		}
		ImageScaler.scaleBilinearByHeight(source1, 45);
		long duration_scale = System.currentTimeMillis() - timeBeforeCalculation;
		long startZeit = BootProfiler.getStartDuration();

		String out = "TestRoute - Dauer: " + duration_algo + " ms\n" + "Bild-Skalieren - Dauer: " + duration_scale + " ms\n\n" + "Start-Zeit (gesamt): " + startZeit + " ms\n";
		out += BootProfiler.getProfilerString();
		DebugInfo.addDebugString(out);

		DebugInfo.showDebugInfo(gc);
	}

	public void activateWaitScreen(String message) {
		guiWaitScreen = new GuiWaitScreen(message);
		guis[STATE_WAIT_SCREEN] = guiWaitScreen;
		switchToState(STATE_WAIT_SCREEN, PRESSED_OK);
	}

	public void keyPressed(int keyCode) {
		keyHandler(keyCode);
		try {
			if (keyThread != null) {
				synchronized (keyThread) {
					keyThread.notify();
				}
			}
		} catch (OutOfMemoryError oom) {
			PushandrideMidlet.OutOfMemoryException(getClass(), oom, "zzs92");
		} catch (Exception e) {
			Logger.warnException(getClass(), e, "k8njms");
		}
	}

	public void keyReleased(int keyCode) {
		gc.getKeyStates();
	}

	/*
	 * -------------------------------------------------- Pointer pressed
	 * -------------------------------------------------
	 */

	public static int POINTER_X_MIDDLE = 0;
	public static int POINTER_X_LEFT = 1;
	public static int POINTER_X_RIGHT = 2;

	public static int checkPointerXPos(int x) {
		// int leftx = Math.min(20, Utils.scale(Manager.SCREEN_WIDTH, 3));
		int leftx = GfxUtils.scale(Manager.SCREEN_WIDTH, 4);

		// System.out.println("Manager.SCREEN_WIDTH " + Manager.SCREEN_WIDTH);

		// System.out.println("leftx " + leftx);
		int rightx = Manager.SCREEN_WIDTH - leftx;
		if (x <= leftx) {
			return POINTER_X_LEFT;
		}
		if (x >= rightx) {
			return POINTER_X_RIGHT;
		}
		return POINTER_X_MIDDLE;
	}

	private static int magic_pointer_pressed = 0;

	private static int pointerDownAtX;
	private static int pointerDownAtY;
	private static int lastPointerScrollDiff;
	private static final int POINTERMODE_DOWN = 1;
	private static final int POINTERMODE_UP = 2;
	private static final int POINTERMODE_HANDLED = 3;
	private static int pointerMode = POINTERMODE_UP;
	private static boolean hasMoved;

	synchronized public void pointerReleased(int x, int y) {
		pointerMode = POINTERMODE_UP;
		// System.out.println("pointerReleased (" + x + "," + y + ")");

		if (!hasMoved) {
			// magic key handling
			if (currentState == STATE_HOME) {
				if (y <= Manager.HEADER_HEIGHT) {
					magic_pointer_pressed++;
					if (magic_pointer_pressed == 10) {
						magic_pointer_pressed = 0;
						if (x < Manager.SCREEN_WIDTH * 1 / 4) {
							new TestKeys();
						} else {
							if (x < Manager.SCREEN_WIDTH * 2 / 4) {
								showProfilerResults();
							} else {
								if (x < Manager.SCREEN_WIDTH * 3 / 4) {
									Device.drawInfoTo(GameCanvasInstance.returnGraphics(), gc);
								} else {
									DebugInfo.showDebugInfo(gc);
								}
							}
						}
						return;
					}
				} else {
					magic_pointer_pressed = 0;
				}
			}
			// normal behaviour
			Gui activeGui = getActiveGui();
			if (currentState == STATE_MESSAGESCREEN) {
				switchToState(activeGui.getPreviousGui(), PRESSED_BACK, guiMessageScreen.getFaderType());
				return;
			}
			if (currentState == STATE_BEDIENUNGSHINWEIS) {
				switchToState(activeGui.getPreviousGui(), PRESSED_OK, FADER_TYPE_OVERLAY);
				return;
			}
			// left right - funktioniert wenn man nicht im Footer ist
			int yMax = Manager.SCREEN_HEIGHT - activeGui.getFooterHeight();
			if (activeGui instanceof GuiSelectionLocation) {
				if (Device.showSoftwareKeyboard()) {
					GuiSelectionLocation gsl = (GuiSelectionLocation) activeGui;
					yMax -= gsl.getKeyHeight();
				}
			}
			if (y < yMax) {
				int pointerPos = checkPointerXPos(x);
				if (pointerPos == POINTER_X_LEFT) {
					keyHandler(Keys.getKeyCode(Keys.LEFT));
					return;
				}
				if (pointerPos == POINTER_X_RIGHT) {
					if (!(activeGui instanceof GuiSelectionHome && ((GuiSelectionHome) activeGui).getItemAtPos(x, y) instanceof ItemHomeHaltestellenPaar)) {
						keyHandler(Keys.getKeyCode(Keys.RIGHT));
						return;
					}
				}
			}

			// Im Footer
			if (y >= Manager.SCREEN_HEIGHT - activeGui.getFooterHeight()) {
				if (x < Manager.SCREEN_WIDTH / 2) {
					keyHandler(Keys.getKeyCode(Keys.MENU_LEFT));
				} else {
					keyHandler(Keys.getKeyCode(Keys.MENU_RIGHT));
				}
				// Im Header oder Main
			} else {
				if (y < Manager.HEADER_HEIGHT) {
					activeGui.pointerPressed(x, y);
				} else {
					if (activeGui.pointerPressed(x, y)) {
						markFollowingGuisInvalid(currentState);
						keyHandler(Keys.getKeyCode(Keys.OK));
					}
				}
			}
		}

	}

	// Ziehgesten
	public void pointerDragged(int x, int y) {
		// System.out.println("pointerDragged (" + x + "," + y + ")");
		if (pointerMode == POINTERMODE_DOWN) {
			int threasholdLeftRight = GfxUtils.scale(Math.min(SCREEN_WIDTH, SCREEN_HEIGHT), 30);
			int dx = Math.abs(x - pointerDownAtX);
			int dy = Math.abs(y - pointerDownAtY);

			// falls zusätzlich nach oben/unten gescrollt wird, wird links
			// rechts gesperrt
			if (dx >= threasholdLeftRight && dy * 2 < dx) {
				pointerMode = POINTERMODE_HANDLED;
				// fixme: hier sollte Command zum links und rechts scrollen hin
				// und nicht der Tastenkey
				if (x > pointerDownAtX) {
					keyHandler(Keys.getKeyCode(Keys.LEFT));
				} else {
					keyHandler(Keys.getKeyCode(Keys.RIGHT));
				}
				hasMoved = true;
			} else {
				int lineHight = FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);

				int pointerScrollDiff = (y - pointerDownAtY) / lineHight;

				if (lastPointerScrollDiff != pointerScrollDiff) {
					hasMoved = true;
					if (lastPointerScrollDiff - pointerScrollDiff > 0) {
						for (int i = 0; i < lastPointerScrollDiff - pointerScrollDiff; i++) {
							getActiveGui().scrollDown();
						}
					} else {
						for (int i = 0; i < pointerScrollDiff - lastPointerScrollDiff; i++) {
							getActiveGui().scrollUp();
						}
					}
					lastPointerScrollDiff = pointerScrollDiff;
				}

			}
		}

	}

	synchronized public void pointerPressed(int x, int y) {
		Device.checkForSoftwareKeyboard();

		// System.out.println("pointerPressed (" + x + "," + y + ")");
		pointerMode = POINTERMODE_DOWN;
		pointerDownAtX = x;
		pointerDownAtY = y;
		lastPointerScrollDiff = 0;
		hasMoved = false;
	}

	/**
	 * setzt alle Daten in den Guis auf Ungültig, die nach 'state' kommen
	 * 
	 * @param state
	 *            - STATE_xyz
	 */
	private void markFollowingGuisInvalid(int state) {
		for (int i = state + 1; i < guis.length; i++) {
			if (guis[i] != null) {
				guis[i].dataInvalid();
			}
		}
	}

	public static String getFooterDebugString() {
		return Manager.footerDebugString;
	}

	/**
	 * Zeigt einen Message Screen ein
	 * 
	 * @param header
	 *            Kopfzeile
	 * @param text
	 *            Haupttext - neue Zeilen mit '\n'
	 * @param type
	 *            GuiMessageScreen.TYPE_MESSAGE_FULLSCREEN oder
	 *            GuiMessageScreen.TYPE_MESSAGE_OVERLAY
	 * 
	 */

	public static void showMessage(String header, String text, int type) {
		if (Manager.manager != null) {
			Manager.manager.viewScreen(header, text, type);
		} else {
			System.err.println("showMessage kann nicht aufgerufen werden, Manager noch nicht inizialisiert!");
		}
	}

	private void viewScreen(String header, String text, int type) {
		guiMessageScreen.showMessage(header, text, type);
		switchToState(STATE_MESSAGESCREEN, PRESSED_OK, type);
	}

	public void timeIncreased() {
		getActiveGui().redrawCurrentTime();
	}

	public static void reminderCallback(Route route) {
		PushandrideMidlet.getManager().guiReminder.activate(route);
		PushandrideMidlet.getManager().switchToState(STATE_REMINDER, PRESSED_OK, FADER_TYPE_SPLITTER);
	}

	private boolean lockScreen() {
		synchronized (this) {
			if (isLocked) {
				return false;
			} else {
				isLocked = true;
				return true;
			}
		}
	}

	private void unlockScreen() {
		synchronized (this) {
			isLocked = false;
		}
	}

	public static int getFooterHeight() {
		return FOOTER_HEIGHT;
	}

	private void showFirst() {
		// ACHTUNG - es kann immer nur eine Message angezeigt werden - das letzt
		// aufgerufene showMessage wird angezeigt !

		// Handling der Anzeige von Hinweisen
		int gueltigVon = Data.getGueltigVon();
		int gueltigBis = Data.getGueltigBis();
		// Speichern wir zwischen, da nicht ganz billig.
		int now = DatumZeit.Now();
		// prüfe ob die Fahrplandaten Gültig sind - falls nicht müssen die
		// Messages deaktiviert werden
		if ((gueltigVon == DatumZeit.NULL) || (gueltigBis == DatumZeit.NULL)) {
			Manager.showMessage("Info", "Ungültiger Datensatz! - keine Gültigkeitszeitraum", GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
		} else {
			if ((gueltigVon != DatumZeit.NULL) && (now < gueltigVon)) {
				Manager.showMessage("Info", "Die Fahrplandaten dieser Version sind erst ab " + DatumZeit.toDateYearString(gueltigVon) + " gültig. Evtl. ist die Zeit des Handys nicht richtig eingestellt." + "\nAktuelles Datum laut Handy: " + DatumZeit.toDateYearString(now), GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
			} else {
				if ((gueltigBis != DatumZeit.NULL) && (now > gueltigBis)) {
					Manager.showMessage("Info", "Die Fahrplandaten dieser Version sind nur bis " + DatumZeit.toDateYearString(gueltigBis) + " gültig. Bitte installiere eine neue Version von push&ride." + "\nAktuelles Datum laut Handy: " + DatumZeit.toDateYearString(now), GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
				} else {
					activateBedinungshinweise(STATE_HOME);
				}
			}
		}
	}

	boolean changedsize = false;
	int changeWidth = 0;
	int changeHeight = 0;

	public void sizeChanged(int width, int height) {
		if (width > 0 && SCREEN_HEIGHT > 0) {
			if (changedsize) {
				if (width != changeWidth || height != changeHeight) {
					changeWidth = width;
					changeHeight = height;
					repaintAll();
					if (width == SCREEN_WIDTH && height == SCREEN_HEIGHT) {
						changedsize = false;
					}
				}
			} else {
				if (width != SCREEN_WIDTH || height != SCREEN_HEIGHT) {
					changeWidth = width;
					changeHeight = height;
					repaintAll();
					changedsize = true;
				}
			}
		}
	}

	public void returnFromOptions() {
		OptionManager.backFromOptions();
		switchToState(getActiveGui().getPreviousGui(), PRESSED_BACK, FADER_TYPE_SPLITTER);
	}

	public void resetSettings() {
		Persistence.deleteRecordStore();
		guiSelectionOptions.prepare();
		guiSelectionOptions.drawTo(GameCanvasInstance.returnGraphics());
	}

	public void setMainScreenInvalid() {
		guiHome.dataInvalid();
	}

	public Graphics getGameCanvasGraphics() {
		return GameCanvasInstance.returnGraphics();
	}

	synchronized public void berechneScreenCancel() {
		guiBerechne.cancel();
	}

	synchronized public void berechneScreenMoveOn() {
		switchToState(STATE_UEBERSICHT, PRESSED_OK);
	}

	public void openAbout() {
		if (guis[STATE_ABOUT] == null) {
			guis[STATE_ABOUT] = new GuiAbout(gc, GameCanvasInstance.returnGraphics());
		}
		switchToState(STATE_ABOUT, PRESSED_OK);
	}

	public void repaintAll() {
		guis[currentState].drawTo(GameCanvasInstance.returnGraphics());
		gc.repaint();
	}

	private void showStats() {
		DebugInfo.addDebugString(Stats.getStatsInfo());
		DebugInfo.showDebugInfo(gc);
	}

	public static void datasetSelectWrapper(int id) {
		manager.datasetSelect(id);
	}

	private void datasetSelect(int id) {
		switch (id) {
		case GuiSelectionDatasetSelect.ITEM_ID_DATASET_SMALL:
			UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_DATENSATZ_TYP, Data.DATSET_KLEIN);
			break;
		case GuiSelectionDatasetSelect.ITEM_ID_DATASET_BIG:
			UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_DATENSATZ_TYP, Data.DATSET_ALLES);
			break;
		}

		switch (id) {
		case GuiSelectionDatasetSelect.ITEM_ID_DATASET_SMALL:
		case GuiSelectionDatasetSelect.ITEM_ID_DATASET_BIG:
			GuiNewDataset gui = new GuiNewDataset(gc, getGameCanvasGraphics());
			guis[STATE_NEW_DATASET] = gui;
			gui.triggerRestart();
			switchToState(STATE_NEW_DATASET, PRESSED_OK);
			break;
		case GuiSelectionDatasetSelect.ITEM_ID_DATASET_INDIVIDUELL:
			guis[STATE_DATASET_COSTUMIZE] = new GuiSelectionDatasetCustomize(gc, getGameCanvasGraphics());
			switchToState(STATE_DATASET_COSTUMIZE, PRESSED_OK);
			break;
		}

	}

	public static void showRestartGuiWrapper() {
		manager.showRestartGui();
	}

	private void showRestartGui() {
		GuiNewDataset gui = new GuiNewDataset(gc, getGameCanvasGraphics());
		guis[STATE_NEW_DATASET] = gui;
		gui.triggerRestart();
		switchToState(STATE_NEW_DATASET, PRESSED_OK);
	}

	public static GuiNewDataset downloadDatasetWrapper() {
		return manager.downloadDataset();
	}

	private GuiNewDataset downloadDataset() {
		GuiNewDataset gui = new GuiNewDataset(gc, getGameCanvasGraphics());
		guis[STATE_NEW_DATASET] = gui;
		gui.triggerDownload();
		switchToState(STATE_NEW_DATASET, PRESSED_OK);
		return gui;
	}

}
