// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import gui.guis.GuiMessageScreen;
import gui.guis.GuiSelectionHome;
import gui.items.Item;
import gui.items.ItemAbout;
import gui.items.ItemOption;
import gui.items.ItemOptionCall;
import gui.items.ItemOptionSelectLaufwegeZeit;
import gui.items.ItemOptionToggle;
import gui.items.ItemSpace;
import gui.options.AdviceFriend;
import gui.options.BluetoothChooseDeviceScreen;
import gui.options.dev.TestFonts;
import gui.options.dev.TestImageScaler;
import gui.options.dev.TestKeys;
import gui.options.dev.TestTime;
import gui.options.dev.TestVonNach;
import gui.utils.SMSWithInput;

import java.util.Vector;

import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;

import algo.RoutenCache;
import control.Device;
import control.Global;
import control.Pushandride;
import control.PushandrideMidlet;
import control.locale.Locale;
import dev.Logger;

public class OptionManager {

	public final static int ITEM_ID_BACK = -1;
	public final static int ITEM_ID_CALL_RESET_SETTINGS = 1;
	public final static int ITEM_ID_CALL_FEEDBACK_SCHREIBEN = 2;
	public final static int ITEM_ID_CALL_NEUE_VERSION_PRUEFEN = 3;
	//public final static int ITEM_ID_CALL_SEND_LOGGINGS = 4; Gelöscht
	public final static int ITEM_ID_CALL_SEND_PUSHNRIDE = 5;
	public final static int ITEM_ID_CALL_ABOUT = 6;
	public final static int ITEM_ID_CALL_ADVICE_FRIEND = 7;
	public final static int ITEM_ID_CHANGE_TIMEOFFSET = 8;
	public final static int ITEM_ID_CHANGE_LAUFZEIT_FAKTOR = 9;
	public final static int ITEM_ID_GFX_SOFTWAREKEYBOARD = 20;
	public final static int ITEM_ID_GFX_QUALITY = 21;
	public final static int ITEM_ID_GFX_TOUCH_SIZE = 22;
	public final static int ITEM_ID_TEST_IMAGESCALE = 90;
	public final static int ITEM_ID_TEST_FONTS = 91;
	public final static int ITEM_ID_TEST_VONNACH = 92;
	public final static int ITEM_ID_TEST_KEYS = 93;
	public final static int ITEM_ID_TEST_VIBRATE = 94;
	public final static int ITEM_ID_TEST_TIME = 95;
	public final static int ITEM_ID_TEST_DEVICE_INFOS = 96;

	public static Item[] getInitItemsDataset() {
		Vector items = new Vector();

		int space = FontMetrics.getFont(Font.STYLE_PLAIN, ItemOption.OPTION_ITEM_FONT_SIZE).getHeight() / 4;

		items.addElement(new ItemSpace(space));

		items.addElement(new ItemAbout("Datensatz auswählen", false));

		items.addElement(new ItemSpace(space));

		Item[] optionItems = new Item[items.size()];
		items.copyInto(optionItems);

		return optionItems;
	}

	public static ItemOption[] getInitItems() {
		Vector items = new Vector();

		// Anzeigeelemenente auf dem Optionscreen
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_BACK));

		if (!Device.isAndroid) {
			items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_CHANGE_TIMEOFFSET));
		}
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_CHANGE_LAUFZEIT_FAKTOR));
		if (!Device.isAndroid) {
			items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_GFX_QUALITY));
		}
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_GFX_SOFTWAREKEYBOARD));
		if (!Device.isAndroid) {
			items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_GFX_TOUCH_SIZE));
		}

		// per SMS oder WAP
		if (!Device.isAndroid) {
			items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_CALL_FEEDBACK_SCHREIBEN));
		}

		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_CALL_ABOUT));
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_CALL_RESET_SETTINGS));

		ItemOption[] optionItems = new ItemOption[items.size()];
		items.copyInto(optionItems);

		return optionItems;
	}

	public static ItemOption[] getHiddenItems() {
		Vector items = new Vector();

		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_CALL_NEUE_VERSION_PRUEFEN));
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_TEST_IMAGESCALE));
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_TEST_FONTS));
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_TEST_VONNACH));
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_TEST_KEYS));
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_TEST_VIBRATE));
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_TEST_TIME));
		items.addElement(OptionManager.getOptionItem(OptionManager.ITEM_ID_TEST_DEVICE_INFOS));

		ItemOption[] optionItems = new ItemOption[items.size()];
		items.copyInto(optionItems);

		return optionItems;
	}

	/**
	 * liefert das Item entsprechend seiner Id zurück, diese MUSS eindeutig sein
	 */
	public static ItemOption getOptionItem(int id) {
		switch (id) {
		case ITEM_ID_BACK:
			return new ItemOptionCall(id, Locale.Back);
		case ITEM_ID_CHANGE_TIMEOFFSET:
			String[] values = new String[25];
			for (int i = -12; i <= 12; i++) {
				String stundenStr = "Stunden";
				if (i == -1 || i == 1) {
					stundenStr = "Stunde";
				}
				String string = i + " " + stundenStr;
				if (i > 0) {
					string = "+" + string;
				}
				values[i + 12] = string;
			}
			int timeOffsetInHours = Device.getTimeOffset() / 60;
			if (timeOffsetInHours < -12) {
				Logger.error(OptionManager.class, "timeOffset zu klein");
				timeOffsetInHours = -12;
			}
			if (timeOffsetInHours > 12) {
				Logger.error(OptionManager.class, "timeOffset zu groß");
				timeOffsetInHours = +12;
			}
			return new ItemOptionToggle(id, "Zeitverschiebung", values, timeOffsetInHours + 12);
		case ITEM_ID_CHANGE_LAUFZEIT_FAKTOR:
			return new ItemOptionSelectLaufwegeZeit(id);
		case ITEM_ID_GFX_SOFTWAREKEYBOARD:
			return new ItemOptionToggle(id, Locale.SW_Keyboard, new String[] { Locale.inactive, Locale.active }, 0);
		case ITEM_ID_GFX_QUALITY:
			return new ItemOptionToggle(id, Locale.GraphicsQuality, new String[] { Locale.low, Locale.mid, Locale.high }, 0);
		case ITEM_ID_GFX_TOUCH_SIZE:
			return new ItemOptionToggle(id, "Schaltflächen vergrößern", new String[] { "aus", "an" }, 0);
		case ITEM_ID_CALL_RESET_SETTINGS:
			return new ItemOptionCall(id, Locale.ResetSettings);
		case ITEM_ID_CALL_FEEDBACK_SCHREIBEN:
			return new ItemOptionCall(id, Locale.WriteFeedback);
		case ITEM_ID_CALL_NEUE_VERSION_PRUEFEN:
			return new ItemOptionCall(id, Locale.CheckForNewVersion);
		case ITEM_ID_CALL_ADVICE_FRIEND:
			return new ItemOptionCall(id, "Installationslink versenden");
		case ITEM_ID_CALL_SEND_PUSHNRIDE:
			return new ItemOptionCall(id, "push&ride per Blueooth schicken");
		case ITEM_ID_CALL_ABOUT:
			return new ItemOptionCall(id, Locale.About);
		case ITEM_ID_TEST_IMAGESCALE:
			return new ItemOptionCall(id, "(Test) Bilder skalieren");
		case ITEM_ID_TEST_FONTS:
			return new ItemOptionCall(id, "(Test) Schriftarten");
		case ITEM_ID_TEST_VONNACH:
			return new ItemOptionCall(id, "(Test) Von - Nach");
		case ITEM_ID_TEST_KEYS:
			return new ItemOptionCall(id, "(Test) Keys");
		case ITEM_ID_TEST_VIBRATE:
			return new ItemOptionCall(id, "(Test) Handyvibration");
		case ITEM_ID_TEST_TIME:
			return new ItemOptionCall(id, "(Test) Zeit");
		case ITEM_ID_TEST_DEVICE_INFOS:
			return new ItemOptionCall(id, "(Test) Device-Infos");
		}
		return null;
	}

	/**
	 * Event-Handler für CallItems
	 */
	public static void handleOptionCall(int id) {
		switch (id) {
		case ITEM_ID_BACK:
			PushandrideMidlet.getManager().returnFromOptions();
			break;
		case ITEM_ID_CALL_ABOUT:
			PushandrideMidlet.getManager().openAbout();
			break;
		case ITEM_ID_CALL_RESET_SETTINGS:
			PushandrideMidlet.getManager().resetSettings();
			Manager.showMessage(Locale.Info, Locale.ResetSettings_Success, GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
			break;
		case ITEM_ID_CALL_FEEDBACK_SCHREIBEN:
			//new Feedback().setVisible(true);
			new SMSWithInput("Dein Feedback", Global.FEEDBACK_TELEPHON_NUMBER, "");
			break;
		case ITEM_ID_CALL_ADVICE_FRIEND:
			Displayable last = Display.getDisplay(Pushandride.getMidlet()).getCurrent();
			new AdviceFriend(last).setVisible(true);
			break;
		case ITEM_ID_TEST_IMAGESCALE:
			new TestImageScaler();
			break;
		case ITEM_ID_TEST_FONTS:
			new TestFonts();
			break;
		case ITEM_ID_TEST_VONNACH:
			new TestVonNach();
			break;
		case ITEM_ID_TEST_KEYS:
			new TestKeys();
			break;
		case ITEM_ID_CALL_SEND_PUSHNRIDE:
			try {
				new BluetoothChooseDeviceScreen();
			} catch (Exception e) {
				Manager.showMessage("Info", "Bitte Bluetooth aktvieren. Falls dieses bereits aktiv ist kann push&ride nicht versendet werden.", GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
			}
			break;
		case ITEM_ID_TEST_VIBRATE:
			if (Display.getDisplay(Pushandride.getMidlet()).vibrate(1000)) {
				Manager.showMessage(Locale.Info, "Das Handy sollte jetzt vibriert haben!", -1);
			} else {
				Manager.showMessage(Locale.Info, "Virbration wird vom Handy über JAVA nicht unterstützt!", -1);
			}
			break;
		case ITEM_ID_TEST_TIME:
			new TestTime();
			break;
		case ITEM_ID_TEST_DEVICE_INFOS:
			Manager.showMessage("Device-Info", Device.printDeviceInfo(), GuiMessageScreen.TYPE_MESSAGE_FULLSCREEN);
			break;
		default:
			Manager.showMessage("HINWEIS", "nicht implementiert", -1);
		}
	}

	private static boolean zeitChange = false;
	private static boolean laufgeschwindigkeitChange = false;

	public static void backFromOptions() {
		if (laufgeschwindigkeitChange) {
			RoutenCache.removeAllEntries();
		}
		if (laufgeschwindigkeitChange || zeitChange) {
			BgStartScreen.forceTimeInvalid();
		}
		laufgeschwindigkeitChange = false;
		zeitChange = false;
	}

	public static void postAction(ItemOption io) {
		switch (io.getId()) {
		case ITEM_ID_GFX_SOFTWAREKEYBOARD:
			if (((ItemOptionToggle) io).getSelectedIndex() == 1) {
				Device.setShowSoftwareKeyboard(true);
			} else {
				Device.setShowSoftwareKeyboard(false);
			}
			break;
		case ITEM_ID_GFX_QUALITY:
			int selectedIndex = ((ItemOptionToggle) io).getSelectedIndex();
			switch (selectedIndex) {
			case 0:
				Device.setGfxQuality(Device.GFX_QUALITY_LOW);
				break;
			case 1:
				Device.setGfxQuality(Device.GFX_QUALITY_MID);
				break;
			case 2:
				Device.setGfxQuality(Device.GFX_QUALITY_HIGH);
				break;
			default:
				Logger.error(OptionManager.class, "unbekannte Gfx-Quality");
			}
			break;
		case ITEM_ID_CHANGE_TIMEOFFSET:
			Device.setTimeOffset((((ItemOptionToggle) io).getSelectedIndex() - 12) * 60);
			PushandrideMidlet.getManager().getActiveGui().redrawCurrentTime();
			zeitChange = true;
			break;
		case ITEM_ID_CHANGE_LAUFZEIT_FAKTOR:
			laufgeschwindigkeitChange = true;
			break;
		case ITEM_ID_GFX_TOUCH_SIZE:
			if (((ItemOptionToggle) io).getSelectedIndex() == 1) {
				Device.setTouchSize(true);
			} else {
				Device.setTouchSize(false);
			}
			((GuiSelectionHome) Manager.guis[Manager.STATE_HOME]).calculateFreeHaltestellenPaarSlots();
			Manager.guis[Manager.STATE_HOME].dataInvalid();
			break;
		}
	}
}
