// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import gui.guis.Gui;
import gui.launch.GameCanvasInstance;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.Sprite;

public class Overlay {
	public static final int FROM_LEFT = 0;
	public static final int TO_RIGHT = 1;
	public static final int TO_LEFT = 2;
	public static final int SCROLLZEIT = 450; //ms
	//	public static final int SCROLLZEIT = 5000; //ms
	public static final int FPS_LIMIT = 33;
	public static final int STATE_FINISHED = 0;
	public static final int STATE_UNFINISHED = 1;

	private static int info_height = -1;
	private static int info_width = -1;
	private static int info_y = -1;
	private static int info_x = -1;
	private static double PI_H = Math.PI / 2.0;

	/**
	 * liefert die Scrollposition zurück (Wert zwischen 0..1) in Abhängigkeit
	 * von der Startzeit und der Aktuellen Zeit
	 * 
	 * @param startZeit
	 * @param aktuelleZeit
	 * @return
	 */
	private static double getPos(long startZeit, long aktuelleZeit, int direction) {
		int timeDiff = (int) (aktuelleZeit - startZeit);
		if (timeDiff > SCROLLZEIT)
			return 1;
		double tpos = (timeDiff / (double) SCROLLZEIT);

		double sinValue = 0;

		switch (direction) {
		case FROM_LEFT:
			double angle = PI_H * tpos;
			sinValue = Math.sin(angle);
			break;
		case TO_LEFT:
		case TO_RIGHT:
			double angle2 = PI_H + PI_H * tpos;
			sinValue = 1.0 - Math.sin(angle2);
			break;
		}

		return sinValue;

		// linear:
		//		return (aktuelleZeit - startZeit) / (double) SCROLLZEIT;
	}

	private static int getXAncor(double pos, int direction) {
		int x = 0;
		switch (direction) {
		case FROM_LEFT:
			x = (int) Math.floor(pos * (info_width + info_x)) - info_width;
			break;
		case TO_RIGHT:
			x = info_x + (int) Math.ceil(pos * (info_width + info_x));
			break;
		case TO_LEFT:
			x = info_x - (int) Math.ceil(pos * (info_width + info_x));
			break;
		}
		return x;
	}

	private static boolean first = true;
	private static long startZeit;
	private static long letzteZeit;
	private static int picCnt = 0;

	private static Image iMessage;
	private static Image iBackground;
	private static IOverlayable messageScreen;

	private static void reset() {
		first = true;
		letzteZeit = startZeit;
		picCnt = 0;
	}

	public static int move(Manager manager, Gui guiTo, int direction) {

		double pos = 0;

		Graphics gMain = GameCanvasInstance.returnGraphics();

		Image bufferTo = Fader.getBufferTo();
		Image bufferFrom = Fader.getBufferFrom();

		if (first == true) {
			first = false;
			switch (direction) {
			case FROM_LEFT:
				messageScreen = (IOverlayable) guiTo;
				iMessage = bufferTo;
				iBackground = bufferFrom;
				break;
			case TO_RIGHT:
			case TO_LEFT:
				messageScreen = (IOverlayable) manager.getActiveGui();
				iMessage = bufferFrom;
				iBackground = bufferTo;
			}

			GfxUtils.clearClip(bufferTo.getGraphics());
			guiTo.drawTo(bufferTo.getGraphics());

			// copy screen into buffer
			GfxUtils.clearClip(bufferFrom.getGraphics());
			//			manager.paint(imageBuffer.getGraphics());
			manager.getActiveGui().drawTo(bufferFrom.getGraphics());

			info_width = messageScreen.getInfoWidth();
			info_height = messageScreen.getInfoHeight();
			info_x = messageScreen.getInfoX();
			info_y = messageScreen.getInfoY();

			GfxUtils.clearClip(gMain);
			gMain.setClip(0, info_y, Manager.SCREEN_WIDTH, info_height);

			startZeit = System.currentTimeMillis() - (1000 / FPS_LIMIT) - 1;
		}

		long aktuelleZeit = System.currentTimeMillis();

		if ((aktuelleZeit - letzteZeit) < (1000 / FPS_LIMIT)) {
			try {
				Thread.sleep((1000 / FPS_LIMIT) - (aktuelleZeit - letzteZeit));
			} catch (InterruptedException e) {
			}
			aktuelleZeit = System.currentTimeMillis();
		}
		letzteZeit = aktuelleZeit;

		pos = getPos(startZeit, aktuelleZeit, direction);

		if (aktuelleZeit - startZeit < SCROLLZEIT) {
			int xAncor;
			xAncor = getXAncor(pos, direction);

			gMain.drawRegion(iBackground, 0, info_y, Manager.SCREEN_WIDTH, info_height, Sprite.TRANS_NONE, 0, info_y, Graphics.LEFT | Graphics.TOP);
			gMain.drawRegion(iMessage, info_x, info_y, info_width, info_height, Sprite.TRANS_NONE, xAncor, info_y, Graphics.LEFT | Graphics.TOP);
		}

		GameCanvasInstance.getInstance().flushGraphics();
		picCnt++;

		if (aktuelleZeit - startZeit >= SCROLLZEIT) {
			gMain.drawRegion(iBackground, 0, info_y, Manager.SCREEN_WIDTH, info_height, Sprite.TRANS_NONE, 0, info_y, Graphics.LEFT | Graphics.TOP);
			if (direction == FROM_LEFT) {
				gMain.drawRegion(iMessage, info_x, info_y, info_width, info_height, Sprite.TRANS_NONE, info_x, info_y, Graphics.LEFT | Graphics.TOP);
			}

			GameCanvasInstance.getInstance().flushGraphics();

			reset();

			return STATE_FINISHED;
		}
		return STATE_UNFINISHED;
	}

}
