// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import gui.guis.Gui;
import gui.guis.GuiSelectionLocation;
import gui.launch.GameCanvasInstance;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.Sprite;

import control.Device;

public class Slider {
	public static final int DIRECTION_LEFT = 0;
	public static final int DIRECTION_RIGHT = 1;
	public static final int DIRECTION_UP = 2;
	public static final int DIRECTION_DOWN = 3;
	public static final int SCROLLZEIT = 450; //ms
	//	public static final int SCROLLZEIT = 5000; //ms
	public static final int FPS_LIMIT = 33;
	public static final int STATE_FINISHED = 0;
	public static final int STATE_UNFINISHED = 1;

	/**
	 * liefert die Scrollposition zurück (Wert zwischen 0..1) in Abhängigkeit von der Startzeit und
	 * der Aktuellen Zeit
	 * 
	 * @param startZeit
	 * @param aktuelleZeit
	 * @return
	 */
	private static double getPos(long startZeit, long aktuelleZeit) {
		int timeDiff = (int) (aktuelleZeit - startZeit);
		if (timeDiff > SCROLLZEIT)
			return 1;
		double tpos = (timeDiff / (double) SCROLLZEIT);
		double angle = Math.PI * tpos;

		double cosValue = -Math.cos(angle);

		return (cosValue + 1) / 2.0;

		// linear:
		//		return (aktuelleZeit - startZeit) / (double) SCROLLZEIT;
	}

	private static int getXAncor(double pos, int direction) {
		if ((direction == DIRECTION_DOWN) || (direction == DIRECTION_UP)) {
			return 0;
		}
		int x;
		if (direction == DIRECTION_LEFT) {
			x = -(int) Math.floor(pos * Manager.SCREEN_WIDTH);
		} else {
			x = (int) Math.ceil(pos * Manager.SCREEN_WIDTH);
		}
		return x;
	}

	private static int getYAncor(double pos, int direction) {
		if ((direction == DIRECTION_LEFT) || (direction == DIRECTION_RIGHT)) {
			return 0;
		}
		int y;
		if (direction == DIRECTION_UP) {
			y = -(int) Math.floor(pos * Manager.SCREEN_HEIGHT);
		} else {
			y = (int) Math.ceil(pos * Manager.SCREEN_HEIGHT);
		}
		return y;
	}

	private static boolean first = true;
	private static long startZeit;
	private static long letzteZeit;
	private static int picCnt = 0;

	private static void reset() {
		first = true;
		letzteZeit = startZeit;
		picCnt = 0;
	}

	public static int move(Manager manager, Gui guiTo, int direction) {

		double pos = 0;

		Graphics gMain = GameCanvasInstance.returnGraphics();
		
		Image bufferTo = Fader.getBufferTo();
		Image bufferFrom = Fader.getBufferFrom();

		if (first == true) {
			//			System.out.println("start scrollling");
			first = false;
			startZeit = System.currentTimeMillis();

			GfxUtils.clearClip(gMain);
			gMain.setClip(0, 0, Manager.SCREEN_WIDTH, Manager.SCREEN_HEIGHT);

			GfxUtils.clearClip(bufferTo.getGraphics());
			guiTo.drawTo(bufferTo.getGraphics());

			// copy screen into buffer
			GfxUtils.clearClip(bufferFrom.getGraphics());
			//			manager.paint(imageBuffer.getGraphics());
			manager.getActiveGui().drawTo(bufferFrom.getGraphics());
		}

		long aktuelleZeit = System.currentTimeMillis();

		if ((aktuelleZeit - letzteZeit) < (1000 / FPS_LIMIT)) {
			try {
				Thread.sleep((1000 / FPS_LIMIT) - (aktuelleZeit - letzteZeit));
			} catch (InterruptedException e) {
			}
			aktuelleZeit = System.currentTimeMillis();
		}
		letzteZeit = aktuelleZeit;

		pos = getPos(startZeit, aktuelleZeit);

		if (aktuelleZeit - startZeit < SCROLLZEIT) {
			int xAncor;
			int yAncor;
			xAncor = getXAncor(pos, direction);
			yAncor = getYAncor(pos, direction);

			int height = Manager.SCREEN_HEIGHT - Manager.FOOTER_HEIGHT;
			if (Device.showSoftwareKeyboard()) {
				if ((manager.getActiveGui() instanceof GuiSelectionLocation) && (guiTo instanceof GuiSelectionLocation)) {
					height -= ((GuiSelectionLocation) guiTo).getKeyHeight();
				} else {
					if ((manager.getActiveGui() instanceof GuiSelectionLocation) || (guiTo instanceof GuiSelectionLocation)) {
						height += Manager.FOOTER_UPPER_SPACE;
					}
				}
			}
			if (direction == DIRECTION_LEFT) {
				gMain.drawRegion(bufferFrom, 0, 0, Manager.SCREEN_WIDTH + xAncor, height, Sprite.TRANS_NONE, Manager.SCREEN_WIDTH, 0, Graphics.TOP | Graphics.RIGHT);

				gMain.drawRegion(bufferTo, Manager.SCREEN_WIDTH + xAncor, 0, -xAncor, height, Sprite.TRANS_NONE, 0, 0, Graphics.TOP | Graphics.LEFT);
			}
			if (direction == DIRECTION_RIGHT) {
				gMain.drawRegion(bufferFrom, xAncor, 0, Manager.SCREEN_WIDTH - xAncor, height, Sprite.TRANS_NONE, 0, 0, Graphics.TOP | Graphics.LEFT);

				gMain.drawRegion(bufferTo, 0, 0, xAncor, height, Sprite.TRANS_NONE, Manager.SCREEN_WIDTH, 0, Graphics.TOP | Graphics.RIGHT);
			}
			if (direction == DIRECTION_UP) {
				gMain.drawRegion(bufferFrom, 0, 0, Manager.SCREEN_WIDTH, Manager.SCREEN_HEIGHT + yAncor, Sprite.TRANS_NONE, 0, Manager.SCREEN_HEIGHT, Graphics.BOTTOM | Graphics.LEFT);

				gMain.drawRegion(bufferTo, 0, Manager.SCREEN_HEIGHT + yAncor, Manager.SCREEN_WIDTH, -yAncor, Sprite.TRANS_NONE, 0, 0, Graphics.TOP | Graphics.LEFT);
			}
			if (direction == DIRECTION_DOWN) {
				gMain.drawRegion(bufferFrom, 0, yAncor, Manager.SCREEN_WIDTH, Manager.SCREEN_HEIGHT - yAncor, Sprite.TRANS_NONE, 0, 0, Graphics.TOP | Graphics.LEFT);

				gMain.drawRegion(bufferTo, 0, 0, Manager.SCREEN_WIDTH, yAncor, Sprite.TRANS_NONE, 0, Manager.SCREEN_HEIGHT, Graphics.BOTTOM | Graphics.LEFT);
			}
		}

		GameCanvasInstance.getInstance().flushGraphics();
		picCnt++;

		if (aktuelleZeit - startZeit >= SCROLLZEIT) {
			gMain.drawImage(bufferTo, 0, 0, Graphics.TOP | Graphics.LEFT);

			//			long gesamtZeit = System.currentTimeMillis() - startZeit;
			//			double fps = picCnt / (double) gesamtZeit * 1000.0;

			GameCanvasInstance.getInstance().flushGraphics();

			reset();

			return STATE_FINISHED;
		}
		return STATE_UNFINISHED;
	}

}
