// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import gui.guis.Gui;
import gui.guis.GuiSelectionLocation;
import gui.launch.GameCanvasInstance;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.Sprite;


public class SplitToHaltestellen {
	public static final int DIRECTION_LEFT = 0;
	public static final int DIRECTION_RIGHT = 1;
	public static final int DIRECTION_TO_HALTESTELLE = 2;
	public static final int DIRECTION_TO_ORT = 3;
	public static final int SCROLLZEIT = 400; //ms
	//	public static final int SCROLLZEIT = 3000; //ms
	public static final int FPS_LIMIT = 33;
	public static final int STATE_FINISHED = 0;
	public static final int STATE_UNFINISHED = 1;

	// Bild das den Überlapp vom Ortscreen für den Haltescreen speichert
	private static Image header1;
	private static Image header2;

	/**
	 * liefer den Überlapp vom Ortsscreen zurück;
	 */
	public static Image getHeader1() {
		return header1;
	}

	public static Image getHeader2() {
		return header2;
	}

	/**
	 * liefert die Scrollposition zurück (Wert zwischen 0..1) in Abhängigkeit von der Startzeit und
	 * der Aktuellen Zeit
	 * 
	 * @param startZeit
	 * @param aktuelleZeit
	 * @return
	 */
	private static double getPos(long startZeit, long aktuelleZeit) {
		int timeDiff = (int) (aktuelleZeit - startZeit);
		if (timeDiff > SCROLLZEIT)
			return 1;
		double tpos = (timeDiff / (double) SCROLLZEIT);
		double angle = Math.PI * tpos;

		double cosValue = -Math.cos(angle);

		return (cosValue + 1) / 2.0;

		// linear:
		//		return (aktuelleZeit - startZeit) / (double) SCROLLZEIT;
	}

	private static boolean first = true;
	private static long startZeit;
	private static long letzteZeit;
	private static int picCnt = 0;

	private static int posSelectedBase;
	private static int posMainY;
	private static int sizeHeader;
	private static int sizeBottom;
	private static int sizeSelectedItem;
	private static int topDifferenz;
	private static int bottomDifferenz;

	private static void reset() {
		first = true;
		letzteZeit = startZeit;
		picCnt = 0;
	}

	public static int move(Manager manager, Gui guiTo, int direction) {

		double pos = 0;

		Graphics gMain = GameCanvasInstance.returnGraphics();
		
		Image bufferTo = Fader.getBufferTo();
		Image bufferFrom = Fader.getBufferFrom();

		GuiSelectionLocation ortScreen;
		if (direction == DIRECTION_TO_HALTESTELLE) {
			ortScreen = (GuiSelectionLocation) manager.getActiveGui();
		} else {
			ortScreen = (GuiSelectionLocation) guiTo;
		}

		if (first == true) {

			first = false;
			startZeit = System.currentTimeMillis();

			GfxUtils.clearClip(gMain);
			gMain.setClip(0, 0, Manager.SCREEN_WIDTH, Manager.SCREEN_HEIGHT);

			GfxUtils.clearClip(bufferTo.getGraphics());
			guiTo.drawTo(bufferTo.getGraphics());

			GfxUtils.clearClip(bufferFrom.getGraphics());
			manager.getActiveGui().drawTo(bufferFrom.getGraphics());

			// FIXME Bad hack....
			if (direction == DIRECTION_TO_HALTESTELLE) {
				posMainY = ortScreen.getMainY();
				sizeHeader = ortScreen.getHeaderHeight();
				sizeSelectedItem = ortScreen.getItemHeight();
				sizeBottom = Manager.FOOTER_HEIGHT + ortScreen.getKeyHeight();
				posSelectedBase = ortScreen.getLineBeneathSelected();
				topDifferenz = posSelectedBase - sizeHeader;
				bottomDifferenz = Manager.SCREEN_HEIGHT - sizeBottom - posSelectedBase;
				int topStatic = ((posMainY + sizeSelectedItem) - sizeHeader);
				if (ortScreen.getTyp() == GuiSelectionLocation.TYPE_VON_ORT) {
					header1 = Image.createImage(bufferFrom, 0, posSelectedBase - topStatic, Manager.SCREEN_WIDTH, topStatic, 0);
				} else {
					header2 = Image.createImage(bufferFrom, 0, posSelectedBase - topStatic, Manager.SCREEN_WIDTH, topStatic, 0);
				}
			} else {
				posMainY = ortScreen.getMainY();
				sizeHeader = ortScreen.getHeaderHeight();
				sizeSelectedItem = ortScreen.getItemHeight();
				sizeBottom = Manager.FOOTER_HEIGHT  + ortScreen.getKeyHeight();
				posSelectedBase = ortScreen.getLineBeneathSelected();
				topDifferenz = posSelectedBase - sizeHeader;
				bottomDifferenz = Manager.SCREEN_HEIGHT - sizeBottom - posSelectedBase;
			}

			startZeit = System.currentTimeMillis() - (1000 / FPS_LIMIT) - 1;

		}

		long aktuelleZeit = System.currentTimeMillis();

		if ((aktuelleZeit - letzteZeit) < (1000 / FPS_LIMIT)) {
			try {
				Thread.sleep((1000 / FPS_LIMIT) - (aktuelleZeit - letzteZeit));
			} catch (InterruptedException e) {
			}
			aktuelleZeit = System.currentTimeMillis();
		}
		letzteZeit = aktuelleZeit;

		int x_src, y_src, width, height;
		int x_dest, y_dest;

		pos = getPos(startZeit, aktuelleZeit);

		if (aktuelleZeit - startZeit < SCROLLZEIT) {
			if (direction == DIRECTION_TO_HALTESTELLE) {
				pos = 1 - pos;
			} else {
				//pos = pos;
			}

			int topStatic = ((posMainY + sizeSelectedItem) - sizeHeader);
			int topMoveHeight = (int) ((topDifferenz - topStatic) * pos);
			int bottomHeight = (int) (bottomDifferenz * pos);

			if (direction == DIRECTION_TO_HALTESTELLE) {
				x_src = x_dest = 0;
				width = Manager.SCREEN_WIDTH;

				y_src = sizeHeader + topDifferenz - topMoveHeight - topStatic;
				height = topMoveHeight + topStatic;
				y_dest = sizeHeader;

				//				System.out.println("1 - y_src: " + y_src + "  height: " + height + "  y_dest:" + y_dest);

				gMain.drawRegion(bufferFrom, x_src, y_src, width, height, Sprite.TRANS_NONE, x_dest, y_dest, Graphics.TOP | Graphics.LEFT);

				y_src = y_dest + height;
				height = topDifferenz + bottomDifferenz - topMoveHeight - topStatic - bottomHeight;
				y_dest = y_src;

				//				System.out.println("   2 - y_src: " + y_src + "  height: " + height + "  y_dest:" + y_dest);
				gMain.drawRegion(bufferTo, x_src, y_src, width, height, Sprite.TRANS_NONE, x_dest, y_dest, Graphics.TOP | Graphics.LEFT);

				y_src = Manager.SCREEN_HEIGHT - sizeBottom - bottomDifferenz;
				height = bottomHeight;
				y_dest = Manager.SCREEN_HEIGHT - sizeBottom - bottomHeight;

				//				System.out.println("      3 - y_src: " + y_src + "  height: " + height + "  y_dest:" + y_dest);
				gMain.drawRegion(bufferFrom, x_src, y_src, width, height, Sprite.TRANS_NONE, x_dest, y_dest, Graphics.TOP | Graphics.LEFT);
			}
			if (direction == DIRECTION_TO_ORT) {
				x_src = x_dest = 0;
				width = Manager.SCREEN_WIDTH;

				y_src = sizeHeader + topDifferenz - topMoveHeight - topStatic;
				height = topMoveHeight + topStatic;
				y_dest = sizeHeader;

				//				System.out.println("1 - y_src: " + y_src + "  height: " + height + "  y_dest:" + y_dest);

				gMain.drawRegion(bufferTo, x_src, y_src, width, height, Sprite.TRANS_NONE, x_dest, y_dest, Graphics.TOP | Graphics.LEFT);

				y_src = y_dest + height;
				height = topDifferenz + bottomDifferenz - topMoveHeight - topStatic - bottomHeight;
				y_dest = y_src;

				//				System.out.println("   2 - y_src: " + y_src + "  height: " + height + "  y_dest:" + y_dest);
				gMain.drawRegion(bufferFrom, x_src, y_src, width, height, Sprite.TRANS_NONE, x_dest, y_dest, Graphics.TOP | Graphics.LEFT);

				y_src = Manager.SCREEN_HEIGHT - sizeBottom - bottomDifferenz;
				height = bottomHeight;
				y_dest = Manager.SCREEN_HEIGHT - sizeBottom - bottomHeight;

				//				System.out.println("      3 - y_src: " + y_src + "  height: " + height + "  y_dest:" + y_dest);
				gMain.drawRegion(bufferTo, x_src, y_src, width, height, Sprite.TRANS_NONE, x_dest, y_dest, Graphics.TOP | Graphics.LEFT);
			}
		}

		GameCanvasInstance.getInstance().flushGraphics();
		picCnt++;

		if (aktuelleZeit - startZeit >= SCROLLZEIT) {
			gMain.drawImage(bufferTo, 0, 0, Graphics.TOP | Graphics.LEFT);
			if (direction == DIRECTION_TO_HALTESTELLE) {
				if (ortScreen.getTyp() == GuiSelectionLocation.TYPE_VON_ORT) {
					gMain.drawImage(header1, 0, sizeHeader, 0);
				} else {
					gMain.drawImage(header2, 0, sizeHeader, 0);
				}
			}

			//			long gesamtZeit = System.currentTimeMillis() - startZeit;
			//			double fps = picCnt / (double) gesamtZeit * 1000.0;

			GameCanvasInstance.getInstance().flushGraphics();

			reset();

			return STATE_FINISHED;
		}
		return STATE_UNFINISHED;
	}

}
