// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui;

import gui.guis.Gui;
import gui.launch.GameCanvasInstance;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.Sprite;


public class Splitter {
	public static final int DIRECTION_MOVEIN = 0;
	public static final int DIRECTION_MOVEOUT = 1;
	public static final int SCROLLZEIT = 500; //ms

	private static final double SCROLL_A = 1.0 / (double) (SCROLLZEIT * SCROLLZEIT);

	public static final int FPS_LIMIT = 33;
	public static final int STATE_FINISHED = 0;
	public static final int STATE_UNFINISHED = 1;

	/**
	 * liefert die Scrollposition zurück (Wert zwischen 0..1) in Abhängigkeit von der Startzeit und
	 * der Aktuellen Zeit
	 * 
	 * @param startZeit
	 * @param aktuelleZeit
	 * @return
	 */
	private static double getPos(long startZeit, long aktuelleZeit, int direction) {
		int timeDiff = (int) (aktuelleZeit - startZeit);
		if (timeDiff > SCROLLZEIT)
			return 1;

		if (direction == DIRECTION_MOVEIN) {
			timeDiff = SCROLLZEIT - timeDiff;
		}

		double x = timeDiff;
		double y;

		y = SCROLL_A * x * x;

		return y;
		// linear:
		//		return (timeDiff) / (double) SCROLLZEIT;
	}

	private static boolean first = true;
	private static long startZeit;
	private static long letzteZeit;
	private static int picCnt = 0;

	/**
	 * initialisiert die Bildpuffer
	 */
	private static void reset() {
		first = true;
		letzteZeit = startZeit;
		picCnt = 0;
	}

	public static int split(Manager manager, Gui guiTo, int direction) {

		double pos = 0;

		Graphics gMain = GameCanvasInstance.returnGraphics();
		
		Image bufferTo = Fader.getBufferTo();
		Image bufferFrom = Fader.getBufferFrom();

		if (first == true) {
			//			System.out.println("start scrollling");
			first = false;

			GfxUtils.clearClip(gMain);
			gMain.setClip(0, 0, Manager.SCREEN_WIDTH, Manager.SCREEN_HEIGHT);

			GfxUtils.clearClip(bufferTo.getGraphics());
			guiTo.drawTo(bufferTo.getGraphics());

			// copy screen into buffer
			GfxUtils.clearClip(bufferFrom.getGraphics());
			//			manager.paint(imageBuffer.getGraphics());
			manager.getActiveGui().drawTo(bufferFrom.getGraphics());

			startZeit = System.currentTimeMillis() - (1000 / FPS_LIMIT) - 1;
		}

		long aktuelleZeit = System.currentTimeMillis();

		if ((aktuelleZeit - letzteZeit) < (1000 / FPS_LIMIT)) {
			try {
				Thread.sleep((1000 / FPS_LIMIT) - (aktuelleZeit - letzteZeit));
			} catch (InterruptedException e) {
			}
			aktuelleZeit = System.currentTimeMillis();
		}
		letzteZeit = aktuelleZeit;

		pos = getPos(startZeit, aktuelleZeit, direction);

		if (aktuelleZeit - startZeit < SCROLLZEIT) {

			Image upBuffer;
			Image downBuffer;

			if (direction == DIRECTION_MOVEIN) {
				downBuffer = bufferFrom;
				upBuffer = bufferTo;
			} else {
				downBuffer = bufferTo;
				upBuffer = bufferFrom;
			}
			int width_above = (int) (Manager.SCREEN_WIDTH * (1.0 - pos) / 2.0);
			int height_above = (int) (Manager.SCREEN_HEIGHT * (1.0 - pos) / 2.0);
			int midX = Manager.SCREEN_WIDTH / 2;
			int midY = Manager.SCREEN_HEIGHT / 2;

			// Quadranten
			//
			// 1 | 2
			// -----
			// 3 | 4

			// unteres Bild
			gMain.drawRegion(downBuffer, 0, 0, Manager.SCREEN_WIDTH, Manager.SCREEN_HEIGHT, Sprite.TRANS_NONE, 0, 0, Graphics.TOP | Graphics.LEFT);
			// oberes Bild
			// Quadrant 1
			gMain.drawRegion(upBuffer, midX - width_above, midY - height_above, width_above, height_above, Sprite.TRANS_NONE, 0, 0, Graphics.TOP | Graphics.LEFT);
			// Quadrant 2
			gMain.drawRegion(upBuffer, midX, midY - height_above, width_above, height_above, Sprite.TRANS_NONE, Manager.SCREEN_WIDTH - width_above, 0, Graphics.TOP | Graphics.LEFT);
			// Quadrant 3
			gMain.drawRegion(upBuffer, midX - width_above, midY, width_above, height_above, Sprite.TRANS_NONE, 0, Manager.SCREEN_HEIGHT - height_above, Graphics.TOP | Graphics.LEFT);
			// Quadrant 4
			gMain.drawRegion(upBuffer, midX, midY, width_above, height_above, Sprite.TRANS_NONE, Manager.SCREEN_WIDTH - width_above, Manager.SCREEN_HEIGHT - height_above, Graphics.TOP | Graphics.LEFT);

			int LINEWIDTH = Math.max(1, (int) (6 * Math.sqrt(pos) + 1.5));
			for (int lineno = 0; lineno < LINEWIDTH; lineno++) {
				double colorPos;
				if (LINEWIDTH <= 1) {
					colorPos = 1;
				} else {
					colorPos = ((double) lineno) * (1.0 / (double) (LINEWIDTH - 1));
				}
				//				gMain.setColor(Color.mixColor(Color.gray2color(200), Color.gray2color(80), colorPos * colorPos));
				//				System.out.println("pos " + pos);
				//				System.out.println("colorPos " + colorPos);
				int ces = 210 + (int) ((1 - pos) * 35.0);
				int cev = 50 + (int) ((1 - pos) * 205.0);
				//				System.out.println("ColorStartValue " + ces);
				//				System.out.println("ColorEndValue " + cev);
				//				System.out.println();

				gMain.setColor(Color.mixColor(Color.gray2color(ces), Color.gray2color(cev), colorPos * colorPos));

				// Quadrant 1
				if (width_above + lineno < midX) {
					// senkrecht 
					gMain.drawLine(width_above + lineno, 0, width_above + lineno, height_above + lineno);
				}

				if (height_above + lineno < midY) {
					// waagerecht
					gMain.drawLine(0, height_above + lineno, width_above + lineno, height_above + lineno);
				}
				// Quadrant 2
				if (width_above + lineno < midX) {
					// senkrecht 
					gMain.drawLine(Manager.SCREEN_WIDTH - (width_above + lineno + 1), 0, Manager.SCREEN_WIDTH - (width_above + lineno + 1), height_above + lineno);
				}
				if (height_above + lineno < midY) {
					// waagerecht
					gMain.drawLine(Manager.SCREEN_WIDTH - 0, height_above + lineno, Manager.SCREEN_WIDTH - (width_above + lineno), height_above + lineno);
				}

				if (width_above + lineno < midX) {
					// Quadrant 3
					// senkrecht 
					gMain.drawLine(width_above + lineno, Manager.SCREEN_HEIGHT - (height_above + lineno), width_above + lineno, Manager.SCREEN_HEIGHT);
				}
				if (height_above + lineno < midY) {
					// waagerecht
					gMain.drawLine(0, Manager.SCREEN_HEIGHT - (height_above + lineno + 1), width_above + lineno, Manager.SCREEN_HEIGHT - (height_above + lineno + 1));
				}
				// Quadrant 4
				if (width_above + lineno < midX) {
					// senkrecht 
					gMain.drawLine(Manager.SCREEN_WIDTH - (width_above + lineno + 1), Manager.SCREEN_HEIGHT - (height_above + lineno + 1), Manager.SCREEN_WIDTH - (width_above + lineno + 1), Manager.SCREEN_HEIGHT);
				}
				if (height_above + lineno < midY) {
					// waagerecht
					gMain.drawLine(Manager.SCREEN_WIDTH - 0, Manager.SCREEN_HEIGHT - (height_above + lineno + 1), Manager.SCREEN_WIDTH - (width_above + lineno + 1), Manager.SCREEN_HEIGHT - (height_above + lineno + 1));
				}
			}

		}

		GameCanvasInstance.getInstance().flushGraphics();
		picCnt++;

		if (aktuelleZeit - startZeit >= SCROLLZEIT) {
			gMain.drawImage(bufferTo, 0, 0, Graphics.TOP | Graphics.LEFT);

			//			long gesamtZeit = System.currentTimeMillis() - startZeit;
			//			double fps = picCnt / (double) gesamtZeit * 1000.0;

			GameCanvasInstance.getInstance().flushGraphics();

			reset();

			return STATE_FINISHED;
		}
		return STATE_UNFINISHED;
	}

}
