// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.elements;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.Point;
import gui.utils.GfxStringUtils;
import gui.utils.GfxUtils;
import gui.utils.ImageScaler;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import model.AbstractLocation;
import model.Haltestelle;
import control.Device;

/**
 * Von/Nach-Anzeige im Übersicht/Haltestellenscreen
 */
public class ElementVonNach extends GuiElement {

	private AbstractLocation von = null;
	private AbstractLocation nach = null;

	private Point coordinateVon = null;
	private Point coordinateNach = null;

	private boolean isDrawn = false;

	private static final int textHeight = Manager.HEADER_BASE_UNIT;

	public ElementVonNach() {
	}

	private static final int VON_LEFT_OFFSET = GfxUtils.scale(Manager.SCREEN_WIDTH, 2);
	private static final int NACH_LEFT_OFFSET = GfxUtils.scale(Manager.SCREEN_WIDTH, 2);

	//	private static final int[] BG_COLORS = { Color.COLOR_BG_ACTIVE, Color.gray2color(128), Color.COLOR_BG_FINAL };
	// FIXME: cleanup old framework
	protected static final int BG_COLOR = Color.COLOR_BG_ACTIVE;

	protected void drawHQBackground(Graphics g, int x, int y) {
		g.setColor(Color.WHITE);
		GfxUtils.fillFullRectangle(g, y, getHeight());
	}
	
	public void drawTo(Graphics g, int x, int y) {
		if (Device.getGfxQuality() != Device.GFX_QUALITY_LOW) {
			drawHQBackground(g, x, y);

			// loading Images
			Image imageVon = getVonImage();
			Image imageNach = getNachImage();

			// headSpace
			y += Manager.HEADER_TOP_SPACE;

			// Von
			x = VON_LEFT_OFFSET;

			g.drawImage(imageVon, x, y, Graphics.LEFT | Graphics.TOP);

			x += imageVon.getWidth();

			coordinateVon = new Point(x, y);

			y += textHeight;

			// MidSpace
			y += Manager.HEADER_MID_SPACE;

			// Nach
			x = NACH_LEFT_OFFSET;
			g.drawImage(imageNach, x, y, Graphics.LEFT | Graphics.TOP);
			x += imageNach.getWidth();
			coordinateNach = new Point(x, y);
			y += textHeight;
		} else {
			g.setColor(Color.WHITE);
			GfxUtils.fillFullRectangle(g, y, getHeight());
			Font fb = FontMetrics.getFont(Font.STYLE_BOLD, Manager.ITEMS_FONT_SIZE);
			g.setFont(fb);

			// headSpace
			y += Manager.HEADER_TOP_SPACE;

			// Von
			x = VON_LEFT_OFFSET;

			g.setColor(Color.BLACK);

			g.drawString("VON", x, y, Graphics.LEFT | Graphics.TOP);

			x += fb.stringWidth("VON") + VON_LEFT_OFFSET;
			;

			coordinateVon = new Point(x, y);

			y += textHeight;

			// MidSpace
			y += Manager.HEADER_MID_SPACE;

			// Nach
			x = NACH_LEFT_OFFSET;

			g.setColor(Color.BLACK);

			g.drawString("NACH", x, y, Graphics.LEFT | Graphics.TOP);

			x += fb.stringWidth("NACH") + VON_LEFT_OFFSET;
			coordinateNach = new Point(x, y);
		}

		isDrawn = true;

		drawVonString(g);
		drawNachString(g);

	}

	protected int getVonMaxWidth() {
		return Manager.SCREEN_WIDTH - coordinateVon.X;
	}

	protected int getNachMaxWidth() {
		return Manager.SCREEN_WIDTH - coordinateNach.X;
	}

	protected void drawVonString(Graphics g) {
		if (isDrawn) {
			g.setColor(BG_COLOR);
			GfxUtils.fillFullRectangleFromX(g, coordinateVon.X, coordinateVon.Y, textHeight);

			String text = "";
			Font f;
			f = FontMetrics.getFont(Font.STYLE_PLAIN, Manager.HEADER_FONT_SIZE);
			g.setFont(f);
			g.setColor(Color.WHITE);

			text = GfxStringUtils.getAdjustedLocation(f, von, getVonMaxWidth());

			GfxUtils.drawText(g, text, coordinateVon.X, coordinateVon.Y);
		}
	}

	protected void drawNachString(Graphics g) {
		if (isDrawn) {
			g.setColor(BG_COLOR);
			GfxUtils.fillFullRectangleFromX(g, coordinateNach.X, coordinateNach.Y, textHeight);

			String text = "";
			Font f;
			f = FontMetrics.getFont(Font.STYLE_PLAIN, Manager.HEADER_FONT_SIZE);
			g.setFont(f);
			g.setColor(Color.WHITE);

			text = GfxStringUtils.getAdjustedLocation(f, nach, getNachMaxWidth());

			GfxUtils.drawText(g, text, coordinateNach.X, coordinateNach.Y);

		}
	}

	public int getHeight() {
		return Manager.getHeaderHeight();
	}

	public void setCurrentText() {

	}

	private static Image vonImage = null;
	private static Image nachImage = null;

	protected Image getVonImage() {
		if (vonImage == null) {
			vonImage = ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/von_act.png"), textHeight);
		}
		return vonImage;
	}

	protected Image getNachImage() {
		if (nachImage == null) {
			nachImage = ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/nach_final_plain.png"), textHeight);
		}
		return nachImage;
	}
	
	protected int getTextHeight() {
		return textHeight;
	}
	
	protected Point getCoordinateVon() {
		return coordinateVon;
	}

	protected Point getCoordinateNach() {
		return coordinateNach;
	}
	
	protected boolean isDrawn() {
		return isDrawn;
	}
	
	protected AbstractLocation getVon() {
		return von;
	}

	protected AbstractLocation getNach() {
		return nach;
	}
	
	protected void setVon(AbstractLocation von) {
		this.von = von;
	}

	protected void setNach(AbstractLocation nach) {
		this.nach = nach;
	}

	public void setLocations(Haltestelle von, Haltestelle nach) {
		this.von = von;
		this.nach = nach;
	}
}
