// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.elements;

import gui.Color;
import gui.Cursor;
import gui.FontMetrics;
import gui.Imager;
import gui.Manager;
import gui.Point;
import gui.guis.GuiSelectionLocation;
import gui.items.ItemLocation;
import gui.utils.GfxStringUtils;
import gui.utils.GfxUtils;
import gui.utils.ImageScaler;
import gui.utils.IntelligentKey;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import model.AbstractLocation;
import model.Haltestelle;
import model.Ort;

public class ElementVonNachDynamic extends ElementVonNach {

	private static final int TYPE_ACTIVE = 0;
	private static final int TYPE_DEACTIVATED = 1;

	private static final int FIELD_VON = 0;
	private static final int FIELD_NACH = 1;

	private final int typeVon;
	private final int typeNach;
	private final int activeField;
	
	private boolean firstAfterNewItems;

	private String selectedText = "";

	ItemLocation selectedLocationItem;

	private GuiSelectionLocation parentGui;

	private int cursorPosX = 0;

	private boolean intelligentKeyIsInUse = false;
	private String alphaKey = null;

	public ElementVonNachDynamic(GuiSelectionLocation parentGui) {
		super();
		this.parentGui = parentGui;
		int guiType = parentGui.getTyp();
		switch (guiType) {
		case GuiSelectionLocation.TYPE_VON_ORT:
		case GuiSelectionLocation.TYPE_VON_HALTESTELLE:
			typeVon = TYPE_ACTIVE;
			typeNach = TYPE_DEACTIVATED;
			activeField = FIELD_VON;
			break;
		case GuiSelectionLocation.TYPE_NACH_ORT:
		case GuiSelectionLocation.TYPE_NACH_HALTESTELLE:
			typeNach = TYPE_ACTIVE;
			typeVon = TYPE_DEACTIVATED;
			activeField = FIELD_NACH;
			break;
		default:
			typeNach = -1;
			typeVon = -1;
			activeField = -1;
		}
		
		firstAfterNewItems = true;

	}

	private final static Image[] vonImages = new Image[2];
	private final static Image[] nachImages = new Image[2];

	protected Image getVonImage() {
		if (vonImages[typeVon] == null) {
			if (typeVon == TYPE_ACTIVE) {
				vonImages[typeVon] = super.getVonImage();
			} else {
				vonImages[typeVon] = ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/von_deact.png"), getTextHeight());
			}
		}
		return vonImages[typeVon];
	}

	protected Image getNachImage() {
		if (nachImages[typeNach] == null) {
			if ((typeNach == TYPE_ACTIVE)) {
				nachImages[typeNach] = ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/nach_act.png"), getTextHeight());
			} else {
				nachImages[typeNach] = ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/nach_deact.png"), getTextHeight());
			}
		}
		return nachImages[typeNach];
	}

	public void newSelectedItem(Graphics g, ItemLocation selectedItem, IntelligentKey intelligentKey, boolean repaint, boolean firstAfterNewItems, String alphaKey) {
		this.firstAfterNewItems  = firstAfterNewItems;
		this.alphaKey = alphaKey;

		selectedLocationItem = selectedItem;

		setActiveItem(intelligentKey);

		drawActiveItem(g, intelligentKey);

		if (repaint && isDrawn()) {
			repaintActiveItem();
		}
	}

	private void drawActiveItem(Graphics g, IntelligentKey intelligentKey) {
		if (!isDrawn()) {
			return;
		}
		switch (activeField) {
		case FIELD_VON:
			drawVonString(g);
			break;
		case FIELD_NACH:
			drawNachString(g);
			break;
		}
	}

	private void setActiveItem(IntelligentKey intelligentKey) {
		Font f;
		f = FontMetrics.getFont(Font.STYLE_PLAIN, Manager.HEADER_FONT_SIZE);
		Point dest = null;

		// FIXME: eigentlich stimmt das nicht - falls drawTo noch nicht aufgerufen wurde denkt er er hätte mehr Platz
		int maxWidth = Manager.SCREEN_WIDTH;

		switch (activeField) {
		case FIELD_VON:
			dest = getCoordinateVon();
			break;
		case FIELD_NACH:
			dest = getCoordinateNach();
			break;
		}

		if (dest != null) {
			maxWidth = Manager.SCREEN_WIDTH - dest.X;
		}

		String text = "";

		if (intelligentKey != null && intelligentKey.isInUse()) {
			intelligentKeyIsInUse = true;
		} else {
			intelligentKeyIsInUse = false;
		}

		if (selectedLocationItem == null) {
			if ((intelligentKey != null) && intelligentKey.isInUse()) {
				// keine Haltestelle auswählbar ==> kein gülter IntelligentKey String
				text = intelligentKey.getIntelligentKeyHeaderString();
				
				// hack for ALPHAKEYS
				if (alphaKey != null) {
					text = alphaKey;
				}

			} else {
				text = "";
			}
		} else {
			// Normalfall:
			AbstractLocation selectedLocation = selectedLocationItem.getLocation();

			if ((intelligentKey != null) && intelligentKey.isInUse()) {
				int intelligentKeyMatchPos = selectedLocationItem.getIntelligentKeyMatchPos();
				int intelligentKeyLength = intelligentKey.getKeysLength();

				if (selectedLocation instanceof Ort) {
					String ortsName = ((Ort) selectedLocation).getScreenName();
					if (intelligentKeyMatchPos >= 0) {
						text = ortsName.substring(intelligentKeyMatchPos, intelligentKeyMatchPos + intelligentKeyLength);
					} else {
						// kein Match von IntelligentKey auf Ortsname
						text = ortsName;
					}
				} else {
					// selectedLocation instanceof Haltestelle
					if (intelligentKeyMatchPos >= 0) {
						text = ((Haltestelle) selectedLocation).getScreenName().substring(intelligentKeyMatchPos, intelligentKeyMatchPos + intelligentKeyLength);
					} else {
						text = GfxStringUtils.getAdjustedLocation(f, selectedLocation, maxWidth);
					}
				}
			} else {
				if (parentGui.getTyp() == GuiSelectionLocation.TYPE_NACH_HALTESTELLE) {
					Ort ort = ((Haltestelle) selectedLocation).getOrt();
					text = ort.getScreenName();
				} else {
					text = GfxStringUtils.getAdjustedLocation(f, selectedLocation, maxWidth);
				}
			}
		}
		selectedText = text;
	}

	private void repaintActiveItem() {
		int x = 0;
		int y = 0;
		if (activeField == FIELD_VON) {
			x = getCoordinateVon().X;
			y = getCoordinateVon().Y;
		}
		if (activeField == FIELD_NACH) {
			x = getCoordinateNach().X;
			y = getCoordinateNach().Y;
		}
		if (parentGui.isVisible()) {
			((GuiSelectionLocation) parentGui).getGameCanvas().repaint(x, y, Manager.SCREEN_WIDTH - x, getTextHeight());
		}
	}

	public void setLocations(AbstractLocation von, AbstractLocation nach) {
		setVon(von);
		setNach(nach);
	}

	// FIXME mit drawNachString zusammenfassen
	protected void drawVonString(Graphics g) {
		if (isDrawn()) {
			g.setColor(BG_COLOR);

			Point base = getCoordinateVon();
			GfxUtils.fillFullRectangleFromX(g, base.X, base.Y, getTextHeight());

			String text = "";
			Font f;
			f = FontMetrics.getFont(Font.STYLE_PLAIN, Manager.HEADER_FONT_SIZE);
			g.setFont(f);
			g.setColor(Color.WHITE);

			if (activeField == FIELD_VON) {
				if (firstAfterNewItems) {
					g.setColor(Color.gray2color(180));
				}
				text = selectedText;
				if (intelligentKeyIsInUse) {
					cursorPosX = base.X + f.stringWidth(text);
				} else {
					cursorPosX = base.X;
				}
			} else {
				if (getVon() != null) {
					text = GfxStringUtils.getAdjustedLocation(f, getVon(), getVonMaxWidth());
				}
			}
			GfxUtils.drawText(g, text, base.X, base.Y);
		}
	}

	protected void drawNachString(Graphics g) {
		if (isDrawn()) {
			g.setColor(BG_COLOR);

			Point base = getCoordinateNach();

			GfxUtils.fillFullRectangleFromX(g, base.X, base.Y, getTextHeight());

			String text = "";
			Font f;
			f = FontMetrics.getFont(Font.STYLE_PLAIN, Manager.HEADER_FONT_SIZE);
			g.setFont(f);
			g.setColor(Color.WHITE);

			if (activeField == FIELD_NACH) {
				if (firstAfterNewItems) {
					g.setColor(Color.gray2color(180));
				}
				text = selectedText;
				if (intelligentKeyIsInUse) {
					cursorPosX = base.X + f.stringWidth(text);
				} else {
					cursorPosX = base.X;
				}
			} else {
				if (getNach() != null) {
					text = GfxStringUtils.getAdjustedLocation(f, getNach(), getNachMaxWidth());
				}
			}
			GfxUtils.drawText(g, text, base.X, base.Y);
		}
	}

	protected void drawHQBackground(Graphics g, int x, int y) {
		g.drawImage(Imager.getHeaderBackgroundImage(), 0, y, Graphics.LEFT | Graphics.TOP);
	}

	public void updateCursor(Cursor cursor, Graphics g) {
		if (isDrawn()) {
			if ((cursor != null) && (cursor.isVisible())) {
				int cursorYPos;

				if (activeField == FIELD_VON) {
					cursorYPos = getCoordinateVon().Y;
				} else {
					cursorYPos = getCoordinateNach().Y;
				}

				cursor.drawTo(new Point(cursorPosX, cursorYPos), g);
			} else {
				if (activeField == FIELD_VON) {
					drawVonString(g);
				} else {
					drawNachString(g);
				}
			}
			parentGui.getGameCanvas().repaint(0, 0, Manager.SCREEN_WIDTH, parentGui.getHeaderHeight());
		}
	}

	public void drawTo(Graphics g, int x, int y, boolean firstAfterNewItems) {
		this.firstAfterNewItems = firstAfterNewItems;

		super.drawTo(g, x, y);	
	}
}
