// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.Color;
import gui.FontMetrics;
import gui.Imager;
import gui.Manager;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;

import model.DatumZeit;
import control.Device;

public abstract class Gui {

	private String Name;
	protected GameCanvas gc;
	private Graphics gc_graphics;
	private boolean visible;
	private int statePriviousGui;

	private boolean showTime;

	private boolean validData;

	private String leftFooterString = "";
	private Image leftFooterIcon = null;
	private Image rightFooterIcon = null;
	private String rightFooterString = "";

	protected void setLeftFooterString(String caption, Image icon) {
		this.leftFooterString = caption;
		this.leftFooterIcon = icon;
	}

	protected void setRightFooterString(String caption, Image icon) {
		if (!caption.equals("")) {
			showTime = false;
		}
		this.rightFooterString = caption;
		this.rightFooterIcon = icon;
	}

	protected void setRightFooterIcon(Image icon) {
		this.rightFooterIcon = icon;
	}

	public String getName() {
		return Name;
	}

	/**
	 * gibt zurück ob die Daten in der aktuellen Gui aktuell sind
	 */
	protected boolean hasValidData() {
		return validData;
	}

	/**
	 * setzt die Daten in der Gui auf aktuell gültig
	 */
	protected void dataValid() {
		validData = true;
	}

	/**
	 * die aktuell gespeicherten Daten sind nicht mehr gültig
	 */
	public void dataInvalid() {
		validData = false;
	}

	public Gui(String Name, GameCanvas gc, Graphics g) {
		this.Name = Name;
		this.gc_graphics = g;
		this.gc = gc;
		this.visible = false;

		this.statePriviousGui = 0;

		showTime = true;

		dataInvalid();
	}

	protected void hideTime() {
		showTime = false;
	}

	protected void showTime() {
		showTime = true;
	}

	protected void setGameCanvas(GameCanvas gc) {
		this.gc = gc;
	}

	protected GameCanvas getGameCanvas() {
		return gc;
	}

	public Graphics getGameCanvasGraphics() {
		return gc_graphics;
	}

	protected void setName(String Name) {
		this.Name = Name;
	}

	public abstract void drawTo(Graphics g);

	protected void drawHeaderString(Graphics g) {
		g.setColor(Manager.HEADER_COLOR_FONT);
		Font f = FontMetrics.getFont(Manager.HEADER_FONT_STYLE, Manager.HEADER_FONT_SIZE);
		g.setFont(f);
		//		System.out.println("drawing " + this.Name +" @ "+((Manager.getHeaderHeight() - f.getHeight()) / 2));
		GfxUtils.drawHCenteredText(g, this.Name, (Manager.getHeaderHeight() - f.getHeight()) / 2);
	}
	
	protected void clearTopHeader(Graphics g) {
		g.setColor(Color.WHITE);
		int headerHeight = Manager.HEADER_HEIGHT;
		g.fillRect(0, 0, Manager.SCREEN_WIDTH, headerHeight);
		g.setColor(Color.BLACK);
		GfxUtils.drawFullLineHoriz(g, headerHeight - 1);
	}

	protected void drawHeader(Graphics g) {
		if (Device.getGfxQuality() != Device.GFX_QUALITY_LOW) {
			g.drawImage(Imager.getHeaderImage(this), 0, 0, Graphics.LEFT | Graphics.TOP);
		} else {
			int headerHeight = Manager.HEADER_HEIGHT;
			clearTopHeader(g);
			Font f = FontMetrics.getFont(Manager.HEADER_FONT_STYLE, Manager.HEADER_FONT_SIZE);
			g.setFont(f);
			g.setColor(Color.BLACK);
			GfxUtils.drawHCenteredText(g, Name, (headerHeight - f.getHeight()) / 2);
		}
	}

	protected void drawFooter(Graphics g) {
		// workaround für Android - Left-Footer-String wird nicht neu gezeichnet nachdem man die Erinnerung aktiviert hat 
		if (Device.isAndroid) {
			GfxUtils.clearClip(g);
		}

		g.drawImage(Imager.getFooterImage(), 0, Manager.SCREEN_HEIGHT - Manager.FOOTER_HEIGHT, Graphics.LEFT | Graphics.TOP);

		Font f = FontMetrics.getFont(Manager.FOOTER_FONT_STYLE, Manager.FOOTER_FONT_SIZE);
		g.setFont(f);

		g.setColor(Manager.FOOTER_COLOR_BACKGROUND);
		GfxUtils.fillFullRectangle(g, Manager.SCREEN_HEIGHT - (Manager.FOOTER_HEIGHT - Manager.FOOTER_UPPER_SPACE), Manager.FOOTER_HEIGHT - Manager.FOOTER_UPPER_SPACE);

		g.setColor(Manager.FOOTER_COLOR_FONT);
		int xpos = Manager.FOOTER_LEFT_SPACE;
		int ypos = Manager.SCREEN_HEIGHT - f.getHeight() - Manager.FOOTER_LOWER_SPACE;
//		System.out.println("drawing LeftFooter ### " + Name + ":" + leftFooterString + "@" + xpos + "," + ypos);
		GfxUtils.drawText(g, leftFooterString, xpos, ypos);
		if (leftFooterIcon != null) {
			int xoff = f.stringWidth(leftFooterString + " ");
			g.drawImage(leftFooterIcon, xpos + xoff, ypos, Graphics.LEFT|Graphics.TOP);
		}

		if (!rightFooterString.equals("")) {
			int width = f.stringWidth(rightFooterString);
			xpos = Manager.SCREEN_WIDTH - Manager.FOOTER_LEFT_SPACE - width;
			ypos = Manager.SCREEN_HEIGHT - f.getHeight() - Manager.FOOTER_LOWER_SPACE;
			GfxUtils.drawText(g, rightFooterString, xpos, ypos);
		}

		// falls showTime an ist wird das Icon in drawTime gezeichnet
		if (rightFooterIcon != null && !showTime) {
			int xoff = f.stringWidth(" ");
			g.drawImage(rightFooterIcon, xpos - xoff - rightFooterIcon.getWidth(), ypos , Graphics.LEFT | Graphics.TOP);
		}
		
		if (showTime) {
			drawTime(g, false);
		}
	}

	protected void repaintFooter() {
		drawFooter(getGameCanvasGraphics());
		if (isVisible()) {
			gc.repaint(0, Manager.SCREEN_HEIGHT - Manager.FOOTER_HEIGHT, Manager.SCREEN_WIDTH, Manager.FOOTER_HEIGHT);
		}
	}

	synchronized protected void drawTime(Graphics g, boolean redraw) {
		int width = -1;
		int height = -1;
		String timeStr = DatumZeit.toTimeString(DatumZeit.Now());
		Font f = FontMetrics.getFont(Manager.FOOTER_FONT_STYLE, Manager.FOOTER_FONT_SIZE);
		g.setFont(f);

		width = f.stringWidth(timeStr);
		height = f.getHeight();

		int x_pos = Manager.SCREEN_WIDTH - Manager.FOOTER_LEFT_SPACE - width;
		//		int y_pos =  Manager.SCREEN_HEIGHT - height - (Manager.FOOTER_HEIGHT - height) / 2;
		int y_pos = Manager.SCREEN_HEIGHT - (Manager.FOOTER_HEIGHT - Manager.FOOTER_UPPER_SPACE);

		//		g.drawRegion(Imager.getFooterImage(), x_pos, y_pos - y_footer, width, height,Sprite.TRANS_NONE, x_pos, y_pos, Graphics.LEFT | Graphics.TOP);

		g.setColor(Manager.FOOTER_COLOR_BACKGROUND);

		GfxUtils.fillFullRectangleFromX(g, x_pos, y_pos, height);

		g.setColor(Manager.FOOTER_COLOR_FONT);

		GfxUtils.drawText(g, timeStr, x_pos, y_pos);
		
		if (rightFooterIcon != null) {
			int xoff = f.stringWidth(" ");
			g.drawImage(rightFooterIcon, x_pos - xoff - rightFooterIcon.getWidth(), y_pos , Graphics.LEFT | Graphics.TOP);
		}

		if (redraw && isVisible()) {
			gc.repaint(x_pos, y_pos, width, height);
		}
	}

	public void redrawCurrentTime() {
		if (isVisible() & showTime) {
			drawTime(gc_graphics, true);
		}
	}

	/**
	 * Rückgabewert gibt zurück, ob der Key in der entsprechenden Methode behandelt wurde 
	 */
	public boolean keyPressed(int keyCode, Canvas canvas) {
		return false;
	}

	/**
	 * Rückgabewert gibt zurück, ob durch den übergebenen Parameter eine neue Auswahl getroffen werden konnte bzw. auf die akuelle Auswahl geklickt wurde 
	 */
	public boolean pointerPressed(int x, int y) {
		return false;
	}


	protected int getHeaderHeight() {
		return Manager.HEADER_HEIGHT;
	}

	/**
	 * gibt die Pixelhöhe der Hauptelemente dieser Gui zurück
	 */
	protected int getMainHeight() {
		return Manager.SCREEN_HEIGHT - getMainY() - Manager.MAIN_INNER_SPACE - Manager.FOOTER_HEIGHT;
	}

	/**
	 * gibt die Y-Position zurück, bei dem die Hauptelemente dieser Gui anfangen
	 * 
	 * @return y-Pixelposition
	 */
	protected int getMainY() {
		return getHeaderHeight() + Manager.MAIN_INNER_SPACE;
	}

	/**
	 * speichert den Rücklink auf die Gui, von dem es aufgerufen wurde. Daraus wird die Gui
	 * ermittelt, die angezeigt werden soll falls die Zurück-Taste gedrückt wurde
	 * 
	 * @param previousGui -
	 *            STATE_xyz der vorgänger Gui
	 */
	public void setPreviousGui(int previousGui) {
		this.statePriviousGui = previousGui;
	}

	/**
	 * gibt den STATE_xyz zurück, der angezeigt werden soll falls die Zurück-Taste gedrückt wurde
	 */
	public int getPreviousGui() {
		return statePriviousGui;
	}

	public void show() {
		onShow();
		this.visible = true;
	}

	public void hide() {
		onHide();
		this.visible = false;
	}

	public boolean isVisible() {
		return this.visible;
	}

	protected void onShow() {
	}

	protected void onHide() {
	}
	
	public int getFooterHeight() {
		return Manager.FOOTER_HEIGHT;
	}
	
	public void beforeSwitchTo() {		
	}

	public void scrollUp() {
	}

	public void scrollDown() {
	}

}
