// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.FontMetrics;
import gui.Manager;
import gui.items.Item;
import gui.items.ItemAbout;
import gui.items.ItemAboutPushandride;
import gui.items.ItemSpace;
import gui.utils.GfxUtils;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.InputStream;
import java.util.Vector;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import control.Device;
import control.Global;
import control.Keys;
import control.locale.Locale;
import dev.Logger;

public class GuiAbout extends GuiView {

	public GuiAbout(GameCanvas gc, Graphics g) {
		super("Über", gc, g);
		setLeftFooterString(Locale.Back, null);
		generateItems();
	}

	private void generateItems() {
		int fontheight = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE).getHeight();
		int ySpacing = fontheight / 4;

		Vector v = new Vector();
		v.addElement(new ItemAboutPushandride());
		v.addElement(new ItemAbout(new String[] { "Fahrplan 2011", "kontakt@pushandride.de", "http://pushandride.de" }, true));

		v.addElement(new ItemSpace(ySpacing));

		v.addElement(new ItemAbout("Copyright", false));
		v.addElement(new ItemAbout("GPL V3", true));

		v.addElement(new ItemSpace(ySpacing));

		v.addElement(new ItemAbout("Version", false));
		v.addElement(new ItemAbout(Global.getMidletVersion(), true));

		v.addElement(new ItemSpace(ySpacing));
		
		if (!Device.isAndroid) {
			v.addElement(new ItemAbout("Enthalten", false));
			String[] sets = GfxUtils.splitupInLines(FontMetrics.getFont(Manager.ITEMS_FONT_SIZE), new String[] { Global.getSetInfo() }, Manager.SCREEN_WIDTH - 3 * Manager.MAIN_LEFT_SPACE);
			v.addElement(new ItemAbout(sets, true));
	
			v.addElement(new ItemAbout("Zusätzliche Linien", false));
	
			String[] binInfo = Global.getBinaryInfo();
			String linienString = binInfo[Global.BINARY_INFO_LINIEN];
			String downloadUrl = binInfo[Global.BINARY_INFO_URL];
	
			String[] linien = GfxUtils.splitupInLines(FontMetrics.getFont(Manager.ITEMS_FONT_SIZE), new String[] { linienString }, Manager.SCREEN_WIDTH - 3 * Manager.MAIN_LEFT_SPACE);
			v.addElement(new ItemAbout(linien, true));
	
			v.addElement(new ItemSpace(ySpacing));
	
			v.addElement(new ItemAbout("Direkter Download", false));
			v.addElement(new ItemAbout(downloadUrl, true));
	
			v.addElement(new ItemSpace(ySpacing));
		}

		Item[] items = new Item[v.size()];
		v.copyInto(items);
		setItems(items);
		
		initialized = true;
	}
	
	private boolean initialized = false;
	
	public void drawTo(Graphics g) {
		if (!initialized) {
			generateItems();
		}
		super.drawTo(g);
	}
	
	
	public void scrollUp() {
		keyPressed(Keys.getKeyCode(Keys.UP), getGameCanvas());
	}

	public void scrollDown() {
		keyPressed(Keys.getKeyCode(Keys.DOWN), getGameCanvas());
	}
}
