// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.Color;
import gui.FontMetrics;
import gui.IOverlayable;
import gui.Manager;
import gui.items.Item;
import gui.items.ItemTastenHinweis;
import gui.utils.GfxUtils;

import java.util.Vector;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import control.Keys;
import data.persistence.Stats;
import data.persistence.UserSettingsInt;
import dev.Logger;

public class GuiBedienungsHinweis extends Gui implements IOverlayable {
	private static int headerHeight;
	private static int innerSpace;
	private static int textHeight;
	
	private final int info_height;
	private final int info_width;
	private final int info_y;
	private final int info_x;
	private static int content_width;

	private static int itemSpace;
	private static int endSpace;
	private static String[] bottomLines;
	
	private static boolean initialized = false;

	private final Item items[];
	
	private final int hinweisId;

	private static final int HINWEIS_HAUPTBILDSCHIRM_1 = 0;
	private static final int HINWEIS_HAUPTBILDSCHIRM_2 = 1;
	private static final int HINWEIS_ORT_VON = 2;
	private static final int HINWEIS_UEBERSICHT_1 = 3;
	private static final int HINWEIS_UEBERSICHT_2 = 4;
	private static final int HINWEIS_DETAILS_1 = 5;
//	private static final int HINWEIS_DETAILS_2 = 7;

	private static int alreadySeenMask = -1;

	/**
	 * falls showImmediately gesetzt ist wird nicht erst darauf gewartet, dass einige Hinweise erst nach dem x. Start angezeigt werden
	 */
	private static final boolean showImmediately = false;

	/**
	 * prüft ob die übergebene hinweisId schon angezeigt wurde
	 */
	private static boolean hasSeen(int hinweisId) {
		boolean alreadySeen = ((alreadySeenMask & (1 << hinweisId)) != 0);
		return alreadySeen;
	}

	/**
	 * cached die noch nicht angezeigten Hinweise (ACHTUNG NOCH NICHT GEPRÜFT)
	 */
	public static void createCache() {
		if (!initialized) {
			initialize();
		}
		int startNo = Stats.get(Stats.STARTS);
		for (int i = 0; i < Manager.guis.length; i++) {
			getItemsFor(i, startNo);
		}
	}

	/**
	 *  initialisiert die statischen Variablen
	 */
	private static void initialize() {
		alreadySeenMask = UserSettingsInt.get(UserSettingsInt.SETTINGS_ID_HINWEISE_MASK);
		if (alreadySeenMask == UserSettingsInt.NULL_VALUE) {
			alreadySeenMask = 0;
		}
		
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		textHeight = f.getHeight();
		innerSpace = GfxUtils.scale(textHeight, 20);
		headerHeight = textHeight + 1;

		itemSpace = GfxUtils.scale(f.getHeight(), 60);
		endSpace = GfxUtils.scale(f.getHeight(), 70);

		int info_width = calcInfoWidth();
		content_width = info_width - 2 * innerSpace;
		
		String[] bottomLines = {"Beliebige Taste für weiter."};
		GuiBedienungsHinweis.bottomLines = GfxUtils.splitupInLines(f, bottomLines, content_width);
		
		initialized = true;
	}

	/**
	 * Hilfsklasse für einen Rückgabewert
	 */
	private static class ReturnHinweis {
		private final int hinweisId;
		private final Item[] items;

		public ReturnHinweis(int hinweisId, Item[] items) {
			this.hinweisId = hinweisId;
			this.items = items;
		}
	}

	/**
	 * gibt einen entsprechenden Hinweis für den entsprechenden guiState zurück
	 */
	private static ReturnHinweis getItemsFor(int guiState, int startNo) {
		if (alreadySeenMask < 0) {
			// FIXME: init first time
			alreadySeenMask = 0;
		}

		int hinweis = -1;
		Vector v = new Vector();
		switch (guiState) {
		case Manager.STATE_HOME:
			hinweis = HINWEIS_HAUPTBILDSCHIRM_1;
			if (!hasSeen(hinweis)) {
				int button = Keys.OK;
				String text = "Bestätigen / Weiter";
				v.addElement(new ItemTastenHinweis(button, text, content_width));
				break;
			}
			hinweis = HINWEIS_HAUPTBILDSCHIRM_2;
			if (!hasSeen(hinweis) && (startNo >= 3 || showImmediately)) {
				int button = Keys.MENU_LEFT;
				String text = "von / nach Haltestelle tauschen";
				v.addElement(new ItemTastenHinweis(button, text, content_width));
				break;
			}
			break;
		case Manager.STATE_ORT_VON:
			hinweis = HINWEIS_ORT_VON;
			if (!hasSeen(hinweis)) {
				int button = Keys.NUMBER;
				String text = "Textsuche ähnlich zu IntelligentKey";
				v.addElement(new ItemTastenHinweis(button, text, content_width));
				button = Keys.LEFT;
				text = "zurück zum Hauptbildschirm";
				v.addElement(new ItemTastenHinweis(button, text, content_width));
				break;
			}
			break;
		case Manager.STATE_UEBERSICHT:
			hinweis = HINWEIS_UEBERSICHT_1;
			if (!hasSeen(hinweis)) {
				int button = Keys.OK;
				String text = "Details der ausgewählten Strecke anzeigen";
				v.addElement(new ItemTastenHinweis(button, text, content_width));
				button = Keys.LEFT;
				text = "zurück zur Haltestellenauswahl";
				v.addElement(new ItemTastenHinweis(button, text, content_width));
				break;
			}
			hinweis = HINWEIS_UEBERSICHT_2;
			if (!hasSeen(hinweis) && (startNo >= 5 || showImmediately)) {
				int button = Keys.MENU_RIGHT;
				String text = "Abfahrtszeit wählen";
				v.addElement(new ItemTastenHinweis(button, text, content_width));
				break;
			}
			break;
		case Manager.STATE_DETAILS:
			hinweis = HINWEIS_DETAILS_1;
			if (!hasSeen(hinweis)) {
				int button = Keys.OK;
				String text = "zurück zum Hauptbildschirm";
				v.addElement(new ItemTastenHinweis(button, text, content_width));
				button = Keys.LEFT;
				text = "zurück zur Übersicht";
				v.addElement(new ItemTastenHinweis(button, text, content_width));
				break;
			}
			break;
		}

		Item items[] = new Item[v.size()];
		v.copyInto(items);

		return new ReturnHinweis(hinweis, items);
	}

	/**
	 * prüft ob ein Hinweis zum aktuellen guiState verfügbar ist. Falls nein wird 'null' zurückgeliefert
	 */
	public static GuiBedienungsHinweis getBedienungsHinweis(GameCanvas gc, Graphics g, int guiState) {
		if (!initialized) {
			initialize();
		}

		if (showImmediately) {
			Logger.info(GuiBedienungsHinweis.class, "showImmediately activiated");
		}
		
		int startNo = Stats.get(Stats.STARTS);

		ReturnHinweis returnHinweis = getItemsFor(guiState, startNo);

		Item items[] = returnHinweis.items;

		if (items.length == 0) {
			return null;
		}

		setSeen(returnHinweis.hinweisId);
		return new GuiBedienungsHinweis(gc, g, items, returnHinweis.hinweisId);
	}

	/**
	 * setzt in alreadySeen das passende Bit über die hinweisId
	 */
	private static void setSeen(int hinweisId) {
		alreadySeenMask |= (1 << hinweisId);

		UserSettingsInt.set(UserSettingsInt.SETTINGS_ID_HINWEISE_MASK, alreadySeenMask);
	}

	private static int calcInfoX() {
		return GfxUtils.scale(Manager.SCREEN_WIDTH, 10);
	}

	private static int calcInfoWidth() {
		return Manager.SCREEN_WIDTH - 2 * calcInfoX();
	}

	private GuiBedienungsHinweis(GameCanvas gc, Graphics g, Item[] items, int hinweisId) {
		super("Bedienhinweis", gc, g);
		this.hinweisId = hinweisId;
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);

		info_x = calcInfoX();
		info_width = calcInfoWidth();

		this.items = items;

		int tmp_height = headerHeight + innerSpace;

		for (int i = 0; i < items.length; i++) {
			tmp_height += items[i].getHeight();
			if (i != items.length - 1) {
				tmp_height += itemSpace;
			}
		}
		
		// beim ersten Screen wird 'Beliebige Taste für weiter' nicht angezeigt
		if (hinweisId != HINWEIS_HAUPTBILDSCHIRM_1) {
			tmp_height += endSpace;
			
			tmp_height += f.getHeight() * bottomLines.length;
		}
		
		tmp_height += innerSpace + 1;

		info_height = tmp_height;

		info_y = (Manager.SCREEN_HEIGHT - info_height) / 2;
	}

	public int getInfoHeight() {
		return info_height;
	}

	public int getInfoWidth() {
		return info_width;
	}

	public int getInfoX() {
		return info_x;
	}

	public int getInfoY() {
		return info_y;
	}

	private final static int header_Background_Color = Color.RED;
	private final static int header_Font_Color = Color.WHITE;
	private final static int main_Background_Color = Color.WHITE;

	protected void drawHeader(Graphics g) {
		int info_add_y = 1;
		// Header
		Font fb = FontMetrics.getFont(Font.STYLE_BOLD, Manager.ITEMS_FONT_SIZE);
		g.setFont(fb);
		g.setColor(header_Background_Color);
		g.fillRect(info_x + 1, info_y + 1, info_width - 2, headerHeight - 1);
		g.setColor(header_Font_Color);
		GfxUtils.drawHCenteredText(g, "Bedienungshinweis", info_y + info_add_y);

		// Rect
		g.setColor(Color.BLACK);
		g.drawRect(info_x, info_y, info_width - 1, info_height - 1);
	}

	synchronized private void drawMain(Graphics g) {
		int y_base = getMainY();

		int mainHeight = getMainHeight();

		int y_add = innerSpace;

		int x_base = info_x + innerSpace;

		Item items[] = getItems();

		g.setColor(main_Background_Color);

		g.fillRect(info_x + 1, y_base, info_width - 2, mainHeight - 1);

		for (int i = 0; i < items.length; i++) {
			y_add += items[i].draw(this, g, x_base, y_base + y_add, i);
			if (i != items.length - 1) {
				y_add += itemSpace;
			}
		}
		// beim ersten Screen wird 'Beliebige Taste für weiter' nicht angezeigt
		if (hinweisId != HINWEIS_HAUPTBILDSCHIRM_1) {
			y_add += endSpace;
			
			GfxUtils.drawText(g, bottomLines, x_base, y_base + y_add);
		}
	}

	private Item[] getItems() {
		return items;
	}

	protected int getHeaderHeight() {
		return headerHeight;
	}

	protected int getMainY() {
		return info_y + getHeaderHeight();
	}

	protected int getMainHeight() {
		return info_height - getHeaderHeight();
	}

	public void drawTo(Graphics g) {
		drawHeader(g);

		drawMain(g);

		if (isVisible()) {
			gc.repaint();
		}
	}

}
