// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.launch.GuiLaunchScreen;

import java.util.Timer;
import java.util.TimerTask;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;

import control.Global;
import control.PushandrideMidlet;

public class GuiBerechne extends Gui {

	// Timer und Task sind dafür zuständig die mindestzeit in Global abzuwarten um auf den nächsten Screen zu wechseln
	private Timer timer;
	private TimerTask task;

	// status Flag ob der aktuelle Task abgebrochen wurde
	private boolean canceled = false;
	
	// taskId wird hochgezählt bei jedem erstellen von einem Task
	private int activeTaskId = 0;
	
	private final Image logo;

	public GuiBerechne(GameCanvas gc, Graphics g) {
		super("berechne", gc, g);
		timer = new Timer();
		logo = GuiLaunchScreen.getHandImage();
	}

	protected void clearMainBackground(Graphics g) {
		g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
		int headerHeight = getHeaderHeight();
		gui.utils.GfxUtils.fillFullRectangle(g, headerHeight, Manager.SCREEN_HEIGHT - Manager.FOOTER_HEIGHT - headerHeight);
	}

	/**
	 * zeichnet die komplette Gui
	 */
	synchronized public void drawTo(Graphics g) {
		clearMainBackground(g);

		drawHeader(g);

		drawFooter(g);

		drawMain(g);

		if (isVisible()) {
			gc.repaint();
		}
	}

	private void drawMain(Graphics g) {
		g.setColor(Color.BLACK);
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		int textHeight = f.getHeight();
		g.setFont(f);
		
		int logoHeight = 0;
		if (logo != null) {
			logoHeight = logo.getHeight();
		}
		
		int spacing = getMainHeight() - textHeight * 2 - logoHeight;
		
		int yPos = getMainY();

		yPos += spacing / 3;
		
		g.drawString("berechne...", Manager.MAIN_LEFT_SPACE, yPos, Graphics.LEFT | Graphics.TOP);
		yPos += textHeight;
		g.drawString("bitte warten...", Manager.MAIN_LEFT_SPACE, yPos, Graphics.LEFT | Graphics.TOP);
		yPos += textHeight;
		
		yPos += spacing / 3;
		if (logo != null) {
			g.drawImage(logo, (Manager.SCREEN_WIDTH - logo.getWidth()) / 2, yPos, Graphics.LEFT | Graphics.TOP);
		}
	}

	synchronized protected void onShow() {
		activeTaskId++;
		cancel();
		canceled = false;
		timer.schedule(task = new CallbackTask(activeTaskId), Global.TIME_MIN_BERECHNE_MS);
	}

	synchronized protected void onHide() {
		cancel();
	}

	synchronized protected void moveOn(int taskId) {
		if (!canceled && (this.activeTaskId == taskId)) {
			task = null;
			PushandrideMidlet.getManager().berechneScreenMoveOn();
		}
	}

	synchronized public void cancel() {
		canceled = true;
		if (task != null) {
			task.cancel();
			task = null;
		}
	}

	// FIXME: ist das wirklich sicher falls der Event kommt und gleichzeitig zurück gedrückt wird ?
	private class CallbackTask extends TimerTask {
		private final int id;
		public CallbackTask(int id) {
			this.id = id;
		}
		public void run() {
			moveOn(id);
		}

	}
}
