// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.Color;
import gui.FontMetrics;
import gui.IOverlayable;
import gui.Manager;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import dev.Logger;

public class GuiMessageScreen extends Gui implements IOverlayable {
	private final static int COLOR_BACKGROUND = Color.WHITE;
	private final static int COLOR_HEADER = Color.BLACK;
	private final static int COLOR_TEXT = Color.BLACK;

	public final static int TYPE_MESSAGE_FULLSCREEN = 0;
	public final static int TYPE_MESSAGE_OVERLAY = 5;

	private String messageHeader;
	private String messageText;
	private int messageType;

	public GuiMessageScreen(GameCanvas gc, Graphics g) {
		super("Hinweis", gc, g);
		hideTime();
	}

	protected int getMainY() {
		return Manager.HEADER_HEIGHT * 2;
	}

	protected int getMainHeight() {
		return Manager.SCREEN_HEIGHT - getMainY();
	}

	/**
	 * zeichnet den Text auf dem Graphics
	 */
	private void drawMain(Graphics g) {
		int y_base = Manager.HEADER_HEIGHT;

		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);

		g.setFont(f);

		int lineHeight = f.getHeight();

		// Aufteiles der Nachricht auf meherere Zeilen - Trennzeichen '\n'
		String[] lines = utils.StringUtils.split(messageText, "\n");

		int maxwidth = GfxUtils.scale(Manager.SCREEN_WIDTH, 90);
		int posX = (Manager.SCREEN_WIDTH - maxwidth) / 2;

		String[] output = GfxUtils.splitupInLines(f, lines, maxwidth);

		g.setColor(COLOR_TEXT);

		for (int i = 0; i < output.length; i++) {
			GfxUtils.drawText(g, output[i], posX, y_base + i * lineHeight);
		}

	}

	protected void drawHeader(Graphics g) {
		Font f;
		f = FontMetrics.getFont(Font.STYLE_BOLD, Manager.HEADER_FONT_SIZE);
		g.setFont(f);

		int headerTextWidth = f.stringWidth(messageHeader);
		int headerTextHeight = f.getHeight();

		//		int spaceing = GfxUtils.scaleWithMinimun(headerTextHeight, 10, 1);

		int midX = Manager.SCREEN_WIDTH / 2;
		int midY = getHeaderHeight() / 2;

		g.setColor(COLOR_HEADER);
		g.setFont(FontMetrics.getFont(Manager.ITEMS_FONT_SIZE));
		//		g.drawRect(midX - headerTextWidth / 2 - spaceing, midY - headerTextHeight / 2 - spaceing, headerTextWidth + 2 * spaceing, headerTextHeight + 2 * spaceing);

		GfxUtils.drawText(g, messageHeader, midX - headerTextWidth / 2, midY - headerTextHeight / 2);
	}

	/**
	 * Initialisiert den Message-Screen
	 * 
	 * @param header
	 *            Kopfzeile
	 * @param text
	 *            Haupttext - neue Zeilen mit '\n'
	 * @param noch
	 *            nicht implementiert
	 */
	public void showMessage(String header, String text, int type) {
		setType(type);
		this.messageHeader = header;
		this.messageText = text;
	}

	public void drawTo(Graphics g) {
		switch (messageType) {
		case TYPE_MESSAGE_FULLSCREEN:
			g.setColor(COLOR_BACKGROUND);
			g.setFont(FontMetrics.getFont(Manager.ITEMS_FONT_SIZE));

			GfxUtils.fillFullRectangle(g, 0, Manager.SCREEN_HEIGHT);

			drawHeader(g);

			drawMain(g);

			break;
		case TYPE_MESSAGE_OVERLAY:
			drawInfo(g, messageText);
			break;
		default:
		}
		if (isVisible()) {
			gc.repaint();
		}
	}

	private int info_height = -1;
	private int info_width = -1;
	private int info_y = -1;
	private int info_x = -1;

	public int getInfoHeight() {
		return info_height;
	}

	public int getInfoWidth() {
		return info_width;
	}

	public int getInfoX() {
		return info_x;
	}

	public int getInfoY() {
		return info_y;
	}

	public void drawInfo(Graphics g, String s) {
		int header_Background_Color = Color.RED;
		int main_Background_Color = Color.WHITE;
		int header_Font_Color = Color.WHITE;
		int main_Font_Color = Color.BLACK;
		int info_space = GfxUtils.scale(Manager.SCREEN_WIDTH, 10);
		info_x = info_space;
		info_width = Manager.SCREEN_WIDTH - 2 * info_space;
		info_y = getHeaderHeight() * 2;

		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		Font fb = FontMetrics.getFont(Font.STYLE_BOLD, Manager.ITEMS_FONT_SIZE);

		info_height = 0;
		info_height += 1; // top Line
		info_height += f.getHeight(); // Header

		int space = f.getHeight() / 3;
		info_height += space; // space between Header and Content

		String[] lines = utils.StringUtils.split(s, "\n");
		lines = GfxUtils.splitupInLines(f, lines, Manager.SCREEN_WIDTH - 3 * info_space);

		info_height += lines.length * f.getHeight(); // Content Height

		info_height += space; // space between Text and Press any key 

		String[] bottomLines = { "Beliebige Taste für weiter." };
		bottomLines = GfxUtils.splitupInLines(f, bottomLines, Manager.SCREEN_WIDTH - 3 * info_space);
		info_height += f.getHeight() * bottomLines.length; // press any key

		info_height += space; // space between Content and Footer

		info_height += 1; // bottom Line

		// Position mittig in Y-Richtung zentrieren
		info_y = Math.max(0, (Manager.SCREEN_HEIGHT - info_height) / 2);

		int info_add_y = 1;

		// Header
		g.setFont(fb);
		g.setColor(header_Background_Color);
		g.fillRect(info_space, info_y + info_add_y, info_width, f.getHeight());
		g.setColor(header_Font_Color);
		GfxUtils.drawHCenteredText(g, "Hinweis", info_y + info_add_y);
		info_add_y += f.getHeight();

		g.setFont(f);
		// Space
		g.setColor(main_Background_Color);
		g.fillRect(info_space, info_y + info_add_y, info_width, space);
		info_add_y += space;

		// Text
		for (int i = 0; i < lines.length; i++) {
			g.setColor(main_Background_Color);
			g.fillRect(info_space, info_y + info_add_y, info_width, f.getHeight());
			g.setColor(main_Font_Color);
			GfxUtils.drawText(g, lines[i], info_space + info_space / 2, info_y + info_add_y);
			info_add_y += f.getHeight();
		}

		g.setColor(main_Background_Color);
		g.fillRect(info_space, info_y + info_add_y, info_width, space);
		info_add_y += space; // space between Text and Press any key 

		// press any key
		for (int i = 0; i < bottomLines.length; i++) {
			g.setColor(main_Background_Color);
			g.fillRect(info_space, info_y + info_add_y, info_width, f.getHeight());
			g.setColor(main_Font_Color);
			GfxUtils.drawText(g, bottomLines[i], info_space + info_space / 2, info_y + info_add_y);
			info_add_y += f.getHeight();
		}

		// Space
		g.setColor(main_Background_Color);
		g.fillRect(info_space, info_y + info_add_y, info_width, space);
		info_add_y += space;

		info_add_y += 1;

		if (info_height != info_add_y) {
			System.err.println("drawInfo - falsche height");
		}

		g.setColor(Color.BLACK);
		g.drawRect(info_x, info_y, info_width - 1, info_height - 1);
	}

	public int getFaderType() {
		switch (messageType) {
		case TYPE_MESSAGE_FULLSCREEN:
			return Manager.FADER_TYPE_SPLITTER;
		case TYPE_MESSAGE_OVERLAY:
			return Manager.FADER_TYPE_OVERLAY;
		default:
			Logger.error(this.getClass(), "wrong messageType:" + messageType);
			return -1;
		}
	}

	public void setType(int messageType) {
		this.messageType = messageType;
	}

}
