// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.Manager;
import gui.items.Item;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import control.Keys;


public class GuiSelection extends Gui {

	public final static int EINFUEGEN_HINTEN = 0;
	public final static int EINFUEGEN_VORNE = 1;
	protected final static int SELECTION_NONE = -1;
	protected final static int OVERFLOW_TYPE_NONE = 0;
	protected final static int OVERFLOW_TYPE_ROUND = 1;
	protected final static int OVERFLOW_TYPE_ENDLESS = 2;

	// Wieviel Platz ist noch im Main
	private int mainFreeItemSpace;
	private Item items[];
	
	/**
	 * Anzahl der sichtbaren Elemente
	 */
	// FIXME: privat machen und nicht mehr durch alles durchreichen
	protected int visibleItems;
	protected int overflowType;
	/**
	 * die ItemID, welche ganz oben angezeigt wird
	 */
	protected int firstId;
	/**
	 * die aktuell selektierte ItemId
	 */
	protected int selectedId;

	public GuiSelection(String Name, GameCanvas gc, Graphics g, Item items[], int overFlowType) {
		super(Name, gc, g);

		this.items = items;
		this.overflowType = overFlowType;

		resetSelectionAndFirstPos();
	}

	public GuiSelection(String Name, GameCanvas gc, Graphics g, Item items[]) {
		super(Name, gc, g);
		newItems(items);
	}

	public GuiSelection(String Name, GameCanvas gc, Graphics g) {
		super(Name, gc, g);
	}

	protected void resetSelectionAndFirstPos() {
		firstId = 0;
		visibleItems = -1;

		selectedId = SELECTION_NONE;

		for (int i = 0; i < items.length; i++) {
			if (items[i].isSelectable()) {
				selectedId = i;
				if ((overflowType != OVERFLOW_TYPE_ENDLESS) || (selectedId > 0)) {
					break;
				}
			}
		}

	}

	protected int getLastId() {
		return getValidId(items.length - 1);
	}

	private int getLastVisibleId() {
		return getValidId(firstId + visibleItems - 1);
	}

	protected void setOverflowType(int overFlowType) {
		this.overflowType = overFlowType;
	}

	protected int getOverflowType() {
		return this.overflowType;
	}

	/**
	 * Ein Element über dem aktuell selektierten soll ausgewählt werden
	 */
	synchronized protected void moveUp(boolean repaint) {
		if (selectedId == SELECTION_NONE) {
			return;
		}
		int oldSelectedId = selectedId;

		do {
			if ((overflowType == OVERFLOW_TYPE_NONE) && (selectedId == 0)) {
				selectedId = oldSelectedId;
				return;
			}
			selectedId = getValidId(selectedId - 1);
			// komplette Elemente nach oben schieben wenn der passende
			// Overflowtyp aktiv ist
			if ((overflowType == OVERFLOW_TYPE_ENDLESS) || ((overflowType == OVERFLOW_TYPE_NONE) && !(this instanceof GuiSelectionDatasetSelect))) {
				if (selectedId == firstId) {
					firstId = getValidId(selectedId - 1);
				}
			}

		} while (!items[selectedId].isSelectable());

		onChangeSelected();

		if (repaint) {
			drawMain(getGameCanvasGraphics(), true);
		}
	}

	private void calculateVisibleItems() {
		visibleItems = 0;
		int height = 0;
		int mainHeight = getMainHeight();
		for (int i = 0; i < items.length; i++) {
			height += items[getValidId(firstId + i)].getHeight();
			if (height <= mainHeight) {
				visibleItems++;
			} else {
				break;
			}
		}
	}

	/**
	 * Ein Element ünter dem aktuell selektierten soll ausgewählt werden
	 */
	synchronized protected void moveDown(boolean repaint) {
		if (selectedId == SELECTION_NONE) {
			return;
		}
		int oldSelectedId = selectedId;
		int lastId;

		do {

			lastId = getLastId();
			if ((overflowType == OVERFLOW_TYPE_NONE) && (lastId == selectedId)) {
				selectedId = oldSelectedId;
				return;
			}
			selectedId = getValidId(selectedId + 1);
			// komplette Elemente nach unten schieben wenn der passende
			// Overflowtyp aktiv ist
			if ((overflowType == OVERFLOW_TYPE_ENDLESS) || ((overflowType == OVERFLOW_TYPE_NONE) && !(this instanceof GuiSelectionDatasetSelect))) {
				while (selectedId == getLastVisibleId()) {
					firstId = getValidId(firstId + 1);
					calculateVisibleItems();
				}
			}

		} while (!items[selectedId].isSelectable());

		onChangeSelected();

		if (repaint) {
			drawMain(getGameCanvasGraphics(), true);
		}
	}

	/**
	 * wandelt die id in eine gültige ItemId zurück (wenn sie z.B. negativ oder großer als die
	 * Anzahl der Elemente ist)
	 * 
	 * @return gültige ItemID (0..Items.size)
	 */
	protected int getValidId(int id) {
		if (items.length == 0) {
			System.err.println("getValidId - Item Size 0");
			return -1;
		}
		while (id < 0) {
			id += items.length;
		}

		return (id % items.length);
	}

	/**
	 * zeichnet die Hauptelemente der Gui neu
	 */
	synchronized protected void drawMain(Graphics g, boolean clearBackgroundAndRepaint) {
		int y_base = getMainY();
		int mainHeight = getMainHeight();

		// clear Background
		if (clearBackgroundAndRepaint && isVisible()) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
			gui.utils.GfxUtils.fillFullRectangle(g, y_base, mainHeight + Manager.MAIN_INNER_SPACE);
		}

		int currentId = firstId;

		visibleItems = 0;
		int y_add = 0;

		// FIXME: HACK damit im Übersichtscreen ein einzelnes/zwei Element(e) an Zeile 2 angezeigt wird
		if (this instanceof GuiSelectionUebersicht && items.length < 3) {
			y_add += items[currentId].getHeight();
		}

		if ((items != null) && (items.length != 0)) {
			/* es werden so lange Elemente neu gezeichnet bis das nächste Element nicht mehr komplett hineinpassen würde
			 */
			while (y_add + items[currentId].getHeight() <= mainHeight) {
				boolean selected;
				if (currentId == selectedId) {
					selected = true;
				} else {
					selected = false;
				}
				y_add += items[currentId].draw(this, g, Manager.MAIN_LEFT_SPACE, y_base + y_add, selected, currentId);
				currentId = getValidId(currentId + 1);
				visibleItems++;
				if ((overflowType != OVERFLOW_TYPE_ENDLESS) && (visibleItems == items.length)) {
					break;
				}
				if ((overflowType == OVERFLOW_TYPE_NONE) && (currentId == 0)) {
					break;
				}
			}
		}

		mainFreeItemSpace = mainHeight - y_add;

		drawAdditionalMainStuff(g);

		if (clearBackgroundAndRepaint && isVisible()) {
			gc.repaint(0, y_base, Manager.SCREEN_WIDTH, mainHeight + Manager.MAIN_INNER_SPACE);
		}
	}

	protected void clearMainBackground(Graphics g) {
		g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
		int headerHeight = getHeaderHeight();
		gui.utils.GfxUtils.fillFullRectangle(g, headerHeight, Manager.SCREEN_HEIGHT - Manager.FOOTER_HEIGHT - headerHeight);
	}

	/**
	 * zeichnet die komplette Gui bis auf den Header neu
	 */
	synchronized public void drawTo(Graphics g) {
		clearMainBackground(g);
		drawHeader(g);
		drawFooter(g);
		drawMain(g, false);
		drawAdditionalStuff(g);

		if (isVisible()) {
			gc.repaint();
		}
	}

	protected void drawAdditionalMainStuff(Graphics g) {
	}

	protected void drawAdditionalStuff(Graphics g) {
	}

	/*--------------------------------------------------
	 * Key code event handling
	 *-------------------------------------------------*/
	public boolean keyPressed(int keyCode, Canvas canvas) {
		if (Keys.isKey(keyCode, Keys.UP)) {
			moveUp(true);
			return true;
		}
		if (Keys.isKey(keyCode, Keys.DOWN)) {
			moveDown(true);
			return true;
		}
		return false;
	}

	/**
	 * übernimmt die übergebenen Items
	 */
	synchronized protected void setItems(Item items[]) {
		this.items = items;
	}

	/**
	 * übernimmt die übergebenen Items und resettet die Auswahl / Anzeige
	 */
	synchronized protected void newItems(Item items[]) {
		setItems(items);
		resetSelectionAndFirstPos();
	}

	protected Item[] getItems() {
		return items;
	}

	/**
	 * fügt ein Item hinzu, mit dem Parameter kann es oben oder unten hinzugefügt werden
	 * 
	 * @param i -
	 *            Item
	 * @param einfuegenPosition -
	 *            EINFUEGEN_VORNE oder EINFUEGEN_HINTEN
	 */
	synchronized protected void addItem(Item i, int einfuegenPosition) {
		Item items[];
		items = new Item[this.items.length + 1];
		if (einfuegenPosition == EINFUEGEN_VORNE) {
			System.arraycopy(this.items, 0, items, 1, this.items.length);
			items[0] = i;
		}
		if (einfuegenPosition == EINFUEGEN_HINTEN) {
			System.arraycopy(this.items, 0, items, 0, this.items.length);
			items[this.items.length] = i;
			int itemHeight = i.getHeight();
			if (itemHeight <= mainFreeItemSpace) {
				visibleItems++;
				mainFreeItemSpace -= itemHeight;
			}
		}
		this.items = items;

		if ((selectedId != SELECTION_NONE) && (einfuegenPosition == EINFUEGEN_VORNE)) {
			selectedId += 1;
		}

		if ((selectedId == SELECTION_NONE) && (i.isSelectable())) {
			if (einfuegenPosition == EINFUEGEN_VORNE) {
				selectedId = 0;
			}
			if (einfuegenPosition == EINFUEGEN_HINTEN) {
				selectedId = items.length - 1;
			}
		}
	}

	/**
	 * erstetzt ein Item an der angegeben Position
	 */
	synchronized protected void replaceItem(int pos, Item item) {
		if ((pos < 0) || (pos >= items.length)) {
			System.err.println("replaceItem - falscher Index: pos " + pos + "  max " + items.length);
			return;
		}
		items[pos] = item;
	}

	/**
	 * kann in abgeleiteten Klassen überschrieben werden um Änderungen am selektierten Element
	 * rückgemeldet zu bekommen
	 */
	protected void onChangeSelected() {
	}

	public Item getItemAtPos(int x, int y) {
		int y_pos = getMainY();
		if (y < y_pos) {
			return null;
		}
		for (int i = 0; i < visibleItems; i++) {
			Item item = items[getValidId(firstId + i)];
			int itemHeight = item.getHeight();
			y_pos += itemHeight;
			if (y < y_pos) {
				return item;
			}
		}
		return null;
	}

	public int getItemIdAtPos(int x, int y) {
		int y_pos = getMainY();
		if (y < y_pos) {
			return -1;
		}
		for (int i = 0; i < visibleItems; i++) {
			Item item = items[getValidId(firstId + i)];
			int itemHeight = item.getHeight();
			y_pos += itemHeight;
			if (y < y_pos) {
				return getValidId(firstId + i);
			}
		}

		return -1;
	}

	public boolean select(Item newSelection) {
		for (int newSelectedId = 0; newSelectedId < items.length; newSelectedId++) {
			if (items[newSelectedId].equals(newSelection)) {
				if (selectedId == newSelectedId) {
					return true;
				}
				if (selectedId < newSelectedId) {
					for (int down = 0; down < newSelectedId - selectedId; down++) {
						moveDown(false);
					}
					return true;
				}
				if (selectedId > newSelectedId) {
					for (int up = 0; up < selectedId - newSelectedId; up++) {
						moveUp(false);
					}
					return true;
				}
			}
		}
		System.err.println("GuiSeleciton->Select -- can't be here");
		return false;
	}

	public boolean selectId(int newId) {
		if (!items[newId].isSelectable()) {
			return false;
		}

		if (newId == selectedId) {
			return true;
		}

		if (newId > selectedId) {
			while (selectedId != newId) {
				moveDown(false);
			}
			return true;
		}

		if (newId < selectedId) {
			while (selectedId != newId) {
				int lastSelectedId = selectedId;
				moveUp(false);
				if (this instanceof GuiSelectionUebersicht) {
					// Spezialfall, falls der oberste Eintrag gewählt wurde  
					if (newId == 0 && lastSelectedId == 1) {
						return true;
					}
				}
			}
			return true;
		}

		System.err.println("GuiSeleciton->Select -- can't be here");
		return false;
	}
	
	public boolean pointerPressed(int x, int y) {
		int newSelectedId = this.getItemIdAtPos(x, y);
		if (newSelectedId != SELECTION_NONE) {
			if (this.selectId(newSelectedId)) {
				return true;
			}
		}
		return false;
	}
	
	public void scrollUp() {
		if (overflowType == OVERFLOW_TYPE_ENDLESS) {
			int tmpId = firstId; 
			firstId = getValidId(firstId - 1);
			if (firstId == tmpId -1) {
				moveUp(false);	
			}
			drawMain(getGameCanvasGraphics(), true);
		}
	}

	public void scrollDown() {
		if (overflowType == OVERFLOW_TYPE_ENDLESS) {
			int tmpId = firstId; 
			firstId = getValidId(firstId + 1);
			if (firstId == tmpId +1) {
				moveDown(false);	
			}
			drawMain(getGameCanvasGraphics(), true);
		}
	}
}
