// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.items.Item;
import gui.items.ItemCheckbox;
import gui.items.ItemLocationDescription;

import java.io.DataInputStream;
import java.util.Vector;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import data.persistence.RemoteDataBinary;
import dev.Logger;

public class GuiSelectionDatasetCustomize extends GuiSelection {
	public GuiSelectionDatasetCustomize(GameCanvas gc, Graphics g) {
		super("Datensatz", gc, g);

		setOverflowType(OVERFLOW_TYPE_ENDLESS);

		Item[] items = null;

		Vector v = new Vector();
		String[] lastTimeSelected = RemoteDataBinary.getCustomLinien();
		for (int i = 0; i < lastTimeSelected.length; i++) {
			v.addElement(lastTimeSelected[i]);
		}

		try {
			DataInputStream dis = new DataInputStream(getClass().getResourceAsStream("/res/bus.bin"));
			int anzahl = dis.readShort();
			items = new Item[anzahl + 1];

			for (int i = 0; i < anzahl; i++) {
				String name = dis.readUTF();
				String richtung = dis.readUTF();
				if (v.contains(name)) {
					items[i] = new ItemCheckbox(name, richtung, true);
				} else {
					items[i] = new ItemCheckbox(name, richtung);
				}
			}
			dis.close();
		} catch (Exception e) {
			Logger.fatalException(GuiSelectionDatasetCustomize.class, e, "akjsiss");
		}

		items[items.length - 1] = new ItemLocationDescription("---");

		newItems(items);

		selectedId = 0;
		firstId = getValidId(-1);

		setLeftFooterString("abbrechen", null);
		setRightFooterString("aktivieren", null);
	}

	public boolean keyPressed(int keyCode, Canvas canvas) {
		if (selectedId != GuiSelection.SELECTION_NONE) {
			if (getItems()[selectedId].handleKey(keyCode)) {
				if (isVisible()) {
					drawMain(getGameCanvasGraphics(), true);
				}
				return true;
			}
		}
		return super.keyPressed(keyCode, canvas);
	}

	public String[] getSelectedLinien() {
		Vector v = new Vector();
		Item[] items = getItems();
		for (int i = 0; i < items.length; i++) {
			Item item = (Item) items[i];
			if (item instanceof ItemCheckbox) {
				ItemCheckbox itemCheckbox = (ItemCheckbox) item;
				if (itemCheckbox.isChecked()) {
					v.addElement(itemCheckbox.getCaption());
				}
			}
		}
		String[] s = new String[v.size()];
		v.copyInto(s);
		return s;
	}

}
