// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.FontMetrics;
import gui.Manager;
import gui.items.Item;
import gui.items.ItemAbout;
import gui.items.ItemRadiogroup;
import gui.items.RadioGroupManager;
import gui.utils.GfxUtils;

import java.util.Vector;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import utils.StringUtils;
import control.locale.Locale;
import data.persistence.RemoteDataBinary;
import data.persistence.UserSettingsInt;

public class GuiSelectionDatasetSelect extends GuiSelection {

	public final static int ITEM_ID_DATASET_SMALL = 50;
	public final static int ITEM_ID_DATASET_BIG = 51;
	public final static int ITEM_ID_DATASET_INDIVIDUELL = 52;
	public final RadioGroupManager radioGroupManager;

	public GuiSelectionDatasetSelect(GameCanvas gc, Graphics g) {
		super("Optionen", gc, g);

		Vector v = new Vector();

		radioGroupManager = new RadioGroupManager(UserSettingsInt.get(UserSettingsInt.SETTINGS_ID_DATENSATZ_TYP));
		v.addElement(new ItemRadiogroup(ITEM_ID_DATASET_SMALL, "keine Busse", radioGroupManager));
		v.addElement(new ItemRadiogroup(ITEM_ID_DATASET_BIG, "alle Busse", radioGroupManager));
		v.addElement(new ItemRadiogroup(ITEM_ID_DATASET_INDIVIDUELL, "individuell", radioGroupManager));
		
		v.addElement(new ItemAbout("individuelle Linien:", false));
		String[] s = RemoteDataBinary.getCustomLinien();
		if (s.length == 0) {
			v.addElement(new ItemAbout("keine ausgewählt", true));
		} else {
			String oneLine = StringUtils.arrayToString(s, ", ");
			String[] output = GfxUtils.splitupInLines(FontMetrics.getFont(Manager.ITEMS_FONT_SIZE), new String[] {oneLine}, ItemAbout.getWidthEingerueckt());
			for (int i = 0; i < output.length; i++) {
				v.addElement(new ItemAbout(output[i], true));
			}
		}
		

		Item[] items = new Item[v.size()];

		v.copyInto(items);

		setOverflowType(OVERFLOW_TYPE_NONE);
		
		newItems(items);

		setLeftFooterString(Locale.Back, null);
	}

	public Item getSelected() {
		return getItems()[selectedId];
	}

	public boolean keyPressed(int keyCode, Canvas canvas) {
		if (selectedId != GuiSelection.SELECTION_NONE) {
			if (getSelected().handleKey(keyCode)) {
				if (isVisible()) {
					drawMain(getGameCanvasGraphics(), true);
				}
				return true;
			}
		}
		return super.keyPressed(keyCode, canvas);
	}
	
	public boolean pointerPressed(int x, int y) {
		Item item = getItemAtPos(x, y);
		if (item != null && item instanceof ItemAbout) {
			Manager.datasetSelectWrapper(ITEM_ID_DATASET_INDIVIDUELL);
			return false;
		} else {
			return super.pointerPressed(x, y);
		}
	}
}
