// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.Color;
import gui.FontMetrics;
import gui.IOverlayable;
import gui.Manager;
import gui.items.Item;
import gui.items.ItemDayTime;
import gui.items.ItemDescription;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import model.DatumZeit;
import control.Global;
import control.Keys;
import dev.Logger;

public class GuiSelectionDayTime extends GuiSelection implements IOverlayable {
	private final int headerHeight;
	private final int innerSpace;
	private final int textHeight;
	private final String headerString;

	private final int info_height;
	private final int info_width;
	private final int info_y;
	private final int info_x;

	private final ItemDayTime itemDayTime;

	public GuiSelectionDayTime(GameCanvas gc, Graphics g, int currentTime) {
		super("Tag/Zeit", gc, g);

		setOverflowType(OVERFLOW_TYPE_NONE);
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);

		textHeight = f.getHeight();
		innerSpace = GfxUtils.scale(textHeight, 20);
		headerHeight = textHeight + 1;

		int info_x_space = GfxUtils.scale(Manager.SCREEN_WIDTH, 10);

		info_x = info_x_space;
		info_width = Manager.SCREEN_WIDTH - 2 * info_x_space;

		String[] headerStrings = { "Tag/Zeit wählen", "Tag/Zeit" };
		int no = GfxUtils.selectStringByMaxWidth(headerStrings, f, info_width, GfxUtils.WITH_SPACE);
		if (no == GfxUtils.NO_FIT) {
			Logger.error(this.getClass(), "Headerbreite zu klein um passenden Text hineinschreiben zu können");
			no = headerStrings.length - 1;
		}

		headerString = headerStrings[no];

		Item[] newitems = new Item[2];

		itemDayTime = new ItemDayTime(currentTime);
		newitems[0] = itemDayTime;

		String lines[] = { "" };
		newitems[1] = new ItemDescription(lines, ItemDescription.DESCRIPTION_DAYTIME);

		newItems(newitems);

		info_height = headerHeight + newitems.length * newitems[0].getHeight() + 2 * innerSpace + 1;

		info_y = (Manager.SCREEN_HEIGHT - info_height) / 2;
	}

	public int getInfoHeight() {
		return info_height;
	}

	public int getInfoWidth() {
		return info_width;
	}

	public int getInfoX() {
		return info_x;
	}

	public int getInfoY() {
		return info_y;
	}

	private final static int header_Background_Color = Color.RED;
	private final static int header_Font_Color = Color.WHITE;
	private final static int main_Background_Color = Color.WHITE;

	protected void drawHeader(Graphics g) {
		int info_add_y = 1;

		// Header
		Font fb = FontMetrics.getFont(Font.STYLE_BOLD, Manager.ITEMS_FONT_SIZE);
		g.setFont(fb);
		g.setColor(header_Background_Color);
		g.fillRect(info_x + 1, info_y + 1, info_width - 2, headerHeight - 1);
		g.setColor(header_Font_Color);
		GfxUtils.drawHCenteredText(g, headerString, info_y + info_add_y);

		// Rect
		g.setColor(Color.BLACK);
		g.drawRect(info_x, info_y, info_width - 1, info_height - 1);
	}

	synchronized protected void drawMain(Graphics g, boolean clearBackgroundAndRepaint) {
		int y_base = getMainY();
		int mainHeight = getMainHeight();

		visibleItems = 0;
		int y_add = innerSpace;

		g.setColor(main_Background_Color);
		g.fillRect(info_x + 1, y_base, info_width - 2, mainHeight - 1);

		y_add += itemDayTime.draw(this, g, info_x + Manager.MAIN_LEFT_SPACE, y_base + y_add, true, 0);
		visibleItems++;

		String label = DatumZeit.getWeekdayAsString(itemDayTime.getTime());

		int tag = DatumZeit.getDayType(itemDayTime.getTime());

		if (DatumZeit.isFeiertag(itemDayTime.getTime())) {
			label += " (Feiertag)";
		} else if (tag == Global.TAG_WOCHENTAG_FERIEN) {
			label += " (Ferien)";
		} else if (tag == Global.TAG_WOCHENTAG_SCHULE) {
			label += " (Schule)";
		}
		String lines[] = { label };
		y_add += new ItemDescription(lines, ItemDescription.DESCRIPTION_DAYTIME).draw(this, g, info_x + Manager.MAIN_LEFT_SPACE, y_base + y_add, false, 1);
		visibleItems++;

		if (clearBackgroundAndRepaint && isVisible()) {
			gc.repaint(info_x, y_base, info_width, mainHeight + Manager.MAIN_INNER_SPACE);
		}
	}

	protected void drawFooter(Graphics g) {
		// no footer
	}

	protected int getHeaderHeight() {
		return headerHeight;
	}

	protected int getMainY() {
		return info_y + getHeaderHeight();
	}

	protected int getMainHeight() {
		return info_height - getHeaderHeight();
	}

	public int getSelectedId() {
		return selectedId;
	}

	protected void clearMainBackground(Graphics g) {
		// empty
	}

	public int getTime() {
		return itemDayTime.getTime();
	}

	public boolean keyPressed(int keyCode, Canvas canvas) {
		Graphics g = getGameCanvasGraphics();
		if (itemDayTime.handleKey(keyCode)) {
			drawMain(g, true);
			return true;
		}
		return false;
	}

	public boolean pointerPressed(int x, int y) {
		if (y >= info_y && y <= info_y + info_height && x >= info_x && x <= info_x + info_width) {
			int position = itemDayTime.getPointerItem(x);
			if (position != ItemDayTime.POINTER_UNDEFINDED) {
				itemDayTime.setSelection(position);
				Graphics g = getGameCanvasGraphics();
				drawMain(g, true);
			}
			return false;
		} else {
			return true;
		}
	}

	public void resetTime() {
		itemDayTime.setTime(DatumZeit.Now());
		Graphics g = getGameCanvasGraphics();
		drawMain(g, true);
	}

	public void scrollUp() {
		Graphics g = getGameCanvasGraphics();
		itemDayTime.handleKey(Keys.getKeyCode(Keys.DOWN));
		drawMain(g, true);
	}

	public void scrollDown() {
		Graphics g = getGameCanvasGraphics();
		itemDayTime.handleKey(Keys.getKeyCode(Keys.UP));
		drawMain(g, true);
	}

}
