// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.BgStartScreen;
import gui.FontMetrics;
import gui.Imager;
import gui.Manager;
import gui.Point;
import gui.items.Item;
import gui.items.ItemHomeExecute;
import gui.items.ItemHomeHaltestellenPaar;
import gui.items.ItemSpace;
import gui.utils.GfxUtils;

import java.util.Enumeration;
import java.util.Vector;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;

import model.HaltestellenPaar;
import control.Device;
import control.Keys;
import data.persistence.Favoriten;
import data.persistence.Persistence;
import dev.Logger;

public class GuiSelectionHome extends GuiSelection {
	private int freeHaltestellenPaarSlots = 0;
	private int posHaltestellenPaarSlots;
	private boolean showFlipped = false;
	private BgStartScreen bgThread;

	private Item[] topItems;
	private ItemHomeHaltestellenPaar[] middleItems;
	private Item[] bottomItems;
	private final boolean viewDebug = false;

	private static GuiSelectionHome thisScreen = null;

	public void setBgThread(BgStartScreen bgThread) {
		this.bgThread = bgThread;
	}

	public GuiSelectionHome(GameCanvas gc, Graphics g) {
		super("push&ride", gc, g);
		thisScreen = this;
		setOverflowType(OVERFLOW_TYPE_ROUND);
		initStaticItems();
		middleItems = new ItemHomeHaltestellenPaar[0];
		getValidData();
	}

	public void getValidData() {
		updateDynamicItems();
		resetSelectionAndFirstPos();
		dataValid();
		if (bgThread != null) {
			synchronized (bgThread) {
				bgThread.notify();
			}
		}
	}

	public void flipHaltestellen() {
		showFlipped = !showFlipped;
		Item[] allItems = getItems();
		for (int i = 0; i < middleItems.length; i++) {
			ItemHomeHaltestellenPaar item = (ItemHomeHaltestellenPaar) allItems[i + posHaltestellenPaarSlots];
			item.flipHalstellenPaar();
		}
		redrawMain();
	}

	public void updateDynamicItems() {
		// Haltestellenpaare einholen
		Vector haltestellenPaare = new Vector();

		int freeSlots = freeHaltestellenPaarSlots;

		int lastFavoritSlot = -1;

		Vector favoriten = new Vector();

		if (freeSlots > 0) {
			// Favoriten vorbereiten
			HaltestellenPaar[] favoritenSlots = data.persistence.Favoriten.getFavoriten();
			for (int i = 0; i < favoritenSlots.length; i++) {
				if (favoritenSlots[i] != HaltestellenPaar.NULL) {
					favoriten.addElement(favoritenSlots[i]);
					if (viewDebug) {
						HaltestellenPaar hp = (HaltestellenPaar) favoriten.lastElement();
						System.out.println("all Fav: " + hp.getPaarAsString());
					}
				}
			}

			// Haltestellen aus Favoriten und LastUsed zustammenstellen
			Vector lastUsed = data.persistence.LastHaltestellenPaare.getLastHaltestellenPaare();
			Enumeration enumLastUsed = lastUsed.elements();
			for (int iSlot = 0; iSlot < freeSlots; iSlot++) {
				if (iSlot >= favoritenSlots.length || favoritenSlots[iSlot] == HaltestellenPaar.NULL) {
					while (enumLastUsed.hasMoreElements()) {
						HaltestellenPaar lastUsedHaltestellenPaar = (HaltestellenPaar) enumLastUsed.nextElement();
						if (!favoriten.contains(lastUsedHaltestellenPaar) && !favoriten.contains(lastUsedHaltestellenPaar.switchVonNach())) {
							if (!haltestellenPaare.contains(lastUsedHaltestellenPaar) && !haltestellenPaare.contains(lastUsedHaltestellenPaar.switchVonNach())) {
								if (showFlipped) {
									lastUsedHaltestellenPaar = lastUsedHaltestellenPaar.switchVonNach();
								}
								haltestellenPaare.addElement(lastUsedHaltestellenPaar);
								break;
							}
						}
					}
				} else {
					lastFavoritSlot = iSlot;
					haltestellenPaare.addElement(favoritenSlots[iSlot]);
				}
			}
		}

		HaltestellenPaar[] newHaltestellenPaare = new HaltestellenPaar[haltestellenPaare.size()];
		haltestellenPaare.copyInto(newHaltestellenPaare);

		// evtl. können sich lücken ergeben, die geschlossen werden, damit stimmt die Zuordnung nicht mehr und alles muss neu gespeichert werden. 
		if (lastFavoritSlot >= haltestellenPaare.size()) {
			Logger.warn(this, "lastFavoritSlot  >= haltestellenPaare.size()");
			for (int i = 0; i < newHaltestellenPaare.length; i++) {
				HaltestellenPaar haltestellenPaar = newHaltestellenPaare[i];
				if (favoriten.contains(haltestellenPaar)) {
					Favoriten.setFavorit(haltestellenPaar, i);
				} else {
					Favoriten.setFavorit(HaltestellenPaar.NULL, i);
				}
			}
		}

		ItemHomeHaltestellenPaar[] newItemHaltestellenPaare = new ItemHomeHaltestellenPaar[newHaltestellenPaare.length];

		// Elemente erstellen		
		for (int iSlot = 0; iSlot < newHaltestellenPaare.length; iSlot++) {
			int type;
			HaltestellenPaar haltestellenPaar = newHaltestellenPaare[iSlot];
			if (haltestellenPaar == HaltestellenPaar.NULL) {
				Logger.error(this, "Haltestellenpaar darf nicht Null sein (Fav)");
			}
			if (favoriten.contains(haltestellenPaar)) {
				type = ItemHomeHaltestellenPaar.TYP_FAVORITEN;
			} else {
				type = ItemHomeHaltestellenPaar.TYP_ZULETZTVERWENDETE;
			}
			// versuche ein altes Element weiterzuverwenden
			// falls es hier Probleme gibt einfach immer neu erstellen - durch den Routencache sollte es keine Performance-Probleme geben 
			ItemHomeHaltestellenPaar recoverItem = null;
			for (int oldItemNo = 0; oldItemNo < middleItems.length; oldItemNo++) {
				ItemHomeHaltestellenPaar oldItem = middleItems[oldItemNo];
				if (newHaltestellenPaare[iSlot].equals(oldItem.getHaltestellenPaar())) {
					newItemHaltestellenPaare[iSlot] = oldItem;
					oldItem.setTyp(type);
					recoverItem = oldItem;
					break;
				}
				if (newHaltestellenPaare[iSlot].equals(oldItem.getHaltestellenPaar().switchVonNach())) {
					newItemHaltestellenPaare[iSlot] = oldItem;
					oldItem.flipHalstellenPaar();
					oldItem.setTyp(type);
					recoverItem = oldItem;
					break;
				}
			}

			if (recoverItem != null) {
				newItemHaltestellenPaare[iSlot] = recoverItem;
			} else {
				newItemHaltestellenPaare[iSlot] = new ItemHomeHaltestellenPaar(newHaltestellenPaare[iSlot], type);
			}
		}

		middleItems = newItemHaltestellenPaare;

		setAllItems();

		if (middleItems.length > 0) {
			setLeftFooterString("Rückfahrt", Imager.getFooterIcon(Imager.ICON_FOOTER_RUECKFAHRT));
		} else {
			setLeftFooterString("", null);
		}
	}

	public void setAllItems() {
		Item[] items = new Item[topItems.length + bottomItems.length + middleItems.length];
		System.arraycopy(topItems, 0, items, 0, topItems.length);
		System.arraycopy(middleItems, 0, items, topItems.length, middleItems.length);
		System.arraycopy(bottomItems, 0, items, topItems.length + middleItems.length, bottomItems.length);
		setItems(items);
	}

	/**
	 * gibt die Items des Startscreens zurück
	 */
	private void initStaticItems() {

		int freespace = getMainHeight();
		Vector v = new Vector();

		// fest definierete Items
		Item item;
		item = new ItemHomeExecute(ItemHomeExecute.HOME_ITEM_NEW);
		v.addElement(item);
		freespace -= item.getHeight();

		item = new ItemSpace(Manager.HEADER_BASE_UNIT / 2);
		v.addElement(item);
		freespace -= item.getHeight();

		topItems = new Item[v.size()];

		v.copyInto(topItems);

		posHaltestellenPaarSlots = v.size();

		v = new Vector();

		item = new ItemSpace(Manager.HEADER_BASE_UNIT / 2);
		v.addElement(item);
		freespace -= item.getHeight();

		if (Device.isAndroid) {
			item = new ItemHomeExecute(ItemHomeExecute.HOME_ITEM_DATASET);
			v.addElement(item);
			freespace -= item.getHeight();
		} else {
			item = new ItemHomeExecute(ItemHomeExecute.HOME_ITEM_WEITERGEBEN);
			v.addElement(item);
			freespace -= item.getHeight();
		}

		item = new ItemHomeExecute(ItemHomeExecute.HOME_ITEM_OPTIONS);
		v.addElement(item);
		freespace -= item.getHeight();

		item = new ItemHomeExecute(ItemHomeExecute.HOME_ITEM_EXIT);
		v.addElement(item);
		freespace -= item.getHeight();

		bottomItems = new Item[v.size()];
		v.copyInto(bottomItems);

		calculateFreeHaltestellenPaarSlots();
	}

	public void calculateFreeHaltestellenPaarSlots() {
		int freespace = getMainHeight();
		for (int i = 0; i < topItems.length; i++) {
			freespace -= topItems[i].getHeight();
		}
		for (int i = 0; i < bottomItems.length; i++) {
			freespace -= bottomItems[i].getHeight();
		}
		freeHaltestellenPaarSlots = freespace / ItemHomeHaltestellenPaar.getItemHeight();
	}

	public Item getSelectedItem() {
		return getItems()[selectedId];
	}

	synchronized public ItemHomeHaltestellenPaar[] getAllItemHaltestellenPaare() {
		return middleItems;
	}

	synchronized public void redrawMain() {
		if (isVisible()) {
			drawMain(getGameCanvasGraphics(), true);
		}
	}

	protected int getHeaderHeight() {
		return Manager.HEADER_HEIGHT;
	}

	public boolean selectHaltestellenPaar(int no) {
		if ((no <= 0) || (no > middleItems.length)) {
			return false;
		}
		selectedId = topItems.length - 1 + no;
		return true;
	}

	public void activateScreenDemoMode() {
		Persistence.deleteRecordStore();
		updateDynamicItems();
	}

	public void drawTo(Graphics g) {
		if (!hasValidData()) {
			getValidData();
		}
		super.drawTo(g);
	}

	public static int getFreeHaltestellenPaareSlots() {
		if (thisScreen == null) {
			return 0;
		} else {
			return thisScreen.freeHaltestellenPaarSlots;
		}
	}

	synchronized protected void drawTime(Graphics g, boolean redraw) {
		if (timeState == STATE_SHOW_TIME) {
			super.drawTime(g, redraw);
		} else {
			return;
		}
	}

	protected void drawFooter(Graphics g) {
		Item item = getSelectedItem();
		//		System.out.println("drawFooter");
		if (item instanceof ItemHomeHaltestellenPaar) {
			timeState = STATE_SHOW_HINWEIS;
		} else {
			timeState = STATE_SHOW_TIME;
		}
		super.drawFooter(g);
		if (item instanceof ItemHomeHaltestellenPaar) {
			// falls ein Haltestellenpaar akiv ist evtl. den Hinweis und nicht die Zeit anzeigen
			if (timeState == STATE_SHOW_HINWEIS) {
				Font f = FontMetrics.getFont(Manager.FOOTER_FONT_STYLE, Manager.FOOTER_FONT_SIZE);
				g.setFont(f);

				g.setColor(Manager.FOOTER_COLOR_FONT);
				int xpos = Manager.SCREEN_WIDTH - Manager.FOOTER_LEFT_SPACE;
				int ypos = Manager.SCREEN_HEIGHT - f.getHeight() - Manager.FOOTER_LOWER_SPACE;

				String text = "Favorit";
				GfxUtils.drawAlignedText(text, g, new Point(xpos, ypos), Graphics.RIGHT | Graphics.TOP);
				Image image = Imager.getFooterIcon(Imager.ICON_FOOTER_FAVORITEN);
				if (image != null) {
					int xoff = f.stringWidth(text + " ");
					g.drawImage(image, xpos - xoff - image.getWidth(), ypos, Graphics.LEFT | Graphics.TOP);
				}
			}
		}
	}

	public boolean keyPressed(int keyCode, Canvas canvas) {
		if (Keys.isKey(keyCode, Keys.MENU_RIGHT)) {
			// prüfe ob ein Favorit an-/abgewählt wurde
			Item i = getSelectedItem();
			if (i instanceof ItemHomeHaltestellenPaar) {
				ItemHomeHaltestellenPaar item = (ItemHomeHaltestellenPaar) i;
				int typ = item.getTyp();
				int position = -1;
				for (int iPos = 0; iPos < middleItems.length; iPos++) {
					if (middleItems[iPos] == i) {
						position = iPos;
						break;
					}
				}
				int neuerTyp;
				if (typ == ItemHomeHaltestellenPaar.TYP_FAVORITEN) {
					neuerTyp = ItemHomeHaltestellenPaar.TYP_ZULETZTVERWENDETE;
					Favoriten.setFavorit(HaltestellenPaar.NULL, position);
				} else {
					neuerTyp = ItemHomeHaltestellenPaar.TYP_FAVORITEN;
					Favoriten.setFavorit(item.getHaltestellenPaar(), position);
				}
				//				System.out.println("neuerTyp" + neuerTyp);
				item.setTyp(neuerTyp);
				redrawMain();
				return true;
			}
		}
		boolean value = super.keyPressed(keyCode, canvas);
		repaintFooter();
		return value;
	}
	
	public boolean pointerPressed(int x, int y) {
		Item i = getItemAtPos(x, y);
		if (i instanceof ItemHomeHaltestellenPaar) {
			ItemHomeHaltestellenPaar iHaltestellenPaar = (ItemHomeHaltestellenPaar) i;
			if (x > iHaltestellenPaar.getLastIconDrawPos()) {
				int lastSelectedId = selectedId;
				selectedId = getItemIdAtPos(x, y);
				keyPressed(Keys.getKeyCode(Keys.MENU_RIGHT), null);
				selectedId = lastSelectedId;
				redrawMain();
				return false;
			}
		} 
		return super.pointerPressed(x, y);
	}

	private final int STATE_SHOW_TIME = 0;
	private final int STATE_SHOW_HINWEIS = 1;
	private int timeState = STATE_SHOW_HINWEIS;

}
