// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.SplitToHaltestellen;
import gui.utils.GfxUtils;

import java.util.Vector;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;

import model.AbstractLocation;
import model.Ort;
import control.Device;

public class GuiSelectionLocationHaltestellen extends GuiSelectionLocation {
	private Ort selectedOrt = null;

	public GuiSelectionLocationHaltestellen(int typ, GameCanvas gc, Graphics g) {
		super(typ, gc, g);

		updateAdditionalItems();
	}

	public static int getColorInside() {
		return GfxUtils.getBackgroundColorByPosWithDarkBg(0);
	}

	public static int getColorFrame() {
		return Color.gray2color(127);
	}

	public static int getXFrame() {
		return Manager.MAIN_LEFT_SPACE / 2;
	}

	/**
	 * holt sich die Daten vom vorhergehenden Screen
	 */
	private void getValidData() {
		AbstractLocation location = getPreviousGuiSelectionLocation(getGuiType()).getSelected();
		if (!(location instanceof Ort)) {
			System.err.println("GuiSelectionLocationHaltestellen - vorgänger hat keinen Ort markiert");
			return;
		}

		// holt sich die Haltestellen aus der vorherigen Ortsauswahl
		selectedOrt = (Ort) location;
		Vector alleHaltestellen = selectedOrt.getAlleHaltestellen();

		AbstractLocation locations[] = new AbstractLocation[alleHaltestellen.size()];
		alleHaltestellen.copyInto(locations);

		setInitItems(locations);

		resetStoredData();

		setAllItems(WITH_RESET);
	}

	/**
	 * zeichnet die komplette Gui neu
	 */
	public void drawTo(Graphics g) {
		if (!hasValidData()) {
			getValidData();
		}
		super.drawTo(g);
	}

	protected void drawHeader(Graphics g) {
		// Ort anzeigen, falls Gfx-Mode != High
		if (Device.getGfxQuality() != Device.GFX_QUALITY_HIGH && selectedOrt != null) {
			Font font = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
			g.setFont(font);
			//			g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
			int h = font.getHeight();
			int y = getMainY() - h;
			int xframe = GuiSelectionLocationHaltestellen.getXFrame();
			int xtext = Manager.MAIN_LEFT_SPACE;
			xtext = Math.max(xtext, xframe + h / 4);
			int textSpace = xtext - xframe;
			String ortsName = selectedOrt.getScreenName();
			int textWidth = font.stringWidth(ortsName);
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
			//			GfxUtils.fillFullRectangle(g, getHeaderHeight(), y - getHeaderHeight() + 10);

			g.setColor(GuiSelectionLocationHaltestellen.getColorFrame());
			g.drawLine(xframe, y - 1, xframe, y + h);
			g.drawLine(xframe, y - 1, xframe + 2 * textSpace + textWidth, y - 1);
			g.drawLine(xframe + 2 * textSpace + textWidth, y - 1, xframe + 2 * textSpace + textWidth, y + h);
			g.setColor(Manager.ITEMS_COLOR_FONT);
			GfxUtils.drawText(g, ortsName, xtext, y);
		} else {
			// Ort vom vorherigen Screen (Gfx: hight)
			Image i;
			if (getTyp() == GuiSelectionLocation.TYPE_VON_HALTESTELLE) {
				i = SplitToHaltestellen.getHeader1();
			} else {
				i = SplitToHaltestellen.getHeader2();
			}

			if (i != null) {
				g.drawImage(i, 0, getHeaderHeight(), 0);
			}
		}
	}

	protected void drawAdditionalMainStuff(Graphics g) {
		// zeichnet den grauen inneren Teil und den Rahmen 
		if (getOverflowType() == OVERFLOW_TYPE_ENDLESS) {
			int baseline = getMainY();
			int xframe = GuiSelectionLocationHaltestellen.getXFrame();
			g.setColor(GuiSelectionLocationHaltestellen.getColorInside());
			g.drawLine(xframe, baseline - 1, Manager.SCREEN_WIDTH - xframe, baseline - 1);
			g.setColor(GuiSelectionLocationHaltestellen.getColorFrame());
			g.drawLine(xframe, baseline, xframe, baseline);
			g.drawLine(Manager.SCREEN_WIDTH - xframe, baseline, Manager.SCREEN_WIDTH - xframe, baseline);
			baseline += visibleItems * getItemHeight();

			// unterer freier Platz
			int space = Manager.SCREEN_HEIGHT - Manager.FOOTER_HEIGHT - getKeyHeight() - baseline;

			g.setColor(GuiSelectionLocationHaltestellen.getColorInside());
			g.fillRect(xframe, baseline, Manager.SCREEN_WIDTH - 2 * xframe, space);
			g.setColor(GuiSelectionLocationHaltestellen.getColorFrame());
			g.drawLine(xframe, baseline, xframe, baseline + space);
			g.drawLine(Manager.SCREEN_WIDTH - xframe, baseline, Manager.SCREEN_WIDTH - xframe, baseline + space);
		} else {
			int baseline = getMainY();
			int xframe = GuiSelectionLocationHaltestellen.getXFrame();
			g.setColor(GuiSelectionLocationHaltestellen.getColorFrame());
			g.drawLine(xframe, baseline, Manager.SCREEN_WIDTH - xframe, baseline);
			baseline += getItems().length * getItemHeight();
			g.drawLine(xframe, baseline, Manager.SCREEN_WIDTH - xframe, baseline);
		}
	}
}
