// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.OptionManager;
import gui.items.Item;
import gui.items.ItemLocationDescription;
import gui.items.ItemOption;
import gui.items.ItemOptionToggle;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import control.Device;
import control.locale.Locale;
import dev.Logger;

public class GuiSelectionOptions extends GuiSelection {
	private static ItemOption[] optionItems;
	private boolean showHiddenOptions;
	private int counterDetectionsHiddenOptions;

	/**
	 * Items vom Optionscreen erstellen
	 */
	public GuiSelectionOptions(GameCanvas gc, Graphics g) {
		super("Optionen", gc, g);

		counterDetectionsHiddenOptions = 0;

		showHiddenOptions = false;

		ItemOption[] optionItems = OptionManager.getInitItems();

		setAllItems(optionItems);

		// FIXME: selectedId sollte eigentlich automatisch 0 sein
		if (optionItems.length > 0) {
			selectedId = 0;
		}

		setLeftFooterString(Locale.Back, null);
	}

	private void setAllItems(ItemOption[] optionItems) {
		GuiSelectionOptions.optionItems = optionItems;

		int mainHeight = getMainHeight();
		int height = 0;
		boolean overflow = false;
		for (int i = 0; i < optionItems.length; i++) {
			height += optionItems[i].getHeight();
			if (height > mainHeight) {
				overflow = true;
				break;
			}
		}
		if (overflow) {
			Item[] items = new Item[optionItems.length + 1];
			System.arraycopy(optionItems, 0, items, 0, optionItems.length);
			items[items.length - 1] = new ItemLocationDescription("---");
			setItems(items);
			setOverflowType(GuiSelection.OVERFLOW_TYPE_ENDLESS);
			firstId = items.length - 1;
			selectedId = 0;
		} else {
			setItems(optionItems);
			setOverflowType(GuiSelection.OVERFLOW_TYPE_ROUND);
		}
		dataValid();
	}

	public ItemOption getSelected() {
		if (selectedId == SELECTION_NONE) {
			return null;
		} else {
			return optionItems[selectedId];
		}
	}

	private void showHiddenOptions() {
		showHiddenOptions = true;
		ItemOption[] additionalOptionItems = OptionManager.getHiddenItems();
		ItemOption[] oldItems = optionItems;
		ItemOption[] newItems = new ItemOption[oldItems.length + additionalOptionItems.length];
		System.arraycopy(oldItems, 0, newItems, 0, oldItems.length);
		System.arraycopy(additionalOptionItems, 0, newItems, oldItems.length, additionalOptionItems.length);
		setAllItems(newItems);
		drawMain(getGameCanvasGraphics(), true);
	}

	public boolean keyPressed(int keyCode, Canvas canvas) {
		if (!showHiddenOptions) {
			if (keyCode == '9') {
				counterDetectionsHiddenOptions++;
				if (counterDetectionsHiddenOptions == 4) {
					showHiddenOptions();
				}
			}
		}

		if (selectedId != GuiSelection.SELECTION_NONE) {
			if (getSelected().handleKey(keyCode)) {
				if (isVisible()) {
					drawMain(getGameCanvasGraphics(), true);
				}
				return true;
			}
		}
		return super.keyPressed(keyCode, canvas);
	}

	private int cntHeaderPressed = 0;
	public boolean pointerPressed(int x, int y) {
		if (!showHiddenOptions) {
			int headerHeight = getHeaderHeight(); 
			if (y <= headerHeight) {
				cntHeaderPressed++;
				if (cntHeaderPressed == 10) {
					showHiddenOptions();
				}
			} else {
				cntHeaderPressed = 0;
			}
		}
		return super.pointerPressed(x,y);
	}

	public void prepare() {
		for (int i = 0; i < optionItems.length; i++) {
			ItemOption optionItem = optionItems[i];
			if (optionItem.getId() == OptionManager.ITEM_ID_GFX_SOFTWAREKEYBOARD) {
				if (Device.showSoftwareKeyboard()) {
					((ItemOptionToggle) optionItem).setSelectedIndex(1);
				} else {
					((ItemOptionToggle) optionItem).setSelectedIndex(0);
				}
			}
			if (optionItem.getId() == OptionManager.ITEM_ID_GFX_TOUCH_SIZE) {
				if (Device.increaseHeightForTouch()) {
					((ItemOptionToggle) optionItem).setSelectedIndex(1);
				} else {
					((ItemOptionToggle) optionItem).setSelectedIndex(0);
				}
			}
			if (optionItem.getId() == OptionManager.ITEM_ID_GFX_QUALITY) {
				int quality = Device.getGfxQuality();
				switch (quality) {
				case Device.GFX_QUALITY_HIGH:
					((ItemOptionToggle) optionItem).setSelectedIndex(2);
					break;
				case Device.GFX_QUALITY_MID:
					((ItemOptionToggle) optionItem).setSelectedIndex(1);
					break;
				case Device.GFX_QUALITY_LOW:
					((ItemOptionToggle) optionItem).setSelectedIndex(0);
					break;
				default:
					Logger.error(this, "unbekannte Gfx-Quality");
				}
			}
		}
	}
}
