// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.BgWorker;
import gui.FontMetrics;
import gui.Imager;
import gui.Manager;
import gui.elements.ElementVonNach;
import gui.items.Item;
import gui.items.ItemDescription;
import gui.items.ItemUebersicht;
import gui.items.ItemUebersichtLaufweg;
import gui.items.ItemUebersichtRechne;
import gui.utils.GfxUtils;

import java.util.Vector;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import model.DatumZeit;
import model.HaltestellenPaar;
import model.Route;
import control.PushandrideMidlet;
import control.locale.Locale;
import data.Data;

/*
 * Gui Seleciton besteht aus
 * - FirstHeader (normale Kopfzeile)
 * - SecondHeader (Anzeige Von, Nach)
 * - ThirdHeader (Tabellenkopf)
 */

public class GuiSelectionUebersicht extends GuiSelection {

	private HaltestellenPaar haltestellenPaar;
	
	private Vector ItemsVorneDummys;
	private Vector ItemsMitte;
	private Vector ItemsHintenDummys;

	// berechnet im Hintergrund Fahrzeiten für die ausgewählte Strecke
	private BgWorker bgWorker;

	private boolean calculatedOneAbove;
	private boolean calculatedOneBelow;

	// Positionen der Tabelle
	private static int POS_ITEMS[];
	private static int POS_LINES[];
	private static boolean tablePositionsInitialized;

	private int heute;

	private Vector visibleDays;

	private boolean keineGueltigeStrecke;

	private final ElementVonNach guiElementVonNach;

	private int firstQueryTime;

	private int addInBeginningCounter;

	/**
	 * gibt ein Element mit "[berechne]" zurück
	 */
	private Item[] getInitItems() {
		return new Item[] { new ItemUebersichtRechne(this) };
	}

	public GuiSelectionUebersicht(GameCanvas gc, Graphics g) {
		super("Fahrtwahl", gc, g, new Item[] {}, GuiSelection.OVERFLOW_TYPE_NONE);

		newItems(getInitItems());

		guiElementVonNach = new ElementVonNach();

		ItemsVorneDummys = new Vector();
		ItemsMitte = new Vector();
		ItemsHintenDummys = new Vector();
		visibleDays = new Vector();

		if (!tablePositionsInitialized) {
			initTablePositions();
		}

		setRightFooterIcon(Imager.getFooterIcon(Imager.ICON_FOOTER_ZEIT));
	}

	/**
	 * berechnet die Lienen und Textpositionen von der Tabelle
	 */
	private static void initTablePositions() {
		tablePositionsInitialized = true;

		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_STYLE, Manager.ITEMS_FONT_SIZE);

		POS_ITEMS = new int[4];
		POS_LINES = new int[5];

		int timeWidth = f.stringWidth("00:00");
		int umsteigenWidth = f.stringWidth("0");
		int minutenWidth = f.stringWidth("000`");

		int sum = 2 * timeWidth + umsteigenWidth + minutenWidth;

		int space = Manager.SCREEN_WIDTH - sum - 2 * Manager.MAIN_LEFT_SPACE;

		int maxSpace = GfxUtils.scale(sum, 100);

		space = Math.min(maxSpace, space);

		int onespace = space / 4;

		if ((onespace % 2) == 0) {
			onespace++;
		}

		int totalWidth = onespace * 4 + sum;

		int leftBase = (Manager.SCREEN_WIDTH - totalWidth) / 2;

		POS_LINES[0] = leftBase;
		POS_ITEMS[0] = POS_LINES[0] + onespace / 2;
		POS_LINES[1] = POS_ITEMS[0] + timeWidth + onespace / 2 + 1;
		POS_ITEMS[1] = POS_LINES[1] + onespace / 2;
		//		POS_LINES[2] = POS_ITEMS[1] + timeWidth + onespace / 2 + 1;
		//		POS_ITEMS[2] = POS_LINES[2] + umsteigenWidth + onespace / 2 ;
		POS_LINES[2] = POS_ITEMS[1] + timeWidth + onespace / 2 + 1;
		POS_ITEMS[2] = POS_LINES[2] + umsteigenWidth + onespace / 2 + 1;
		POS_LINES[3] = POS_ITEMS[2] + onespace / 2 + 1 - 1;
		POS_ITEMS[3] = POS_LINES[3] + minutenWidth + onespace / 2;
		POS_LINES[4] = POS_ITEMS[3] + onespace / 2;
	}

	/**
	 * gibt die X-Position des gewählten Eintrags zurück
	 */
	public static int getPosItems(int no) {
		if (!tablePositionsInitialized) {
			initTablePositions();
		}
		return POS_ITEMS[no];
	}

	/**
	 * gibt die X-Position der gewählten Tabellenlinie zurück
	 */
	public static int getPosLines(int no) {
		if (!tablePositionsInitialized) {
			initTablePositions();
		}
		return POS_LINES[no];
	}

	public void prepare(HaltestellenPaar haltestellenPaar) {
		prepare(haltestellenPaar, DatumZeit.Now());
	}

	/**
	 * erstellt die ursprünglichen Anzeigeelemente und startet den Thread zum
	 * Berechnen von Fahrmöglichkeiten
	 */
	public void prepare(HaltestellenPaar haltestellenPaar, int firstQueryTime) {
		if (!Data.isLoaded()) {
			PushandrideMidlet.getManager().activateWaitScreen("Daten werden geladen");
			while (!Data.isLoaded()) {
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					System.err.println(e.getMessage());
				}
			}
		}

		guiElementVonNach.setLocations(haltestellenPaar.getVon(), haltestellenPaar.getNach());

		this.firstQueryTime = firstQueryTime;

		if (hasValidData()) {
			bgWorker = new BgWorker(haltestellenPaar, this, bgWorker, firstQueryTime);
			bgWorker.start();
		} else {
			this.haltestellenPaar = haltestellenPaar;
			keineGueltigeStrecke = false;

			data.persistence.LastHaltestellenPaare.addLastHaltestellen(haltestellenPaar.getVon(), haltestellenPaar.getNach());

			ItemsVorneDummys.removeAllElements();
			ItemsMitte.removeAllElements();
			ItemsHintenDummys.removeAllElements();

			visibleDays.removeAllElements();

			newItems(getInitItems());
			ItemsHintenDummys.addElement(getItems()[0]);

			calculatedOneBelow = false;
			calculatedOneAbove = false;

			selectedId = 0;
			addInBeginningCounter = 0;
			heute = DatumZeit.getDay(DatumZeit.Now());

			bgWorker = new BgWorker(haltestellenPaar, this, null, firstQueryTime);
			bgWorker.start();
			
			dataValid();
		}
	}

	synchronized protected void drawHeader(Graphics g) {
		guiElementVonNach.drawTo(g, 0, 0);
	}

	
	private static boolean warnungSondertagAngezeigt = false;
	/**
	 * Methode, die vom bgWorker aufgerufen wird um gefunde Routen der Gui
	 * hinzuzufügen
	 * 
	 * return boolean - ob die Route erfolgreich hinzugefügt werden konnte
	 * (keine Fehler)
	 */
	synchronized public boolean addFinalRoute(Route finalRoute, int einfuegenPosition) {
		if (finalRoute.getErrorCode() != Route.STATUS_KEIN_FEHLER) {
			setKeineGueltigeStrecke(Locale.KeineVerbindungGefunden);
			return false;
		}

		// Handling von Sonderdaten
		int now = finalRoute.getAbfahrtszeit();
		if (!warnungSondertagAngezeigt) {
			if ((DatumZeit.getDayOfMonth(now) == 24 || DatumZeit.getDayOfMonth(now) == 31) && DatumZeit.getMonthOfYear(now) == 12) {
				warnungSondertagAngezeigt = true;
				Manager.showMessage("Achtung", "Am " + DatumZeit.toDateString(now) + " gelten Sonderfahrpläne bei der VVS. Bitte beachten Sie die entsprechenden Aushangfahrpläne.", GuiMessageScreen.TYPE_MESSAGE_OVERLAY);
			}
		}
		
		ItemUebersicht item;

		if (finalRoute.nurLaufwege()) {
			item = new ItemUebersichtLaufweg(finalRoute, this);
		} else {
			item = new ItemUebersicht(finalRoute, this);
		}

		switch (einfuegenPosition) {
		case EINFUEGEN_VORNE:
			if (ItemsVorneDummys.size() == 0) {
				System.err.println("ERROR: GuiUebersicht->addFinalRoute einfuegenVorne");
				return true;
			}
			replaceItem(ItemsVorneDummys.size() - 1, item);
			ItemsVorneDummys.removeElementAt(0);
			ItemsMitte.insertElementAt(item, 0);
			break;
		case EINFUEGEN_HINTEN:
			if (ItemsHintenDummys.size() == 0) {
				System.err.println("ERROR: GuiUebersicht->addFinalRoute einfuegenHinten");
				return true;
			}
			replaceItem(ItemsVorneDummys.size() + ItemsMitte.size(), item);
			ItemsHintenDummys.removeElementAt(0);
			ItemsMitte.addElement(item);
			break;
		}

		// gegebenenfalls das Datum dazuschreiben falls ein Tagesumbruch stattfindet
		int abfahrtsTag = DatumZeit.getDay(finalRoute.getAbfahrtszeit());
		if (abfahrtsTag != heute) {
			switch (einfuegenPosition) {
			case EINFUEGEN_VORNE:
				if ((visibleDays.size() == 0) || (((Integer) visibleDays.firstElement()).intValue() > abfahrtsTag)) {
					ItemUebersicht itemDanach = (ItemUebersicht) ItemsMitte.elementAt(1);
					itemDanach.setNewDay(itemDanach.getRoute().getAbfahrtszeit());
					visibleDays.insertElementAt(new Integer(abfahrtsTag), 0);
					checkSelectionPosPre();
				}
				break;
			case EINFUEGEN_HINTEN:
				if ((visibleDays.size() == 0) || (((Integer) visibleDays.lastElement()).intValue() < abfahrtsTag)) {
					item.setNewDay(abfahrtsTag);
					visibleDays.addElement(new Integer(abfahrtsTag));
					checkSelectionPosPre();
				}
				break;
			}
		}

		// falls keine User-Berechnungen mehr stattfinden berechne einige Routen selbst 
		if ((ItemsVorneDummys.size() == 0) && (ItemsHintenDummys.size() == 0)) {
			if (!calculatedOneBelow) {
				calculatedOneBelow = true;
				Item i = new ItemUebersichtRechne(this);
				addItem(i, EINFUEGEN_HINTEN);
				ItemsHintenDummys.addElement(i);
				bgWorker.addToQueue(false, EINFUEGEN_HINTEN);
			} else {
				if (!calculatedOneAbove) {
					calculatedOneAbove = true;
					Item i = new ItemUebersichtRechne(this);
					addItem(i, EINFUEGEN_VORNE);
					ItemsVorneDummys.addElement(i);
					bgWorker.addToQueue(false, EINFUEGEN_VORNE);
				} else {
					// auffüllen, bis der Bildschirm voll ist
					int maxNoItems = getMainHeight() / FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
					if (ItemsMitte.size() < maxNoItems) {
						Item i = new ItemUebersichtRechne(this);
						addItem(i, EINFUEGEN_HINTEN);
						ItemsHintenDummys.addElement(i);
						bgWorker.addToQueue(false, EINFUEGEN_HINTEN);
					}
				}
			}
		}
		if (isVisible()) {
			drawMain(getGameCanvasGraphics(), true);
			checkSelectionPosPost();
		}
		return true;
	}

	/**
	 * überprüft, ob die ausgewählte Haltestelle ganz unten liegt und aufgrund
	 * der Vergrößerung eines Elements oberhalb zu weit nach unten rutscht
	 * (nicht im Dragging-Mode)
	 */
	private void checkSelectionPosPre() {
		if (isDragging)
			return;
		//		System.out.println("KF : firstId " + firstId + "  visibleItems " + visibleItems + " selectedId " + selectedId);
		if ((visibleItems < getItems().length) && (selectedId == firstId + visibleItems - 2)) {
			firstId++;
		}
	}

	/**
	 * überprüft, ob die die Selection im gültigen Bereich liegt (nicht im
	 * Dragging-Mode)
	 */
	private void checkSelectionPosPost() {
		if (isDragging)
			return;

		int lastId = firstId + visibleItems - 1;
		//		System.out.println("firstId " + firstId + "  lastId " + lastId + "  visibleItems " + visibleItems + " selectedId " + selectedId);
		if (selectedId >= lastId) {
			firstId += selectedId - lastId + 1;
			if (isVisible()) {
				drawMain(getGameCanvasGraphics(), true);
			}
		}
	}

	private int getThirdHeaderY() {
		return getHeaderHeight() - getThirdHeaderHeight();
	}

	private int getThirdHeaderHeight() {
		int textHeight = FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
		return 1 * textHeight + Manager.MAIN_INNER_SPACE + 1;
	}

	protected int getMainY() {
		return getHeaderHeight();
	}

	synchronized public void drawTo(Graphics g) {
		drawThirdHeader(g);
		super.drawTo(g);
	}

	/**
	 * ThirdHeader ist die Beschrifung der Tabelle
	 */
	private void drawThirdHeader(Graphics g) {
		int y = getThirdHeaderY();

		// Hintergrund löschen
		g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
		GfxUtils.fillFullRectangle(g, y, getThirdHeaderHeight());

		g.setColor(Manager.ITEMS_COLOR_FONT);
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_STYLE, Manager.ITEMS_FONT_SIZE);
		g.setFont(f);

		g.setStrokeStyle(Graphics.DOTTED);

		// Vertikale Linien
		y += Manager.MAIN_INNER_SPACE;
		for (int i = 1; i < 4; i++) {
			int linePos = GuiSelectionUebersicht.getPosLines(i);
			g.drawLine(linePos, y, linePos, y + f.getHeight());
		}

		// Überschriftentext
		String[] caption = new String[] { "AB", "AN", "X", "MIN" };
		for (int i = 0; i < caption.length; i++) {
			GfxUtils.drawTextInBetween(g, caption[i], getPosLines(i), getPosLines(i + 1), y);
		}

		// horizontale Linie
		y = getHeaderHeight() - 1;
		g.drawLine(getPosLines(0), y, getPosLines(4), y);

		g.setStrokeStyle(Graphics.SOLID);
	}

	/**
	 * liefert die gesamt Header Höhe zurück (von allen drei Headern)
	 */
	public int getHeaderHeight() {
		return super.getHeaderHeight() + getThirdHeaderHeight();
	}

	protected void moveUp(boolean repaint) {
		if (!isGueltigeStrecke()) {
			return;
		}

		// passiert nur am Anfang
		if (selectedId == 0) {
			Item i = new ItemUebersichtRechne(this);
			addInBeginningCounter++;
			ItemsVorneDummys.addElement(i);
			addItem(i, EINFUEGEN_VORNE);

			bgWorker.addToQueue(true, EINFUEGEN_VORNE);
		}

		if (selectedId == firstId + 1) {
			Item i = new ItemUebersichtRechne(this);
			addInBeginningCounter++;
			ItemsVorneDummys.addElement(i);
			addItem(i, EINFUEGEN_VORNE);

			bgWorker.addToQueue(true, EINFUEGEN_VORNE);
		}

		super.moveUp(false);

		drawMain(getGameCanvasGraphics(), true);
	}

	protected void moveUpByDragging() {
		if (!isGueltigeStrecke()) {
			return;
		}

		if (firstId == 0) {
			Item i = new ItemUebersichtRechne(this);
			ItemsVorneDummys.addElement(i);
			addItem(i, EINFUEGEN_VORNE);

			bgWorker.addToQueue(true, EINFUEGEN_VORNE);
		} else {
			firstId--;
		}
		super.moveUp(false);
		drawMain(getGameCanvasGraphics(), true);
	}

	protected void moveDownByDragging() {
		if (!isGueltigeStrecke()) {
			return;
		}

		Item i = new ItemUebersichtRechne(this);
		ItemsHintenDummys.addElement(i);
		addItem(i, EINFUEGEN_HINTEN);

		bgWorker.addToQueue(true, EINFUEGEN_HINTEN);

		if (visibleItems != getItems().length) {
			firstId++;
		}
		super.moveDown(false);
		drawMain(getGameCanvasGraphics(), true);
	}

	protected void moveDown(boolean repaint) {
		if (!isGueltigeStrecke()) {
			return;
		}

		// passiert nur am Anfang
		if (selectedId == getItems().length - 1) {
			Item i = new ItemUebersichtRechne(this);
			ItemsHintenDummys.addElement(i);
			addItem(i, EINFUEGEN_HINTEN);
			bgWorker.addToQueue(true, EINFUEGEN_HINTEN);
		}

		if (selectedId == getLastId() - 1) {
			Item i = new ItemUebersichtRechne(this);
			ItemsHintenDummys.addElement(i);
			addItem(i, EINFUEGEN_HINTEN);
			bgWorker.addToQueue(true, EINFUEGEN_HINTEN);
		}

		super.moveDown(false);

		drawMain(getGameCanvasGraphics(), true);
	}

	public Route getSelectedRoute() {
		// Sonderbehandlung für den Fall, dass die aller erste berechnete Route angeklickt wird
		if (selectedId==1 && ItemsVorneDummys.size() == 0 && ItemsMitte.size() == 1) {
			return ((ItemUebersicht) ItemsMitte.elementAt(0)).getRoute();
		}

		if ((selectedId >= ItemsVorneDummys.size()) && (selectedId < ItemsVorneDummys.size() + ItemsMitte.size())) {
			return ((ItemUebersicht) ItemsMitte.elementAt(selectedId - ItemsVorneDummys.size())).getRoute();
		} else {
			return model.Route.NULL;
		}
	}

	public HaltestellenPaar getHaltestellenPaar() {
		return haltestellenPaar;
	}

	private void setKeineGueltigeStrecke(String[] message) {
		setKeineGueltigeStrecke(message, null);
	}

	private void setKeineGueltigeStrecke(String[] message, Item item) {
		keineGueltigeStrecke = true;
		Item[] items;
		if (item == null) {
			items = new Item[1];
		} else {
			items = new Item[2];
			items[0] = item;
		}

		items[items.length - 1] = new ItemDescription(message, ItemDescription.DESCRIPTION_UEBERSICHT);

		newItems(items);
		if (isVisible()) {
			drawMain(getGameCanvasGraphics(), true);
		}
	}

	public boolean isGueltigeStrecke() {
		return !keineGueltigeStrecke;
	}

	public int getFirstQueryTime() {
		return firstQueryTime;
	}


	private static boolean isDragging = false;

	public void scrollUp() {
		isDragging = true;
		moveUpByDragging();
	}

	public void scrollDown() {
		isDragging = true;
		moveDownByDragging();
	}
	
	public int getAddInBeginningCounter() {
		return addInBeginningCounter;
	}
	
}
