// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.Manager;
import gui.items.Item;
import gui.items.ItemLocationDescription;
import gui.items.ItemMultiline;
import gui.items.ItemSpace;
import gui.items.ItemWeitergebenExecute;

import java.util.Vector;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import control.Device;
import control.Global;
import control.PushandrideMidlet;
import control.locale.Locale;

public class GuiSelectionWeitergeben extends GuiSelection {
	public GuiSelectionWeitergeben(GameCanvas gc, Graphics g) {
		super("Weitergeben", gc, g);

		Item[] items = getInitItems();

		setAllItems(items);

		setLeftFooterString(Locale.Back, null);
	}
	
	public static Item[] getInitItems() {
		Vector itemVector = new Vector();

		itemVector.addElement(new ItemMultiline("Vielen Dank, dass du push&ride weitergeben willst! Du hast folgende Möglichkeiten:", false));
		
		itemVector.addElement(new ItemSpace(Manager.MAIN_INNER_SPACE));

		if (PushandrideMidlet.replicateStatus != Global.REPLICATE_NONE) {
			itemVector.addElement(new ItemWeitergebenExecute(ItemWeitergebenExecute.WEITERGEBEN_ITEM_SEND_VIA_BLUETOOTH));
		} else {
			itemVector.addElement(new ItemMultiline("keine Selbstreplikation", false));
		}

		itemVector.addElement(new ItemSpace(Manager.MAIN_INNER_SPACE));

		// Anzeigeelemenente auf dem Optionscreen
		if (Device.hasMessaging()) {
			// schickt per SMS
			itemVector.addElement(new ItemWeitergebenExecute(ItemWeitergebenExecute.WEITERGEBEN_ITEM_SEND_VIA_SMS));
		}

		Item[] items = new Item[itemVector.size()];
		itemVector.copyInto(items);

		return items;
	}


	private void setAllItems(Item[] items) {
		int mainHeight = getMainHeight();
		int height = 0;
		boolean overflow = false;
		for (int i = 0; i < items.length; i++) {
			height += items[i].getHeight();
			if (height > mainHeight) {
				overflow = true;
				break;
			}
		}
		if (overflow) {
			Item[] itemsOverflow = new Item[items.length + 1];
			System.arraycopy(items, 0, itemsOverflow, 0, items.length);
			itemsOverflow[itemsOverflow.length - 1] = new ItemLocationDescription("---");
			newItems(itemsOverflow);
			setOverflowType(GuiSelection.OVERFLOW_TYPE_ENDLESS);
		} else {
			newItems(items);
			setOverflowType(GuiSelection.OVERFLOW_TYPE_ROUND);
		}
		dataValid();
	}

	public Item getSelected() {
		if (selectedId == SELECTION_NONE) {
			return null;
		} else {
			return getItems()[selectedId];
		}
	}
}
