// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.FontMetrics;
import gui.Manager;
import gui.items.Item;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import control.Keys;

abstract public class GuiView extends Gui {
	private Item items[];
	private int totalItemHeight;
	private int y_scrollPos;

	public GuiView(String name, GameCanvas gc, Graphics g) {
		super(name, gc, g);
	}

	/**
	 * setzt das Clipping Fenster auf die Hauptelemente
	 */
	private void setMainClip(Graphics g) {
		g.setClip(0, getMainY(), Manager.SCREEN_WIDTH, getMainHeight());
	}

	protected int getMainY() {
		return Manager.HEADER_HEIGHT;
	}

	protected int getMainHeight() {
		return Manager.SCREEN_HEIGHT - getMainY() - Manager.FOOTER_HEIGHT;
	}

	/**
	 * zeichnet nur die Hauptelemente neu
	 * 
	 * @param clearBackground
	 */
	private void drawMain(Graphics g, boolean clearBackground) {
		int y_base = getMainY();

		int mainHeight = getMainHeight();

		if (clearBackground) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
			gui.utils.GfxUtils.fillFullRectangle(g, y_base, mainHeight);
		}

		int y_add = Manager.MAIN_INNER_SPACE - y_scrollPos;

		setMainClip(g);

		for (int i = 0; i < items.length; i++) {
			int itemHeight = items[i].getHeight();

			if (y_add + itemHeight < 0) {
				// Item nicht sichtbar
				y_add += itemHeight;
				continue;
			}

			items[i].draw(this, g, 0, y_base + y_add, i);

			y_add += itemHeight;

			if (y_add >= mainHeight) {
				// Item verschwindet im Footer
				break;
			}
		}

		GfxUtils.clearClip(g);

		if (clearBackground && isVisible()) {
			gc.repaint(0, y_base, Manager.SCREEN_WIDTH, mainHeight);
		}
	}

	/**
	 * zeichnet die komplette Gui bis auf den Header neu
	 */
	public void drawTo(Graphics g) {
		drawHeader(g);

		drawFooter(g);

		g.setColor(Manager.ITEMS_COLOR_BACKGROUND);

		int headerHeight = getHeaderHeight();

		// löschen des aktuellen Screens (bis auf den Header)
		gui.utils.GfxUtils.fillFullRectangle(g, headerHeight, Manager.SCREEN_HEIGHT - Manager.FOOTER_HEIGHT - headerHeight);

		// zeichenen des aktuellen Screens (bis auf den Header)
		drawMain(g, false);

		if (isVisible()) {
			gc.repaint();
		}
	}

	/*--------------------------------------------------
	 * Key code event handling
	 *-------------------------------------------------*/
	public boolean keyPressed(int keyCode, Canvas canvas) {
		if (!((Keys.isKey(keyCode, Keys.UP)) || (Keys.isKey(keyCode, Keys.DOWN)))){
			return false;
		}

		int oldScrollPos = y_scrollPos;

		if (Keys.isKey(keyCode, Keys.UP)) {
			y_scrollPos -= FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
			y_scrollPos = Math.max(0, y_scrollPos);
		}
		if (Keys.isKey(keyCode, Keys.DOWN)) {
			int mainHeight = getMainHeight();

			if (totalItemHeight <= mainHeight - Manager.MAIN_INNER_SPACE) {
				// Anzeige passt komplett auf den Bildschirm
				y_scrollPos = 0;
			} else {
				y_scrollPos += FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
				y_scrollPos = Math.min(y_scrollPos, totalItemHeight - mainHeight + 2 * Manager.MAIN_INNER_SPACE);
			}
		}

		if (oldScrollPos != y_scrollPos) {
			drawMain(getGameCanvasGraphics(), true);
		}
		
		return true;
	}
	
	protected void setItems(Item[] items) {
		this.items = items;

		// bestimme totalItemHeight;
		totalItemHeight = 0;
		for (int i = 0; i < items.length; i++) {
			totalItemHeight += ((Item) items[i]).getHeight();
		}

		y_scrollPos = 0;
	}
}
