// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.guis;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.launch.GameCanvasInstance;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import control.locale.Locale;

public class GuiWaitScreen extends Gui {
	private final static int COLOR_BACKGROUND = Color.WHITE;
	private final static int COLOR_TEXT = Color.BLACK;
	
	private final String output[];

	public GuiWaitScreen(String messageText) {
		super("push&ride", GameCanvasInstance.getInstance(),  GameCanvasInstance.returnGraphics());
		hideTime();

		String[] lines = utils.StringUtils.split(messageText, "\n");

		int maxwidth = GfxUtils.scale(Manager.SCREEN_WIDTH, 90);

		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		output = GfxUtils.splitupInLines(f, lines, maxwidth);
	}

	protected int getMainHeight() {
		return Manager.SCREEN_HEIGHT - getMainY();
	}

	
	/**
	 * zeichnet den Text auf dem Graphics
	 */
	private void drawMain(Graphics g) {
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
                
		g.setFont(f);

		g.setColor(COLOR_TEXT);

		// center x
		int maxwidth = GfxUtils.scale(Manager.SCREEN_WIDTH, 90);
		int posX = (Manager.SCREEN_WIDTH - maxwidth) / 2;

		int maxHeight = getMainHeight();
		
		int lineHeight = f.getHeight();
		int textHeight = lineHeight * (output.length + 1) + lineHeight / 2;
		
		int yAdd = 0;
				
		if (maxHeight >= textHeight) {
			// y zentriert
			yAdd = (maxHeight - textHeight) / 2;
			// hochgeschoben
			yAdd = yAdd / 2;
		}
		
		int yBase = getMainY();
		
		f = FontMetrics.getFont(Font.STYLE_BOLD, Manager.ITEMS_FONT_SIZE);
		g.setFont(f);

		GfxUtils.drawHCenteredText(g, Locale.BitteWarten, yBase + yAdd);
		yAdd += lineHeight;

		// halbe Leerzeile
		yAdd += lineHeight /2;
		
		f = FontMetrics.getFont(Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE);
		g.setFont(f);
		// einzelne Zeile horizontal zentrieren
		if (output.length == 1) {
			GfxUtils.drawHCenteredText(g, output[0], yBase + yAdd);
		} else {
			for (int i = 0; i < output.length; i++) {
				GfxUtils.drawText(g, output[i], posX, yBase + yAdd);
				yAdd += lineHeight;
			}
		}

	}

	public void drawTo(Graphics g) {
			g.setColor(COLOR_BACKGROUND);
            g.setFont(FontMetrics.getFont(Manager.ITEMS_FONT_SIZE));
                        
			GfxUtils.fillFullRectangle(g, 0, Manager.SCREEN_HEIGHT);

			drawHeader(g);

			drawMain(g);
		if (isVisible()) {
			gc.repaint();
			// da meinst das Programm nicht in einem eigenen Thread läuft muss Flushgraphics explizit aufgerufen werden
			gc.flushGraphics();
		}
	}
}
