// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

public class ItemAbout extends Item {

	private final String[] message;
	private final boolean eingerueckt;
	private final int height;

	protected static Font getDefaultFont() {
		return FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
	}
	
	protected static Font getItalicFont() {
		return FontMetrics.getFont(Font.STYLE_ITALIC, Manager.ITEMS_FONT_SIZE);
	}
	
	protected static Font getBoldFont() {
		return FontMetrics.getFont(Font.STYLE_BOLD, Manager.ITEMS_FONT_SIZE);
	}
	
	public ItemAbout(String[] message, boolean eingerueckt) {
		setSelectable(false);
		this.message = message;
		this.eingerueckt = eingerueckt;
		height = this.message.length * getDefaultFont().getHeight();
	}
	
	public ItemAbout(String message, boolean eingerueckt) {
		setSelectable(false);
		this.message = new String[]{message};
		this.eingerueckt = eingerueckt;
		height = this.message.length * getDefaultFont().getHeight();
	}
	
	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		Font f;
		if (eingerueckt) {
			f = getDefaultFont();
			x = 2* Manager.MAIN_LEFT_SPACE;
		} else {
			f = getItalicFont();
			x = Manager.MAIN_LEFT_SPACE;
		}
		g.setFont(f);
		
		g.setColor(Manager.ITEMS_COLOR_FONT);
		
		for (int i = 0; i < message.length; i++) {
			GfxUtils.drawText(g, message[i], x, y);
			y += f.getHeight();
		}

		return height;
	}
	
	public static int getWidthEingerueckt() {
		return Manager.SCREEN_WIDTH - 3 * Manager.MAIN_LEFT_SPACE;
	}

	public int getHeight() {
		return height;
	}

}
