// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import control.Device;
import control.Keys;

public class ItemCheckbox extends Item  {
	private final String linie;
	private final String richtung;
	private boolean checked;
	public ItemCheckbox(String linie, String richtung) {
		this.linie = linie;
		this.richtung = richtung;
		setSelectable(true);
		checked = false;
	}

	public ItemCheckbox(String linie, String richtung, boolean checked) {
		this.linie = linie;
		this.richtung = richtung;
		setSelectable(true);
		this.checked = checked;
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		int lineHeight = getLineHeight();
		
		if (selected) {
			g.setColor(gui.Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
		} else {
			g.setColor(GfxUtils.getBackgroundColorByPos(itemNo + 1));
		}		

		GfxUtils.fillFullRectangle(g, y, lineHeight);
		
		Font f;

		f = FontMetrics.getFont(Font.STYLE_PLAIN, ItemOption.OPTION_ITEM_FONT_SIZE);

		g.setFont(f);

		int fontHeight = f.getHeight(); 

		int leftOffset = GfxUtils.scale(fontHeight, 130);
		int circle = GfxUtils.scale(fontHeight, 75);

		if (isSelectable()) {
			g.setColor(Manager.ITEMS_COLOR_FONT);
		} else {
			g.setColor(Manager.ITEMS_COLOR_FONT_DISABLED);
		}

		int yBase = y;
		int xSpaceCircle = (fontHeight - circle) / 2;
		int xLinieOff = Math.max(f.stringWidth("123 "), f.stringWidth("N73 "));
		
		int fontTopSpace = (lineHeight - f.getHeight()) / 2;
		
		GfxUtils.drawText(g, linie, x + leftOffset, yBase + fontTopSpace);

		f = FontMetrics.getFont(Font.STYLE_PLAIN, Font.SIZE_SMALL);
		g.setFont(f);
		int baselinediff = FontMetrics.getBaseLine(ItemOption.OPTION_ITEM_FONT_SIZE) - FontMetrics.getBaseLine(Font.SIZE_SMALL);
		
		GfxUtils.drawText(g, richtung, x + leftOffset + xLinieOff, yBase + fontTopSpace + baselinediff);
		g.drawRoundRect(x + xSpaceCircle, yBase + (lineHeight - circle) / 2, circle, circle, GfxUtils.scaleWithMinimun(circle, 10, 1), GfxUtils.scaleWithMinimun(circle, 10, 1));
		if (isChecked()) {
			g.fillRoundRect(x + xSpaceCircle + 2, yBase + (lineHeight - circle) / 2 + 2, circle - 4, circle - 4, GfxUtils.scaleWithMinimun(circle, 10, 1), GfxUtils.scaleWithMinimun(circle, 10, 1));
		}

		return lineHeight;
	}

	public boolean isChecked() {
		return checked;
	}

	public int getHeight() {
		return getLineHeight();
	}

	private int getLineHeight() {
		if (Device.increaseHeightForTouch()) {
			return GfxUtils.getHeightForTouch(FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE));
		} else {
			return FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
		}
	}
	public boolean handleKey(int keyCode) {
		if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
			checked = !checked;
			return true;
		}
		return false;
	}

	public String getCaption() {
		return linie;
	}
}
