// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import model.DatumZeit;
import control.Keys;
import dev.Logger;

public class ItemDayTime extends Item {
	private int datumZeit;
	private final int height;
	int currentPos = 0;

	private static final int POS_START = 0;
	private static final int POS_END = 1;
	public static final int POINTER_UNDEFINDED = -1;
	public static final int POINTER_DAY = 0;
	public static final int POINTER_HOUR = 1;
	public static final int POINTER_MIN = 2;
	private int[][] pointerPos;

	public ItemDayTime(int datumZeit) {
		setSelectable(false);
		this.datumZeit = datumZeit;
		int fontHeight = FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
		height = fontHeight;
	}

	private String formatTime(int datumZeit) {
		String result = "";
		result = DatumZeit.toDateString(datumZeit).substring(0, 6) + " " + DatumZeit.toTimeString(datumZeit);
		return result;
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		g.setFont(f);
		g.setColor(Manager.ITEMS_COLOR_FONT);
		String currentTimeString = formatTime(datumZeit);

		int centeredX = (Manager.SCREEN_WIDTH - GfxUtils.getTextWidth(currentTimeString, f)) / 2;
		GfxUtils.drawText(g, currentTimeString, centeredX, y);

		int underlineStart = centeredX + GfxUtils.getTextWidth(currentTimeString.substring(0, currentPos), f);
		int underlineWidth = GfxUtils.getTextWidth(currentTimeString.substring(currentPos, currentPos + 2), f);

		int underlineY = y + f.getHeight() - FontMetrics.getBottomHeight(f.getSize()) - FontMetrics.getBottomSpace(f.getSize()) + 2;

		g.drawLine(underlineStart, underlineY, underlineStart + underlineWidth, underlineY);

		// pointer positions
		int add = GfxUtils.getTextWidth(":", f) / 2;
		pointerPos = new int[3][2];
		pointerPos[POINTER_DAY][POS_START] = -add + centeredX;
		pointerPos[POINTER_DAY][POS_END] = +add + centeredX + GfxUtils.getTextWidth(currentTimeString.substring(0, 2), f);
		pointerPos[POINTER_HOUR][POS_START] = -add + centeredX  + GfxUtils.getTextWidth(currentTimeString.substring(0, 7), f);
		pointerPos[POINTER_HOUR][POS_END] = +add + centeredX  + GfxUtils.getTextWidth(currentTimeString.substring(0, 9), f);
		pointerPos[POINTER_MIN][POS_START] = -add + centeredX  + GfxUtils.getTextWidth(currentTimeString.substring(0, 10), f);
		pointerPos[POINTER_MIN][POS_END] = +add + centeredX  + GfxUtils.getTextWidth(currentTimeString.substring(0, 12), f);

		return height;
	}

	public int getHeight() {
		return height;
	}

	public int getTime() {
		return datumZeit;
	}

	//	private static final int[] singlevalidPos = { 0, 1, 3, 4, 7, 8, 10, 11 };
	private static final int[] validPos = { 0, 7, 10 };

	public boolean handleKey(int key) {
		if (Keys.isKey(key, Keys.NUMBER)) {
			return true;
		}
		if (Keys.isKey(key, Keys.LEFT)) {
			int nextPos = currentPos;
			for (int i = 0; i < validPos.length; i++) {
				if (validPos[i] == currentPos) {
					i = i - 1;
					if (i < 0) {
						i = validPos.length - 1;
					}
					nextPos = validPos[i];
					currentPos = nextPos;
					break;
				}
			}
			return true;
		}
		if (Keys.isKey(key, Keys.RIGHT)) {
			int nextPos = currentPos;
			for (int i = 0; i < validPos.length; i++) {
				if (validPos[i] == currentPos) {
					i = i + 1;
					if (i == validPos.length) {
						i = 0;
					}
					nextPos = validPos[i];
					currentPos = nextPos;
					break;
				}
			}
			return true;
		}
		/*
		 * int newZeit = datumZeit; int year = DatumZeit.getYear(datumZeit); int
		 * month = DatumZeit.getMonthOfYear(datumZeit); int day =
		 * DatumZeit.getDayOfMonth(datumZeit); int hours =
		 * DatumZeit.getMinutes(datumZeit) / 60; int minutes =
		 * DatumZeit.getMinutes(datumZeit) % 60;
		 */
		if (Keys.isKey(key, Keys.UP) || Keys.isKey(key, Keys.DOWN)) {
			int delta;
			if (Keys.isKey(key, Keys.UP)) {
				delta = 1;
			} else {
				delta = -1;
			}
			if (currentPos == 0) {
				datumZeit += delta * DatumZeit.DAY_IN_MIN;
			}
			if (currentPos == 7) {
				datumZeit += delta * 60;
			}
			if (currentPos == 10) {
				datumZeit += delta;
			}
			return true;
		}
		if (Keys.isKey(key, Keys.DOWN)) {
			return true;
		}
		return false;
	}

	public void setTime(int datumZeit) {
		this.datumZeit = datumZeit;
	}

	public int getPointerItem(int x) {
		for (int i = 0; i < 3; i++) {
			if (x >= pointerPos[i][0] && x <= pointerPos[i][1]) {
				return i;
			}
		}
		return POINTER_UNDEFINDED;
	}

	public void setSelection(int pointerSelection) {
		switch (pointerSelection) {
		case POINTER_DAY:
			currentPos = 0;
			break;
		case POINTER_HOUR:
			currentPos = 7;
			break;
		case POINTER_MIN:
			currentPos = 10;
			break;
		default:
			Logger.warn(this, "unknown pointerSelection: " + pointerSelection);
		}
	}

}
