// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;


public class ItemDescription extends Item {
	public static final int DESCRIPTION_UEBERSICHT = 0;
	public static final int DESCRIPTION_DAYTIME = 1;
	private String[] message;
	private final int type;
	private final int height;

	public ItemDescription(String[] message, int type) {
		setSelectable(false);
		this.message = message;
		this.type = type;
		int fontHeight = FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
		switch (type) {
		case DESCRIPTION_UEBERSICHT:
		case DESCRIPTION_DAYTIME:
			height = message.length * fontHeight;
			break;
		default:
			height = -1;
		}
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		g.setFont(f);
		switch (type) {
		case DESCRIPTION_UEBERSICHT:
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
			GfxUtils.fillFullRectangle(g, y, getHeight());
			g.setColor(Manager.ITEMS_COLOR_FONT);
			for (int i = 0; i < message.length; i++) {
				GfxUtils.drawHCenteredText(g, message[i], y);
				y += f.getHeight();
			}
			break;
		case DESCRIPTION_DAYTIME:
			g.setColor(Manager.ITEMS_COLOR_FONT);
			for (int i = 0; i < message.length; i++) {
				GfxUtils.drawText(g, message[i],x,y);
				y += f.getHeight();
			}
			return height;
		}
		return 0;
	}

	public int getHeight() {
		return height;
	}

}
