// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;


import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxStringUtils;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import model.Haltestelle;

public class ItemDetailHaltestelle extends ItemDetail {

	private final Haltestelle haltestelle;
	private static int BASE_X = GfxUtils.scale(Manager.SCREEN_WIDTH, 5);
	private static int innerHorizSpace = GfxUtils.scaleWithMinimun(Manager.SCREEN_WIDTH, 2, 1);
	private final String anzeigeText;

	public static int get_center_xPos() {
		int xpos = BASE_X + 1 + innerHorizSpace;
		int TIMEWIDTH = (Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE)).stringWidth("00:00");
		return xpos + TIMEWIDTH / 2;
	}

	public ItemDetailHaltestelle(Haltestelle haltestelle) {
		this.haltestelle = haltestelle;
		Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE);

		int TIMEWIDTH = f.stringWidth("00:00");
		int text_xpos = BASE_X + 1 + innerHorizSpace + TIMEWIDTH + innerHorizSpace;

		anzeigeText = GfxStringUtils.getAdjustedLocation(f, this.haltestelle, Manager.SCREEN_WIDTH - text_xpos);
	}

	public int getHeight() {
		int textHeight = FontMetrics.getRealHeight(Manager.ITEMS_FONT_SIZE) - 1;
		int totalHeight =  2 * innerSpace - 1 + 2 + textHeight;
		return totalHeight;
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE);
		g.setFont(f);

		int totalHeight = getHeight();

		int COLOR_OUTLINE = Color.gray2color(128);
		int COLOR_BACKGROUND = Color.gray2color(220);

		// INNEN
		g.setColor(COLOR_BACKGROUND);
		int round = totalHeight / 2;
		GfxUtils.fillFullRectangleFromX(g, BASE_X + round, y, totalHeight);
		g.fillArc(BASE_X, y , totalHeight, totalHeight, 90, 180);

		// AUSSEN
		g.setColor(COLOR_OUTLINE);
		GfxUtils.drawFullLineHorizFromX(g, BASE_X + round, y);
		GfxUtils.drawFullLineHorizFromX(g, BASE_X + round, y + totalHeight);
		g.drawArc(BASE_X, y , totalHeight, totalHeight, 90, 180);

		// MITTE
		g.setColor(Color.BLACK);
		int xpos = BASE_X + 1 + innerHorizSpace;

		int TIMEWIDTH = f.stringWidth("00:00");
		xpos += TIMEWIDTH + innerHorizSpace;
		GfxUtils.drawText(g, anzeigeText, xpos, y + innerSpace + 1 - FontMetrics.getTopSpace(f.getSize()));
		return totalHeight;
	}

}
