// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxStringUtils;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import model.DatumZeit;
import model.RoutenAbschnitt;
import dev.Logger;

public class ItemDetailHaltestellenPaar extends ItemDetail {
	// FIXME: Umstellung auf ScaleX
	private static int BASE_X = GfxUtils.scale(Manager.SCREEN_WIDTH, 5);
	private static int innerHorizSpace = GfxUtils.scaleWithMinimun(Manager.SCREEN_WIDTH, 2, 1);
	private final String anzeigeTextHaltestellen[];
	private final String anzeigeTextZeiten[];
	private static int get_center_xpos = -1;

	public static int get_center_xPos() {
		if (get_center_xpos < 0) {
			int xpos = BASE_X + 1 + innerHorizSpace;
			int TIMEWIDTH = (Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE)).stringWidth("00:00");
			get_center_xpos = xpos + TIMEWIDTH / 2;
		}
		return get_center_xpos;
	}

	public ItemDetailHaltestellenPaar(RoutenAbschnitt routenAbschnitt, RoutenAbschnitt[] routenAbschnitte) {
		Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE);

		int noVon = -1;
		int noNach = -1;
		for (int i = 0; i < routenAbschnitte.length; i++) {
			if (noVon < 0) {
				if (routenAbschnitt.getVon() == routenAbschnitte[i].getVon()) {
					noVon = i;
				}
			}
			if (noVon >= 0) {
				if (routenAbschnitt.getNach() == routenAbschnitte[i].getNach()) {
					noNach = i;
					break;
				}
			}
		}

		if (noVon < 0 || noNach < 0) {
			Logger.error(this, "Fehler - konnte keine passenden Routenabschnitte finden");
			anzeigeTextHaltestellen = null;
			anzeigeTextZeiten = null;
			return;
		}

		int anzHaltestellen = 2 + noNach - noVon;

		anzeigeTextHaltestellen = new String[anzHaltestellen];
		anzeigeTextZeiten = new String[anzHaltestellen];
		int TIMEWIDTH = f.stringWidth("00:00");
		int text_xpos = BASE_X + 1 + innerHorizSpace + TIMEWIDTH + innerHorizSpace;

		for (int i = 0; i < anzHaltestellen - 1; i++) {
			anzeigeTextHaltestellen[i] = GfxStringUtils.getAdjustedLocation(f, routenAbschnitte[noVon + i].getVon(), Manager.SCREEN_WIDTH - text_xpos);
			anzeigeTextZeiten[i] = DatumZeit.toTimeString(routenAbschnitte[noVon + i].getAbfahrtsZeit());
		}
		anzeigeTextHaltestellen[anzeigeTextHaltestellen.length - 1] = GfxStringUtils.getAdjustedLocation(f, routenAbschnitte[noNach].getNach(), Manager.SCREEN_WIDTH - text_xpos);
		anzeigeTextZeiten[anzeigeTextHaltestellen.length - 1] = DatumZeit.toTimeString(routenAbschnitte[noNach].getAnkunftsZeit());
	}

	public int getHeight() {
		int anzItems = anzeigeTextHaltestellen.length;
		int textHeight = FontMetrics.getRealHeight(Manager.ITEMS_FONT_SIZE) - 1;
		int innerHeight = anzItems * 2 * innerSpace + (anzItems - 1) + anzItems * textHeight;
		int totalHeight = innerHeight + 1;
		return totalHeight;
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE);
		g.setFont(f);

		int anzItems = anzeigeTextHaltestellen.length;
		int textHeight = FontMetrics.getRealHeight(Manager.ITEMS_FONT_SIZE) - 1;
		int singleHeight = 2 * innerSpace + textHeight;
		int innerHeight = singleHeight * anzItems + (anzItems - 1);
		int totalHeight = innerHeight + 1;

		int round = Manager.SCREEN_WIDTH / 25;

		int COLOR_OUTLINE = Color.gray2color(128);
		int COLOR_BACKGROUND = Color.gray2color(220);

		// overdraw
		//		int overdraw = -textHeight / 2;
		int overdraw = 0;

		// INNEN
		g.setColor(COLOR_BACKGROUND);
		GfxUtils.fillFullRectangleFromX(g, BASE_X, y + overdraw, innerHeight - round - overdraw);
		GfxUtils.fillFullRectangleFromX(g, BASE_X + round, y + innerHeight - round, round);
		// FIXME: I have J2ME - need reimplementation of fillArc und drawArc
		g.fillArc(BASE_X, y + innerHeight - round * 2 + 1, round * 2 - 1, round * 2 - 1, 0, 360);

		// AUSSEN
		g.setColor(COLOR_OUTLINE);
		g.drawLine(BASE_X, y + overdraw, BASE_X, y + innerHeight - round);
		GfxUtils.drawFullLineHorizFromX(g, BASE_X + round, y + innerHeight);
		GfxUtils.doClip(g, BASE_X, y + innerHeight - round + 1, round, round);
		g.drawArc(BASE_X, y + innerHeight - round * 2, round * 2 - 1, round * 2 - 1, 0, 360);
		GfxUtils.restoreClip(g);

		// MITTE
		g.setColor(Color.BLACK);
		int xpos = BASE_X + 1 + innerHorizSpace;
		int center_X = get_center_xPos();
		int triangleHeight = Math.max(2, Manager.SCREEN_WIDTH / 240);
		int triangleWidth = triangleHeight * 2 + 1;
		triangleHeight += 1;

		int yPos = y;

		yPos += singleHeight;

		for (int i = 0; i < anzItems - 1; i++) {
			g.drawLine(xpos, yPos, (int) Math.floor(center_X - triangleWidth / 2.0 - 1.0), yPos);

			GfxUtils.drawFullLineHorizFromX(g, center_X + (int) Math.ceil(triangleWidth / 2.0 + 1.0), yPos);

			GfxUtils.downTriangle(g, center_X, yPos, triangleWidth, triangleHeight);

			yPos += singleHeight + 1;
		}

		yPos = y;
		yPos += innerSpace - FontMetrics.getTopSpace(f.getSize());
		int TIMEWIDTH = f.stringWidth("00:00");
		int xposHaltestelle = xpos + TIMEWIDTH + innerHorizSpace;
		for (int i = 0; i < anzItems; i++) {
			GfxUtils.drawText(g, anzeigeTextZeiten[i], xpos, yPos);
			GfxUtils.drawText(g, anzeigeTextHaltestellen[i], xposHaltestelle, yPos);
			yPos += singleHeight + 1;
		}

		return totalHeight;
	}

}
