// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import model.Fahrt;
import model.RoutenAbschnitt;
import model.Verkehrsmittel;
import dev.Logger;
// FIXME in ...Fahrt umbenennen
public class ItemDetailLinie extends ItemDetail {
	private final RoutenAbschnitt routenAbschnitt;
	private final Fahrt fahrt;

	public ItemDetailLinie(RoutenAbschnitt routenAbschnitt) {
		this.routenAbschnitt = routenAbschnitt;
		if (routenAbschnitt.isLaufweg()) {
			throw new RuntimeException("ItemDetailLinie darf nur mit einer Fahrt initialisiert werden (kein Laufweg)");
		}
		this.fahrt = (Fahrt) routenAbschnitt.getFahrt();
	}

	public int getHeight() {
		int textHeight = FontMetrics.getRealHeight(Manager.ITEMS_FONT_SIZE) - 1;
		return textHeight + innerSpace * 2 + 2;
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {

		Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE);
		Font fb = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Manager.ITEMS_FONT_SIZE);

		int textHeight = FontMetrics.getRealHeight(Manager.ITEMS_FONT_SIZE) - 1;
		int totalHeight = getHeight();
		int leftSpace = Math.min(1, textHeight / 20);

		int rightFontWidth = f.stringWidth("000'");
		int rightSpace = Manager.SCREEN_WIDTH / 100;

		int COLOR_OUTLINE = Color.gray2color(128);

		// Farbe in Abhängigkeit der Fortbewegungsmittels
		int COLOR_BACKGROUND = Color.rgb2color(65, 107, 41);

		switch (fahrt.getLinie().getVerkehrsmittel()) {
		case (Verkehrsmittel.SBAHN):
			COLOR_BACKGROUND = Color.rgb2color(63, 127, 51);
			break;
		case (Verkehrsmittel.UBAHN):
			COLOR_BACKGROUND = Color.rgb2color(0, 127, 175);
			break;
		case (Verkehrsmittel.RBAHN):
			COLOR_BACKGROUND = Color.rgb2color(90, 90, 90);
			break;
		case (Verkehrsmittel.BUS):
			COLOR_BACKGROUND = Color.rgb2color(173, 26, 31);
			break;
		case (Verkehrsmittel.TRAM):
			COLOR_BACKGROUND = Color.rgb2color(215, 54, 41);
			break;
		case (Verkehrsmittel.NACHTBUS):
			COLOR_BACKGROUND = Color.BLACK;
			break;
		default:
			Logger.error(this.getClass(), "Verkehrsmittel nicht existent");
			COLOR_BACKGROUND = Color.BLACK;
		}

		int COLOR_FONT = Color.WHITE;

		int BASE_X = Manager.SCREEN_WIDTH / 40;

		// FÜLLUNG
		g.setColor(COLOR_BACKGROUND);
		g.fillArc(BASE_X, y, totalHeight, totalHeight - 1, 90, 180);
		g.fillRect(BASE_X + totalHeight / 2, y + 1, Manager.SCREEN_WIDTH - BASE_X + totalHeight / 2, totalHeight - 2);

		/*
				// SCHATTIERUNG
				g.setGrayScale(150);
				g.drawArc(SPACE_X_HEADING + 1, y + 1, height, height, 90, 180);
				g.drawLine(SPACE_X_HEADING + height / 2, y + 1, Global.screenWidth, y + 1);
				g.drawLine(SPACE_X_HEADING + height / 2, y + height + 1, Global.screenWidth, y + height + 1);
		*/
		// RAND
		g.setColor(COLOR_OUTLINE);
		g.drawArc(BASE_X, y, totalHeight - 1, totalHeight - 1, 90, 180);
		g.drawLine(BASE_X + totalHeight / 2, y, Manager.SCREEN_WIDTH, y);
		g.drawLine(BASE_X + totalHeight / 2, y + totalHeight - 1, Manager.SCREEN_WIDTH, y + totalHeight - 1);

		// TEXT
		g.setColor(COLOR_FONT);
		GfxUtils.doClip(g, BASE_X, y, Manager.SCREEN_WIDTH - BASE_X - rightSpace - rightFontWidth, totalHeight);
		g.setFont(fb);
		String linienName = fahrt.getLinie().getName();
		if (linienName.equals("015")) {
			linienName = "15E";
		}
		GfxUtils.drawText(g, linienName, BASE_X + totalHeight / 2 + leftSpace, y + 1 + innerSpace - FontMetrics.getTopSpace(Manager.ITEMS_FONT_SIZE));
		int TextWidth = fb.stringWidth("XXX ");
		g.setFont(f);
		GfxUtils.drawText(g, fahrt.getScreenName(), BASE_X + totalHeight / 2 + leftSpace + TextWidth, y + 1 + innerSpace - FontMetrics.getTopSpace(Manager.ITEMS_FONT_SIZE));
		//		g.drawString(streckenAbschnitt.getFahrt().getLinie().getScreenName(), BASE_X + totalHeight / 2 + leftSpace, y + 1 + innerSpace - FontMetrics.getTopSpace(Font.SIZE_SMALL), 0);
		GfxUtils.restoreClip(g);
		GfxUtils.drawText(g, routenAbschnitt.getFahrZeit() + "'", Manager.SCREEN_WIDTH - rightSpace, y + 1 + innerSpace - FontMetrics.getTopSpace(Manager.ITEMS_FONT_SIZE), Graphics.TOP | Graphics.RIGHT);

		return (totalHeight);
	}

}
