// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import java.util.Vector;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import model.Route;
import utils.Sort;
import algo.pricing.ZonePricing;

public class ItemDetailPricing extends ItemDetail {

	private final Font font;
	private final String[] output;

	public ItemDetailPricing(Route route) {
		font = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);

		ZonePricing price = new ZonePricing(route);
		if (price.getStatus() > 0) {
			Vector priceText = new Vector();

			// Preis
			String preis = price.getPreisErwachsener();
			if(!preis.equals("-1")) {
				priceText.addElement("Preis Erw.: " + preis + "€");
			}

			// Info Text
			int maxWidth = Manager.SCREEN_WIDTH - 2 * Manager.MAIN_LEFT_SPACE;
			String[] infoText = GfxUtils.splitupInLines(font, new String[] { price.getInfoText() }, maxWidth);
			for (int i = 0; i < infoText.length; i++) {
				priceText.addElement(infoText[i]);
			}

			// Zonen
			boolean skipZoneOutput = false;
			int[] zonen = price.getZonen();
			boolean first = true;
			String zonenText = "";
			if (zonen.length > 0) {
				Sort.bubbleSortInt(zonen);
				for (int i = 0; i < zonen.length; i++) {
					// FIXME: VVS-Spezifisch
					if(zonen[i] > 1000) {
						skipZoneOutput = true;
						break;
					}
					if (first) {
						zonenText = zonen[i] + "";
						first = false;
					} else {
						zonenText += ", " + zonen[i];
					}
				}
				if (!skipZoneOutput) {
					priceText.addElement(zonenText);
				}
			}
			
			// Ohne Gewähr
			priceText.addElement("Ohne Gewähr");

			output = new String[priceText.size()];
			priceText.copyInto(output);

		} else {
			output = new String[0];
			return;
		}
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		int line = 0;
		g.setFont(font);
		g.setColor(Color.BLACK);
		int textHeight = font.getHeight();
		for (int i = 0; i < output.length; i++) {
			g.drawString(output[i], Manager.SCREEN_WIDTH - Manager.MAIN_LEFT_SPACE, y + Manager.MAIN_INNER_SPACE + line++ * textHeight, Graphics.RIGHT | Graphics.TOP);
		}
		return getHeight();
	}

	public int getHeight() {
		return output.length * font.getHeight() + Manager.MAIN_INNER_SPACE;
	}
}
