// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.Color;
import gui.FontMetrics;
import gui.Imager;
import gui.Manager;
import gui.Point;
import gui.guis.Gui;
import gui.launch.GameCanvasInstance;
import gui.utils.GfxStringUtils;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import model.DatumZeit;
import model.Haltestelle;
import model.HaltestellenPaar;
import model.Route;
import control.Global;

public class ItemHomeHaltestellenPaar extends ItemHome {

	public static final int TYP_UNDEFINED = -1;
	public static final int TYP_FAVORITEN = 0;
	public static final int TYP_ZULETZTVERWENDETE = 1;

	private final HaltestellenPaar[] haltestellenPaar = new HaltestellenPaar[2];
	private final boolean[] hasTime = { false, false };
	private final int[] nextTimeAbfahrt = new int[2];
	private final int[] nextTimeAnkunft = new int[2];
	private int currentState;
	private static final int STATE_NORMAL = 0;
	private static final int STATE_REVERSED = 1;

	private int typ;

	public ItemHomeHaltestellenPaar(HaltestellenPaar haltestellenPaar, int typ) {
		super(haltestellenPaar.getVon().getFullShortName() + " > " + haltestellenPaar.getNach().getFullShortName());
		this.haltestellenPaar[STATE_NORMAL] = haltestellenPaar;
		this.haltestellenPaar[STATE_REVERSED] = haltestellenPaar.switchVonNach();
		this.typ = typ;
		setIcon(typ);
	}

	public void setInvalidTime () {
		hasTime[0] = false;
		hasTime[1] = false;
	}

	private void setIcon(int typ) {
		if (typ == TYP_FAVORITEN) {
			setIcons(Imager.getIcon(Imager.ICON_FAVORIT1, false), Imager.getIcon(Imager.ICON_FAVORIT1, true), Imager.getIcon(Imager.ICON_FAVORIT1, false, true));
		} else {
			if (GameCanvasInstance.getInstance().hasPointerEvents()) {
				setIcons(Imager.getIcon(Imager.ICON_FAVORIT0, false), Imager.getIcon(Imager.ICON_FAVORIT0, true), Imager.getIcon(Imager.ICON_FAVORIT0, false, true));
			} else {
				setIcons(null, null,null);
			}
		}
	}

	/**
	 * gibt zurück ob es eine LetzteFahrt, Favorit oder Smart ist
	 * (ItemStartHaltestellenPaar.TYP_XXX)
	 */
	public int getTyp() {
		return typ;
	}

	/**
	 * setzt den Typ: LetzteFahrt, Favorit oder Smart
	 * (ItemStartHaltestellenPaar.TYP_XXX)
	 */
	public void setTyp(int typ) {
		this.typ = typ;
		setIcon(typ);
	}

	public HaltestellenPaar getHaltestellenPaar() {
		return haltestellenPaar[currentState];
	}

	public void flipHalstellenPaar() {
		currentState = otherState();
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		Font f = FontMetrics.getFont(gui.Manager.ITEMS_FONT_STYLE, gui.Manager.ITEMS_FONT_SIZE);
		g.setFont(f);

		int itemHeight = getHeight();
		int textHeight = f.getHeight();

		// Hintergrund

		if (selected) {
			g.setColor(gui.Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
			GfxUtils.fillFullRectangle(g, y, itemHeight);
		} else {
//			int xoff = Manager.MAIN_LEFT_SPACE / 2;
			int xoff = 0;
			
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
			g.fillRect(0, y, xoff, itemHeight);
			
			g.setColor(GfxUtils.getBackgroundColorByPos(itemNo + 1, true));
			
			GfxUtils.fillFullRectangleFromX(g, xoff, y, textHeight);
			GfxUtils.fillFullRectangleFromX(g, xoff, y + textHeight, textHeight);
		}

		// Hintergrund - Icon
		drawIcon(g, y + textHeight / 2, selected, itemNo+1);


		int timewidth = f.stringWidth("00:00");

		g.setColor(Manager.ITEMS_COLOR_FONT);

		// Zeit
		int center_X = x + timewidth / 2;
		int triangleHeight = Math.max(2, Manager.SCREEN_WIDTH / 240);
		int triangleWidth = triangleHeight * 2 + 1;
		triangleHeight += 1;

		for (int i = 0; i < 2; i++) {
			GfxUtils.downTriangle(g, center_X, y + textHeight, triangleWidth, triangleHeight);
		}

		
		if (hasTime()) {
			GfxUtils.drawAlignedText(DatumZeit.toTimeString(nextTimeAbfahrt[currentState]), g, new Point(x, y), Graphics.LEFT | Graphics.TOP);
			GfxUtils.drawAlignedText(DatumZeit.toTimeString(nextTimeAnkunft[currentState]), g, new Point(x, y + textHeight), Graphics.LEFT | Graphics.TOP);
		} else {
//			int xoff = f.stringWidth("00");
			int x2 = f.stringWidth("00:00");
			
			int yoff = FontMetrics.getTopSpace(Manager.ITEMS_FONT_SIZE);
			int yadd = FontMetrics.getTopHeight(Manager.ITEMS_FONT_SIZE) + FontMetrics.getMidHeight(Manager.ITEMS_FONT_SIZE);
			
			g.setColor(Color.WHITE);

			g.fillRect(x, y + yoff, x2, yadd);
			g.fillRect(x, y + yoff + textHeight, x2, yadd);
		}

		g.setColor(Manager.ITEMS_COLOR_FONT);

		// Haltestellennamen
		Haltestelle[] haltestellen = { haltestellenPaar[currentState].getVon(), haltestellenPaar[currentState].getNach() };

		int text_x_pos = x + timewidth + GfxUtils.scaleWithMinimun(textHeight, 20, 1);
		int maxWidth = Manager.SCREEN_WIDTH - text_x_pos - Manager.MAIN_LEFT_SPACE;
		Image icon = getIcon();
		if (icon != null) {
			maxWidth -= icon.getWidth();
		}
		
		for (int i = 0; i < 2; i++) {
			String haltestellenName = GfxStringUtils.getAdjustedLocation(f, haltestellen[i], maxWidth);
			GfxUtils.drawAlignedText(haltestellenName, g, new Point(text_x_pos, y + i * textHeight), Graphics.LEFT | Graphics.TOP);
		}

		return itemHeight;
	}

	public int getHeight() {
		return ItemHomeHaltestellenPaar.getItemHeight();
	}

	public static int getItemHeight() {
		return 2 * FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
	}

	/**
	 * 
	 * @param route
	 * @return ob ein passendes Haltestellenpaar enthalten ist und gleichzeitig
	 *         aktiviert ist
	 */
	public boolean tryToUpdateNaechsteFahrmoeglichkeit(Route route, HaltestellenPaar paar) {
		for (int i = 0; i < 2; i++) {
			if (haltestellenPaar[i].equals(paar)) {
				if (route.getErrorCode() == Route.STATUS_KEIN_FEHLER) {
					nextTimeAbfahrt[i] = route.getAbfahrtszeit();
					nextTimeAnkunft[i] = route.getAnkunftszeit();
				} else {
					nextTimeAbfahrt[i] = Global.TIME_ERROR;
					nextTimeAnkunft[i] = Global.TIME_ERROR;
				}
				hasTime[i] = true;
				if (i == currentState) {
					return true;
				} else {
					return false;
				}
			}
		}
		System.out.println("!!! FOUND");
		return false;
	}

	public boolean hasTime() {
		return hasTime[currentState];
	}

	private int otherState() {
		if (currentState == STATE_NORMAL) {
			return STATE_REVERSED;
		} else {
			return STATE_NORMAL;
		}
	}

	public boolean hasReversedTime() {
		return hasTime[otherState()];
	}

	public int getAbfahrtsZeit() {
		return nextTimeAbfahrt[currentState];
	}

	public int getAbfahrtsZeitReversed() {
		return nextTimeAbfahrt[otherState()];
	}
}
