// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.Color;
import gui.FontMetrics;
import gui.Imager;
import gui.Manager;
import gui.guis.Gui;
import gui.guis.GuiSelectionLocationHaltestellen;
import gui.utils.GfxUtils;
import gui.utils.IntelligentKey;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import model.AbstractLocation;
import model.Haltestelle;
import model.Ort;
import control.Device;

public class ItemLocation extends Item {
	private short lastIntelligentKeyMatchPos;
	private IntelligentKey intelligentKey;

	protected AbstractLocation location;

	public ItemLocation(AbstractLocation location, IntelligentKey intelligentKey) {
		this.location = location;
		setSelectable(true);
		setIntelligentKeyMatch(IntelligentKey.PATTERN_NOT_SEARCHED);
		this.intelligentKey = intelligentKey;
	}

	public ItemLocation(AbstractLocation location, short lastIntelligentKeyMatchPos, IntelligentKey intelligentKey) {
		this.location = location;
		setSelectable(true);
		setIntelligentKeyMatch(lastIntelligentKeyMatchPos);
		this.intelligentKey = intelligentKey;
	}

	public void setIntelligentKeyMatch(short lastIntelligentKeyMatchPos) {
		this.lastIntelligentKeyMatchPos = lastIntelligentKeyMatchPos;
	}

	public short getIntelligentKeyMatchPos() {
		return lastIntelligentKeyMatchPos;
	}

	protected String getText() {
		return location.getScreenName();
	}

	protected int getUnselectedBgColor(int itemNo) {
		return GfxUtils.getBackgroundColorByPos(itemNo);
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		int bgColor;
		boolean breakItem = false;
		if ((this instanceof ItemLocationDescription) && (((ItemLocationDescription) this).getText().equals("---"))) {
			breakItem = true;
		}

		if (g == null) {
			return getHeight();
		}

		int itemHeight = getHeight();

		if (selected) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
			bgColor = Manager.ITEMS_COLOR_BACKGROUND_SELECTED;
		} else {
			g.setColor(getUnselectedBgColor(itemNo));
			bgColor = getUnselectedBgColor(itemNo);
		}

		if ((!isSelectable()) && (!breakItem)) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_DISABLED);
		}

		if ((parent instanceof GuiSelectionLocationHaltestellen)) {
			g.setColor(Color.WHITE);
			if ((!isSelectable()) && (!breakItem)) {
				g.setColor(Manager.ITEMS_COLOR_BACKGROUND_DISABLED);
			}
			GfxUtils.fillFullRectangle(g, y, itemHeight);
		} else {
			GfxUtils.fillFullRectangle(g, y, itemHeight);
		}

		if ((parent instanceof GuiSelectionLocationHaltestellen) && (isSelectable() || breakItem)) {
			if (selected) {
				g.setColor(Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
			} else {
				g.setColor(GfxUtils.getBackgroundColorByPosWithDarkBg(itemNo));
				bgColor = GfxUtils.getBackgroundColorByPosWithDarkBg(itemNo);
			}
			int xframe = GuiSelectionLocationHaltestellen.getXFrame();
			g.fillRect(xframe, y, Manager.SCREEN_WIDTH - 2 * xframe, itemHeight);
			g.setColor(GuiSelectionLocationHaltestellen.getColorFrame());
			g.drawLine(xframe, y, xframe, y + itemHeight);
			g.drawLine(Manager.SCREEN_WIDTH - xframe, y, Manager.SCREEN_WIDTH - xframe, y + itemHeight);
		}

		Font f;
		if (location instanceof Ort) {
			f = FontMetrics.getFont(Font.STYLE_BOLD, Manager.ITEMS_FONT_SIZE);
		} else {
			f = FontMetrics.getFont(Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE);
		}

		if (this instanceof ItemLocationShowAll) {
			f = FontMetrics.getFont(Font.STYLE_ITALIC, Manager.ITEMS_FONT_SIZE);
		}

		int yAddForTouch;
		if (Device.increaseHeightForTouch()) {
			yAddForTouch = (itemHeight - f.getHeight()) / 2;
		} else {
			yAddForTouch = 0;
		}

		if (location instanceof Haltestelle) {
			x += 10;
		}

		String prefix = "+ ";
		Font prefixFont = null;
		int prefixWidth = 0;

		if (location instanceof Ort) {
			prefixFont = FontMetrics.getFont(Font.STYLE_PLAIN, Manager.ITEMS_FONT_SIZE);
			prefixWidth = prefixFont.stringWidth(prefix);
			x = x / 2;
			x += prefixWidth;
		}

		String text = getText();

		if (lastIntelligentKeyMatchPos >= 0) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_INTELLIGENTKEYMATCH);
			int xpos = x + f.stringWidth(text.substring(0, lastIntelligentKeyMatchPos));
			int width = f.stringWidth(text.substring(lastIntelligentKeyMatchPos, lastIntelligentKeyMatchPos + intelligentKey.getKeysLength()));
			g.fillRect(xpos, y, width, itemHeight);
		}

		g.setColor(Manager.ITEMS_COLOR_FONT);
		if (!isSelectable()) {
			g.setColor(Manager.ITEMS_COLOR_FONT_DISABLED);
		}

		if ((location instanceof Ort) && (!(this instanceof ItemLocationShowAll))) {
			if (((Ort) location).getAlleHaltestellen().size() > 1) {
				g.setFont(prefixFont);
				GfxUtils.drawText(g, prefix, x - prefixWidth, y + yAddForTouch);
			}
		}

		g.setFont(f);
		if (breakItem) {
			GuiSelectionLocationHaltestellen.getColorFrame();
			g.setStrokeStyle(Graphics.DOTTED);

			int xframe = GuiSelectionLocationHaltestellen.getXFrame();
			g.setColor(GuiSelectionLocationHaltestellen.getColorFrame());
			g.drawLine(xframe, y + itemHeight / 2, Manager.SCREEN_WIDTH - xframe, y + itemHeight / 2);
			g.setStrokeStyle(Graphics.SOLID);
		} else {
			GfxUtils.drawText(g, text, x, y + yAddForTouch);
		}

		// Verkehrsmittelstring anzeigen
		if ((!(this instanceof ItemLocationDescription)) && (!(this instanceof ItemLocationShowAll))) {
			int[] verkehrsmittel = location.getVerkehrsmittelTypen();
			int xpos;

			if (parent instanceof GuiSelectionLocationHaltestellen) {
				xpos = Manager.SCREEN_WIDTH - GuiSelectionLocationHaltestellen.getXFrame() - GfxUtils.scale(Manager.SCREEN_WIDTH, 1) - 1;
			} else {
				xpos = Manager.SCREEN_WIDTH - GfxUtils.scale(Manager.SCREEN_WIDTH, 1);
			}

			for (int i = 0; i < verkehrsmittel.length; i++) {
				Image im;
				im = Imager.getAnschlussPic(verkehrsmittel[i], selected, bgColor);
				xpos -= im.getWidth();
				int yadd = (f.getHeight() - im.getHeight()) / 2;
				g.drawImage(im, xpos, y + yadd + yAddForTouch, Graphics.LEFT | Graphics.TOP);
			}
		}

		return itemHeight;
	}

	protected int getFreeTextWidthOfLastUsed() {
		int rightspace = GfxUtils.scale(Manager.SCREEN_WIDTH, 1);
		int[] verkehrsmittel = location.getVerkehrsmittelTypen();

		if (verkehrsmittel.length > 0) {
			Image im = Imager.getAnschlussPic(verkehrsmittel[0]);
			rightspace += im.getWidth() * verkehrsmittel.length;
		}
		int leftspace = Manager.MAIN_LEFT_SPACE;
		return Manager.SCREEN_WIDTH - leftspace - rightspace;
	}

	public int getHeight() {
		if (Device.increaseHeightForTouch()) {
			return GfxUtils.getHeightForTouch(FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE));
		} else {
			return FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
		}
	}

	public AbstractLocation getLocation() {
		return location;
	}

}
