// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import control.Device;

public class ItemMultiline extends Item {
	private Image icon = null;
	private Image iconSelected = null;
	private Image iconDark = null;
	private final String[] text;

	public ItemMultiline(String anzeigeName, boolean selectable) {
		int maxWidth = Manager.SCREEN_WIDTH - 2 * Manager.MAIN_LEFT_SPACE;
		Font f;
		f = FontMetrics.getFont(gui.Manager.ITEMS_FONT_STYLE, gui.Manager.ITEMS_FONT_SIZE);
		text = GfxUtils.splitupInLines(f, new String[]{anzeigeName}, maxWidth);
		setSelectable(selectable);
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {

		int itemHeight = getHeight();

		if (selected) {
			g.setColor(gui.Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
		} else {
			g.setColor(GfxUtils.getBackgroundColorByPos(itemNo));
		}
		GfxUtils.fillFullRectangle(g, y, itemHeight);

		Font f;

		f = FontMetrics.getFont(gui.Manager.ITEMS_FONT_STYLE, gui.Manager.ITEMS_FONT_SIZE);
		
		int yAddForTouch;
		yAddForTouch = (itemHeight - f.getHeight() * text.length) / 2;

		drawIcon(g, y, selected, itemNo);

		g.setFont(f);
		g.setColor(gui.Manager.ITEMS_COLOR_FONT);
		GfxUtils.drawText(g, text, x, y + yAddForTouch);

		return itemHeight;
	}

	public void drawIcon(Graphics g, int yTop, boolean selected, int itemNo) {
		if (icon != null && iconSelected != null && iconDark != null) {
			Image i;

			if (selected) {
				i = iconSelected;
			} else {
				if ((itemNo % 2) == 0) {
					i = icon;
				} else {
					if (Device.increaseHeightForTouch()) { 
						i = iconDark;
					} else {
						i = icon;
					}
				}
			}

			int ICONMAXWIDTH = 2 * FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);

			int iconPos = Manager.SCREEN_WIDTH - Manager.MAIN_LEFT_SPACE - (ICONMAXWIDTH - i.getWidth()) / 2;

			g.drawImage(i, iconPos, yTop + (getHeight() - i.getHeight()) / 2, Graphics.LEFT | Graphics.TOP);
		}
	}

	public int getHeight() {
		Font f;

		f = FontMetrics.getFont(gui.Manager.ITEMS_FONT_STYLE, gui.Manager.ITEMS_FONT_SIZE);

		int itemHeight = f.getHeight() * text.length;

		if (text.length == 1 && Device.increaseHeightForTouch() && isSelectable()) {
			itemHeight = itemHeight  + f.getHeight();
		}

		return itemHeight;
	}

	protected void setIcons(Image icon, Image iconSelected, Image iconDark) {
		this.icon = icon;
		this.iconSelected = iconSelected;
		this.iconDark = iconDark;
	}

	protected Image getIcon() {
		return this.icon;
	}

}

