// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.OptionManager;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import control.Device;
import control.Keys;

public class ItemOptionCall extends ItemOption {
	private final String caption;

	public ItemOptionCall(int id, String caption) {
		super(id);
		this.caption = caption;
		setSelectable(true);
	}

	public int getHeight() {
		if (Device.increaseHeightForTouch() && isSelectable()) {
			return 2 * getYSpace() + GfxUtils.getHeightForTouch(FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE));
		} else {
			return 2 * getYSpace() + FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
		}
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		int itemHeight = getHeight();

		g.setColor(Manager.ITEMS_COLOR_BACKGROUND);

		if (!isSelectable()) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_DISABLED);
		}

		if (selected) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
		}
		GfxUtils.fillFullRectangle(g, y, itemHeight);

		Font f;

		f = FontMetrics.getFont(ItemOption.OPTION_ITEM_FONT_STYLE, ItemOption.OPTION_ITEM_FONT_SIZE);

		g.setFont(f);

		if (isSelectable()) {
			g.setColor(Manager.ITEMS_COLOR_FONT);
		} else {
			g.setColor(Manager.ITEMS_COLOR_FONT_DISABLED);
		}

		// FIXME: bad style - override
		if (caption.equals("zurück")) {
			f = FontMetrics.getFont(ItemOption.OPTION_ITEM_FONT_STYLE | Font.STYLE_ITALIC, ItemOption.OPTION_ITEM_FONT_SIZE);
			g.setFont(f);
		}

		y += (itemHeight - f.getHeight()) / 2;

		GfxUtils.drawText(g, caption, x, y);

		return itemHeight;
	}

	public boolean handleKey(int keyCode) {
		if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
			OptionManager.handleOptionCall(getId());
			return true;
		}
		return false;
	}

	public boolean equals(Object o) {
		if (!(o instanceof ItemOption)) {
			return false;
		}
		return ((ItemOption) o).getId() == this.getId();
	}

}
