// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;


import java.util.Vector;

import utils.StringUtils;
import control.Global;
import control.locale.Locale;
import data.persistence.UserSettingsInt;

public class ItemOptionSelectLaufwegeZeit extends ItemOptionToggle {

	/**
	 * erstellt die Items für den Konstruktor
	 */
	private static String[] getInitItems() {
		Vector v = new Vector();
		for (int i = 4; i <= 30; ) {
			String factorStr = StringUtils.fillInt(i, '0', 2);
			factorStr = factorStr.charAt(0) + "." + factorStr.charAt(1);
			if ( i < 10) {
				v.addElement("schneller (x" + factorStr + ")");
			}
			if (i == 10) {
				v.addElement("normal (x" + factorStr + ")");
			}
			if (i > 10) {
				v.addElement("langsamer (x" + factorStr + ")");
			}
			
			if ( i < 20 ) {
				i++;
			} else {
				i += 5;
			}
		}
		
		String items[] = new String[v.size()];
		v.copyInto(items);
		return items;
	}
		
	public ItemOptionSelectLaufwegeZeit(int id) {
		super(id, Locale.Laufgeschwindigkeit, getInitItems(), 0, false);
		overrideSelectedIndex(getClosestEntry(Global.getLaufzeitFaktor()));
	}
	
	/**
	 * gibt den nächstgelegenen Index zum übergebenen Faktor zurück
	 */
	private int getClosestEntry(int factor) {
		String[] items = getItems();
		for (int i = 0; i < items.length; i++) {
			if (factor <= getValueOfEntry(i)) {
				return i;
			}
		}
		return items.length - 1;
	}
	
	/**
	 * liefert den Faktor aus dem übergebenen Index zurück 
	 */
	private int getValueOfEntry(int index) {
		String[] items = getItems();
		String item = items[index];
		return Integer.valueOf(item.charAt(item.length() - 4) + "" + item.charAt(item.length()-2)).intValue();
	}
	
	/**
	 * wird von der Superklasse aufgerufen, wenn sich der selektierte Eintrag ändert
	 */
	protected void OnChange() {
		int newValue = getValueOfEntry(getSelectedIndex());
		Global.setNewLaufzeitFaktor(newValue);
		UserSettingsInt.set(UserSettingsInt.SETTINGS_LAUFZEIT_FAKTOR, newValue);
	}

}
