// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.OptionManager;
import gui.Point;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import control.Keys;


public class ItemOptionToggle extends ItemOption {

	private final String caption;
	private final String[] items;
	private int selectedIndex;
	private final boolean wrap;

	private final static int INDEX_NOT_VISIBLE = -1;

	public void setSelectedIndex(int index) {
		if (items.length > 0) {
			selectedIndex = index % items.length;
		}
	}

	public ItemOptionToggle(int id, String caption, String[] items, int selectedIndex) {
		super(id);

		this.caption = caption;
		this.items = items;
		this.selectedIndex = selectedIndex;

		for (int i = 0; i < items.length; i++) {
			//System.out.println("items " + items[i]);
		}

		if (items.length <= selectedIndex) {
			selectedIndex = INDEX_NOT_VISIBLE;
		}

		wrap = true;
		
		setSelectable(true);
	}

	public ItemOptionToggle(int id, String caption, String[] items, int selectedIndex, boolean wrap) {
		super(id);

		this.caption = caption;
		this.items = items;
		this.selectedIndex = selectedIndex;

		for (int i = 0; i < items.length; i++) {
			//System.out.println("items " + items[i]);
		}

		if (items.length <= selectedIndex) {
			selectedIndex = INDEX_NOT_VISIBLE;
		}

		this.wrap = wrap;
		
		setSelectable(true);
	}

	public int getHeight() {
		return 2 * getYSpace() + 2 * FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		int itemHeight = getHeight();

		g.setColor(Manager.ITEMS_COLOR_BACKGROUND);

		if (!isSelectable()) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_DISABLED);
		}

		GfxUtils.fillFullRectangle(g, y, itemHeight);

		if (selected) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
			GfxUtils.fillFullRectangle(g, y + getYSpace(), 2 * FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE));
		}

		Font f;

		f = FontMetrics.getFont(ItemOption.OPTION_ITEM_FONT_STYLE, ItemOption.OPTION_ITEM_FONT_SIZE);

		g.setFont(f);

		if (isSelectable()) {
			g.setColor(Manager.ITEMS_COLOR_FONT);
		} else {
			g.setColor(Manager.ITEMS_COLOR_FONT_DISABLED);
		}

		// zeichnen der ersten Zeile (Beschreibung)
		y += getYSpace();
		GfxUtils.drawAlignedText(caption, g, new Point(x, y), Graphics.LEFT | Graphics.TOP);

		// zeichnen der zweiten Zeile (Auswahl)
		if (selectedIndex != INDEX_NOT_VISIBLE) {
			f = FontMetrics.getFont(Font.STYLE_PLAIN, ItemOption.OPTION_ITEM_FONT_SIZE);
			g.setFont(f);
			y += f.getHeight();
			GfxUtils.drawAlignedText(items[selectedIndex], g, new Point(Manager.SCREEN_WIDTH - x, y), Graphics.RIGHT | Graphics.TOP);
		}

		return itemHeight;
	}

	public boolean handleKey(int keyCode) {
		if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
			int itemLength = items.length;
			if (itemLength > 0) {
				// prüft den Überlauf
				if ((selectedIndex+1) >= itemLength && (!wrap)) {
					return true;
				}
				selectedIndex += 1;
				selectedIndex = (selectedIndex + itemLength) % itemLength;
			}
			OnChange();
			OptionManager.postAction(this);
			return true;
		}
		if (Keys.isKey(keyCode, Keys.LEFT)) {
			int itemLength = items.length;
			if (itemLength > 0) {
				// prüft den Überlauf
				if ((selectedIndex-1) < 0 && (!wrap)) {
					return true;
				}
				selectedIndex -= 1;
				selectedIndex = (selectedIndex + itemLength) % itemLength;
			}
			OnChange();
			OptionManager.postAction(this);
			return true;
		}
		return false;
	}

	public int getSelectedIndex() {
		return selectedIndex;
	}
	
	protected void overrideSelectedIndex(int i) {
		selectedIndex = i;
	}
	
	protected String[] getItems() {
		return items;
	}

	/**
	 * Methode die überschrieben werden kann, wird aufgerufen falls sich der selektierte Eintrag ändert
	 */
	protected void OnChange() {		
	}

}
