// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import control.Device;
import control.Keys;

public class ItemRadiogroup extends Item  {
	private final String caption;
	private final RadioGroupManager rgm;
	private final int id;
	public ItemRadiogroup(int id, String caption, RadioGroupManager rgm) {
		this.id = id;
		this.caption = caption;
		this.rgm = rgm;
		setSelectable(true);
		rgm.register(this);
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		int lineHeight = getLineHeight();
		
		g.setColor(Manager.ITEMS_COLOR_BACKGROUND);
		
		if (selected) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
		}

		GfxUtils.fillFullRectangle(g, y, lineHeight);
		
		Font f;

		f = FontMetrics.getFont(Font.STYLE_PLAIN, ItemOption.OPTION_ITEM_FONT_SIZE);

		g.setFont(f);

		int fontHeight = f.getHeight(); 

		int leftOffset = GfxUtils.scale(fontHeight, 130);
		int circle = GfxUtils.scale(fontHeight, 75);

		if (isSelectable()) {
			g.setColor(Manager.ITEMS_COLOR_FONT);
		} else {
			g.setColor(Manager.ITEMS_COLOR_FONT_DISABLED);
		}

		int yBase = y;
		int xSpaceCircle = (fontHeight - circle) / 2;
		
		GfxUtils.drawText(g, caption, x + leftOffset, yBase + (lineHeight - f.getHeight()) / 2);
		g.drawArc(x + xSpaceCircle, yBase + (lineHeight - circle) / 2, circle, circle, 0, 360);
		if (rgm.isSelected(this)) {
			g.fillArc(x + xSpaceCircle + 2, yBase + (lineHeight - circle) / 2 + 2, circle - 4, circle - 4, 0, 360);
		}

		return lineHeight;
	}

	public int getHeight() {
		return getLineHeight();
	}

	private int getLineHeight() {
		if (Device.increaseHeightForTouch()) {
			return GfxUtils.getHeightForTouch(FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE));
		} else {
			return FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
		}
	}
	
	public boolean handleKey(int keyCode) {
		if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
			rgm.setSelectedItem(this);
			Manager.datasetSelectWrapper(id);
			return true;
		}
		return false;
	}
}
