// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.Point;
import gui.guis.Gui;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import control.Device;
import dev.Logger;

public class ItemReminderTime extends Item {
	private static String beschreibung;
	private static int beschreibungsWidth;
	private final int width;
	private final int x_base;
	private final int zeit;
	private static boolean initialized = false;

	synchronized private static boolean isInitialized() {
		return initialized;
	}

	synchronized private static void init(int width) {
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		String[] strings = { " min vor Abfahrt", " min vor Ab.", " min vor" };
		int no = GfxUtils.selectStringByMaxWidth(strings, f, width, GfxUtils.WITH_SPACE);
		if (no == GfxUtils.NO_FIT) {
			Logger.error(ItemReminderTime.class, "breite zu klein um Text hineinschreiben zu können");
			no = strings.length - 1;
		}

		beschreibung = strings[no];

		beschreibungsWidth = f.stringWidth("00" + beschreibung);

		initialized = true;
	}

	public ItemReminderTime(int zeit, int x, int width) {
		setSelectable(true);
		if (!isInitialized()) {
			init(width);
		}
		this.zeit = zeit;
		this.x_base = x;
		this.width = width;
	}

	private final static int main_Font_Color = Color.BLACK;

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		// Text
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		g.setFont(f);
		
		int itemHeight = getHeight();

		if (selected) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
			g.fillRect(x_base, y, width, itemHeight);
		} 
		
		int yAddForTouch;
		yAddForTouch = (itemHeight - f.getHeight()) / 2;

		g.setColor(main_Font_Color);
		if (zeit == -1) {
			GfxUtils.drawHCenteredText(g, "deaktiviert", y + yAddForTouch);
		} else {
			GfxUtils.drawAlignedText(zeit + beschreibung, g, new Point(x_base + (width - beschreibungsWidth) / 2 + beschreibungsWidth, y + yAddForTouch), Graphics.RIGHT | Graphics.TOP);
		}

		return itemHeight;
	}

	public int getHeight() {
		int itemHeight = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE).getHeight();
		
		if (Device.increaseHeightForTouch() && isSelectable()) {
			itemHeight = GfxUtils.getHeightForTouch(itemHeight);
		}
		
		return itemHeight;
	}

}
