// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxStringUtils;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import model.HaltestellenPaar;

public class ItemSaveFavorit extends Item {
	private final int totalHeight;
	private final int innerSpace;
	private final int textHeight;
	private final int width;
	private final int x_base;
	private final HaltestellenPaar haltestellenPaar;
	private final int xTextSpace;

	public ItemSaveFavorit(HaltestellenPaar haltestellenPaar, int x, int width, int itemNo) {
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		setSelectable(true);
		textHeight = f.getHeight();
		innerSpace = textHeight / 6;
		totalHeight = 2 * textHeight + 2 * innerSpace;
		this.haltestellenPaar = haltestellenPaar;
		this.x_base = x;
		this.width = width;
		xTextSpace = GfxUtils.scale(width, 5);
	}

	private final static int main_Font_Color = Color.BLACK;

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		g.setFont(f);

		if (selected) {
			g.setColor(Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
			g.fillRect(x_base, y, width, totalHeight);
		}
		g.setColor(main_Font_Color);
		if (haltestellenPaar == HaltestellenPaar.NULL) {
			g.setColor(Color.gray2color(170));
			GfxUtils.drawHCenteredText(g, "noch keiner", y + innerSpace);
			GfxUtils.drawHCenteredText(g, "gespeichert", y + innerSpace + textHeight);
		} else {
			int maxWidth = width - 2 * xTextSpace;
			String von = GfxStringUtils.getAdjustedLocation(f, haltestellenPaar.getVon(), maxWidth);
			String nach = GfxStringUtils.getAdjustedLocation(f, haltestellenPaar.getNach(), maxWidth);
			GfxUtils.drawText(g, von, x_base + xTextSpace, y + innerSpace);
			GfxUtils.drawText(g, nach, x_base + xTextSpace, y + innerSpace + textHeight);
		}
		
		return totalHeight;
	}

	public int getHeight() {
		return totalHeight;
	}

	public HaltestellenPaar getHaltestellenPaar() {
		return haltestellenPaar;
	}

}
