// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.utils.GfxUtils;
import gui.utils.ImageScaler;

import java.util.Hashtable;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import utils.StringUtils;
import control.Keys;
import dev.Logger;

public class ItemTastenHinweis extends Item {
	private final Image imageKey;
	private final String[] text;
	private final int itemHeight;
	private static final Hashtable htImages = new Hashtable();

	/**
	 * liefert das passende Bild zur Taste 'key' zurück
	 */
	private static Image getImage(int key) {
		Image image = (Image) htImages.get(new Integer(key));
		int baseScale = 150;
		int additionalScale = 100;
		if (image == null) {
			String base = "/res/keys/";
			String picname;
			switch (key) {
			case Keys.OK:
			case Keys.RIGHT:
				picname = "ok";
				break;
			case Keys.NUMBER:
				additionalScale = 150;
				picname = "numbers";
				break;
			case Keys.LEFT:
				picname = "left";
				break;
			case Keys.MENU_LEFT:
				picname = "menu_left";
				break;
			case Keys.MENU_RIGHT:
				picname = "menu_right";
				break;
			default:
				Logger.error(ItemTastenHinweis.class, "kein Bild für Taste " + key);
				return Image.createImage(1, 1);
			}
			Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
			String filename = base + picname + ".png";
			int finalScale;
			if (additionalScale != 100) {
				finalScale = GfxUtils.scale(baseScale, additionalScale);
			} else {
				finalScale = baseScale;
			}
			image = ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage(filename), GfxUtils.scale(f.getHeight(), finalScale));
			if (image == null) {
				Logger.error(ItemTastenHinweis.class, "konnte Bild aus " + filename + " nicht erstellen");
			}
			htImages.put(new Integer(key), image);
		}
		return image;
	}

	public ItemTastenHinweis(int key, String text, int screenWidth) {
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		imageKey = getImage(key);
		String[] lines = StringUtils.split(text, "\n");
		this.text = GfxUtils.splitupInLines(f, lines, screenWidth);
		itemHeight = imageKey.getHeight() + f.getHeight() * this.text.length;
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_SIZE);
		g.setFont(f);
		g.setColor(Manager.ITEMS_COLOR_FONT);
		g.drawImage(imageKey, (Manager.SCREEN_WIDTH - imageKey.getWidth()) / 2, y, Graphics.TOP | Graphics.LEFT);
		GfxUtils.drawText(g, text, x, y + imageKey.getHeight());
		return itemHeight;
	}

	public int getHeight() {
		return itemHeight;
	}

}
