// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.items;

import gui.FontMetrics;
import gui.Manager;
import gui.guis.Gui;
import gui.guis.GuiSelectionUebersicht;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import model.DatumZeit;
import model.Route;
import control.Device;

public class ItemUebersicht extends Item {
	private Route finalRoute;
	private boolean isNewDay;
	private int abfahrtsTag;
	private final GuiSelectionUebersicht parent;

	protected Route getFinalRoute() {
		return finalRoute;
	}

	public ItemUebersicht(Route finalRoute, GuiSelectionUebersicht parent) {
		this.parent = parent;
		this.finalRoute = finalRoute;
		setSelectable(true);
	}

	public void setNewDay(int newDay) {
		isNewDay = true;
		this.abfahrtsTag = newDay;
	}

	public int getHeight() {
		if (isNewDay) {
			return 2 * FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
		} else {
			if (Device.increaseHeightForTouch()) {
				return GfxUtils.getHeightForTouch(FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE));
			} else {
				return FontMetrics.getHeight(Manager.ITEMS_FONT_SIZE);
			}
		}
	}

	public int draw(Gui parent, Graphics g, int x, int y, boolean selected, int itemNo) {

		int height = getHeight();

		if (g == null) {
			return height;
		}

		g.setFont(FontMetrics.getFont(Manager.ITEMS_FONT_STYLE, Manager.ITEMS_FONT_SIZE));

		drawItem(g, x, y, selected, itemNo);

		return height;
	}

	/**
	 * zeichnet die vertikalen Linien für die Tabelle
	 */
	protected void drawLines(Graphics g, int y) {
		g.setStrokeStyle(Graphics.DOTTED);
		for (int i = 1; i < 4; i++) {
			if (i == 1 && isNewDay) {
				int linePos = GuiSelectionUebersicht.getPosLines(i);
				g.drawLine(linePos, y + getHeight() / 2, linePos, y + getHeight());
			} else {
				int linePos = GuiSelectionUebersicht.getPosLines(i);
				g.drawLine(linePos, y, linePos, y + getHeight());
			}
		}
		g.setStrokeStyle(Graphics.SOLID);
	}

	/**
	 * zeichnet den Text in die Tabelle
	 */
	protected void drawText(Graphics g, int y) {
		g.setColor(Manager.ITEMS_COLOR_FONT);
		int fahrDauer = finalRoute.getAnkunftszeit() - finalRoute.getAbfahrtszeit();
		GfxUtils.drawText(g, DatumZeit.toTimeString(finalRoute.getAbfahrtszeit()), GuiSelectionUebersicht.getPosItems(0), y, Graphics.LEFT | Graphics.TOP);
		GfxUtils.drawText(g, DatumZeit.toTimeString(finalRoute.getAnkunftszeit()), GuiSelectionUebersicht.getPosItems(1), y, Graphics.LEFT | Graphics.TOP);
		String anzUmsteigen = finalRoute.getAnzUmsteigen() + "";
		if (finalRoute.isOptimierungAbgebrochen()) {
			anzUmsteigen = "?";
		}
		GfxUtils.drawText(g, anzUmsteigen, GuiSelectionUebersicht.getPosItems(2), y, Graphics.RIGHT | Graphics.TOP);
		g.setColor(Manager.ITEMS_COLOR_FONT);
		GfxUtils.drawText(g, fahrDauer + "'", GuiSelectionUebersicht.getPosItems(3), y, Graphics.RIGHT | Graphics.TOP);
	}

	/**
	 * zeichnet den String des neuen Tags
	 */
	protected int drawDay(Graphics g, int y) {

		Font f = FontMetrics.getFont(Font.STYLE_ITALIC, Manager.ITEMS_FONT_SIZE);
		g.setFont(f);
		String feiertag = (DatumZeit.isFeiertag(abfahrtsTag)) ? " (Feiertag)" : "";
		GfxUtils.drawText(g, DatumZeit.getWeekdayAsString(abfahrtsTag) + feiertag, GuiSelectionUebersicht.getPosLines(0), y, Graphics.LEFT | Graphics.TOP);
		f = FontMetrics.getFont(Manager.ITEMS_FONT_STYLE, Manager.ITEMS_FONT_SIZE);
		g.setFont(f);

		return f.getHeight();
	}

	protected void drawItem(Graphics g, int x, int y, boolean selected, int itemNo) {
		
		itemNo += parent.getAddInBeginningCounter();
		
		if (selected) {
			if (isNewDay) {
				int halfwidth = getHeight() / 2;
				g.setColor(GfxUtils.getBackgroundColorByPos(itemNo));
				GfxUtils.fillFullRectangle(g, y, halfwidth);
				g.setColor(Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
				GfxUtils.fillFullRectangle(g, y + halfwidth, halfwidth);
			} else {
				g.setColor(Manager.ITEMS_COLOR_BACKGROUND_SELECTED);
				GfxUtils.fillFullRectangle(g, y, getHeight());
			}
		} else {
			g.setColor(GfxUtils.getBackgroundColorByPos(itemNo));
			GfxUtils.fillFullRectangle(g, y, getHeight());
		}
		g.setColor(Manager.ITEMS_COLOR_FONT);
		Font f = FontMetrics.getFont(Manager.ITEMS_FONT_STYLE, Manager.ITEMS_FONT_SIZE);
		g.setFont(f);

		g.setColor(Manager.ITEMS_COLOR_FONT);

		drawLines(g, y);

		g.setColor(Manager.ITEMS_COLOR_FONT);

		if (isNewDay) {
			y += drawDay(g, y);
		} else {
			// Anpassung für Touch-Vergrößerung
			y += (getHeight() - f.getHeight()) / 2;
		}

		drawText(g, y);

	}

	public Route getRoute() {
		return finalRoute;
	}

}
