// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.launch;

import gui.Manager;

import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import control.Device;
import control.Keys;
import control.Pushandride;
import control.PushandrideMidlet;
import dev.DebugInfo;
import dev.Logger;

public class GameCanvasInstance extends GameCanvas implements Runnable {
	private static GameCanvasInstance gameCanvasInstance;
	public static boolean requiresFlushGraphics = false;
	private Manager manager = null;

	public static void setRequiresFlushGraphics() {
		requiresFlushGraphics = true;
	}

	public GameCanvasInstance(Pushandride pushandride) {
		super(false);
		setFullScreenMode(true);
		GameCanvasInstance.gameCanvasInstance = this;

		// Startbildschirm erstellen und anzeigen
		Graphics g = getGraphics();
		g.setGrayScale(255);
		g.fillRect(0, 0, getWidth(), getHeight());
		flushGraphics();
		repaint();
		Display.getDisplay(pushandride).setCurrent(this);

		new Thread(this).start();
	}

	public void run() {
		try {
			new GuiLaunchScreen("LaunchScreen", this, getGraphics());
			flushGraphics();
			repaint();
	
			PushandrideMidlet.init();
		} catch (OutOfMemoryError oom) {
			PushandrideMidlet.OutOfMemoryException(getClass(), oom, "s09ns");
		} catch (Exception e) {
			Logger.infoException(getClass(), e, "as89m");
		}
	}

	public void redirectGameCanvasEvents(Manager manager) {
		this.manager = manager;
	}

	private static int cntMagicKey = 0;
	
	private static final long PRE_TRIGGER_ANDROID_RESET = -1;
	private long preTriggerAndroidLeftRightPressed = PRE_TRIGGER_ANDROID_RESET;
	
	protected void keyPressed(int keyCode) {
		if (manager != null) {
			// bei Android Devices wird der erste Klick nach Links oder Rechts verworfen, da die Events viel zu schnell kommen 
			if (Device.isAndroid && (Keys.isKey(keyCode, Keys.LEFT) || Keys.isKey(keyCode, Keys.RIGHT))) {
				long currentTime = System.currentTimeMillis();

				if (currentTime - preTriggerAndroidLeftRightPressed > 500) {
					preTriggerAndroidLeftRightPressed = PRE_TRIGGER_ANDROID_RESET;
				}

				if (preTriggerAndroidLeftRightPressed == PRE_TRIGGER_ANDROID_RESET) {
					preTriggerAndroidLeftRightPressed = currentTime;
					return;
				} else {
					preTriggerAndroidLeftRightPressed = PRE_TRIGGER_ANDROID_RESET;
				}
			}
			manager.keyPressed(keyCode);
		} else {
			if (keyCode == '0') {
				cntMagicKey++;
				if (cntMagicKey == 6) {
					DebugInfo.showDebugInfo(this);
				}
			} else {
				cntMagicKey = 0;
			}
		}
	}

	protected void keyReleased(int keyCode) {
		if (manager != null) {
			manager.keyReleased(keyCode);
		}
	}

	protected void pointerPressed(int x, int y) {
		if (manager != null) {
			if (Device.isAndroid) {
				y += 1;
				y = Math.max(y, 0);
			}
			manager.pointerPressed(x, y);
		} else {
			Device.checkForSoftwareKeyboard();
		}
	}

	protected void pointerReleased(int x, int y) {
		if (manager != null) {
			if (Device.isAndroid) {
				y += 1;
				y = Math.max(y, 0);
			}
			manager.pointerReleased(x, y);
		}
	}

	protected void pointerDragged(int x, int y) {
		if (manager != null) {
			if (Device.isAndroid) {
				y += 1;
				y = Math.max(y, 0);
			}
			manager.pointerDragged(x, y);
		}
	}

	public void paint(Graphics g) {
		if (requiresFlushGraphics) {
			flushGraphics();
		} else {
			super.paint(g);
		}
	}

	protected void sizeChanged(int width, int height) {
		if (manager != null) {
			manager.sizeChanged(width, height);
		}
	}

	public static GameCanvasInstance getInstance() {
		return gameCanvasInstance;
	}

	public static Graphics returnGraphics() {
		return gameCanvasInstance.getGraphics();
	}
	
	protected void showNotify() {
	}
	
}
