// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.launch;

import gui.Color;
import gui.FontMetrics;
import gui.Imager;
import gui.guis.Gui;
import gui.utils.GfxUtils;
import gui.utils.ImageScaler;

import java.util.Vector;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;

import model.Verkehrsmittel;
import control.Device;
import control.Global;

public class GuiLaunchScreen extends Gui {

	protected final int screenWidth;
	protected final int screenHeight;

	private static Font font = null;
	private static Font fontBold = null;
	private static Image logoHand;
	private final Vector fahrmittel;
	
	private final String[] adText;
	
	public GuiLaunchScreen(String Name, GameCanvas gc, Graphics g) {
		super(Name, gc, g);
		this.gc = gc;

		Device.initScreen(gc.getWidth(), gc.getHeight(), 4);

		gc.getKeyStates();

		screenWidth = g.getClipWidth();
		screenHeight = g.getClipHeight();

		int maxWidth = screenWidth * 96 / 100;
		int freeHeight = screenHeight * 96 / 100;

		font = getDefaultFont(screenWidth, screenHeight);
		fontBold = Font.getFont(font.getFace(), Font.STYLE_BOLD, font.getSize());

		int fontHeight = font.getHeight();
		

		fahrmittel = new Vector(); 
		
		freeHeight -= (4 + 2 + 1) * fontHeight;
		
		String[] text = {};
	
		if (Device.getGfxQuality() != Device.GFX_QUALITY_LOW) {
			logoHand = Imager.createLogoHand(maxWidth, freeHeight * 9 / 10, screenWidth * 65 / 100);

			String linienFull = Global.getBinaryInfo()[Global.BINARY_INFO_SETS];
			String linienPart = Global.getBinaryInfo()[Global.BINARY_INFO_SETS_PART];
			
			for (int i = 0; i < linienFull.length(); i++) {
				try {
					int typ = Integer.parseInt(linienFull.charAt(i) + "");
					fahrmittel.addElement(ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/" + Verkehrsmittel.ANSCHLUSSTYP[typ] + "w.png"), fontHeight));
				} catch (Exception e) {
					System.err.println("Exception GuiLaunch" + e.getMessage());
				}
			}

			for (int i = 0; i < linienPart.length(); i++) {
				try {
					int typ = Integer.parseInt(linienPart.charAt(i) + "");
					fahrmittel.addElement(ImageScaler.scaleBilinearByHeight(GfxUtils.loadImage("/res/" + Verkehrsmittel.ANSCHLUSSTYP[typ] + "light.png"), fontHeight));
				} catch (Exception e) {
					System.err.println("Exception GuiLaunch" + e.getMessage());
				}
			}
		} else {
			logoHand = Image.createImage(1, 1);
		}
		
		this.adText = text;
		
		drawTo(g);
	}

	private static int FOOTER_COLOR_BACKGROUND = Color.rgb2color(220, 250, 255);

	synchronized public void drawTo(Graphics g) {
		g.setFont(font);
		int fontHeight = font.getHeight();

		// clear background
		g.setGrayScale(255);
		g.fillRect(0, 0, screenWidth, screenHeight);
		
		// text - init
		FontMetrics.init();
		g.setFont(font);
		g.setColor(Color.BLACK);

		// text
		int freeWidth = screenWidth * 94 / 100;
		int xSpace = (screenWidth - freeWidth) / 2;

		// header
		int y = xSpace;
//		g.drawImage(logoHand, xSpace, y, Graphics.LEFT | Graphics.TOP);
		
		g.setFont(fontBold);
		String text = "push&ride";
		GfxUtils.drawText(g, text, screenWidth - fontBold.stringWidth(text) - xSpace, y);
		y+= fontHeight;
		g.setFont(font);
		text = "Stuttgart";
		GfxUtils.drawText(g, text, screenWidth - font.stringWidth(text) - xSpace, y);
		y+= fontHeight;
		text = "V " + Global.getMidletVersion();
		GfxUtils.drawText(g, text, screenWidth - font.stringWidth(text) - xSpace, y);
		y+= fontHeight;
		
		int yMain = y;

		// Fahrmittel Logos
		int x = screenWidth - xSpace;
		for (int i = fahrmittel.size() - 1; i >=0 ; i--) {
			Image image = (Image) fahrmittel.elementAt(i);
			x -= image.getWidth() - image.getWidth() / 7;
			g.drawImage(image, x, y, Graphics.LEFT | Graphics.TOP);
		}

		// Main
		int yPosLogo = yMain + (screenHeight - logoHand.getHeight() - yMain - fontHeight * 2)  / 2;
		yPosLogo -= adText.length *  fontHeight;
		g.drawImage(logoHand, (screenWidth - logoHand.getWidth()) / 2, yPosLogo, Graphics.LEFT | Graphics.TOP);
		
		if (Device.getGfxQuality() == Device.GFX_QUALITY_LOW) {
			int yPos = yMain + (screenHeight - yMain - fontHeight * 2)  / 3;
			g.setFont(fontBold);
			text = "push&ride";
			g.drawString(text, (screenWidth - fontBold.stringWidth(text))/2, yPos - fontHeight, Graphics.LEFT | Graphics.TOP);
			g.setFont(font);
			text = "Der Fahrplan für dein Handy";
			if (font.stringWidth(text) > screenWidth * 9 / 10) {
				text = "Der Fahrplan";
				g.drawString(text, (screenWidth - font.stringWidth(text))/2, yPos + fontHeight / 3, Graphics.LEFT | Graphics.TOP);
				text = "für dein Handy";
				g.drawString(text, (screenWidth - font.stringWidth(text))/2, yPos + fontHeight / 3 + fontHeight, Graphics.LEFT | Graphics.TOP);
			} else {
				g.drawString(text, (screenWidth - font.stringWidth(text))/2, yPos + fontHeight / 3, Graphics.LEFT | Graphics.TOP);
			}
		}
		
		// footer
		g.setColor(FOOTER_COLOR_BACKGROUND);
		
		if (Device.isAndroid) {
			g.fillRect(0, screenHeight - fontHeight * 1, screenWidth, fontHeight * 1);
		} else {
			g.fillRect(0, screenHeight - fontHeight * 2, screenWidth, fontHeight * 2);
		}

		g.setColor(Color.BLACK);
		int footerPos = screenHeight - fontHeight * 2;
		if (!Device.isAndroid) {
			String bottomText;
			bottomText = "Individuelle Linien:";
			GfxUtils.drawText(g, bottomText, xSpace, footerPos);
		}

		String bottomText2;
		bottomText2 = "http://www.pushandride.de";
		
		GfxUtils.drawText(g, bottomText2, xSpace, footerPos + fontHeight);
	}

	protected Font getFont() {
		return font;
	}

	
	public static final String PROBE_WIDTH_STRING = "Schwieberdingen Glemstal";
	public static final int PROBE_WIDTH_SCALAR = 128;
	public static Font getDefaultFont(int screenWidth, int screenHeight) {
		if (font != null) {
			return font;
		}

		int maxWidth = Math.min(screenWidth, screenHeight);

		Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_LARGE);
		int textWidth = GfxUtils.scale(f.stringWidth(PROBE_WIDTH_STRING), PROBE_WIDTH_SCALAR);
		f = FontMetrics.getFont(Font.SIZE_LARGE);
		if (textWidth > maxWidth) {
			f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_MEDIUM);
			textWidth = GfxUtils.scale(f.stringWidth(PROBE_WIDTH_STRING), PROBE_WIDTH_SCALAR);
			if (textWidth > maxWidth) {
				f = FontMetrics.getFont(Font.SIZE_SMALL);
			}
		}
		return f;
	}
	
	public static Image getHandImage() {
		return logoHand;
	}
}
