// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.options.dev;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.launch.GuiLaunchScreen;
import gui.utils.GfxUtils;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import control.Keys;


public class TestFonts extends AbstractGfxGui implements CommandListener {

	private static int state = 0;

	public TestFonts() {
		super(false);
		setFullScreenMode(true);

		FontMetrics.init();

		showAllFonts();
	}

	public Graphics returnGraphics() {
		return getGraphics();
	}

	/**
	 * commandAction behandelt Funktionstasten
	 */
	public void commandAction(Command c, Displayable d) {

	}

	/*--------------------------------------------------
	 * Pointer pressed
	 *-------------------------------------------------*/
	protected void pointerPressed(int x, int y) {
		state += 1;
		state = state % anzScreens;
		keyPressed(-12345);
	}

	/*--------------------------------------------------
	 * Pointer moved
	 *-------------------------------------------------*/
	protected void pointerDragged(int x, int y) {
	}

	private static final int anzScreens = 5;
	
	protected void keyPressed(int keyCode) {
		if (Keys.isKey(keyCode, Keys.C) || Keys.isKey(keyCode, Keys.BACK)) {
			backToMain();
			return;
		}

		if (Keys.isKeyAction(keyCode, Keys.ACTION_FORWARD)) {
			state += 1;
			state = state % anzScreens;
		}
		if (Keys.isKeyAction(keyCode, Keys.ACTION_BACKWARD)) {
			state -= 1;
			if (state < 0)
				state = anzScreens - 1;
		}
		if (keyCode == '0') {
			state = 0;
		}
		if (keyCode == '1') {
			state = 1;
		}
		if (keyCode == '2') {
			state = 2;
		}
		if (keyCode == '3') {
			state = 3;
		}
		if (keyCode == '4') {
			state = 4;
		}
		if (state == 0) {
			showAllFonts();
		}
		if ((state > 0) && (state < 4)) {
			showFontDetails(state - 1);
		}
		if (state == 4) {
			showFontMetrics();
		}
	}

	private int drawText(Graphics g, String s, int y) {
		Font f;
		f = g.getFont();
		GfxUtils.drawText(g, s, 2, y);
		return f.getHeight();
	}

	private int drawFullLine(Graphics g, int y) {
		g.drawLine(0, y, getWidth(), y);
		return 1;
	}

	private final static String testString = "ABCD GH ÄÜÖß  abcd gh äüö";
	private final static String testStringWidth = "Schwieberdingen Glemstal";

	private void clearScreen() {
		Graphics g = getGraphics();
		g.setGrayScale(255);
		g.fillRect(0, 0, g.getClipWidth(), g.getClipHeight());
	}

	private void showAllFonts() {
		clearScreen();

		Graphics g = getGraphics();

		Font f;

		int y = 0;
		g.setFont(Font.getDefaultFont());
		g.setColor(Color.BLACK);

		y += drawText(g, "Schrifttest: alle Schriftarten", y);
		y += drawText(g, "aktuell: " + FontMetrics.getFontSizeById(FontMetrics.getIdBySize(Manager.ITEMS_FONT_SIZE)) + " maxWidth:" + Math.min(Manager.SCREEN_WIDTH, Manager.SCREEN_HEIGHT) + "px", y);
		y += drawFullLine(g, y);
		y += drawFullLine(g, y);
		for (int i = 0; i < 3; i++) {
			f = FontMetrics.getFontById(0, 0, i);
			g.setFont(f);
			int textWidth = GfxUtils.scale(f.stringWidth(GuiLaunchScreen.PROBE_WIDTH_STRING), GuiLaunchScreen.PROBE_WIDTH_SCALAR);
			y += drawText(g, "" + FontMetrics.getFontSizeById(i) + "  (h:" + f.getHeight() + "px, t: "+textWidth+"px)", y);
			y += drawFullLine(g, y);
			y += drawText(g, testStringWidth, y);
			y += drawFullLine(g, y);
			y += drawFullLine(g, y);
		}
		repaint();
	}

	private void showFontDetails(int type) {
		clearScreen();

		Graphics g = getGraphics();

		Font f;

		int y = 0;
		g.setFont(Font.getDefaultFont());
		g.setColor(Color.BLACK);

		y += drawText(g, "Schrifttest: " + FontMetrics.getFontNameById(type), y);
		y += drawFullLine(g, y);
		y += drawFullLine(g, y);
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 4; j++) {
				f = FontMetrics.getFontById(type, j, i);
				g.setFont(f);
				y += drawText(g, FontMetrics.getFontSizeById(i) + "  (" + f.getHeight() + "px)  " + testString, y);
				y += drawFullLine(g, y);
			}
		}
		repaint();
	}

	protected void showNotify() {
	}

	private void showFontMetrics() {
		clearScreen();

		FontMetrics.init(getGraphics());

		repaint();
	}

}
