// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.options.dev;

import gui.utils.GfxUtils;
import gui.utils.ImageScaler;

import java.io.IOException;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import control.Keys;

public class TestImageScaler extends AbstractGfxGui implements CommandListener {

	public TestImageScaler() {
		super(false);
		setFullScreenMode(true);

		try {
			source1 = Image.createImage("/res/pnr30px.png");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			source2 = Image.createImage("/res/pnr60px.png");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		draw();
	}

	public Graphics returnGraphics() {
		return getGraphics();
	}

	private int currentHeight = 30;

	/**
	 * commandAction behandelt Funktionstasten
	 */
	public void commandAction(Command c, Displayable d) {

	}

	/*--------------------------------------------------
	 * Pointer pressed
	 *-------------------------------------------------*/
	protected void pointerPressed(int x, int y) {
	}

	/*--------------------------------------------------
	 * Pointer moved
	 *-------------------------------------------------*/
	protected void pointerDragged(int x, int y) {
	}

	protected void keyPressed(int keyCode) {
		if (Keys.isKey(keyCode, Keys.C)) {
			backToMain();
			return;
		}
		if (Keys.isKey(keyCode, Keys.RIGHT)) {
			currentHeight += 5;
		}
		if (Keys.isKey(keyCode, Keys.UP)) {
			currentHeight -= 1;
		}
		if (Keys.isKey(keyCode, Keys.LEFT)) {
			currentHeight -= 5;
		}
		if (Keys.isKey(keyCode, Keys.DOWN)) {
			currentHeight += 1;
		}
		draw();
	}

	protected void showNotify() {
	}

	private Image source1;
	private Image source2;

	private void draw() {
		int y;

		y = 0;
		Graphics g = getGraphics();
		g.setGrayScale(255);
		g.fillRect(0, 0, g.getClipWidth(), g.getClipHeight());
		g.setGrayScale(0);

		Font f;
		f = Font.getFont(Font.STYLE_PLAIN, Font.STYLE_PLAIN, Font.SIZE_SMALL);
		g.setFont(f);

		y += f.getHeight() * 2;
		g.drawLine(0, y, g.getClipWidth(), y);
		y += 1;

		Image pic;

		pic = source1;
		g.drawImage(pic, (g.getClipWidth() - pic.getWidth()) / 2, y, 0);
		y += pic.getHeight();
		g.drawLine(0, y, g.getClipWidth(), y);
		y += 1;

		long lasttime;
		long time1;
		lasttime = System.currentTimeMillis();
		pic = ImageScaler.scaleBresenhamsByHeight(source1, currentHeight);
		time1 = System.currentTimeMillis() - lasttime;
		g.drawImage(pic, (g.getClipWidth() - pic.getWidth()) / 2, y, 0);
		y += pic.getHeight();
		g.drawLine(0, y, g.getClipWidth(), y);
		y += 1;

		long time2;
		lasttime = System.currentTimeMillis();
		pic = ImageScaler.scaleBilinearByHeight(source1, currentHeight);
		time2 = System.currentTimeMillis() - lasttime;
		g.drawImage(pic, (g.getClipWidth() - pic.getWidth()) / 2, y, 0);
		y += pic.getHeight();
		g.drawLine(0, y, g.getClipWidth(), y);
		y += 1;

		long time3;
		lasttime = System.currentTimeMillis();
		pic = ImageScaler.scaleBresenhamsByHeight(source2, currentHeight);
		time3 = System.currentTimeMillis() - lasttime;
		g.drawImage(pic, (g.getClipWidth() - pic.getWidth()) / 2, y, 0);
		y += pic.getHeight();
		g.drawLine(0, y, g.getClipWidth(), y);
		y += 1;

		long time4;
		lasttime = System.currentTimeMillis();
		pic = ImageScaler.scaleBilinearByHeight(source2, currentHeight);
		time4 = System.currentTimeMillis() - lasttime;
		g.drawImage(pic, (g.getClipWidth() - pic.getWidth()) / 2, y, 0);
		y += pic.getHeight();
		g.drawLine(0, y, g.getClipWidth(), y);
		y += 1;

		pic = source2;
		g.drawImage(pic, (g.getClipWidth() - pic.getWidth()) / 2, y, 0);
		y += pic.getHeight();
		g.drawLine(0, y, g.getClipWidth(), y);
		y += 1;

		GfxUtils.drawText(g, "Bilin S1: " + source1.getHeight() + "  1->2: " + currentHeight + "  4->3: " + currentHeight + "  S4: " + source2.getHeight(), 0, 0);
		GfxUtils.drawText(g, "bA " + time1 + "ms  lA " + time2 + "ms  bB " + time3 + "ms  lB " + time4 + "ms", 0, f.getHeight());
		GfxUtils.drawText(g, "Zurück mit 'C'", 0, y);

		repaint();
	}
}
