// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.options.dev;

import gui.Color;
import gui.FontMetrics;
import gui.Manager;
import gui.utils.GfxUtils;

import java.util.Vector;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import utils.StringUtils;
import control.Device;
import control.Keys;

public class TestKeys extends AbstractGfxGui implements CommandListener {

	private boolean preparetoquit = false;
	private boolean preparetonext = false;

	private final int returnX;
	private final int returnY;
	private final int boxWidth;
	private final int boxHeight;
	private final int nextX;
	private final int nextY;
	
	private final static int POINTER_PRESSED = 0;
	private final static int POINTER_DRAGGED = 1;
	private final static int POINTER_RELEASED = 2;
	
	private final String[] POINTER_EVENT_STRINGS = {"pressed", "dragged", "released"};
	
	private final Vector eventBuffer;
	private final int eventBufferMinElements = 15;

	private int tab = 0;

	public TestKeys() {
		super(false);
		setFullScreenMode(true);
		showPnrKeys(0);

		returnX = Manager.SCREEN_HEIGHT * 1 / 3;
		returnY = Manager.SCREEN_HEIGHT * 2 / 3;

		boxWidth = Manager.SCREEN_WIDTH / 20;
		boxHeight = Manager.SCREEN_WIDTH / 20;
		
		nextX = Manager.SCREEN_WIDTH - boxWidth;
		nextY = (Manager.SCREEN_HEIGHT / 2) + boxHeight;
		
		eventBuffer = new Vector();
		for (int i = 0; i < eventBufferMinElements; i++) {
			eventBuffer.addElement("");
		}
	}

	public Graphics returnGraphics() {
		return getGraphics();
	}

	/**
	 * commandAction behandelt Funktionstasten
	 */
	public void commandAction(Command c, Displayable d) {

	}

	/*--------------------------------------------------
	 * Pointer pressed
	 *-------------------------------------------------*/
	private int returnCnt = 0;

	protected void pointerPressed(int x, int y) {
		if (x >= returnX && y >= returnY && x <= returnX + boxWidth && y <= returnY + boxHeight) {
			returnCnt++;
			if (returnCnt == 2) {
				backToMain();
				return;
			}
		} else if (x >= nextX && y >= nextY && y <= nextY + boxHeight) {
			if (tab == 0) {
				tab = 1;
			} else {
				tab = 0;
			}
			if (tab != 0) {
				showJ2MEKeys();
				return;
			} else {
				showPnrKeys(0);
				return;
			}
		} else {
			returnCnt = 0;
		}
		showPointerKoordinaten(x, y, POINTER_PRESSED);
	}

	/*--------------------------------------------------
	 * Pointer moved
	 *-------------------------------------------------*/
	protected void pointerDragged(int x, int y) {
		showPointerKoordinaten(x, y, POINTER_DRAGGED);
	}

	protected void pointerReleased(int x, int y) {
		showPointerKoordinaten(x, y, POINTER_RELEASED);
	}

	protected void keyPressed(int keyCode) {
		if (keyCode == '1') {
			if (preparetoquit) {
				backToMain();
				return;
			} else {
				preparetoquit = true;
			}
		} else {
			preparetoquit = false;
		}
		if (keyCode == '2') {
			if (preparetonext) {
				if (tab == 0) {
					tab = 1;
					preparetonext = false;
				} else {
					tab = 0;
					preparetonext = false;
				}
			} else {
				preparetonext = true;
			}
		} else {
			preparetonext = false;
		}
		if (tab == 0) {
			showPnrKeys(keyCode);
		} else {
			showJ2MEKeys();
		}
	}
	
	private int drawText(Graphics g, String s, int y) {
		Font f;
		f = g.getFont();
		GfxUtils.drawText(g, s, 2, y);
		return f.getHeight();
	}

	private int drawFullLine(Graphics g, int y) {
		g.drawLine(0, y, getWidth(), y);
		return 1;
	}

	private void clearScreen() {
		Graphics g = getGraphics();
		g.setGrayScale(255);
		g.fillRect(0, 0, g.getClipWidth(), g.getClipHeight());
	}

	private void showPnrKeys(int keycode) {
		clearScreen();

		Graphics g = getGraphics();
		g.setFont(FontMetrics.getFont(Font.SIZE_SMALL));
		g.setGrayScale(0);

		int y = 0;

		y += drawText(g, "p&r Tastencheck", y);
		y += drawText(g, "plt: " + Device.getPlatform(), y);
		y += drawFullLine(g, y);
		y += drawFullLine(g, y);
		y += drawText(g, "Keycode: " + keycode, y);
		y += drawText(g, "^ p&r: : " + Keys.getKeyString(keycode), y);
		int keyStates = getKeyStates();
		y += drawText(g, "getKeyStates(): " + keyStates, y);
		y += drawText(g, "" + StringUtils.intToBitString(keyStates), y);
		y += drawFullLine(g, y);
		y += drawFullLine(g, y);
		y += drawText(g, "zwei mal '1' drücken zum zurückkehren", y);
		y += drawText(g, "zwei mal '2' drücken für J2ME Keys", y);
		repaint();
	}

	private void showJ2MEKeys() {
		clearScreen();

		Graphics g = getGraphics();
		g.setFont(FontMetrics.getFont(Font.SIZE_SMALL));
		g.setGrayScale(0);

		int y = 0;

		String out = Keys.getKeySettings();

		String[] lines = StringUtils.split(out, "\n");
		for (int i = 0; i < lines.length; i++) {
			y += drawText(g, lines[i], y);
		}
		y += drawFullLine(g, y);
		y += drawText(g, "zwei mal '1' drücken zum zurückkehren", y);
		y += drawText(g, "zwei mal '2' drücken für dynamic Keys", y);
		repaint();
	}

	private void showPointerKoordinaten(int x, int y, int eventtype) {
		if (eventtype != POINTER_DRAGGED) {
			eventBuffer.addElement("(" + x + "," + y + ")" + POINTER_EVENT_STRINGS[eventtype]);
		}
		
		clearScreen();
		int leftx = 0;
		int rightx = 0;
		for (int i = 0; i < Manager.SCREEN_WIDTH; i++) {
			if (Manager.checkPointerXPos(i) == Manager.POINTER_X_LEFT) {
				leftx = i;
			} else {
				break;
			}
		}
		for (int i = 0; i < Manager.SCREEN_WIDTH; i++) {
			if (Manager.checkPointerXPos(i) == Manager.POINTER_X_RIGHT) {
				rightx = i;
				break;
			}
		}
		Graphics g = getGraphics();
		g.setGrayScale(200);
		g.drawRect(0, 0, leftx, Manager.SCREEN_HEIGHT);
		g.drawRect(rightx, 0, Manager.SCREEN_WIDTH, Manager.SCREEN_HEIGHT);

		g.setGrayScale(0);

		int z = 0;
		g.setFont(FontMetrics.getFont(Font.SIZE_SMALL));

		z += drawText(g, "p&r Tastencheck", z);
		z += drawText(g, "plt: " + Device.getPlatform(), z);
		z += drawFullLine(g, z);
		z += drawFullLine(g, z);
		z += drawText(g, "X: " + x, z);
		z += drawFullLine(g, z);
		z += drawFullLine(g, z);
		z += drawText(g, "Y: " + y, z);
		
		g.setColor(Color.gray2color(128));
		
		int eventbuffersize = eventBuffer.size();
		
		for (int i = 0; i < eventBufferMinElements; i++) {
			z += drawText(g, (String) eventBuffer.elementAt(eventbuffersize - 1 - i), z);
			if (z >= Manager.SCREEN_HEIGHT) {
				break;
			}
		}

		g.setColor(Color.BLACK);
		g.drawRect(returnX, returnY, boxWidth, boxHeight);
		g.drawRect(nextX, nextY, boxWidth, boxHeight);

		repaint();
	}

	protected void showNotify() {
	}

}
