// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.options.dev;

import gui.Color;
import gui.FontMetrics;
import gui.utils.GfxUtils;
import gui.utils.ImageScaler;

import java.io.IOException;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import control.Keys;

public class TestVonNach extends AbstractGfxGui implements CommandListener {

	private final int COLOR_BACK_RED = Color.rgb2color(152, 0, 0);

	private Image vonImage;
	private Image nachImage;

	public TestVonNach() {
		super(false);
		setFullScreenMode(true);

		FontMetrics.init();

		try {
			vonImage = Image.createImage("/res/von_30px.png");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			nachImage = Image.createImage("/res/nach_30px.png");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		draw();
	}

	private void draw() {
		clearScreen();

		Graphics g;
		g = getGraphics();
		int y = 0;
		int x = 0;
		int textHeight;

		Font f;
		f = FontMetrics.getFont(Font.SIZE_MEDIUM);
		g.setFont(f);

		textHeight = f.getHeight();
		Image i1;
		Image i2;
		i1 = ImageScaler.scaleBilinearByHeight(vonImage, textHeight);
		i2 = ImageScaler.scaleBilinearByHeight(nachImage, textHeight);

		y += textHeight / 10;

		x = 2;
		g.drawImage(i1, x, y, Graphics.LEFT | Graphics.TOP);
		x += i1.getWidth();
		g.setColor(COLOR_BACK_RED);
		GfxUtils.fillFullRectangleFromX(g, x, y, textHeight);
		g.setColor(Color.WHITE);
		GfxUtils.drawText(g, "Schwieberdingen Glemstal", x, y);
		y += textHeight;

		y += textHeight / 7;
		x = 2;
		g.drawImage(i2, 2, y, Graphics.LEFT | Graphics.TOP);
		x += i2.getWidth();
		g.setColor(COLOR_BACK_RED);
		GfxUtils.fillFullRectangleFromX(g, x, y, textHeight);
		g.setColor(Color.WHITE);
		GfxUtils.drawText(g, "Stuttgart Universität", x, y);
		y += textHeight;

		g.setColor(Color.rgb2color(186, 245, 255));
		GfxUtils.drawFullLineHoriz(g, y);
		y += 1;
		g.setColor(Color.gray2color(177));
		GfxUtils.drawFullLineHoriz(g, y);
		y += 1;

		// ***********************

		f = FontMetrics.getFont(Font.SIZE_SMALL);
		g.setFont(f);

		y += 30;

		textHeight = f.getHeight();
		i1 = ImageScaler.scaleBilinearByHeight(vonImage, textHeight);
		i2 = ImageScaler.scaleBilinearByHeight(nachImage, textHeight);

		y += textHeight / 10;

		x = 2;
		g.drawImage(i1, x, y, Graphics.LEFT | Graphics.TOP);
		x += i1.getWidth();
		g.setColor(COLOR_BACK_RED);
		GfxUtils.fillFullRectangleFromX(g, x, y, textHeight);
		g.setColor(Color.WHITE);
		GfxUtils.drawText(g, "Schwieberdingen Glemstal", x, y);
		y += textHeight;

		y += textHeight / 7;
		x = 2;
		g.drawImage(i2, 2, y, Graphics.LEFT | Graphics.TOP);
		x += i2.getWidth();
		g.setColor(COLOR_BACK_RED);
		GfxUtils.fillFullRectangleFromX(g, x, y, textHeight);
		g.setColor(Color.WHITE);
		GfxUtils.drawText(g, "Stuttgart Universität", x, y);
		y += textHeight;
		g.setColor(Color.rgb2color(186, 245, 255));
		GfxUtils.drawFullLineHoriz(g, y);
		y += 1;
		g.setColor(Color.gray2color(177));
		GfxUtils.drawFullLineHoriz(g, y);

		repaint();
	}

	public Graphics returnGraphics() {
		return getGraphics();
	}

	/**
	 * commandAction behandelt Funktionstasten
	 */
	public void commandAction(Command c, Displayable d) {

	}

	/*--------------------------------------------------
	 * Pointer pressed
	 *-------------------------------------------------*/
	protected void pointerPressed(int x, int y) {
	}

	/*--------------------------------------------------
	 * Pointer moved
	 *-------------------------------------------------*/
	protected void pointerDragged(int x, int y) {
	}

	protected void keyPressed(int keyCode) {
		if (Keys.isKey(keyCode, Keys.C)) {
			backToMain();
			return;
		}
	}

	private void clearScreen() {
		Graphics g = getGraphics();
		g.setGrayScale(255);
		g.fillRect(0, 0, g.getClipWidth(), g.getClipHeight());
	}

	protected void showNotify() {
	}

}
