// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.utils;

import gui.launch.GameCanvasInstance;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;

import control.Pushandride;
import control.locale.Locale;

public class AlertMessage extends Alert implements CommandListener {

	protected final Command command;
	protected final Display display;
	private final String title;
	private final String messageText;
	private final String backButtonText;

	/**
	 * Zeigt eine Meldung an. Bietet einen Back-Command an. (Exit, falls Locale.Beenden übergeben wird)
	 * 
	 * @param title
	 *            Titel der Meldung
	 * @param messageText
	 *            Text-Inhalt der Meldung
	 * @param last
	 *            Letztes angezeigtes Displayable-Objekt (Back-Link)
	 * @param backButtonText
	 *            Textanzeige für den Back-Link (z.B. "Abbrechen", "Zurück",...)
	 */
	public AlertMessage(String title, String messageText, String backButtonText) {
		super(title, messageText, null, null);
		this.title = title;
		this.messageText = messageText;
		this.backButtonText = backButtonText;
		if (backButtonText == Locale.Beenden) {
			this.command = new Command(backButtonText, Command.EXIT, 1);
		} else {
			this.command = new Command(backButtonText, Command.BACK, 1);
		}
		this.setTimeout(Alert.FOREVER);
		this.setCommandListener(this);
		display = Display.getDisplay(Pushandride.getMidlet());
	}

	/**
	 * Zeigt den Alert an oder entfernt ihn vom Screen.
	 */
	public void setVisible(boolean b) {
		if (b) {
			this.setTitle(title);
			this.setString(messageText);
			this.addCommand(command);
			display.setCurrent(this);
		} else {
			this.setTitle("");
			this.setString("");
			this.removeCommand(command);
			display.setCurrent(null);
		}
	}

	public void commandAction(Command c, Displayable d) {
		if (c == command) {
			back();
		}
	}

	protected void back() {
		if (backButtonText == Locale.Beenden) {
			Pushandride.getMidlet().exitProgramm();
		} else {
			display.setCurrent(GameCanvasInstance.getInstance());
			GameCanvasInstance.getInstance().setFullScreenMode(true);
		}
	}
}
