// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.utils;

import javax.microedition.lcdui.Font;

import model.AbstractLocation;
import model.Haltestelle;
import model.Ort;
import control.Global;

public class GfxStringUtils {
	private static String KEIN_STRING = "";

	/**
	 * passt die Länge des Ortes an und gibt je nach Länge
	 * den vollen Namen zurück (Schwieberdingen) 
	 * den gespeicherten Shortname (Schwi)
	 * den ersten Buchstaben mit Punkt (S.)
	 * KEIN_STRING
	 */
	private static String getAdjustedOrt(Font f, Ort ort, int maxwidth) {
		if (maxwidth <= 0) {
			return KEIN_STRING;
		}

		String ortsName = ort.getScreenName();

		int ortsLength = f.stringWidth(ortsName);

		// komplette
		if (ortsLength <= maxwidth) {
			return ortsName;
		}

		// kurzer Ort
		ortsName = ort.getShortName();
		ortsLength = f.stringWidth(ortsName);
		if (ortsLength <= maxwidth) {
			return ortsName;
		}

		// sehr kurzer Ort
		ortsName = ortsName.charAt(0) + ".";
		ortsLength = f.stringWidth(ortsName);
		if (ortsLength <= maxwidth) {
			return ortsName;
		}

		return KEIN_STRING;
	}

	/**
	 *  passt die Länge der übergebenen Location an
	 *  zuerst wird der Ort gekürzt, danach die Haltestelle
	 */
	public static String getAdjustedLocation(Font f, AbstractLocation location, int maxwidth) {
		if (location instanceof Haltestelle) {
			Haltestelle h = (Haltestelle) location;

			Ort ort = h.getOrt();
			
			String haltestellenName = h.getScreenName();
			int haltestellenLength = f.stringWidth(haltestellenName);

			int spaceLength = f.stringWidth(" ");
			int maxOrtsWidth = maxwidth - haltestellenLength - spaceLength;

			// FIXME hack
			if (h.getUniqueId().equals(Global.haltestelleFlughafenUniqueId)) {
				if (maxwidth - haltestellenLength >= 0) {
					return haltestellenName;
				}
			}
			
			String ortsName = getAdjustedOrt(f, ort, maxOrtsWidth);
			
			if (!ortsName.equals(KEIN_STRING)) {
				return ortsName + " " + haltestellenName;
			} else {
				ortsName = ort.getScreenName().charAt(0) + ".";
				if (h.getUniqueId().equals(Global.haltestelleFlughafenUniqueId)) {
					ortsName = "";
				}
				int ortsLength = f.stringWidth(ortsName);
				
				int pointLength = f.stringWidth(".");

				// kurzer Haltestellenname;
				int freeHaltestellenSpace = maxwidth - ortsLength - spaceLength - pointLength;
				for (int i = haltestellenName.length() - 1; i >= 0; i--) {
					haltestellenName = haltestellenName.substring(0, i);
					haltestellenLength = f.stringWidth(haltestellenName);
					if (haltestellenLength <= freeHaltestellenSpace) {
						break;
					}
				}
				return ortsName + " " + haltestellenName + ".";	
			} 
		} else {
			Ort ort = (Ort) location;
			return getAdjustedOrt(f, ort, maxwidth);
		}
	}
}
