// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.utils;

import gui.launch.GameCanvasInstance;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextField;

import control.Pushandride;
import control.locale.Locale;

/**
 * Frägt eine Telefonnummer ab und sendet eine SMS an diese Nummer
 */
public abstract class PhoneNumberInputGui extends Thread implements CommandListener {

	static final Command backCommand = new Command(Locale.Back, Command.BACK, 1);
	static final Command sendCommand = new Command("Senden", Command.OK, 2);

	protected TextField numberInputField = new TextField(null, null, 20, TextField.PHONENUMBER);
	private StringItem userInfo = new StringItem(null, null);

	private Display display;
	protected Displayable last;
	protected Form mainForm;

	/**
	 * Frägt eine Telefonnummer ab und sendet eine SMS an diese Nummer
	 * 
	 * @param last
	 * @param textFieldMsg
	 */
	public PhoneNumberInputGui(Displayable last, String title) {
		this.last = last;
		mainForm = new Form(title);
		mainForm.append(userInfo);
		mainForm.append(numberInputField);
		mainForm.addCommand(backCommand);
		mainForm.addCommand(sendCommand);
		mainForm.setCommandListener(this);
		display = Display.getDisplay(Pushandride.getMidlet());
	}

	public void setVisible(boolean b) {
		if (b)
			display.setCurrent(mainForm);
		else {
			if (mainForm.isShown()) {
				mainForm.deleteAll();
				display.setCurrent(null);
			}
		}
	}

	protected void setNumberInputFieldLabel(String label) {
		numberInputField.setLabel(label);
	}

	protected void setUserInfo(String label, String msg) {
		userInfo.setLabel(label);
		userInfo.setText(msg);
	}

	public void commandAction(Command c, Displayable d) {
		if (c == backCommand) {
			back();
		} else if (c == sendCommand) {
			start();
		}
	}

	protected void back() {
		display.setCurrent(last);
		GameCanvasInstance.getInstance().setFullScreenMode(true);
	}

	public void run() {
		sendSMS();
	}

	/**
	 * Diese Methode muss so implementiert werden, dass eine SMS an die Telefonnummer
	 * "textField.getString()" gesendet wird
	 */
	protected abstract void sendSMS();
}
