// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package gui.utils;

import gui.launch.GameCanvasInstance;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.TextBox;
import javax.microedition.lcdui.TextField;

import control.Device;
import control.Pushandride;
import control.connections.sms.SMS;
import control.locale.Locale;

public class SMSWithInput extends Thread implements CommandListener {

	private static final Command backCommand = new Command(Locale.Back, Command.BACK, 1);
	private static final Command sendCommand = new Command("Senden", Command.SCREEN, 2);

	private final Display display;
	private final Displayable last;
	private final TextBox textBox;
	private final String title;
	private final String number;

	public SMSWithInput(String title, String targetNumber, String presetText) {
		this.title = title;
		this.number = targetNumber;
		this.textBox = new TextBox(title, presetText, 160, TextField.ANY);
		this.textBox.setCommandListener(this);
		this.display = Display.getDisplay(Pushandride.getMidlet());
		this.last = display.getCurrent();
		setVisible(true);
	}

	public void setVisible(boolean visible) {
		if (visible) {
			textBox.setTitle(title);
			display.setCurrent(textBox);
			textBox.addCommand(backCommand);
			textBox.addCommand(sendCommand);
			display.setCurrent(textBox);
		} else if (textBox.isShown()) {
			textBox.setTitle("");
			textBox.setString("");
			textBox.removeCommand(backCommand);
			textBox.removeCommand(sendCommand);
			display.setCurrent(null);
		}
	}

	public void commandAction(Command c, Displayable d) {
		if (c == backCommand) {
			back();
		} else if (c == sendCommand) {
			if (textBox.getString().trim().length() > 0) {
				start();
			} else {
				new AlertMessage("Bitte Text eingeben.", "Bitte Text eingeben.", "Zurück").setVisible(true);
			}
		}
	}

	private void back() {
		display.setCurrent(last);
		GameCanvasInstance.getInstance().setFullScreenMode(true);
	}

	public void run() {
		// Muss vor dem setVisible(false) gemacht werden.
		String msg = textBox.getString();

		this.setVisible(false);
		if (Device.hasMessaging()) {
			new AlertMessage("SMS senden nicht möglich", "Senden von SMS wird von diesem Gerät nicht unterstützt.", "Zurück").setVisible(true);
		}

		if (SMS.sendSMS(number, msg)) {
			new AlertMessage("Erfolgreich gesendet", "Deine SMS wurde erfolgreich gesendet.\nVielen Dank!", "Zurück").setVisible(true);
		} else {
			new AlertMessage("SMS senden nicht möglich", "Tut und leid, der SMS-Versand ist fehlgeschlagen.", "Zurück").setVisible(true);
		}
	}
}
