// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package model;

import dev.Memory;

public abstract class AbstractLocation {
	
	// NULL-Werte
	public static final byte VERKEHRSMITTEL_KEINE = (byte) 0;

	private byte verkehrsmittelMask;
	
	public AbstractLocation(byte verkehrsmittelMask) {
		this.verkehrsmittelMask = verkehrsmittelMask;
	}

	public AbstractLocation() {
		this.verkehrsmittelMask = 0;
	}

	public abstract String getScreenName();

	/**
	 * IntelligentKey
	 */
	int[] intelligentKeyCondensedValues;
	
	public int[] getIntelligentKeyCondensedValues() {
		return intelligentKeyCondensedValues;
	}
	
	public void setIntelligentKeyCondensedValues(int[] intelligentKeyCondensedValues) {
		this.intelligentKeyCondensedValues = intelligentKeyCondensedValues;
	}

	/**
	 * Hat dieses Item (Ort bzw. Haltestelle) Anschluss an eine Bus-Verbindung?
	 */
	public boolean hasBus() {
		return (verkehrsmittelMask & (1 << Verkehrsmittel.BUS)) != 0;
	}

	/**
	 * Hat dieses Item (Ort bzw. Haltestelle) Anschluss an eine SBahn-Verbindung?
	 */
	public boolean hasSBahn() {
		return (verkehrsmittelMask & (1 << Verkehrsmittel.SBAHN)) != 0;
	}

	/**
	 * Hat dieses Item (Ort bzw. Haltestelle) Anschluss an eine UBahn-Verbindung?
	 */
	public boolean hasUBahn() {
		return (verkehrsmittelMask & (1 << Verkehrsmittel.UBAHN)) != 0;
	}

	/**
	 * Hat dieses Item (Ort bzw. Haltestelle) Anschluss an eine RegionalBahn-Verbindung?
	 */
	public boolean hasRegionalBahn() {
		return (verkehrsmittelMask & (1 << Verkehrsmittel.RBAHN)) != 0;
	}

	/**
	 * Hat dieses Item (Ort bzw. Haltestelle) Anschluss an den angefragten Verbindungs-Typ?
	 * 
	 * @param typ -
	 *            Verbindungstyp (SBahn,UBahn,RegionalBahn,Bus)
	 * @return
	 */
	public boolean hasAnschluss(int typ) {
		return (verkehrsmittelMask & (1 << typ)) != 0;
	}

	/**
	 * Liefert einen String zurück mit allen gegebenen Anschlussmöglichkeiten Bei S-Bahn und R-Bahn
	 * zum Beispiel 'SR'
	 */
	public String getAnschlussString() {
		String s;
		s = "";
		for (int typ = 0; typ < Verkehrsmittel.ANSCHLUSSTYP.length; typ++) {
			if (hasAnschluss(typ)) {
				s += Verkehrsmittel.ANSCHLUSSTYP[typ];
			}
		}
		return s;
	}

	/**
	 * Liefert einen int[] zurück mit allen gegebenen Anschlussmöglichkeiten
	 */
	public int[] getVerkehrsmittelTypen() {
		int[] preResult = new int[Verkehrsmittel.VERKEHRSMITTEL_NUMMERN.length];
		int cnt = 0;
		for (int i = 0; i < Verkehrsmittel.VERKEHRSMITTEL_NUMMERN.length; i++) {
			if (i == Verkehrsmittel.LAUFWEG) {
				continue;
			}
			if ((this.verkehrsmittelMask & (1 << i)) != 0) {
				preResult[cnt++] = i;
			}
		}
		int[] result = new int[cnt];
		System.arraycopy(preResult, 0, result, 0, cnt);
		return result;
	}
	
	/**
	 * ein EINZELNES Verkehrsmittel kann hinzugefügt werden
	 * z.B: addVerkehrsmittel(Global.SBAHN);
	 * Mehrfachaufrufe mit dem selben Verkehrsmittel werden ignorieren
	 */
	public void addVerkehrsmittel(int verkehrsmittel) {
		this.verkehrsmittelMask |= 1 << verkehrsmittel; 
	}

	/**
	 * eine komplette Verkehrsmittelmaske kann der Location hinzugefügt werden
	 * NICHT 1 << Global.SBAHN 
	 */
	public void addVerkehrsmittel(byte verkehrsmittelMask) {
		this.verkehrsmittelMask |= verkehrsmittelMask; 
	}
	
	public byte getVerkehrsmittelMask() {
		return this.verkehrsmittelMask;
	}
	
	public void clearVerkehrsmittel() {
		this.verkehrsmittelMask = 0;
	}

	public int getMemory() {
		int mem = Memory.getObjSize();
		
		mem += Memory.getByteSize();
		
		return mem;
	}
}
