// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package model;

/**
 * Das Objekt HaltestellenPaar repräsentiert zwei Haltestellen
 */
public class HaltestellenPaar {
	
	// NULL-Werte
	public static final HaltestellenPaar NULL = new HaltestellenPaar(Haltestelle.NULL, Haltestelle.NULL);
	
	protected final Haltestelle von;
	protected final Haltestelle nach;

	public HaltestellenPaar(Haltestelle von, Haltestelle nach) {
		this.von = von;
		this.nach = nach;
	}

	/**
	 * Gibt die Haltestelle zurueck, von der diese Abfrage gestartet ist (Anfangsort)
	 * 
	 * @return von - Haltestelle, von der diese Abfrage gestartet ist (Anfangsort)
	 */
	public Haltestelle getVon() {
		return this.von;
	}

	/**
	 * Gibt die Haltestelle zurueck, an der diese Abfrage geendet ist (Zielort)
	 * 
	 * @return nach - Haltestelle, an der diese Abfrage geendet ist (Zielort)
	 */
	public Haltestelle getNach() {
		return this.nach;
	}
	
	/**
	 * Gibt ein NEUES Objekt zurueck, welches von und nach vertauscht enthaelt.
	 * Das HaltestllenPaar auf dem die Funktion verwendet wird, wird NICHT
	 * veraendert!
	 */
	public HaltestellenPaar switchVonNach() {
		return new HaltestellenPaar(this.nach, this.von);
	}

	/**
	 * Gibt den String des Haltestellenpaars zurück (z.B. 'Stuttgart Stadtmitte, Stuttgart
	 * Universität')
	 */
	public String getPaarAsString() {
		return this.von.getFullName() + ", " + this.nach.getFullName();
	}

	public int hashCode() {
		int hash = 7;
		hash = 53 * hash + (this.von != null ? this.von.hashCode() : 0);
		hash = 53 * hash + (this.nach != null ? this.nach.hashCode() : 0);
		return hash;
	}

	/**
	 * Zwei Objekte HaltestellenPaar sind gleich, wenn beide Haltestellen uebereinstimmen
	 */
	public boolean equals(Object obj) {
		if (obj instanceof HaltestellenPaar) {
			boolean uebereinstimmung = this.von.getId() == ((HaltestellenPaar) obj).von.getId() && this.nach.getId() == ((HaltestellenPaar) obj).nach.getId();
			return uebereinstimmung;
		}
		return super.equals(obj);
	}

	public String toString() {
		if (this == HaltestellenPaar.NULL) {
			return "NULL_HALTESTELLEN_PAAR";
		}
		String vonString;
		String nachString;
		if (this.von == null) {
			vonString = "NULL";
		} else {
			vonString = this.von.getScreenName();
		}
		if (this.nach == null) {
			nachString = "NULL";
		} else {
			nachString = this.nach.getScreenName();
		}
		return vonString + " -> " + nachString;
	}
	
	/**
	 * prüft, ob eine der beiden Haltestellen null oder NULL ist
	 */
	public boolean isValid() {
		if(von == null || nach == null || von == Haltestelle.NULL || nach == Haltestelle.NULL) {
			return false;
		} else {
			return true;
		}
	}
}
