// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package model;

import control.Global;
import data.Data;

public class IFahrtStatic {
	
	private static final Haltestelle SONDERFALL_CHARLOTTENPLATZ = Data.getHaltestelleByUniqueId("6075");
	
	/**
	 * Gibt die Zeit zurück, welche für einen Umstieg zwischen den beiden übergebenen Linien nötig ist.<br>
	 * Die Funktion bedenkt auch wenn keine Umsteigezeit nötig ist und gibt entsprechend 0 zurück.<br>
	 * Die Reihenfolge der Linien ist egal.
	 * 
	 * @param linie1
	 * @param linie2
	 * @return Umsteigezeit
	 */
	public static int getUmsteigezeit(Linie linie1, Linie linie2, Haltestelle aktuelleHaltestelle) {
		/**
		 * Keine Strafzeit fürs umsteigen wenn:<br>
		 * + Letzte Linie null war (Startknoten von Dijkstra)<br>
		 * + letzte Linie ein Laufweg war<br>
		 * + nächste Linie ein Laufweg war<br>
		 * + Weiterfahen auf der gleichen Linie
		 */
		if(linie1 == null
			|| linie2 == null
			|| (linie1 != null && linie1.getVerkehrsmittel() == Verkehrsmittel.LAUFWEG)
			|| (linie2 != null && linie2.getVerkehrsmittel() == Verkehrsmittel.LAUFWEG)
			|| linie1 == linie2) {
			return 0;
		}
		
		/**
		 * Wir müssen also eine Zeit zum Umsteigen berechnen.
		 */
		if(linie1.getVerkehrsmittel() != linie2.getVerkehrsmittel()) {
			// Wenn wir auf oder von einem Bus umsteigen verwenden wir nochmal eine höhere Umsteigezeit
			// da die Buss normalerweise weiter entfernt sind von der eigentlichen Haltestelle als bei den
			// anderen Umstiegen.
			if(linie1.getVerkehrsmittel() == Verkehrsmittel.BUS || linie2.getVerkehrsmittel() == Verkehrsmittel.BUS) {
				return Global.UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL_VONAUF_BUS;
			} else {
				return Global.UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL;
			}
		} else {
			//
			if(aktuelleHaltestelle == SONDERFALL_CHARLOTTENPLATZ) {
				// Oben: U5/6/7/15
				// Unten: U1/2/4
				int linie1Nummer = Integer.parseInt(linie1.getName().substring(1));
				int linie2Nummer = Integer.parseInt(linie2.getName().substring(1));
				if((linie1Nummer < 5 && linie2Nummer >= 5) || (linie2Nummer < 5 && linie1Nummer >= 5)) {
					return Global.UMSTEIGEZEIT_VERKEHRSMITTELWECHSEL;
				}
			}
			
			return Global.UMSTEIGEZEIT;
		}
	}
}
