// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package model;

import control.Global;
import data.Data;
import dev.Memory;

public class Laufweg implements IFahrt {

	// Laufzeit ohne Faktor
	private final byte laufzeitOhneFaktor;
	private final Haltestelle haltestelleA;
	private final Haltestelle haltestelleB;

	/**
	 * Konstruktor Wird mit super an die Klasse Fahrt weitergereicht.
	 * factor == 10 enstpricht 1
	 * factor ==  1 entspricht 0.1
	 * factor == 50 entspricht 5
	 */
	public Laufweg(Haltestelle haltestelleA, Haltestelle haltestelleB, byte laufzeitOhneFaktor) {
		this.haltestelleA = haltestelleA;
		this.haltestelleB = haltestelleB;
		this.laufzeitOhneFaktor = laufzeitOhneFaktor;
	}

	/**
	 * getAbfahrtAfterTime() Überschreiben, weil man bei Laufwegen direkt
	 * loslaufen kann und nicht warten muss.
	 */
	public int getAbfahrtAfterTime(Haltestelle haltestelle, int zeit) {
		return zeit;
	}

	/**
	 * Überschreiben, weil es grundsätzlich anders funktioniert, als bei der
	 * Fahrt. Es reicht einfach die Laufzeit zu addieren.
	 */
	public int getTimeAtNextHaltestelle(Haltestelle act, int zeit, Linie linie) {
		return zeit + getLaufzeit();
	}
	
	/**
	 * gibt die Laufzeit zurück, die evtl. mit dem Laufzeitmultiplikator versehen ist
	 */
	public int getLaufzeit() {
		return calcLaufzeit(laufzeitOhneFaktor, Global.LAUFWEGE_ZEIT_FAKTOR);
	}
	
	public static int calcLaufzeit(byte laufzeit, int factor) {
		int i = laufzeit * factor;
		int result = ((i+5) / 10);
		return  Math.max(1, result);
	}
	
	/**
	 * gibt die Laufzeit zurück, mit der das Objekt initialisiert wurde
	 */
	public byte getBaseLaufzeit() {
		return laufzeitOhneFaktor;
	}
	
	public boolean isLaufweg() {
		return true;
	}

	
	public int getMemory() {
		int mem = Memory.getObjSize();

		mem += Memory.getObjSize();

		mem += Memory.getObjSize();

		mem += Memory.getByteSize();
		
		return mem;
	}


	public Linie getLinie() {
		return Linie.LAUFWEG_LINIE;
	}

	public String toString() {
		return "Laufweg " + Data.getIdOfFahrt(this) + " " + haltestelleA + " <-> " + haltestelleB + " in " + getLaufzeit() + " (base: " + getBaseLaufzeit() + ")";
	}

	public Haltestelle getNextHaltestelle(Haltestelle haltestelle) {
		if (haltestelle == haltestelleA) {
			return haltestelleB;
		} else {
			return haltestelleA;
		}
	}

	public Haltestelle getNextHaltestelleReverse(Haltestelle haltestelle) {
		if (haltestelle == haltestelleA) {
			return haltestelleB;
		} else {
			return haltestelleA;
		}
	}

	public int getTimeToNextHaltestelle(Haltestelle haltestelleVon) {
		return getLaufzeit();
	}

	public int getTimeAtNextHaltestelleReverse(Haltestelle act, int zeit, Linie lastUsedLinie) {
		return zeit - getLaufzeit();
	}

	public Haltestelle[] getAlleHaltestellen() {
		return new Haltestelle[] {haltestelleA, haltestelleB};
	}

	public String getLinienInfo() {
		if (getLaufzeit() == getBaseLaufzeit()) {
			return "Laufweg " + getLaufzeit();
		} else {
			return "Laufweg " + getLaufzeit() + " (base: " + getBaseLaufzeit() + ")";
		}
	}
}
