// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package model;

import dev.Memory;

public class Linie {
	
	// NULL- und andere besondere Werte
	public static final Linie NULL = new Linie("NULL-LINIE", null, -1);	
	public static final Linie NULL_RUECKLINIE = new Linie("NULL-RUECKLINIE", null, -2);
	public static final Linie LAUFWEG_LINIE = new Linie("Laufweg", "Laufweg", -3);

	private final String name;
	private final String fahrstrecke;
	private final int id;

	public Linie(String name, String fahrstrecke, int id) {
		this.name = name;
		this.fahrstrecke = fahrstrecke;
		this.id = id;
	}

	/**
	 * Gibt wirklich NUR den Namen der Linie zurück (z.B. S6) Wenn man den
	 * ausführlichen Namen will, muss man getScreenName() verwenden.
	 * 
	 * @return
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Gibt den Name der Linie, der bei der Ausgabe angezeigt werden soll
	 * zurück. (z.B. S6 Schwabstraße)
	 * 
	 * @return
	 */
	public String getScreenName() {
		return (this.name + " " + this.fahrstrecke);
	}

	public String getFahrstrecke() {
		return this.fahrstrecke;
	}

	/**
	 * Gibt die Id des Verkehrsmittels zurück (z.B: 0 == SBAHN)
	 */
	// FIXME: Verkehrsmittel sind short, diese Funktion umstellen.
	public int getVerkehrsmittel() {
		if ((this.name == null) || (this.name.equals(""))) {
			return -1;
		}
		for (int i = 0; i < Verkehrsmittel.ANSCHLUSSTYP.length; i++) {
			if (this.name.charAt(0) == Verkehrsmittel.ANSCHLUSSTYP[i]) {
				return i;
			}
		}
		if (this.name.charAt(0) >= '0' && this.name.charAt(0) <= '9') {
			return Verkehrsmittel.BUS;
		}
		return -1;
	}

	public int getId() {
		return id;
	}

	public int getMemory() {
		int mem = Memory.getObjSize();

		mem += Memory.getStringBaseSize() + name.length() * Memory.getStringCharakterSize();

		mem += Memory.getStringBaseSize() + fahrstrecke.length() * Memory.getStringCharakterSize();

		mem += Memory.getObjSize();

		mem += Memory.getIntSize();

		return mem;
	}
	
	public String toString() {
		return this.getScreenName();
	}
}
