// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package model;

public class RoutenAbschnitt extends HaltestellenPaar {

	private final int abfahrtsZeit;
	private final int ankunftsZeit;
	private final IFahrt fahrt;

	public RoutenAbschnitt(Haltestelle von, Haltestelle nach, int abfahrtsZeit, int ankunftsZeit, IFahrt fahrt) {
		super(von, nach);
		this.abfahrtsZeit = abfahrtsZeit;
		this.ankunftsZeit = ankunftsZeit;
		this.fahrt = fahrt;
	}

	public RoutenAbschnitt(Haltestelle von, Haltestelle nach, IFahrt fahrt) {
		super(von, nach);
		this.abfahrtsZeit = DatumZeit.NULL;
		this.ankunftsZeit = DatumZeit.NULL;
		this.fahrt = fahrt;
	}

	public int getAbfahrtsZeit() {
		return abfahrtsZeit;
	}

	public int getAnkunftsZeit() {
		return ankunftsZeit;
	}

	public int getFahrZeit() {
		return ankunftsZeit - abfahrtsZeit;
	}

	public IFahrt getFahrt() {
		return fahrt;
	}
	
	public boolean isLaufweg() {
		return this.getFahrt().isLaufweg();
	}

	public String toString() {
		return "RoutenAbschnitt: " + von.getId() + "-" + von.getFullName() + ">>>TO>>>" + nach.getId() + "-" + nach.getFullName() + " @ " + DatumZeit.toDateTimeString(abfahrtsZeit) + ">>>" + (ankunftsZeit - abfahrtsZeit) + "min>>>" + DatumZeit.toDateTimeString(ankunftsZeit) + ">>>USING>>>" + fahrt;
	}
}
