// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package utils;

import java.util.Enumeration;
import java.util.Vector;

import model.Haltestelle;

public class Queue {
	Vector theQueue = new Vector();
	int[] distance = null;
	boolean reverse = false;

	public Queue(int[] dist) {
		this.distance = dist;
	}
	
	public Queue(int[] dist, boolean reverse) {
		this.distance = dist;
		this.reverse = reverse;
	}

	/**
	 * Zerstoerendes getNext: Element wird geloescht.
	 * 
	 * @return
	 */
	public Haltestelle getNext() {
		Haltestelle closestHaltestelle = null;
		// Siehe Kommentar unten
		int closestHaltestelleDist = (reverse) ? Integer.MIN_VALUE : Integer.MAX_VALUE;
		
		for (Enumeration e = theQueue.elements(); e.hasMoreElements();) {
			Haltestelle tmpHaltestelle = (Haltestelle) e.nextElement();
			int tmpHaltestelleDist = distance[tmpHaltestelle.getId()];
			
			/* Fallunterscheidung Reverse vs. Normale Queue
			 * Reverse: Da wir rückwärts laufen, ist das Ziel so spät wie
			 *	möglich anzukommen, was aus der reverse Sicht ja wieder das
			 *  frühste ist.
			 * Normal: Wir wollen so früh wie möglich ankommen und geben deshalb
			 *  das kleinste Element zurück.
			*/
			if ((closestHaltestelle == null)
					|| ((reverse) ? (tmpHaltestelleDist > closestHaltestelleDist) : (tmpHaltestelleDist < closestHaltestelleDist))) {
				closestHaltestelle = tmpHaltestelle;
				closestHaltestelleDist = distance[closestHaltestelle.getId()];
			}
		}
		remove(closestHaltestelle);
		return closestHaltestelle;
	}

	public boolean isEmpty() {
		return this.theQueue.isEmpty();
	}

	public void add(Haltestelle act) {
		theQueue.addElement(act);
	}

	public void remove(Haltestelle act) {
		theQueue.removeElement(act);
	}

	public boolean contains(Haltestelle nextHaltestelle) {
		return theQueue.contains(nextHaltestelle);
	}
}
