// This file is part of push&ride.

// Copyright 2006-2011 Tobias Binz, Andreas Moosbrugger, Michael Schlagmüller

// push&ride is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
// The application is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with push&ride.  If not, see <http://www.gnu.org/licenses/>.

package utils;

import java.util.Vector;

import dev.Logger;

public class StringUtils {

	/**
	 * Füllt einen gegebenen String mit einem übergebenen Charakter 'c' auf die
	 * Mindestlänge 'anz' auf
	 */
	public static String fillStringFront(String s, char c, int anz) {
		while (s.length() < anz) {
			s = c + s;
		}
		return s;
	}

	/**
	 * füllt einen gegebenen String mit einem übergebenen Charakter 'c' auf die
	 * Mindestlänge 'anz' auf
	 */
	public static String fillStringBack(String s, char c, int anz) {
		while (s.length() < anz) {
			s = s + c;
		}
		return s;
	}

	/**
	 * wie substring - nur kann der String auch kürzer sein als (endIndex + 1).
	 * Der String wird dann mit Leerzeichen aufgefüllt
	 */
	public static String safeSubString(String s, int beginIndex, int endIndex) {
		String result = fillStringBack(s, ' ', endIndex + 1);
		return result.substring(beginIndex, endIndex);
	}

	/**
	 * füllt einen gegebenen integer-Wert mit einem übergebenen Charakter 'c'
	 * auf die Mindestlänge 'anz' auf
	 */
	public static String fillInt(int s, char c, int anz) {
		return StringUtils.fillStringFront(s + "", c, anz);
	}

	/**
	 * füllt einen gegebenen integer-Wert mit einem übergebenen Charakter 'c'
	 * auf die Mindestlänge 'anz' auf
	 */
	public static String fillInt(long s, char c, int anz) {
		return StringUtils.fillStringFront(s + "", c, anz);
	}

	/**
	 * gibt einen integer Wert 'value' als Bitfolge aus
	 */
	public static void printBits(int value) {
		for (int i = 31; i >= 0; i--) {
			if ((value & (1 << i)) != 0) {
				System.out.print("1");
			} else {
				System.out.print("0");
			}
			if (i % 4 == 0) {
				System.out.print(" ");
			}
		}
		System.out.print("  " + value + "\n");
	}

	/**
	 * gibt einen integer Wert 'value' als Bitfolge aus, wobei mit 'decimal' die
	 * Kommastelle angegeben wird
	 */
	public static void printBits(int value, int decimal) {
		for (int i = 31; i >= 0; i--) {
			if ((value & (1 << i)) != 0) {
				System.out.print("1");
			} else {
				System.out.print("0");
			}
			if (i % 4 == 0) {
				System.out.print(" ");
			}
		}
		System.out.print("  " + (value / (double) (1 << decimal)) + "\n");
	}

	/**
	 * @param data
	 *            String[] to check if contains is equal to one of the strings
	 *            !!!WARNING!!! nur wenn der volle String gleich ist wird true
	 *            zurückgegeben
	 * @param contains
	 * @return false of one or both of the inputs is null or empty
	 */
	public static boolean containsFull(String[] data, String contains) {
		if (data == null || data.length == 0 || contains == null || contains.equals("")) {
			return false;
		}
		for (int i = 0; i < data.length; i++) {
			if (data[i].equals(contains)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Splittet einen String am Trennzeichen in einzelnen Strings auf und gibt
	 * diese zurück.
	 * 
	 * @param string
	 *            zu trennender String
	 * @param trennzeichen
	 *            Trennzeichenkette (kann eins oder mehrere Zeichen umfassen)
	 * @return Ergebnisvector mit den einzelnen Elementen oder null wenn eine
	 *         der Eingaben leer oder null war. Falls das Trennzeichen nicht
	 *         gefunden wurde, wird ein String Vector mit dem gesamten string
	 *         als erstes Element zurückgegeben.
	 */
	public static Vector splitAsVector(String string, String trennzeichen) {
		if (string != null && trennzeichen != null && !string.equals("") && !trennzeichen.equals("")) {
			Vector results = new Vector();
			int index;
			while ((index = string.indexOf(trennzeichen)) != -1) {
				// Der Substring vor dem Trennzeichen wird zum Result hinzugefügt.
				results.addElement(string.substring(0, index));
				// Der Rest des Strings, abzüglich des Trennzeichens wird weiterverarbeitet.
				string = string.substring(index + trennzeichen.length(), string.length());
			}
			// Der Rest, oder der gesamte String falls das Trennzeichen nicht gefunden wurde wird als Element
			// zurückgegeben.
			results.addElement(string);
			return results;
		} else {
			return null;
		}
	}

	/**
	 * Splittet einen String am Trennzeichen in einzelnen Strings auf und gibt
	 * diese zurück.
	 * 
	 * @param string
	 *            zu trennender String
	 * @param trennzeichen
	 *            Trennzeichenkette (kann eins oder mehrere Zeichen umfassen)
	 * @return Ergebnisarray mit den einzelnen Elementen oder null wenn eine der
	 *         Eingaben leer oder null war. Falls das Trennzeichen nicht
	 *         gefunden wurde, wird ein String Array mit dem gesamten string als
	 *         erstes Element zurückgegeben.
	 */
	public static String[] split(String string, String trennzeichen) {
		Vector results = splitAsVector(string, trennzeichen);
		if (results != null) {
			String[] stringResults = new String[results.size()];
			results.copyInto(stringResults);
			return stringResults;
		} else {
			return null;
		}
	}

	/**
	 * Wandelt eine String Array in einen String um in dem die einzelnen Array
	 * Elemente durch das Trennzeichen getrennt sind.
	 * <p>
	 * Achtung: Falls das Trennzeichen in einem der Strings vorkommt wird es
	 * gelöscht, damit es später beim splitten zu keinen Problemen kommt!
	 * 
	 * @param array
	 * @param trennzeichen
	 *            ein oder mehrere Zeichen
	 * @return String
	 */
	public static String arrayToString(String[] array, String trennzeichen) {
		String result = "";
		for (int i = 0; i < array.length; i++) {
			if (i != 0) {
				result += trennzeichen;
			}
			result += StringUtils.replace(array[i], trennzeichen, "");
		}
		return result;
	}

	public static String arrayToString(int[] data) {
		String result = "(";
		for (int i = 0; i < data.length; i++) {
			if (i != 0) {
				result += ", ";
			}
			result += data[i];
		}
		result += ")";
		return result;
	}

	/**
	 * replace Ersetzt alle Vorkommen von toReplace in einem Text durch
	 * replaceWith
	 * 
	 * @param text
	 * @param toReplace
	 *            - der zu ersetzende String
	 * @param replaceWith
	 *            - der ersetzende String
	 * @return String
	 */
	public static String replace(String text, String toReplace, String replaceWith) {
		if (text == null || text.length() == 0) {
			return text;
		}
		int index = -1;
		do {
			index = text.indexOf(toReplace, index + 1);
			if (index != -1) {
				String tmp1 = text.substring(0, index);
				String tmp2 = text.substring(index + toReplace.length(), text.length());
				text = tmp1 + replaceWith + tmp2;
			}
		} while (index != -1);
		return text;
	}

	/**
	 * UTF8_encode Ersetzt alle Umlaute, Leerzeichen und Scharf-S in einem Text
	 * durch die entsprechenden UTF8-Kodierungen
	 * 
	 * @param text
	 *            der zu enkodierende String
	 * @return String der enkodierte String
	 */
	public static String UTF8_encode(String text) {
		text = replace(text, " ", "%20");
		text = replace(text, "ä", "%C3%A4");
		text = replace(text, "ö", "%C3%B6");
		text = replace(text, "ü", "%C3%BC");
		text = replace(text, "Ä", "%C3%84");
		text = replace(text, "Ö", "%C3%96");
		text = replace(text, "Ü", "%C3%9C");
		text = replace(text, "ß", "%C3%9F");
		return text;
	}

	/**
	 * UTF8_decode Ersetzt alle gefundenen UTF-Sonderzeichen in einem Text durch
	 * die entsprechenden Umlaute oder Scharf-S
	 * 
	 * @param text
	 *            der zu dekodierende String
	 * @return String der dekodierte String
	 */
	public static String UTF8_decode(String text) {
		text = replace(text, "Ã¤", "ä");
		text = replace(text, "Ã¶", "ö");
		text = replace(text, "Ã¼", "ü");
		text = replace(text, "Ã„", "Ä");
		text = replace(text, "Ã–", "Ö");
		text = replace(text, "Ãœ", "Ü");
		text = replace(text, "ÃŸ", "ß");
		return text;
	}

	/**
	 * zeigt die ersten 16 Bit einer Zahl als Charater an (z.B. value = 0x00F1,
	 * result = 0000 0000 1111 0001)
	 */
	public static String intToBitString(int value) {
		String result = "";
		for (int i = 15; i >= 0; i--) {
			if ((value & (1 << i)) != 0) {
				result += "1";
			} else {
				result += "0";
			}
			if (i % 4 == 0) {
				result += " ";
			}
		}
		return result;
	}

	public static String formatDouble(double value, int vorKomma, int nachKomma) {
		String result = "";
		for (int i = 0; i < nachKomma; i++) {
			value *= 10;
		}
		result = new Long((long) value).toString();
		result = fillStringFront(result, '0', nachKomma + 1);
		if (nachKomma > 0) {
			result = result.substring(0, result.length() - nachKomma) + "." + result.substring(result.length() - nachKomma);
			return fillStringFront(result, ' ', vorKomma + 1 + nachKomma);
		} else {
			return fillStringFront(result, ' ', vorKomma);
		}
	}

	public static boolean startsWith(String searchFor, String string) {
		if (searchFor.length() > string.length()) {
			return false;
		}
		return string.substring(0, searchFor.length()).equals(searchFor);
	}

	/**
	 * Returns true if and only if this string contains the specified sequence
	 * of char values.
	 * 
	 * @param string
	 *            the sequence to search in
	 * @param ausschnitt
	 *            the sequence to search for
	 * @return true if this string contains <code>s</code>, false otherwise
	 */
	public static boolean contains(String string, String ausschnitt) {
		int indexOf = indexOf(string.toCharArray(), 0, string.length(), ausschnitt.toCharArray(), 0, ausschnitt.length(), 0);
		return indexOf > -1;
	}

	/**
	 * Code shared by String and StringBuffer to do searches. The source is the
	 * character array being searched, and the target is the string being
	 * searched for.
	 * 
	 * @param source
	 *            the characters being searched.
	 * @param sourceOffset
	 *            offset of the source string.
	 * @param sourceCount
	 *            count of the source string.
	 * @param target
	 *            the characters being searched for.
	 * @param targetOffset
	 *            offset of the target string.
	 * @param targetCount
	 *            count of the target string.
	 * @param fromIndex
	 *            the index to begin searching from.
	 */
	private static int indexOf(char[] source, int sourceOffset, int sourceCount, char[] target, int targetOffset, int targetCount, int fromIndex) {
		if (fromIndex >= sourceCount) {
			return (targetCount == 0 ? sourceCount : -1);
		}
		if (fromIndex < 0) {
			fromIndex = 0;
		}
		if (targetCount == 0) {
			return fromIndex;
		}

		char first = target[targetOffset];
		int max = sourceOffset + (sourceCount - targetCount);

		for (int i = sourceOffset + fromIndex; i <= max; i++) {
			/* Look for first character. */
			if (source[i] != first) {
				while (++i <= max && source[i] != first)
					;
			}

			/* Found first character, now look at the rest of v2 */
			if (i <= max) {
				int j = i + 1;
				int end = j + targetCount - 1;
				for (int k = targetOffset + 1; j < end && source[j] == target[k]; j++, k++)
					;

				if (j == end) {
					/* Found whole string. */
					return i - sourceOffset;
				}
			}
		}
		return -1;
	}
	
	public static String trimToNumber(String s) {
		s = s.trim();
		boolean valid = true;
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c < '0' || c > '9') {
				valid = false;
			} 
		}
		if (valid) {
			return s;
		}
		// im Fehlerfall einen gültigen String erstellen
		String validString = ""; 
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c < '0' || c > '9') {
				// #160 = non breaking space
				if ((int)(c) != 160) {
					Logger.error(StringUtils.class, "trimToNumber:"+(int)c);
				}
			} else {
				validString += c;
			}
		}
		return validString;
	}
	
	public static String addDots(long l) {
		String result = l + "";
		int cnt = 0;
		for (int i = result.length() -1; i > 0; i--) {
			cnt++;
			if (cnt % 3 == 0 && cnt != 0) {
				result = result.substring(0, i) + "." + result.substring(i);
			}
		}
		return result;
	}

	public static String addDots(String l) {
		String result = l + "";
		int cnt = 0;
		for (int i = result.length() -1; i > 0; i--) {
			cnt++;
			if (cnt % 3 == 0 && cnt != 0) {
				result = result.substring(0, i) + "." + result.substring(i);
			}
		}
		return result;
	}

	/**
	 * Returns the first position of the specified sequence (-1 for none)
	 * 
	 * @param string
	 *            the sequence to search in
	 * @param ausschnitt
	 *            the sequence to search for
	 * @return 
	 */
	public static int firstPos(String string, String ausschnitt) {
		return indexOf(string.toCharArray(), 0, string.length(), ausschnitt.toCharArray(), 0, ausschnitt.length(), 0);
	}
}
